(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[98],{13584:function(e,n,r){"use strict";var l=r(49332);r(62462);r(46709);(0,l.MT)()},42747:function(e,n,r){"use strict";r.d(n,{Dk:function(){return l.Dk},sA:function(){return l.sA},tz:function(){return l.tz}});var l=r(77680);r(13584)},50098:function(e,n,r){"use strict";r.r(n),r.d(n,{AppComponent:function(){return v}});var l=r(46709),t=r(53108),o=r(42747),i=r(27757),a=r(73408);const d=({disabled:e=!1,children:n})=>(0,a.Y)(i.an,{isDarkMode:!1,children:n});window.matchMedia("(prefers-color-scheme: dark)");var c=r(51212),s=r(39595);r(4102),r(16725);const u=l.lazy((()=>Promise.all([r.e(5759),r.e(3368),r.e(7299),r.e(9491),r.e(1095),r.e(6722)]).then(r.bind(r,28189)).then((e=>({default:e.ModelTraceExplorerOSSNotebookRenderer}))))),f=()=>r.e(7299).then(r.bind(r,27299)),h=l.lazy((()=>f().then((e=>({default:e.DesignSystemProvider}))))),m=l.lazy((()=>f().then((e=>({default:e.ApplyGlobalStyles}))))),k=l.lazy((()=>f().then((e=>({default:e.DesignSystemEventProvider}))))),w=({children:e})=>(0,a.Y)(d,{children:(0,a.Y)(k,{callback:()=>{},children:(0,a.FD)(h,{children:[(0,a.Y)(m,{}),e]})})}),b={"databricks.fe.designsystem.useNewTagColors":!0,"databricks.fe.traceExplorer.enableSummaryView":!0},v=()=>{const e=(0,l.useMemo)((()=>new c.E),[]);return window.__databricks_mfe_rpc||Object.defineProperty(window,"__databricks_mfe_rpc",{configurable:!1,writable:!1,value:{makeCall:(e,n)=>(null===n||void 0===n?void 0:n.flagName)in b?b[null===n||void 0===n?void 0:n.flagName]:null===n||void 0===n?void 0:n.defaultValue,hasHandlerFor:()=>!0,registerHandler:()=>{},unregisterHandler:()=>{}}}),(0,a.Y)(l.Suspense,{fallback:null,children:(0,a.Y)(w,{children:(0,a.Y)(o.Dk,{locale:"en",children:(0,a.Y)(s.Ht,{client:e,children:(0,a.Y)(u,{})})})})})};t.render((0,a.Y)(v,{}),document.getElementById("root"))},51531:function(e){function n(e){return Promise.resolve().then((function(){var n=new Error("Cannot find module '"+e+"'");throw n.code="MODULE_NOT_FOUND",n}))}n.keys=function(){return[]},n.resolve=n,n.id=51531,e.exports=n}}]);