"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[76],{42762:function(e,t,n){n.d(t,{m:function(){return o}});var i=n(27299),a=n(28126),s=n(73408);const o=({sourceType:e,className:t})=>e===a.Y1.NOTEBOOK?(0,s.Y)(i.NotebookIcon,{className:t}):e===a.Y1.LOCAL?(0,s.Y)(i.FileCodeIcon,{className:t}):e===a.Y1.PROJECT?(0,s.Y)(i.FolderBranchIcon,{className:t}):e===a.Y1.JOB?(0,s.Y)(i.WorkflowsIcon,{className:t}):null},54741:function(e,t,n){n.d(t,{L:function(){return s}});var i=n(81641);const a=i.J1`
  query MlflowGetExperimentQuery($input: MlflowGetExperimentInput!) @component(name: "MLflow.ExperimentRunTracking") {
    mlflowGetExperiment(input: $input) {
      apiError {
        code
        message
      }
      experiment {
        artifactLocation
        creationTime
        experimentId
        lastUpdateTime
        lifecycleStage
        name
        tags {
          key
          value
        }
      }
    }
  }
`,s=({experimentId:e,options:t={}})=>{var n;const{data:s,loading:o,error:r,refetch:c}=(0,i.IT)(a,{variables:{input:{experimentId:e}},skip:!e,...t});return{loading:o,data:null===s||void 0===s||null===(n=s.mlflowGetExperiment)||void 0===n?void 0:n.experiment,refetch:c,apolloError:r,apiError:(()=>{var e;return null===s||void 0===s||null===(e=s.mlflowGetExperiment)||void 0===e?void 0:e.apiError})()}}},58355:function(e,t,n){n.d(t,{A:function(){return i}});class i{static getStoreForComponent(e,t){return new a([e,t].join("-"),"localStorage")}static getSessionScopedStoreForComponent(e,t){return new a([e,t].join("-"),"sessionStorage")}}i.version="1.1";class a{constructor(e,t){this.scope=void 0,this.storageObj=void 0,this.scope=e,this.storageObj="localStorage"===t?window.localStorage:window.sessionStorage}loadComponentState(){const e=this.getItem(a.reactComponentStateKey);return e?JSON.parse(e):{}}saveComponentState(e){const t="function"===typeof e.toJSON?e.toJSON():e;this.setItem(a.reactComponentStateKey,JSON.stringify(t))}withScopePrefix(e){return["MLflowLocalStorage",i.version,this.scope,e].join("-")}setItem(e,t){this.storageObj.setItem(this.withScopePrefix(e),t)}getItem(e){return this.storageObj.getItem(this.withScopePrefix(e))}}a.reactComponentStateKey="ReactComponentState"},71625:function(e,t,n){n.d(t,{t:function(){return l}});var i=n(68248),a=n(46709),s=n(11522),o=n(27757),r=n(42762),c=n(73408);var u={name:"1gz2b5f",styles:"overflow:hidden;text-overflow:ellipsis"};const l=a.memo((({value:e})=>{var t;const{theme:n}=(0,o.u)();if(!e)return(0,c.Y)(c.FK,{children:"-"});const a=(null===(t=e[s.A.sourceTypeTag])||void 0===t?void 0:t.value)||"",l=s.A.renderSource(e||{},void 0,void 0);return l?(0,c.FD)("div",{css:(0,i.AH)({display:"flex",gap:n.spacing.xs,alignItems:"center"},""),children:[(0,c.Y)(r.m,{sourceType:a,css:(0,i.AH)({color:n.colors.textSecondary},"")}),(0,c.Y)("span",{css:u,children:l})]}):(0,c.Y)(c.FK,{children:"-"})}))},82168:function(e,t,n){n.r(t);var i=n(46709),a=n(59930),s=n.n(a),o=n(91105),r=n(10363),c=n(73408);t.default=()=>{const{experimentId:e}=(0,o.g)();s()(e,"Experiment ID must be defined");const t=(0,i.useMemo)((()=>[e]),[e]);return(0,c.Y)(r.$,{experimentIds:t})}},91944:function(e,t,n){n.d(t,{$g:function(){return d},BH:function(){return o},UI:function(){return l},bf:function(){return E},kk:function(){return s},q5:function(){return u},qp:function(){return c},uk:function(){return r}});var i=n(44105),a=n(77680);const s="mlflow.experimentKind",o=e=>{var t;return null===e||void 0===e||null===(t=e.find((e=>e.key===s)))||void 0===t?void 0:t.value},r=e=>e===i.Gk.GENAI_DEVELOPMENT_INFERRED||e===i.Gk.CUSTOM_MODEL_DEVELOPMENT_INFERRED||e===i.Gk.NO_INFERRED_TYPE||e===i.Gk.GENAI_DEVELOPMENT||e===i.Gk.CUSTOM_MODEL_DEVELOPMENT||e===i.Gk.EMPTY,c=e=>e===i.Gk.GENAI_DEVELOPMENT||e===i.Gk.GENAI_DEVELOPMENT_INFERRED,u=e=>e===i.Gk.GENAI_DEVELOPMENT_INFERRED?i.Gk.GENAI_DEVELOPMENT:e===i.Gk.CUSTOM_MODEL_DEVELOPMENT_INFERRED?i.Gk.CUSTOM_MODEL_DEVELOPMENT:e,l={[i.Gk.GENAI_DEVELOPMENT]:(0,a.zR)({id:"PewjIJ",defaultMessage:"GenAI apps & agents"}),[i.Gk.CUSTOM_MODEL_DEVELOPMENT]:(0,a.zR)({id:"nF10K1",defaultMessage:"Machine learning"}),[i.Gk.GENAI_DEVELOPMENT_INFERRED]:(0,a.zR)({id:"ACNZd9",defaultMessage:"GenAI apps & agents"}),[i.Gk.CUSTOM_MODEL_DEVELOPMENT_INFERRED]:(0,a.zR)({id:"NUjE6C",defaultMessage:"Machine learning"}),[i.Gk.NO_INFERRED_TYPE]:(0,a.zR)({id:"qr7dhT",defaultMessage:"None"}),[i.Gk.FINETUNING]:(0,a.zR)({id:"G4x6G7",defaultMessage:"Finetuning"}),[i.Gk.REGRESSION]:(0,a.zR)({id:"OWfx/j",defaultMessage:"Regression"}),[i.Gk.CLASSIFICATION]:(0,a.zR)({id:"P/Uvf4",defaultMessage:"Classification"}),[i.Gk.FORECASTING]:(0,a.zR)({id:"qDiJI2",defaultMessage:"Forecasting"}),[i.Gk.AUTOML]:(0,a.zR)({id:"wZtuCR",defaultMessage:"AutoML"}),[i.Gk.EMPTY]:(0,a.zR)({id:"0z0lH2",defaultMessage:"None"})},E={[i.Gk.GENAI_DEVELOPMENT]:(0,a.zR)({id:"Fg/zU/",defaultMessage:"GenAI apps & agents"}),[i.Gk.CUSTOM_MODEL_DEVELOPMENT]:(0,a.zR)({id:"T/46y0",defaultMessage:"Machine learning"}),[i.Gk.GENAI_DEVELOPMENT_INFERRED]:(0,a.zR)({id:"CPO2ro",defaultMessage:"GenAI apps & agents"}),[i.Gk.CUSTOM_MODEL_DEVELOPMENT_INFERRED]:(0,a.zR)({id:"KYJxmN",defaultMessage:"Machine learning"}),[i.Gk.NO_INFERRED_TYPE]:(0,a.zR)({id:"jTqRO+",defaultMessage:"None"}),[i.Gk.FINETUNING]:(0,a.zR)({id:"61P6Da",defaultMessage:"finetuning"}),[i.Gk.REGRESSION]:(0,a.zR)({id:"W60/kq",defaultMessage:"regression"}),[i.Gk.CLASSIFICATION]:(0,a.zR)({id:"KwAeyt",defaultMessage:"classification"}),[i.Gk.FORECASTING]:(0,a.zR)({id:"VwxvII",defaultMessage:"forecasting"}),[i.Gk.AUTOML]:(0,a.zR)({id:"lFnpMi",defaultMessage:"AutoML"}),[i.Gk.EMPTY]:(0,a.zR)({id:"+khKtf",defaultMessage:"None"})},d=()=>[i.Gk.GENAI_DEVELOPMENT,i.Gk.CUSTOM_MODEL_DEVELOPMENT]},96584:function(e,t,n){n.d(t,{q:function(){return c}});var i=n(56606),a=n(23403),s=n(60783);class o extends a.${constructor(e,t){super(e,t)}bindMethods(){super.bindMethods(),this.fetchNextPage=this.fetchNextPage.bind(this),this.fetchPreviousPage=this.fetchPreviousPage.bind(this)}setOptions(e,t){super.setOptions({...e,behavior:(0,s.PL)()},t)}getOptimisticResult(e){return e.behavior=(0,s.PL)(),super.getOptimisticResult(e)}fetchNextPage({pageParam:e,...t}={}){return this.fetch({...t,meta:{fetchMore:{direction:"forward",pageParam:e}}})}fetchPreviousPage({pageParam:e,...t}={}){return this.fetch({...t,meta:{fetchMore:{direction:"backward",pageParam:e}}})}createResult(e,t){var n,i,a,o,r,c;const{state:u}=e,l=super.createResult(e,t),{isFetching:E,isRefetching:d}=l,f=E&&"forward"===(null==(n=u.fetchMeta)||null==(i=n.fetchMore)?void 0:i.direction),g=E&&"backward"===(null==(a=u.fetchMeta)||null==(o=a.fetchMore)?void 0:o.direction);return{...l,fetchNextPage:this.fetchNextPage,fetchPreviousPage:this.fetchPreviousPage,hasNextPage:(0,s.rB)(t,null==(r=u.data)?void 0:r.pages),hasPreviousPage:(0,s.RQ)(t,null==(c=u.data)?void 0:c.pages),isFetchingNextPage:f,isFetchingPreviousPage:g,isRefetching:d&&!f&&!g}}}var r=n(93769);function c(e,t,n){const a=(0,i.vh)(e,t,n);return(0,r.t)(a,o)}}}]);