# This file was auto-generated by update_ml_package_versions.py.
# Please do not edit it manually.

_ML_PACKAGE_VERSIONS = {
    "semantic_kernel": {
        "package_info": {
            "pip_release": "semantic-kernel"
        },
        "autologging": {
            "minimum": "1.34.0",
            "maximum": "1.39.0"
        }
    },
    "openai": {
        "package_info": {
            "pip_release": "openai"
        },
        "models": {
            "minimum": "1.57.3",
            "maximum": "2.9.0"
        },
        "autologging": {
            "minimum": "1.57.3",
            "maximum": "2.9.0"
        }
    },
    "dspy": {
        "package_info": {
            "pip_release": "dspy"
        },
        "models": {
            "minimum": "2.5.43",
            "maximum": "3.0.4"
        },
        "autologging": {
            "minimum": "2.5.43",
            "maximum": "3.0.4"
        }
    },
    "langchain": {
        "package_info": {
            "pip_release": "langchain"
        },
        "models": {
            "minimum": "0.3.12",
            "maximum": "1.1.3"
        },
        "autologging": {
            "minimum": "0.3.12",
            "maximum": "1.1.3"
        }
    },
    "langgraph": {
        "package_info": {
            "pip_release": "langgraph"
        },
        "models": {
            "minimum": "0.2.60",
            "maximum": "1.0.4"
        },
        "autologging": {
            "minimum": "0.2.60",
            "maximum": "1.0.4"
        }
    },
    "llama_index": {
        "package_info": {
            "pip_release": "llama-index",
            "module_name": "llama_index.core"
        },
        "models": {
            "minimum": "0.12.6",
            "maximum": "0.14.10"
        },
        "autologging": {
            "minimum": "0.12.6",
            "maximum": "0.14.10"
        }
    },
    "ag2": {
        "package_info": {
            "pip_release": "ag2",
            "module_name": "autogen"
        },
        "autologging": {
            "minimum": "0.7.0",
            "maximum": "0.10.2"
        }
    },
    "autogen": {
        "package_info": {
            "pip_release": "autogen-agentchat",
            "module_name": "autogen_agentchat"
        },
        "autologging": {
            "minimum": "0.4.9",
            "maximum": "0.7.5"
        }
    },
    "gemini": {
        "package_info": {
            "pip_release": "google-genai",
            "module_name": "google.genai"
        },
        "autologging": {
            "minimum": "1.0.0",
            "maximum": "1.55.0"
        }
    },
    "anthropic": {
        "package_info": {
            "pip_release": "anthropic"
        },
        "autologging": {
            "minimum": "0.41.0",
            "maximum": "0.75.0"
        }
    },
    "crewai": {
        "package_info": {
            "pip_release": "crewai",
            "module_name": "crewai"
        },
        "autologging": {
            "minimum": "0.95.0",
            "maximum": "1.7.0"
        }
    },
    "agno": {
        "package_info": {
            "pip_release": "agno",
            "module_name": "agno"
        },
        "autologging": {
            "minimum": "1.7.0",
            "maximum": "2.3.10"
        }
    },
    "pydantic_ai": {
        "package_info": {
            "pip_release": "pydantic-ai",
            "module_name": "pydantic_ai"
        },
        "autologging": {
            "minimum": "0.1.9",
            "maximum": "1.30.0"
        }
    },
    "smolagents": {
        "package_info": {
            "pip_release": "smolagents",
            "module_name": "smolagents"
        },
        "autologging": {
            "minimum": "1.14.0",
            "maximum": "1.23.0"
        }
    },
    "strands": {
        "package_info": {
            "pip_release": "strands-agents",
            "module_name": "strands"
        },
        "autologging": {
            "minimum": "1.4.0",
            "maximum": "1.19.0"
        }
    },
    "mistral": {
        "package_info": {
            "pip_release": "mistralai",
            "module_name": "mistralai"
        },
        "autologging": {
            "minimum": "1.2.6",
            "maximum": "1.9.11"
        }
    },
    "litellm": {
        "package_info": {
            "pip_release": "litellm"
        },
        "autologging": {
            "minimum": "1.63.14",
            "maximum": "1.74.9"
        }
    },
    "groq": {
        "package_info": {
            "pip_release": "groq"
        },
        "autologging": {
            "minimum": "0.13.1",
            "maximum": "0.37.1"
        }
    },
    "bedrock": {
        "package_info": {
            "pip_release": "boto3",
            "module_name": "boto3"
        },
        "autologging": {
            "minimum": "1.35.80",
            "maximum": "1.42.7"
        }
    },
    "sklearn": {
        "package_info": {
            "pip_release": "scikit-learn"
        },
        "models": {
            "minimum": "1.4.0",
            "maximum": "1.8.0"
        },
        "autologging": {
            "minimum": "1.4.0",
            "maximum": "1.8.0"
        }
    },
    "pytorch": {
        "package_info": {
            "pip_release": "torch",
            "module_name": "torch"
        },
        "models": {
            "minimum": "2.1.2",
            "maximum": "2.9.1"
        },
        "autologging": {
            "minimum": "2.1.2",
            "maximum": "2.9.1"
        }
    },
    "pytorch-lightning": {
        "package_info": {
            "pip_release": "pytorch-lightning",
            "module_name": "lightning"
        },
        "models": {
            "minimum": "2.1.3",
            "maximum": "2.6.0"
        },
        "autologging": {
            "minimum": "2.1.3",
            "maximum": "2.6.0"
        }
    },
    "keras": {
        "package_info": {
            "pip_release": "keras"
        },
        "models": {
            "minimum": "3.0.2",
            "maximum": "3.12.0"
        },
        "autologging": {
            "minimum": "3.0.2",
            "maximum": "3.12.0"
        }
    },
    "tensorflow": {
        "package_info": {
            "pip_release": "tensorflow"
        },
        "models": {
            "minimum": "2.16.0",
            "maximum": "2.20.0"
        },
        "autologging": {
            "minimum": "2.16.0",
            "maximum": "2.20.0"
        }
    },
    "xgboost": {
        "package_info": {
            "pip_release": "xgboost"
        },
        "models": {
            "minimum": "2.0.3",
            "maximum": "3.1.2"
        },
        "autologging": {
            "minimum": "2.0.3",
            "maximum": "3.1.2"
        }
    },
    "lightgbm": {
        "package_info": {
            "pip_release": "lightgbm"
        },
        "models": {
            "minimum": "4.2.0",
            "maximum": "4.6.0"
        },
        "autologging": {
            "minimum": "4.2.0",
            "maximum": "4.6.0"
        }
    },
    "catboost": {
        "package_info": {
            "pip_release": "catboost"
        },
        "models": {
            "minimum": "1.2.3",
            "maximum": "1.2.8"
        }
    },
    "onnx": {
        "package_info": {
            "pip_release": "onnx"
        },
        "models": {
            "minimum": "1.17.0",
            "maximum": "1.20.0"
        }
    },
    "spacy": {
        "package_info": {
            "pip_release": "spacy"
        },
        "models": {
            "minimum": "3.7.4",
            "maximum": "3.8.11"
        }
    },
    "statsmodels": {
        "package_info": {
            "pip_release": "statsmodels"
        },
        "models": {
            "minimum": "0.14.1",
            "maximum": "0.14.6"
        },
        "autologging": {
            "minimum": "0.14.1",
            "maximum": "0.14.6"
        }
    },
    "spark": {
        "package_info": {
            "pip_release": "pyspark",
            "module_name": "pyspark"
        },
        "models": {
            "minimum": "3.2.1",
            "maximum": "4.0.1"
        },
        "autologging": {
            "minimum": "3.3.0",
            "maximum": "4.0.1"
        }
    },
    "prophet": {
        "package_info": {
            "pip_release": "prophet"
        },
        "models": {
            "minimum": "1.1.6",
            "maximum": "1.2.1"
        }
    },
    "pmdarima": {
        "package_info": {
            "pip_release": "pmdarima"
        },
        "models": {
            "minimum": "2.1.0",
            "maximum": "2.1.1"
        }
    },
    "h2o": {
        "package_info": {
            "pip_release": "h2o"
        },
        "models": {
            "minimum": "3.44.0.3",
            "maximum": "3.46.0.9"
        }
    },
    "shap": {
        "package_info": {
            "pip_release": "shap"
        },
        "models": {
            "minimum": "0.44.1",
            "maximum": "0.50.0"
        }
    },
    "paddle": {
        "package_info": {
            "pip_release": "paddlepaddle"
        },
        "models": {
            "minimum": "2.6.2",
            "maximum": "3.2.2"
        },
        "autologging": {
            "minimum": "2.6.2",
            "maximum": "3.2.2"
        }
    },
    "transformers": {
        "package_info": {
            "pip_release": "transformers"
        },
        "models": {
            "minimum": "4.38.2",
            "maximum": "4.57.3"
        },
        "autologging": {
            "minimum": "4.38.2",
            "maximum": "4.57.3"
        }
    },
    "haystack": {
        "package_info": {
            "pip_release": "haystack-ai",
            "module_name": "haystack"
        },
        "autologging": {
            "minimum": "2.0.0",
            "maximum": "2.21.0"
        }
    },
    "sentence_transformers": {
        "package_info": {
            "pip_release": "sentence-transformers"
        },
        "models": {
            "minimum": "2.3.1",
            "maximum": "5.1.2"
        }
    },
    "johnsnowlabs": {
        "package_info": {
            "pip_release": "johnsnowlabs"
        },
        "models": {
            "minimum": "5.2.0",
            "maximum": "6.2.0"
        }
    }
}

# A mapping of flavor name to the module name to be imported for autologging.
# This is used for checking version compatibility in autologging.
# DO NOT EDIT MANUALLY

# GenAI packages
GENAI_FLAVOR_TO_MODULE_NAME = {
    "semantic_kernel": "semantic_kernel",
    "openai": "openai",
    "dspy": "dspy",
    "langchain": "langchain",
    "llama_index": "llama_index.core",
    "ag2": "autogen",
    "autogen": "autogen_agentchat",
    "gemini": "google.genai",
    "anthropic": "anthropic",
    "crewai": "crewai",
    "agno": "agno",
    "pydantic_ai": "pydantic_ai",
    "smolagents": "smolagents",
    "strands": "strands",
    "mistral": "mistralai",
    "litellm": "litellm",
    "groq": "groq",
    "bedrock": "boto3"
}

# Non-GenAI packages
NON_GENAI_FLAVOR_TO_MODULE_NAME = {
    "sklearn": "sklearn",
    "pytorch": "torch",
    "pytorch-lightning": "lightning",
    "keras": "keras",
    "tensorflow": "tensorflow",
    "xgboost": "xgboost",
    "lightgbm": "lightgbm",
    "statsmodels": "statsmodels",
    "spark": "pyspark",
    "paddle": "paddle",
    "transformers": "transformers",
    "haystack": "haystack",
    "pyspark.ml": "pyspark"
}

# Combined mapping for backward compatibility
FLAVOR_TO_MODULE_NAME = NON_GENAI_FLAVOR_TO_MODULE_NAME | GENAI_FLAVOR_TO_MODULE_NAME
