.mlflow-sticky-header{left:0;position:sticky;position:-webkit-sticky}.mlflow-compare-run-table{display:block;overflow:auto;width:100%}.mlflow-compare-table th.inter-title{background:transparent;padding:20px 0 0}.mlflow-compare-table .head-value{overflow:hidden;overflow-wrap:break-word}.mlflow-compare-table td.data-value,.mlflow-compare-table th.data-value{max-width:120px;overflow:hidden;text-overflow:ellipsis}.mlflow-responsive-table-container{overflow-x:auto;width:100%}.mlflow-compare-table .diff-row .data-value{background-color:hsla(48,83%,86%,.5);color:#555}.mlflow-compare-table .diff-row .head-value,.mlflow-compare-table .diff-row:hover{background-color:#f9edbe;color:#555}.mlflow-compare-table{margin-bottom:20px;max-width:100%;width:100%}.mlflow-compare-table td,.mlflow-compare-table th{border-bottom:1px solid #e8e8e8;padding:12px 8px}.mlflow-compare-table th{background-color:#fafafa;color:rgba(0,0,0,.85);font-weight:500;text-align:left}.mlflow-compare-table>tbody>tr:hover:not(.diff-row)>td:not(.highlight-data){background-color:#fafafa}