"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[8844],{27925:function(e,t,n){n.d(t,{w:function(){return i}});var r=n(46709);function i({ref:e,debounceTimeMs:t}){const n=(0,r.useRef)({width:-1,height:-1}),i=(0,r.useRef)(),[c,s]=(0,r.useState)(null),u=(0,r.useCallback)((e=>{if(0===e.length)return;const r=e[0].contentRect;if(-1===n.current.width)return n.current={width:r.width,height:r.height},void s(n.current);if(r.width!==n.current.width||r.height!==n.current.height){if(n.current.width=r.width,n.current.height=r.height,!t)return void s({...n.current});clearTimeout(i.current),i.current=setTimeout((()=>{s({...n.current})}),t)}}),[t]),a=(0,r.useRef)();return a.current||(a.current=new ResizeObserver(u)),(0,r.useEffect)((()=>{const t="function"===typeof e?e():e.current;if(t){const e=a.current;return e.observe(t),()=>e.unobserve(t)}})),c}},42747:function(e,t,n){n.d(t,{sA:function(){return r.sA},tz:function(){return r.tz},zR:function(){return r.zR}});var r=n(77680);n(13584)},49633:function(e,t,n){n.d(t,{Mj:function(){return u.Mj},Ub:function(){return c},wY:function(){return s.w}});var r=n(46709),i=n(69103);function c(e){return function(e){return(0,i.useSyncExternalStore)(r.useCallback((t=>(e.addEventListener("change",t),()=>{e.removeEventListener("change",t)})),[e]),(()=>e.matches))}(r.useMemo((()=>window.matchMedia(e)),[e]))}var s=n(27925),u=n(53637)},78844:function(e,t,n){n.r(t);var r=n(68248),i=n(27757),c=n(27299),s=n(42747),u=n(49633),a=n(47045),l=n(19960),o=n(46709),d=n(73408);var f={name:"13z90ae",styles:"display:flex;justify-content:space-between;align-items:center;max-width:600px"};t.default=()=>{const{theme:e}=(0,i.u)(),t=(0,s.tz)(),[n,h]=(0,u.Mj)({key:a.kV,version:a.j6,initialValue:!0}),m=(0,o.useCallback)((e=>{h(e),e?l.R.start():l.R.shutdown()}),[h]);return(0,d.FD)("div",{css:(0,r.AH)({padding:e.spacing.md},""),children:[(0,d.Y)(i.T.Title,{level:2,children:(0,d.Y)(s.sA,{id:"wRV8PN",defaultMessage:"Settings"})}),(0,d.FD)("div",{css:f,children:[(0,d.FD)("div",{css:(0,r.AH)({display:"flex",flexDirection:"column",marginRight:e.spacing.lg},""),children:[(0,d.Y)(i.T.Title,{level:4,children:(0,d.Y)(s.sA,{id:"hg+bcy",defaultMessage:"Enable telemetry"})}),(0,d.Y)(i.T.Text,{children:(0,d.Y)(s.sA,{id:"LXz6c5",defaultMessage:"This setting enables UI telemetry data collection. Learn more about what types of data are collected in our {documentation}.",values:{documentation:(0,d.Y)(i.T.Link,{componentId:"mlflow.settings.telemetry.documentation-link",href:"https://mlflow.org/docs/latest/community/usage-tracking.html",openInNewTab:!0,children:(0,d.Y)(s.sA,{id:"h2398a",defaultMessage:"documentation"})})}})})]}),(0,d.Y)(c.Switch,{componentId:"mlflow.settings.telemetry.toggle-switch",checked:n,onChange:m,label:" ",activeLabel:t.formatMessage({id:"+CGMk6",defaultMessage:"On"}),inactiveLabel:t.formatMessage({id:"aCzpU3",defaultMessage:"Off"}),disabledLabel:" "})]})]})}}}]);