"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[7238],{7238:function(e,t,o){o.r(t);var d=o(46709),i=o(27299),n=o(65765),a=o(73408);var l={name:"82a6rk",styles:"flex:1"};t.default=({runUuid:e,path:t,getArtifact:o=n.UQ,isLoggedModelsMode:c,loggedModelId:r})=>{const[s,u]=(0,d.useState)(),[f,h]=(0,d.useState)(!0);(0,d.useEffect)((()=>{let d;const i=c&&r?(0,n.qk)(t,r):(0,n.To)(t,e);return o(i).then((e=>{d=URL.createObjectURL(e),u(d),h(!1)})),()=>{d&&URL.revokeObjectURL(d)}}),[e,t,c,r,o]);const v={videoOuterContainer:{padding:10,overflow:"hidden",background:"black",minHeight:"100%"},hidden:{display:"none"},video:{maxWidth:"100%",maxHeight:"62.5vh",objectFit:"fit",display:"block"}};return(0,a.Y)("div",{css:l,children:(0,a.FD)("div",{css:v.videoOuterContainer,children:[f&&(0,a.Y)(i.LegacySkeleton,{active:!0}),s&&(0,a.Y)("video",{css:f?v.hidden:v.video,src:s,controls:!0,preload:"auto","aria-label":"video",children:(0,a.Y)("track",{kind:"captions",srcLang:"en",src:"",default:!0})})]})})}}}]);