"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[7419],{7785:function(e,n,t){t.r(n);var s=t(68248),i=t(46709),a=t(7655),r=t(5690),l=t(11522),o=t(46665),u=t(11573),c=t(27757),p=t(27299),d=t(36802),f=t(55840),g=t(73408);const m={setExperimentTagApi:r.EJ,getExperimentApi:r.yc,setCompareExperiments:r.I_};n.default=()=>{const{formatMessage:e}=(0,a.A)(),{theme:n}=(0,c.u)(),t=(0,f.z)();(0,i.useEffect)((()=>{const n=e({id:"5GCYzy",defaultMessage:"Experiment Runs - Databricks"});l.A.updatePageTitle(n)}),[e]);const r=t.length>1,{isLoading:E,isEnabled:k}=(0,d.B)({enabled:!r,experimentId:t[0]});return E?(0,g.Y)(p.LegacySkeleton,{}):k?null:(0,g.Y)(p.PageWrapper,{css:(0,s.AH)({height:"100%",paddingTop:n.spacing.md},""),children:(0,g.Y)(o.Z,{actions:m,children:(0,g.Y)(u.n,{})})})}},36802:function(e,n,t){t.d(n,{B:function(){return c}});var s=t(46709),i=t(91105),a=t(60284),r=t(44105),l=t(54741),o=t(91944),u=t(21678);const c=({enabled:e,experimentId:n})=>{const t=(0,i.Zp)(),{data:c,loading:p}=(0,l.L)({experimentId:n,options:{skip:!e}}),d=(0,s.useMemo)((()=>{if(p||!c)return null;const e=c&&"tags"in c?null===c||void 0===c?void 0:c.tags:[];if(c){const n=(0,o.BH)(e);return(0,u.SK)(r.Gk,n,r.Gk.NO_INFERRED_TYPE)}return null}),[c,p]);return(0,s.useEffect)((()=>{if(!e||!d)return;let s=r.fM.Runs;d===r.Gk.GENAI_DEVELOPMENT&&(s=r.fM.Traces),t(a.Ay.getExperimentPageTabRoute(n,s),{replace:!0})}),[t,n,e,d]),{isEnabled:e,isLoading:e&&p}}}}]);