"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[223],{40223:function(e,l,n){n.r(l);var t=n(59930),f=n.n(t),u=n(91105),m=n(40697),r=n(73408);l.default=()=>{const{experimentId:e}=(0,u.g)();return f()(e,"Experiment ID must be defined"),(0,r.Y)(m.default,{experimentId:e})}}}]);