import databricks_pb2 as _databricks_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from scalapb import scalapb_pb2 as _scalapb_pb2
import unity_catalog_prompt_messages_pb2 as _unity_catalog_prompt_messages_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class UnityCatalogPromptService(_service.service): ...

class UnityCatalogPromptService_Stub(UnityCatalogPromptService): ...
