
import google.protobuf
from packaging.version import Version
if Version(google.protobuf.__version__).major >= 5:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_tracing.proto
  # Protobuf Python Version: 5.26.0
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf.internal import builder as _builder
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import assessments_pb2 as assessments__pb2
  from . import databricks_pb2 as databricks__pb2
  from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
  from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
  from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
  from opentelemetry.proto.trace.v1 import trace_pb2 as opentelemetry_dot_proto_dot_trace_dot_v1_dot_trace__pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x64\x61tabricks_tracing.proto\x12\x11mlflow.databricks\x1a\x11\x61ssessments.proto\x1a\x10\x64\x61tabricks.proto\x1a\x1egoogle/protobuf/duration.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a(opentelemetry/proto/trace/v1/trace.proto\x1a\x15scalapb/scalapb.proto\"z\n\x10UCSchemaLocation\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x13\n\x0bschema_name\x18\x02 \x01(\t\x12\x1d\n\x15otel_spans_table_name\x18\x03 \x01(\t\x12\x1c\n\x14otel_logs_table_name\x18\x04 \x01(\t\"1\n\x18MlflowExperimentLocation\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\"1\n\x16InferenceTableLocation\x12\x17\n\x0f\x66ull_table_name\x18\x01 \x01(\t\"\x9e\x03\n\rTraceLocation\x12@\n\x04type\x18\x01 \x01(\x0e\x32\x32.mlflow.databricks.TraceLocation.TraceLocationType\x12H\n\x11mlflow_experiment\x18\x02 \x01(\x0b\x32+.mlflow.databricks.MlflowExperimentLocationH\x00\x12\x44\n\x0finference_table\x18\x03 \x01(\x0b\x32).mlflow.databricks.InferenceTableLocationH\x00\x12\x38\n\tuc_schema\x18\x04 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\"s\n\x11TraceLocationType\x12#\n\x1fTRACE_LOCATION_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11MLFLOW_EXPERIMENT\x10\x01\x12\x13\n\x0fINFERENCE_TABLE\x10\x02\x12\r\n\tUC_SCHEMA\x10\x03\x42\x0c\n\nidentifier\"\x9b\x05\n\tTraceInfo\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x12\x19\n\x11\x63lient_request_id\x18\x02 \x01(\t\x12\x38\n\x0etrace_location\x18\x03 \x01(\x0b\x32 .mlflow.databricks.TraceLocation\x12\x17\n\x0frequest_preview\x18\x04 \x01(\t\x12\x18\n\x10response_preview\x18\x05 \x01(\t\x12\x30\n\x0crequest_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12\x65xecution_duration\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x31\n\x05state\x18\x08 \x01(\x0e\x32\".mlflow.databricks.TraceInfo.State\x12G\n\x0etrace_metadata\x18\t \x03(\x0b\x32/.mlflow.databricks.TraceInfo.TraceMetadataEntry\x12\x32\n\x0b\x61ssessments\x18\n \x03(\x0b\x32\x1d.mlflow.databricks.Assessment\x12\x34\n\x04tags\x18\x0b \x03(\x0b\x32&.mlflow.databricks.TraceInfo.TagsEntry\x1a\x34\n\x12TraceMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0f\n\x0bIN_PROGRESS\x10\x03\"\xcf\x01\n\x0f\x43reateTraceInfo\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x36\n\ntrace_info\x18\x02 \x01(\x0b\x32\x1c.mlflow.databricks.TraceInfoB\x04\xf8\x86\x19\x01\x1a<\n\x08Response\x12\x30\n\ntrace_info\x18\x01 \x01(\x0b\x32\x1c.mlflow.databricks.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"c\n\tTracePath\x12>\n\x0etrace_location\x18\x01 \x01(\x0b\x32 .mlflow.databricks.TraceLocationB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\"l\n\x05Trace\x12\x30\n\ntrace_info\x18\x01 \x01(\x0b\x32\x1c.mlflow.databricks.TraceInfo\x12\x31\n\x05spans\x18\x02 \x03(\x0b\x32\".opentelemetry.proto.trace.v1.Span\"\xbb\x01\n\x0e\x42\x61tchGetTraces\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\ttrace_ids\x18\x02 \x03(\t\x12\x18\n\x10sql_warehouse_id\x18\x03 \x01(\t\x1a\x34\n\x08Response\x12(\n\x06traces\x18\x01 \x03(\x0b\x32\x18.mlflow.databricks.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xba\x01\n\x0cGetTraceInfo\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08location\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x03 \x01(\t\x1a\x33\n\x08Response\x12\'\n\x05trace\x18\x01 \x01(\x0b\x32\x18.mlflow.databricks.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9b\x01\n\x0bSetTraceTag\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0blocation_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\r\n\x05value\x18\x04 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa9\x01\n\x0e\x44\x65leteTraceTag\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0blocation_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb2\x02\n\x0cSearchTraces\x12\x33\n\tlocations\x18\x01 \x03(\x0b\x32 .mlflow.databricks.TraceLocation\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x18\n\x0bmax_results\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x18\n\x10sql_warehouse_id\x18\x05 \x01(\t\x12\x12\n\npage_token\x18\x06 \x01(\t\x1aV\n\x08Response\x12\x31\n\x0btrace_infos\x18\x01 \x03(\x0b\x32\x1c.mlflow.databricks.TraceInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf5\x01\n\x1c\x43reateTraceUCStorageLocation\x12\x38\n\tuc_schema\x18\x01 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\x12\x1e\n\x10sql_warehouse_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x42\n\x08Response\x12\x36\n\tuc_schema\x18\x01 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocation:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]B\n\n\x08location\"\xbd\x01\n\x1fLinkExperimentToUCTraceLocation\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x38\n\tuc_schema\x18\x02 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]B\n\n\x08location\"\xbf\x01\n!UnLinkExperimentToUCTraceLocation\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x38\n\tuc_schema\x18\x02 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]B\n\n\x08location\"\xe0\x04\n\nAssessment\x12\x15\n\rassessment_id\x18\x01 \x01(\t\x12\x1d\n\x0f\x61ssessment_name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x38\n\x0etrace_location\x18\x04 \x01(\x0b\x32 .mlflow.databricks.TraceLocation\x12\x0f\n\x07span_id\x18\x05 \x01(\t\x12\x34\n\x06source\x18\x06 \x01(\x0b\x32$.mlflow.assessments.AssessmentSource\x12/\n\x0b\x63reate_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x10last_update_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x08\x66\x65\x65\x64\x62\x61\x63k\x18\t \x01(\x0b\x32\x1c.mlflow.assessments.FeedbackH\x00\x12\x36\n\x0b\x65xpectation\x18\n \x01(\x0b\x32\x1f.mlflow.assessments.ExpectationH\x00\x12\x11\n\trationale\x18\x0b \x01(\t\x12=\n\x08metadata\x18\x0c \x03(\x0b\x32+.mlflow.databricks.Assessment.MetadataEntry\x12\x11\n\toverrides\x18\r \x01(\t\x12\x13\n\x05valid\x18\x0e \x01(\x08:\x04true\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x07\n\x05value\"\xec\x01\n\x10\x43reateAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x37\n\nassessment\x18\x02 \x01(\x0b\x32\x1d.mlflow.databricks.AssessmentB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x03 \x01(\t\x1a=\n\x08Response\x12\x31\n\nassessment\x18\x01 \x01(\x0b\x32\x1d.mlflow.databricks.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xe5\x01\n\rGetAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a=\n\x08Response\x12\x31\n\nassessment\x18\x01 \x01(\x0b\x32\x1d.mlflow.databricks.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa3\x02\n\x10UpdateAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x37\n\nassessment\x18\x02 \x01(\x0b\x32\x1d.mlflow.databricks.AssessmentB\x04\xf8\x86\x19\x01\x12\x35\n\x0bupdate_mask\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a=\n\x08Response\x12\x31\n\nassessment\x18\x01 \x01(\x0b\x32\x1d.mlflow.databricks.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb5\x01\n\x10\x44\x65leteAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x92\x01\n\x13\x42\x61tchLinkTraceToRun\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\ttrace_ids\x18\x02 \x03(\t\x12\x14\n\x06run_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x96\x01\n\x17\x42\x61tchUnlinkTraceFromRun\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\ttrace_ids\x18\x02 \x03(\t\x12\x14\n\x06run_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]2\xb0\x18\n\x19\x44\x61tabricksTrackingService\x12\xb8\x01\n\x0f\x63reateTraceInfo\x12\".mlflow.databricks.CreateTraceInfo\x1a\x1c.mlflow.databricks.TraceInfo\"c\xf2\x86\x19_\nE\n\x04POST\x12\x37/mlflow/traces/{location_id}/{trace_info.trace_id}/info\x1a\x04\x08\x04\x10\x00\x10\x03*\x14\x43reate Trace Info v4\x12\xaa\x01\n\x0e\x62\x61tchGetTraces\x12!.mlflow.databricks.BatchGetTraces\x1a*.mlflow.databricks.BatchGetTraces.Response\"I\xf2\x86\x19\x45\n2\n\x03GET\x12%/mlflow/traces/{location_id}/batchGet\x1a\x04\x08\x04\x10\x00\x10\x03*\rGet Traces V4\x12\xa8\x01\n\x0cgetTraceInfo\x12\x1f.mlflow.databricks.GetTraceInfo\x1a(.mlflow.databricks.GetTraceInfo.Response\"M\xf2\x86\x19I\n6\n\x03GET\x12)/mlflow/traces/{location}/{trace_id}/info\x1a\x04\x08\x04\x10\x00\x10\x03*\rGet TraceInfo\x12\xaa\x01\n\x0bsetTraceTag\x12\x1e.mlflow.databricks.SetTraceTag\x1a\'.mlflow.databricks.SetTraceTag.Response\"R\xf2\x86\x19N\n;\n\x05PATCH\x12,/mlflow/traces/{location_id}/{trace_id}/tags\x1a\x04\x08\x04\x10\x00\x10\x03*\rSet Trace Tag\x12\xbd\x01\n\x0e\x64\x65leteTraceTag\x12!.mlflow.databricks.DeleteTraceTag\x1a*.mlflow.databricks.DeleteTraceTag.Response\"\\\xf2\x86\x19X\nB\n\x06\x44\x45LETE\x12\x32/mlflow/traces/{location_id}/{trace_id}/tags/{key}\x1a\x04\x08\x04\x10\x00\x10\x03*\x10\x44\x65lete Trace Tag\x12\x95\x01\n\x0csearchTraces\x12\x1f.mlflow.databricks.SearchTraces\x1a(.mlflow.databricks.SearchTraces.Response\":\xf2\x86\x19\x36\n#\n\x04POST\x12\x15/mlflow/traces/search\x1a\x04\x08\x04\x10\x00\x10\x03*\rSearch Traces\x12\xda\x01\n\x1c\x63reateTraceUCStorageLocation\x12/.mlflow.databricks.CreateTraceUCStorageLocation\x1a\x38.mlflow.databricks.CreateTraceUCStorageLocation.Response\"O\xf2\x86\x19K\n%\n\x04POST\x12\x17/mlflow/traces/location\x1a\x04\x08\x04\x10\x00\x10\x03* Create Trace UC Storage Location\x12\xfc\x01\n\x1flinkExperimentToUCTraceLocation\x12\x32.mlflow.databricks.LinkExperimentToUCTraceLocation\x1a;.mlflow.databricks.LinkExperimentToUCTraceLocation.Response\"h\xf2\x86\x19\x64\n:\n\x04POST\x12,/mlflow/traces/{experiment_id}/link-location\x1a\x04\x08\x04\x10\x00\x10\x03*$Link Experiment to UC Trace Location\x12\x86\x02\n!unlinkExperimentToUCTraceLocation\x12\x34.mlflow.databricks.UnLinkExperimentToUCTraceLocation\x1a=.mlflow.databricks.UnLinkExperimentToUCTraceLocation.Response\"l\xf2\x86\x19h\n<\n\x04POST\x12./mlflow/traces/{experiment_id}/unlink-location\x1a\x04\x08\x04\x10\x00\x10\x03*&Unlink Experiment to UC Trace Location\x12\xce\x01\n\x10\x63reateAssessment\x12#.mlflow.databricks.CreateAssessment\x1a,.mlflow.databricks.CreateAssessment.Response\"g\xf2\x86\x19\x63\nL\n\x04POST\x12>/mlflow/traces/{location_id}/{assessment.trace_id}/assessments\x1a\x04\x08\x04\x10\x00\x10\x03*\x11\x43reate Assessment\x12\xc6\x01\n\rgetAssessment\x12 .mlflow.databricks.GetAssessment\x1a).mlflow.databricks.GetAssessment.Response\"h\xf2\x86\x19\x64\nP\n\x03GET\x12\x43/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x04\x10\x00\x10\x03*\x0eGet Assessment\x12\xeb\x01\n\x10updateAssessment\x12#.mlflow.databricks.UpdateAssessment\x1a,.mlflow.databricks.UpdateAssessment.Response\"\x83\x01\xf2\x86\x19\x7f\nh\n\x05PATCH\x12Y/mlflow/traces/{location_id}/{assessment.trace_id}/assessments/{assessment.assessment_id}\x1a\x04\x08\x04\x10\x00\x10\x03*\x11Update Assessment\x12\xd5\x01\n\x10\x64\x65leteAssessment\x12#.mlflow.databricks.DeleteAssessment\x1a,.mlflow.databricks.DeleteAssessment.Response\"n\xf2\x86\x19j\nS\n\x06\x44\x45LETE\x12\x43/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x04\x10\x00\x10\x03*\x11\x44\x65lete Assessment\x12\xce\x01\n\x13\x62\x61tchLinkTraceToRun\x12&.mlflow.databricks.BatchLinkTraceToRun\x1a/.mlflow.databricks.BatchLinkTraceToRun.Response\"^\xf2\x86\x19Z\nB\n\x04POST\x12\x34/mlflow/traces/{location_id}/link-to-run/batchCreate\x1a\x04\x08\x04\x10\x00\x10\x03*\x12Link Traces to Run\x12\xe4\x01\n\x17\x62\x61tchUnlinkTraceFromRun\x12*.mlflow.databricks.BatchUnlinkTraceFromRun\x1a\x33.mlflow.databricks.BatchUnlinkTraceFromRun.Response\"h\xf2\x86\x19\x64\nH\n\x06\x44\x45LETE\x12\x38/mlflow/traces/{location_id}/unlink-from-run/batchDelete\x1a\x04\x08\x04\x10\x00\x10\x03*\x16Unlink Traces from RunB\x03\x90\x01\x01')

  _globals = globals()
  _builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
  _builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'databricks_tracing_pb2', _globals)
  if not _descriptor._USE_C_DESCRIPTORS:
    _globals['DESCRIPTOR']._loaded_options = None
    _globals['DESCRIPTOR']._serialized_options = b'\220\001\001'
    _globals['_TRACEINFO_TRACEMETADATAENTRY']._loaded_options = None
    _globals['_TRACEINFO_TRACEMETADATAENTRY']._serialized_options = b'8\001'
    _globals['_TRACEINFO_TAGSENTRY']._loaded_options = None
    _globals['_TRACEINFO_TAGSENTRY']._serialized_options = b'8\001'
    _globals['_CREATETRACEINFO'].fields_by_name['location_id']._loaded_options = None
    _globals['_CREATETRACEINFO'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_CREATETRACEINFO'].fields_by_name['trace_info']._loaded_options = None
    _globals['_CREATETRACEINFO'].fields_by_name['trace_info']._serialized_options = b'\370\206\031\001'
    _globals['_CREATETRACEINFO']._loaded_options = None
    _globals['_CREATETRACEINFO']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_TRACEPATH'].fields_by_name['trace_location']._loaded_options = None
    _globals['_TRACEPATH'].fields_by_name['trace_location']._serialized_options = b'\370\206\031\001'
    _globals['_TRACEPATH'].fields_by_name['trace_id']._loaded_options = None
    _globals['_TRACEPATH'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_BATCHGETTRACES'].fields_by_name['location_id']._loaded_options = None
    _globals['_BATCHGETTRACES'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_BATCHGETTRACES']._loaded_options = None
    _globals['_BATCHGETTRACES']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETTRACEINFO'].fields_by_name['trace_id']._loaded_options = None
    _globals['_GETTRACEINFO'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETTRACEINFO'].fields_by_name['location']._loaded_options = None
    _globals['_GETTRACEINFO'].fields_by_name['location']._serialized_options = b'\370\206\031\001'
    _globals['_GETTRACEINFO']._loaded_options = None
    _globals['_GETTRACEINFO']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETTRACETAG'].fields_by_name['trace_id']._loaded_options = None
    _globals['_SETTRACETAG'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_SETTRACETAG'].fields_by_name['location_id']._loaded_options = None
    _globals['_SETTRACETAG'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_SETTRACETAG'].fields_by_name['key']._loaded_options = None
    _globals['_SETTRACETAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_SETTRACETAG']._loaded_options = None
    _globals['_SETTRACETAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETETRACETAG'].fields_by_name['trace_id']._loaded_options = None
    _globals['_DELETETRACETAG'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETRACETAG'].fields_by_name['location_id']._loaded_options = None
    _globals['_DELETETRACETAG'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETRACETAG'].fields_by_name['key']._loaded_options = None
    _globals['_DELETETRACETAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETRACETAG']._loaded_options = None
    _globals['_DELETETRACETAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHTRACES']._loaded_options = None
    _globals['_SEARCHTRACES']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CREATETRACEUCSTORAGELOCATION'].fields_by_name['sql_warehouse_id']._loaded_options = None
    _globals['_CREATETRACEUCSTORAGELOCATION'].fields_by_name['sql_warehouse_id']._serialized_options = b'\370\206\031\001'
    _globals['_CREATETRACEUCSTORAGELOCATION']._loaded_options = None
    _globals['_CREATETRACEUCSTORAGELOCATION']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION']._loaded_options = None
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION']._loaded_options = None
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_ASSESSMENT_METADATAENTRY']._loaded_options = None
    _globals['_ASSESSMENT_METADATAENTRY']._serialized_options = b'8\001'
    _globals['_ASSESSMENT'].fields_by_name['assessment_name']._loaded_options = None
    _globals['_ASSESSMENT'].fields_by_name['assessment_name']._serialized_options = b'\370\206\031\001'
    _globals['_ASSESSMENT'].fields_by_name['trace_id']._loaded_options = None
    _globals['_ASSESSMENT'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEASSESSMENT'].fields_by_name['location_id']._loaded_options = None
    _globals['_CREATEASSESSMENT'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEASSESSMENT'].fields_by_name['assessment']._loaded_options = None
    _globals['_CREATEASSESSMENT'].fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEASSESSMENT']._loaded_options = None
    _globals['_CREATEASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETASSESSMENT'].fields_by_name['location_id']._loaded_options = None
    _globals['_GETASSESSMENT'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETASSESSMENT'].fields_by_name['trace_id']._loaded_options = None
    _globals['_GETASSESSMENT'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETASSESSMENT'].fields_by_name['assessment_id']._loaded_options = None
    _globals['_GETASSESSMENT'].fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETASSESSMENT']._loaded_options = None
    _globals['_GETASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_UPDATEASSESSMENT'].fields_by_name['location_id']._loaded_options = None
    _globals['_UPDATEASSESSMENT'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_UPDATEASSESSMENT'].fields_by_name['assessment']._loaded_options = None
    _globals['_UPDATEASSESSMENT'].fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _globals['_UPDATEASSESSMENT'].fields_by_name['update_mask']._loaded_options = None
    _globals['_UPDATEASSESSMENT'].fields_by_name['update_mask']._serialized_options = b'\370\206\031\001'
    _globals['_UPDATEASSESSMENT']._loaded_options = None
    _globals['_UPDATEASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETEASSESSMENT'].fields_by_name['location_id']._loaded_options = None
    _globals['_DELETEASSESSMENT'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEASSESSMENT'].fields_by_name['trace_id']._loaded_options = None
    _globals['_DELETEASSESSMENT'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEASSESSMENT'].fields_by_name['assessment_id']._loaded_options = None
    _globals['_DELETEASSESSMENT'].fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEASSESSMENT']._loaded_options = None
    _globals['_DELETEASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_BATCHLINKTRACETORUN'].fields_by_name['location_id']._loaded_options = None
    _globals['_BATCHLINKTRACETORUN'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_BATCHLINKTRACETORUN'].fields_by_name['run_id']._loaded_options = None
    _globals['_BATCHLINKTRACETORUN'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_BATCHLINKTRACETORUN']._loaded_options = None
    _globals['_BATCHLINKTRACETORUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_BATCHUNLINKTRACEFROMRUN'].fields_by_name['location_id']._loaded_options = None
    _globals['_BATCHUNLINKTRACEFROMRUN'].fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _globals['_BATCHUNLINKTRACEFROMRUN'].fields_by_name['run_id']._loaded_options = None
    _globals['_BATCHUNLINKTRACEFROMRUN'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_BATCHUNLINKTRACEFROMRUN']._loaded_options = None
    _globals['_BATCHUNLINKTRACEFROMRUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['createTraceInfo']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['createTraceInfo']._serialized_options = b'\362\206\031_\nE\n\004POST\0227/mlflow/traces/{location_id}/{trace_info.trace_id}/info\032\004\010\004\020\000\020\003*\024Create Trace Info v4'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['batchGetTraces']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['batchGetTraces']._serialized_options = b'\362\206\031E\n2\n\003GET\022%/mlflow/traces/{location_id}/batchGet\032\004\010\004\020\000\020\003*\rGet Traces V4'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['getTraceInfo']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['getTraceInfo']._serialized_options = b'\362\206\031I\n6\n\003GET\022)/mlflow/traces/{location}/{trace_id}/info\032\004\010\004\020\000\020\003*\rGet TraceInfo'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['setTraceTag']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['setTraceTag']._serialized_options = b'\362\206\031N\n;\n\005PATCH\022,/mlflow/traces/{location_id}/{trace_id}/tags\032\004\010\004\020\000\020\003*\rSet Trace Tag'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['deleteTraceTag']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['deleteTraceTag']._serialized_options = b'\362\206\031X\nB\n\006DELETE\0222/mlflow/traces/{location_id}/{trace_id}/tags/{key}\032\004\010\004\020\000\020\003*\020Delete Trace Tag'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['searchTraces']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['searchTraces']._serialized_options = b'\362\206\0316\n#\n\004POST\022\025/mlflow/traces/search\032\004\010\004\020\000\020\003*\rSearch Traces'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['createTraceUCStorageLocation']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['createTraceUCStorageLocation']._serialized_options = b'\362\206\031K\n%\n\004POST\022\027/mlflow/traces/location\032\004\010\004\020\000\020\003* Create Trace UC Storage Location'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['linkExperimentToUCTraceLocation']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['linkExperimentToUCTraceLocation']._serialized_options = b'\362\206\031d\n:\n\004POST\022,/mlflow/traces/{experiment_id}/link-location\032\004\010\004\020\000\020\003*$Link Experiment to UC Trace Location'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['unlinkExperimentToUCTraceLocation']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['unlinkExperimentToUCTraceLocation']._serialized_options = b'\362\206\031h\n<\n\004POST\022./mlflow/traces/{experiment_id}/unlink-location\032\004\010\004\020\000\020\003*&Unlink Experiment to UC Trace Location'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['createAssessment']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['createAssessment']._serialized_options = b'\362\206\031c\nL\n\004POST\022>/mlflow/traces/{location_id}/{assessment.trace_id}/assessments\032\004\010\004\020\000\020\003*\021Create Assessment'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['getAssessment']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['getAssessment']._serialized_options = b'\362\206\031d\nP\n\003GET\022C/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\032\004\010\004\020\000\020\003*\016Get Assessment'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['updateAssessment']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['updateAssessment']._serialized_options = b'\362\206\031\177\nh\n\005PATCH\022Y/mlflow/traces/{location_id}/{assessment.trace_id}/assessments/{assessment.assessment_id}\032\004\010\004\020\000\020\003*\021Update Assessment'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['deleteAssessment']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['deleteAssessment']._serialized_options = b'\362\206\031j\nS\n\006DELETE\022C/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\032\004\010\004\020\000\020\003*\021Delete Assessment'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['batchLinkTraceToRun']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['batchLinkTraceToRun']._serialized_options = b'\362\206\031Z\nB\n\004POST\0224/mlflow/traces/{location_id}/link-to-run/batchCreate\032\004\010\004\020\000\020\003*\022Link Traces to Run'
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['batchUnlinkTraceFromRun']._loaded_options = None
    _globals['_DATABRICKSTRACKINGSERVICE'].methods_by_name['batchUnlinkTraceFromRun']._serialized_options = b'\362\206\031d\nH\n\006DELETE\0228/mlflow/traces/{location_id}/unlink-from-run/batchDelete\032\004\010\004\020\000\020\003*\026Unlink Traces from Run'
    _globals['_UCSCHEMALOCATION']._serialized_start=248
    _globals['_UCSCHEMALOCATION']._serialized_end=370
    _globals['_MLFLOWEXPERIMENTLOCATION']._serialized_start=372
    _globals['_MLFLOWEXPERIMENTLOCATION']._serialized_end=421
    _globals['_INFERENCETABLELOCATION']._serialized_start=423
    _globals['_INFERENCETABLELOCATION']._serialized_end=472
    _globals['_TRACELOCATION']._serialized_start=475
    _globals['_TRACELOCATION']._serialized_end=889
    _globals['_TRACELOCATION_TRACELOCATIONTYPE']._serialized_start=760
    _globals['_TRACELOCATION_TRACELOCATIONTYPE']._serialized_end=875
    _globals['_TRACEINFO']._serialized_start=892
    _globals['_TRACEINFO']._serialized_end=1559
    _globals['_TRACEINFO_TRACEMETADATAENTRY']._serialized_start=1394
    _globals['_TRACEINFO_TRACEMETADATAENTRY']._serialized_end=1446
    _globals['_TRACEINFO_TAGSENTRY']._serialized_start=1448
    _globals['_TRACEINFO_TAGSENTRY']._serialized_end=1491
    _globals['_TRACEINFO_STATE']._serialized_start=1493
    _globals['_TRACEINFO_STATE']._serialized_end=1559
    _globals['_CREATETRACEINFO']._serialized_start=1562
    _globals['_CREATETRACEINFO']._serialized_end=1769
    _globals['_CREATETRACEINFO_RESPONSE']._serialized_start=1664
    _globals['_CREATETRACEINFO_RESPONSE']._serialized_end=1724
    _globals['_TRACEPATH']._serialized_start=1771
    _globals['_TRACEPATH']._serialized_end=1870
    _globals['_TRACE']._serialized_start=1872
    _globals['_TRACE']._serialized_end=1980
    _globals['_BATCHGETTRACES']._serialized_start=1983
    _globals['_BATCHGETTRACES']._serialized_end=2170
    _globals['_BATCHGETTRACES_RESPONSE']._serialized_start=2073
    _globals['_BATCHGETTRACES_RESPONSE']._serialized_end=2125
    _globals['_GETTRACEINFO']._serialized_start=2173
    _globals['_GETTRACEINFO']._serialized_end=2359
    _globals['_GETTRACEINFO_RESPONSE']._serialized_start=2263
    _globals['_GETTRACEINFO_RESPONSE']._serialized_end=2314
    _globals['_SETTRACETAG']._serialized_start=2362
    _globals['_SETTRACETAG']._serialized_end=2517
    _globals['_SETTRACETAG_RESPONSE']._serialized_start=1664
    _globals['_SETTRACETAG_RESPONSE']._serialized_end=1674
    _globals['_DELETETRACETAG']._serialized_start=2520
    _globals['_DELETETRACETAG']._serialized_end=2689
    _globals['_DELETETRACETAG_RESPONSE']._serialized_start=1664
    _globals['_DELETETRACETAG_RESPONSE']._serialized_end=1674
    _globals['_SEARCHTRACES']._serialized_start=2692
    _globals['_SEARCHTRACES']._serialized_end=2998
    _globals['_SEARCHTRACES_RESPONSE']._serialized_start=2867
    _globals['_SEARCHTRACES_RESPONSE']._serialized_end=2953
    _globals['_CREATETRACEUCSTORAGELOCATION']._serialized_start=3001
    _globals['_CREATETRACEUCSTORAGELOCATION']._serialized_end=3246
    _globals['_CREATETRACEUCSTORAGELOCATION_RESPONSE']._serialized_start=3123
    _globals['_CREATETRACEUCSTORAGELOCATION_RESPONSE']._serialized_end=3189
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION']._serialized_start=3249
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION']._serialized_end=3438
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION_RESPONSE']._serialized_start=1664
    _globals['_LINKEXPERIMENTTOUCTRACELOCATION_RESPONSE']._serialized_end=1674
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION']._serialized_start=3441
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION']._serialized_end=3632
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION_RESPONSE']._serialized_start=1664
    _globals['_UNLINKEXPERIMENTTOUCTRACELOCATION_RESPONSE']._serialized_end=1674
    _globals['_ASSESSMENT']._serialized_start=3635
    _globals['_ASSESSMENT']._serialized_end=4243
    _globals['_ASSESSMENT_METADATAENTRY']._serialized_start=4187
    _globals['_ASSESSMENT_METADATAENTRY']._serialized_end=4234
    _globals['_CREATEASSESSMENT']._serialized_start=4246
    _globals['_CREATEASSESSMENT']._serialized_end=4482
    _globals['_CREATEASSESSMENT_RESPONSE']._serialized_start=4376
    _globals['_CREATEASSESSMENT_RESPONSE']._serialized_end=4437
    _globals['_GETASSESSMENT']._serialized_start=4485
    _globals['_GETASSESSMENT']._serialized_end=4714
    _globals['_GETASSESSMENT_RESPONSE']._serialized_start=4376
    _globals['_GETASSESSMENT_RESPONSE']._serialized_end=4437
    _globals['_UPDATEASSESSMENT']._serialized_start=4717
    _globals['_UPDATEASSESSMENT']._serialized_end=5008
    _globals['_UPDATEASSESSMENT_RESPONSE']._serialized_start=4376
    _globals['_UPDATEASSESSMENT_RESPONSE']._serialized_end=4437
    _globals['_DELETEASSESSMENT']._serialized_start=5011
    _globals['_DELETEASSESSMENT']._serialized_end=5192
    _globals['_DELETEASSESSMENT_RESPONSE']._serialized_start=1664
    _globals['_DELETEASSESSMENT_RESPONSE']._serialized_end=1674
    _globals['_BATCHLINKTRACETORUN']._serialized_start=5195
    _globals['_BATCHLINKTRACETORUN']._serialized_end=5341
    _globals['_BATCHLINKTRACETORUN_RESPONSE']._serialized_start=1664
    _globals['_BATCHLINKTRACETORUN_RESPONSE']._serialized_end=1674
    _globals['_BATCHUNLINKTRACEFROMRUN']._serialized_start=5344
    _globals['_BATCHUNLINKTRACEFROMRUN']._serialized_end=5494
    _globals['_BATCHUNLINKTRACEFROMRUN_RESPONSE']._serialized_start=1664
    _globals['_BATCHUNLINKTRACEFROMRUN_RESPONSE']._serialized_end=1674
    _globals['_DATABRICKSTRACKINGSERVICE']._serialized_start=5497
    _globals['_DATABRICKSTRACKINGSERVICE']._serialized_end=8617
  _builder.BuildServices(DESCRIPTOR, 'databricks_tracing_pb2', _globals)
  # @@protoc_insertion_point(module_scope)

else:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: databricks_tracing.proto
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import message as _message
  from google.protobuf import reflection as _reflection
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf import service as _service
  from google.protobuf import service_reflection
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import assessments_pb2 as assessments__pb2
  from . import databricks_pb2 as databricks__pb2
  from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
  from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
  from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
  from opentelemetry.proto.trace.v1 import trace_pb2 as opentelemetry_dot_proto_dot_trace_dot_v1_dot_trace__pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x64\x61tabricks_tracing.proto\x12\x11mlflow.databricks\x1a\x11\x61ssessments.proto\x1a\x10\x64\x61tabricks.proto\x1a\x1egoogle/protobuf/duration.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a(opentelemetry/proto/trace/v1/trace.proto\x1a\x15scalapb/scalapb.proto\"z\n\x10UCSchemaLocation\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x13\n\x0bschema_name\x18\x02 \x01(\t\x12\x1d\n\x15otel_spans_table_name\x18\x03 \x01(\t\x12\x1c\n\x14otel_logs_table_name\x18\x04 \x01(\t\"1\n\x18MlflowExperimentLocation\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\"1\n\x16InferenceTableLocation\x12\x17\n\x0f\x66ull_table_name\x18\x01 \x01(\t\"\x9e\x03\n\rTraceLocation\x12@\n\x04type\x18\x01 \x01(\x0e\x32\x32.mlflow.databricks.TraceLocation.TraceLocationType\x12H\n\x11mlflow_experiment\x18\x02 \x01(\x0b\x32+.mlflow.databricks.MlflowExperimentLocationH\x00\x12\x44\n\x0finference_table\x18\x03 \x01(\x0b\x32).mlflow.databricks.InferenceTableLocationH\x00\x12\x38\n\tuc_schema\x18\x04 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\"s\n\x11TraceLocationType\x12#\n\x1fTRACE_LOCATION_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11MLFLOW_EXPERIMENT\x10\x01\x12\x13\n\x0fINFERENCE_TABLE\x10\x02\x12\r\n\tUC_SCHEMA\x10\x03\x42\x0c\n\nidentifier\"\x9b\x05\n\tTraceInfo\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x12\x19\n\x11\x63lient_request_id\x18\x02 \x01(\t\x12\x38\n\x0etrace_location\x18\x03 \x01(\x0b\x32 .mlflow.databricks.TraceLocation\x12\x17\n\x0frequest_preview\x18\x04 \x01(\t\x12\x18\n\x10response_preview\x18\x05 \x01(\t\x12\x30\n\x0crequest_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12\x65xecution_duration\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x31\n\x05state\x18\x08 \x01(\x0e\x32\".mlflow.databricks.TraceInfo.State\x12G\n\x0etrace_metadata\x18\t \x03(\x0b\x32/.mlflow.databricks.TraceInfo.TraceMetadataEntry\x12\x32\n\x0b\x61ssessments\x18\n \x03(\x0b\x32\x1d.mlflow.databricks.Assessment\x12\x34\n\x04tags\x18\x0b \x03(\x0b\x32&.mlflow.databricks.TraceInfo.TagsEntry\x1a\x34\n\x12TraceMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0f\n\x0bIN_PROGRESS\x10\x03\"\xcf\x01\n\x0f\x43reateTraceInfo\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x36\n\ntrace_info\x18\x02 \x01(\x0b\x32\x1c.mlflow.databricks.TraceInfoB\x04\xf8\x86\x19\x01\x1a<\n\x08Response\x12\x30\n\ntrace_info\x18\x01 \x01(\x0b\x32\x1c.mlflow.databricks.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"c\n\tTracePath\x12>\n\x0etrace_location\x18\x01 \x01(\x0b\x32 .mlflow.databricks.TraceLocationB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\"l\n\x05Trace\x12\x30\n\ntrace_info\x18\x01 \x01(\x0b\x32\x1c.mlflow.databricks.TraceInfo\x12\x31\n\x05spans\x18\x02 \x03(\x0b\x32\".opentelemetry.proto.trace.v1.Span\"\xbb\x01\n\x0e\x42\x61tchGetTraces\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\ttrace_ids\x18\x02 \x03(\t\x12\x18\n\x10sql_warehouse_id\x18\x03 \x01(\t\x1a\x34\n\x08Response\x12(\n\x06traces\x18\x01 \x03(\x0b\x32\x18.mlflow.databricks.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xba\x01\n\x0cGetTraceInfo\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08location\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x03 \x01(\t\x1a\x33\n\x08Response\x12\'\n\x05trace\x18\x01 \x01(\x0b\x32\x18.mlflow.databricks.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9b\x01\n\x0bSetTraceTag\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0blocation_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\r\n\x05value\x18\x04 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa9\x01\n\x0e\x44\x65leteTraceTag\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0blocation_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb2\x02\n\x0cSearchTraces\x12\x33\n\tlocations\x18\x01 \x03(\x0b\x32 .mlflow.databricks.TraceLocation\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x18\n\x0bmax_results\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x18\n\x10sql_warehouse_id\x18\x05 \x01(\t\x12\x12\n\npage_token\x18\x06 \x01(\t\x1aV\n\x08Response\x12\x31\n\x0btrace_infos\x18\x01 \x03(\x0b\x32\x1c.mlflow.databricks.TraceInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf5\x01\n\x1c\x43reateTraceUCStorageLocation\x12\x38\n\tuc_schema\x18\x01 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\x12\x1e\n\x10sql_warehouse_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x42\n\x08Response\x12\x36\n\tuc_schema\x18\x01 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocation:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]B\n\n\x08location\"\xbd\x01\n\x1fLinkExperimentToUCTraceLocation\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x38\n\tuc_schema\x18\x02 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]B\n\n\x08location\"\xbf\x01\n!UnLinkExperimentToUCTraceLocation\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x38\n\tuc_schema\x18\x02 \x01(\x0b\x32#.mlflow.databricks.UCSchemaLocationH\x00\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]B\n\n\x08location\"\xe0\x04\n\nAssessment\x12\x15\n\rassessment_id\x18\x01 \x01(\t\x12\x1d\n\x0f\x61ssessment_name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x38\n\x0etrace_location\x18\x04 \x01(\x0b\x32 .mlflow.databricks.TraceLocation\x12\x0f\n\x07span_id\x18\x05 \x01(\t\x12\x34\n\x06source\x18\x06 \x01(\x0b\x32$.mlflow.assessments.AssessmentSource\x12/\n\x0b\x63reate_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x10last_update_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x08\x66\x65\x65\x64\x62\x61\x63k\x18\t \x01(\x0b\x32\x1c.mlflow.assessments.FeedbackH\x00\x12\x36\n\x0b\x65xpectation\x18\n \x01(\x0b\x32\x1f.mlflow.assessments.ExpectationH\x00\x12\x11\n\trationale\x18\x0b \x01(\t\x12=\n\x08metadata\x18\x0c \x03(\x0b\x32+.mlflow.databricks.Assessment.MetadataEntry\x12\x11\n\toverrides\x18\r \x01(\t\x12\x13\n\x05valid\x18\x0e \x01(\x08:\x04true\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x07\n\x05value\"\xec\x01\n\x10\x43reateAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x37\n\nassessment\x18\x02 \x01(\x0b\x32\x1d.mlflow.databricks.AssessmentB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x03 \x01(\t\x1a=\n\x08Response\x12\x31\n\nassessment\x18\x01 \x01(\x0b\x32\x1d.mlflow.databricks.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xe5\x01\n\rGetAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a=\n\x08Response\x12\x31\n\nassessment\x18\x01 \x01(\x0b\x32\x1d.mlflow.databricks.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa3\x02\n\x10UpdateAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x37\n\nassessment\x18\x02 \x01(\x0b\x32\x1d.mlflow.databricks.AssessmentB\x04\xf8\x86\x19\x01\x12\x35\n\x0bupdate_mask\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a=\n\x08Response\x12\x31\n\nassessment\x18\x01 \x01(\x0b\x32\x1d.mlflow.databricks.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb5\x01\n\x10\x44\x65leteAssessment\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x08trace_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x18\n\x10sql_warehouse_id\x18\x04 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x92\x01\n\x13\x42\x61tchLinkTraceToRun\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\ttrace_ids\x18\x02 \x03(\t\x12\x14\n\x06run_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x96\x01\n\x17\x42\x61tchUnlinkTraceFromRun\x12\x19\n\x0blocation_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\ttrace_ids\x18\x02 \x03(\t\x12\x14\n\x06run_id\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]2\xb0\x18\n\x19\x44\x61tabricksTrackingService\x12\xb8\x01\n\x0f\x63reateTraceInfo\x12\".mlflow.databricks.CreateTraceInfo\x1a\x1c.mlflow.databricks.TraceInfo\"c\xf2\x86\x19_\nE\n\x04POST\x12\x37/mlflow/traces/{location_id}/{trace_info.trace_id}/info\x1a\x04\x08\x04\x10\x00\x10\x03*\x14\x43reate Trace Info v4\x12\xaa\x01\n\x0e\x62\x61tchGetTraces\x12!.mlflow.databricks.BatchGetTraces\x1a*.mlflow.databricks.BatchGetTraces.Response\"I\xf2\x86\x19\x45\n2\n\x03GET\x12%/mlflow/traces/{location_id}/batchGet\x1a\x04\x08\x04\x10\x00\x10\x03*\rGet Traces V4\x12\xa8\x01\n\x0cgetTraceInfo\x12\x1f.mlflow.databricks.GetTraceInfo\x1a(.mlflow.databricks.GetTraceInfo.Response\"M\xf2\x86\x19I\n6\n\x03GET\x12)/mlflow/traces/{location}/{trace_id}/info\x1a\x04\x08\x04\x10\x00\x10\x03*\rGet TraceInfo\x12\xaa\x01\n\x0bsetTraceTag\x12\x1e.mlflow.databricks.SetTraceTag\x1a\'.mlflow.databricks.SetTraceTag.Response\"R\xf2\x86\x19N\n;\n\x05PATCH\x12,/mlflow/traces/{location_id}/{trace_id}/tags\x1a\x04\x08\x04\x10\x00\x10\x03*\rSet Trace Tag\x12\xbd\x01\n\x0e\x64\x65leteTraceTag\x12!.mlflow.databricks.DeleteTraceTag\x1a*.mlflow.databricks.DeleteTraceTag.Response\"\\\xf2\x86\x19X\nB\n\x06\x44\x45LETE\x12\x32/mlflow/traces/{location_id}/{trace_id}/tags/{key}\x1a\x04\x08\x04\x10\x00\x10\x03*\x10\x44\x65lete Trace Tag\x12\x95\x01\n\x0csearchTraces\x12\x1f.mlflow.databricks.SearchTraces\x1a(.mlflow.databricks.SearchTraces.Response\":\xf2\x86\x19\x36\n#\n\x04POST\x12\x15/mlflow/traces/search\x1a\x04\x08\x04\x10\x00\x10\x03*\rSearch Traces\x12\xda\x01\n\x1c\x63reateTraceUCStorageLocation\x12/.mlflow.databricks.CreateTraceUCStorageLocation\x1a\x38.mlflow.databricks.CreateTraceUCStorageLocation.Response\"O\xf2\x86\x19K\n%\n\x04POST\x12\x17/mlflow/traces/location\x1a\x04\x08\x04\x10\x00\x10\x03* Create Trace UC Storage Location\x12\xfc\x01\n\x1flinkExperimentToUCTraceLocation\x12\x32.mlflow.databricks.LinkExperimentToUCTraceLocation\x1a;.mlflow.databricks.LinkExperimentToUCTraceLocation.Response\"h\xf2\x86\x19\x64\n:\n\x04POST\x12,/mlflow/traces/{experiment_id}/link-location\x1a\x04\x08\x04\x10\x00\x10\x03*$Link Experiment to UC Trace Location\x12\x86\x02\n!unlinkExperimentToUCTraceLocation\x12\x34.mlflow.databricks.UnLinkExperimentToUCTraceLocation\x1a=.mlflow.databricks.UnLinkExperimentToUCTraceLocation.Response\"l\xf2\x86\x19h\n<\n\x04POST\x12./mlflow/traces/{experiment_id}/unlink-location\x1a\x04\x08\x04\x10\x00\x10\x03*&Unlink Experiment to UC Trace Location\x12\xce\x01\n\x10\x63reateAssessment\x12#.mlflow.databricks.CreateAssessment\x1a,.mlflow.databricks.CreateAssessment.Response\"g\xf2\x86\x19\x63\nL\n\x04POST\x12>/mlflow/traces/{location_id}/{assessment.trace_id}/assessments\x1a\x04\x08\x04\x10\x00\x10\x03*\x11\x43reate Assessment\x12\xc6\x01\n\rgetAssessment\x12 .mlflow.databricks.GetAssessment\x1a).mlflow.databricks.GetAssessment.Response\"h\xf2\x86\x19\x64\nP\n\x03GET\x12\x43/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x04\x10\x00\x10\x03*\x0eGet Assessment\x12\xeb\x01\n\x10updateAssessment\x12#.mlflow.databricks.UpdateAssessment\x1a,.mlflow.databricks.UpdateAssessment.Response\"\x83\x01\xf2\x86\x19\x7f\nh\n\x05PATCH\x12Y/mlflow/traces/{location_id}/{assessment.trace_id}/assessments/{assessment.assessment_id}\x1a\x04\x08\x04\x10\x00\x10\x03*\x11Update Assessment\x12\xd5\x01\n\x10\x64\x65leteAssessment\x12#.mlflow.databricks.DeleteAssessment\x1a,.mlflow.databricks.DeleteAssessment.Response\"n\xf2\x86\x19j\nS\n\x06\x44\x45LETE\x12\x43/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x04\x10\x00\x10\x03*\x11\x44\x65lete Assessment\x12\xce\x01\n\x13\x62\x61tchLinkTraceToRun\x12&.mlflow.databricks.BatchLinkTraceToRun\x1a/.mlflow.databricks.BatchLinkTraceToRun.Response\"^\xf2\x86\x19Z\nB\n\x04POST\x12\x34/mlflow/traces/{location_id}/link-to-run/batchCreate\x1a\x04\x08\x04\x10\x00\x10\x03*\x12Link Traces to Run\x12\xe4\x01\n\x17\x62\x61tchUnlinkTraceFromRun\x12*.mlflow.databricks.BatchUnlinkTraceFromRun\x1a\x33.mlflow.databricks.BatchUnlinkTraceFromRun.Response\"h\xf2\x86\x19\x64\nH\n\x06\x44\x45LETE\x12\x38/mlflow/traces/{location_id}/unlink-from-run/batchDelete\x1a\x04\x08\x04\x10\x00\x10\x03*\x16Unlink Traces from RunB\x03\x90\x01\x01')



  _UCSCHEMALOCATION = DESCRIPTOR.message_types_by_name['UCSchemaLocation']
  _MLFLOWEXPERIMENTLOCATION = DESCRIPTOR.message_types_by_name['MlflowExperimentLocation']
  _INFERENCETABLELOCATION = DESCRIPTOR.message_types_by_name['InferenceTableLocation']
  _TRACELOCATION = DESCRIPTOR.message_types_by_name['TraceLocation']
  _TRACEINFO = DESCRIPTOR.message_types_by_name['TraceInfo']
  _TRACEINFO_TRACEMETADATAENTRY = _TRACEINFO.nested_types_by_name['TraceMetadataEntry']
  _TRACEINFO_TAGSENTRY = _TRACEINFO.nested_types_by_name['TagsEntry']
  _CREATETRACEINFO = DESCRIPTOR.message_types_by_name['CreateTraceInfo']
  _CREATETRACEINFO_RESPONSE = _CREATETRACEINFO.nested_types_by_name['Response']
  _TRACEPATH = DESCRIPTOR.message_types_by_name['TracePath']
  _TRACE = DESCRIPTOR.message_types_by_name['Trace']
  _BATCHGETTRACES = DESCRIPTOR.message_types_by_name['BatchGetTraces']
  _BATCHGETTRACES_RESPONSE = _BATCHGETTRACES.nested_types_by_name['Response']
  _GETTRACEINFO = DESCRIPTOR.message_types_by_name['GetTraceInfo']
  _GETTRACEINFO_RESPONSE = _GETTRACEINFO.nested_types_by_name['Response']
  _SETTRACETAG = DESCRIPTOR.message_types_by_name['SetTraceTag']
  _SETTRACETAG_RESPONSE = _SETTRACETAG.nested_types_by_name['Response']
  _DELETETRACETAG = DESCRIPTOR.message_types_by_name['DeleteTraceTag']
  _DELETETRACETAG_RESPONSE = _DELETETRACETAG.nested_types_by_name['Response']
  _SEARCHTRACES = DESCRIPTOR.message_types_by_name['SearchTraces']
  _SEARCHTRACES_RESPONSE = _SEARCHTRACES.nested_types_by_name['Response']
  _CREATETRACEUCSTORAGELOCATION = DESCRIPTOR.message_types_by_name['CreateTraceUCStorageLocation']
  _CREATETRACEUCSTORAGELOCATION_RESPONSE = _CREATETRACEUCSTORAGELOCATION.nested_types_by_name['Response']
  _LINKEXPERIMENTTOUCTRACELOCATION = DESCRIPTOR.message_types_by_name['LinkExperimentToUCTraceLocation']
  _LINKEXPERIMENTTOUCTRACELOCATION_RESPONSE = _LINKEXPERIMENTTOUCTRACELOCATION.nested_types_by_name['Response']
  _UNLINKEXPERIMENTTOUCTRACELOCATION = DESCRIPTOR.message_types_by_name['UnLinkExperimentToUCTraceLocation']
  _UNLINKEXPERIMENTTOUCTRACELOCATION_RESPONSE = _UNLINKEXPERIMENTTOUCTRACELOCATION.nested_types_by_name['Response']
  _ASSESSMENT = DESCRIPTOR.message_types_by_name['Assessment']
  _ASSESSMENT_METADATAENTRY = _ASSESSMENT.nested_types_by_name['MetadataEntry']
  _CREATEASSESSMENT = DESCRIPTOR.message_types_by_name['CreateAssessment']
  _CREATEASSESSMENT_RESPONSE = _CREATEASSESSMENT.nested_types_by_name['Response']
  _GETASSESSMENT = DESCRIPTOR.message_types_by_name['GetAssessment']
  _GETASSESSMENT_RESPONSE = _GETASSESSMENT.nested_types_by_name['Response']
  _UPDATEASSESSMENT = DESCRIPTOR.message_types_by_name['UpdateAssessment']
  _UPDATEASSESSMENT_RESPONSE = _UPDATEASSESSMENT.nested_types_by_name['Response']
  _DELETEASSESSMENT = DESCRIPTOR.message_types_by_name['DeleteAssessment']
  _DELETEASSESSMENT_RESPONSE = _DELETEASSESSMENT.nested_types_by_name['Response']
  _BATCHLINKTRACETORUN = DESCRIPTOR.message_types_by_name['BatchLinkTraceToRun']
  _BATCHLINKTRACETORUN_RESPONSE = _BATCHLINKTRACETORUN.nested_types_by_name['Response']
  _BATCHUNLINKTRACEFROMRUN = DESCRIPTOR.message_types_by_name['BatchUnlinkTraceFromRun']
  _BATCHUNLINKTRACEFROMRUN_RESPONSE = _BATCHUNLINKTRACEFROMRUN.nested_types_by_name['Response']
  _TRACELOCATION_TRACELOCATIONTYPE = _TRACELOCATION.enum_types_by_name['TraceLocationType']
  _TRACEINFO_STATE = _TRACEINFO.enum_types_by_name['State']
  UCSchemaLocation = _reflection.GeneratedProtocolMessageType('UCSchemaLocation', (_message.Message,), {
    'DESCRIPTOR' : _UCSCHEMALOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.UCSchemaLocation)
    })
  _sym_db.RegisterMessage(UCSchemaLocation)

  MlflowExperimentLocation = _reflection.GeneratedProtocolMessageType('MlflowExperimentLocation', (_message.Message,), {
    'DESCRIPTOR' : _MLFLOWEXPERIMENTLOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.MlflowExperimentLocation)
    })
  _sym_db.RegisterMessage(MlflowExperimentLocation)

  InferenceTableLocation = _reflection.GeneratedProtocolMessageType('InferenceTableLocation', (_message.Message,), {
    'DESCRIPTOR' : _INFERENCETABLELOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.InferenceTableLocation)
    })
  _sym_db.RegisterMessage(InferenceTableLocation)

  TraceLocation = _reflection.GeneratedProtocolMessageType('TraceLocation', (_message.Message,), {
    'DESCRIPTOR' : _TRACELOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.TraceLocation)
    })
  _sym_db.RegisterMessage(TraceLocation)

  TraceInfo = _reflection.GeneratedProtocolMessageType('TraceInfo', (_message.Message,), {

    'TraceMetadataEntry' : _reflection.GeneratedProtocolMessageType('TraceMetadataEntry', (_message.Message,), {
      'DESCRIPTOR' : _TRACEINFO_TRACEMETADATAENTRY,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.TraceInfo.TraceMetadataEntry)
      })
    ,

    'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
      'DESCRIPTOR' : _TRACEINFO_TAGSENTRY,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.TraceInfo.TagsEntry)
      })
    ,
    'DESCRIPTOR' : _TRACEINFO,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.TraceInfo)
    })
  _sym_db.RegisterMessage(TraceInfo)
  _sym_db.RegisterMessage(TraceInfo.TraceMetadataEntry)
  _sym_db.RegisterMessage(TraceInfo.TagsEntry)

  CreateTraceInfo = _reflection.GeneratedProtocolMessageType('CreateTraceInfo', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATETRACEINFO_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.CreateTraceInfo.Response)
      })
    ,
    'DESCRIPTOR' : _CREATETRACEINFO,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.CreateTraceInfo)
    })
  _sym_db.RegisterMessage(CreateTraceInfo)
  _sym_db.RegisterMessage(CreateTraceInfo.Response)

  TracePath = _reflection.GeneratedProtocolMessageType('TracePath', (_message.Message,), {
    'DESCRIPTOR' : _TRACEPATH,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.TracePath)
    })
  _sym_db.RegisterMessage(TracePath)

  Trace = _reflection.GeneratedProtocolMessageType('Trace', (_message.Message,), {
    'DESCRIPTOR' : _TRACE,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.Trace)
    })
  _sym_db.RegisterMessage(Trace)

  BatchGetTraces = _reflection.GeneratedProtocolMessageType('BatchGetTraces', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _BATCHGETTRACES_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.BatchGetTraces.Response)
      })
    ,
    'DESCRIPTOR' : _BATCHGETTRACES,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.BatchGetTraces)
    })
  _sym_db.RegisterMessage(BatchGetTraces)
  _sym_db.RegisterMessage(BatchGetTraces.Response)

  GetTraceInfo = _reflection.GeneratedProtocolMessageType('GetTraceInfo', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETTRACEINFO_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.GetTraceInfo.Response)
      })
    ,
    'DESCRIPTOR' : _GETTRACEINFO,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.GetTraceInfo)
    })
  _sym_db.RegisterMessage(GetTraceInfo)
  _sym_db.RegisterMessage(GetTraceInfo.Response)

  SetTraceTag = _reflection.GeneratedProtocolMessageType('SetTraceTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETTRACETAG_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.SetTraceTag.Response)
      })
    ,
    'DESCRIPTOR' : _SETTRACETAG,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.SetTraceTag)
    })
  _sym_db.RegisterMessage(SetTraceTag)
  _sym_db.RegisterMessage(SetTraceTag.Response)

  DeleteTraceTag = _reflection.GeneratedProtocolMessageType('DeleteTraceTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETETRACETAG_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.DeleteTraceTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETETRACETAG,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.DeleteTraceTag)
    })
  _sym_db.RegisterMessage(DeleteTraceTag)
  _sym_db.RegisterMessage(DeleteTraceTag.Response)

  SearchTraces = _reflection.GeneratedProtocolMessageType('SearchTraces', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHTRACES_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.SearchTraces.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHTRACES,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.SearchTraces)
    })
  _sym_db.RegisterMessage(SearchTraces)
  _sym_db.RegisterMessage(SearchTraces.Response)

  CreateTraceUCStorageLocation = _reflection.GeneratedProtocolMessageType('CreateTraceUCStorageLocation', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATETRACEUCSTORAGELOCATION_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.CreateTraceUCStorageLocation.Response)
      })
    ,
    'DESCRIPTOR' : _CREATETRACEUCSTORAGELOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.CreateTraceUCStorageLocation)
    })
  _sym_db.RegisterMessage(CreateTraceUCStorageLocation)
  _sym_db.RegisterMessage(CreateTraceUCStorageLocation.Response)

  LinkExperimentToUCTraceLocation = _reflection.GeneratedProtocolMessageType('LinkExperimentToUCTraceLocation', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LINKEXPERIMENTTOUCTRACELOCATION_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.LinkExperimentToUCTraceLocation.Response)
      })
    ,
    'DESCRIPTOR' : _LINKEXPERIMENTTOUCTRACELOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.LinkExperimentToUCTraceLocation)
    })
  _sym_db.RegisterMessage(LinkExperimentToUCTraceLocation)
  _sym_db.RegisterMessage(LinkExperimentToUCTraceLocation.Response)

  UnLinkExperimentToUCTraceLocation = _reflection.GeneratedProtocolMessageType('UnLinkExperimentToUCTraceLocation', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UNLINKEXPERIMENTTOUCTRACELOCATION_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.UnLinkExperimentToUCTraceLocation.Response)
      })
    ,
    'DESCRIPTOR' : _UNLINKEXPERIMENTTOUCTRACELOCATION,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.UnLinkExperimentToUCTraceLocation)
    })
  _sym_db.RegisterMessage(UnLinkExperimentToUCTraceLocation)
  _sym_db.RegisterMessage(UnLinkExperimentToUCTraceLocation.Response)

  Assessment = _reflection.GeneratedProtocolMessageType('Assessment', (_message.Message,), {

    'MetadataEntry' : _reflection.GeneratedProtocolMessageType('MetadataEntry', (_message.Message,), {
      'DESCRIPTOR' : _ASSESSMENT_METADATAENTRY,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.Assessment.MetadataEntry)
      })
    ,
    'DESCRIPTOR' : _ASSESSMENT,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.Assessment)
    })
  _sym_db.RegisterMessage(Assessment)
  _sym_db.RegisterMessage(Assessment.MetadataEntry)

  CreateAssessment = _reflection.GeneratedProtocolMessageType('CreateAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEASSESSMENT_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.CreateAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEASSESSMENT,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.CreateAssessment)
    })
  _sym_db.RegisterMessage(CreateAssessment)
  _sym_db.RegisterMessage(CreateAssessment.Response)

  GetAssessment = _reflection.GeneratedProtocolMessageType('GetAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETASSESSMENT_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.GetAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _GETASSESSMENT,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.GetAssessment)
    })
  _sym_db.RegisterMessage(GetAssessment)
  _sym_db.RegisterMessage(GetAssessment.Response)

  UpdateAssessment = _reflection.GeneratedProtocolMessageType('UpdateAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEASSESSMENT_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.UpdateAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEASSESSMENT,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.UpdateAssessment)
    })
  _sym_db.RegisterMessage(UpdateAssessment)
  _sym_db.RegisterMessage(UpdateAssessment.Response)

  DeleteAssessment = _reflection.GeneratedProtocolMessageType('DeleteAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEASSESSMENT_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.DeleteAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEASSESSMENT,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.DeleteAssessment)
    })
  _sym_db.RegisterMessage(DeleteAssessment)
  _sym_db.RegisterMessage(DeleteAssessment.Response)

  BatchLinkTraceToRun = _reflection.GeneratedProtocolMessageType('BatchLinkTraceToRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _BATCHLINKTRACETORUN_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.BatchLinkTraceToRun.Response)
      })
    ,
    'DESCRIPTOR' : _BATCHLINKTRACETORUN,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.BatchLinkTraceToRun)
    })
  _sym_db.RegisterMessage(BatchLinkTraceToRun)
  _sym_db.RegisterMessage(BatchLinkTraceToRun.Response)

  BatchUnlinkTraceFromRun = _reflection.GeneratedProtocolMessageType('BatchUnlinkTraceFromRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _BATCHUNLINKTRACEFROMRUN_RESPONSE,
      '__module__' : 'databricks_tracing_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.databricks.BatchUnlinkTraceFromRun.Response)
      })
    ,
    'DESCRIPTOR' : _BATCHUNLINKTRACEFROMRUN,
    '__module__' : 'databricks_tracing_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.databricks.BatchUnlinkTraceFromRun)
    })
  _sym_db.RegisterMessage(BatchUnlinkTraceFromRun)
  _sym_db.RegisterMessage(BatchUnlinkTraceFromRun.Response)

  _DATABRICKSTRACKINGSERVICE = DESCRIPTOR.services_by_name['DatabricksTrackingService']
  if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'\220\001\001'
    _TRACEINFO_TRACEMETADATAENTRY._options = None
    _TRACEINFO_TRACEMETADATAENTRY._serialized_options = b'8\001'
    _TRACEINFO_TAGSENTRY._options = None
    _TRACEINFO_TAGSENTRY._serialized_options = b'8\001'
    _CREATETRACEINFO.fields_by_name['location_id']._options = None
    _CREATETRACEINFO.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _CREATETRACEINFO.fields_by_name['trace_info']._options = None
    _CREATETRACEINFO.fields_by_name['trace_info']._serialized_options = b'\370\206\031\001'
    _CREATETRACEINFO._options = None
    _CREATETRACEINFO._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _TRACEPATH.fields_by_name['trace_location']._options = None
    _TRACEPATH.fields_by_name['trace_location']._serialized_options = b'\370\206\031\001'
    _TRACEPATH.fields_by_name['trace_id']._options = None
    _TRACEPATH.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _BATCHGETTRACES.fields_by_name['location_id']._options = None
    _BATCHGETTRACES.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _BATCHGETTRACES._options = None
    _BATCHGETTRACES._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETTRACEINFO.fields_by_name['trace_id']._options = None
    _GETTRACEINFO.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _GETTRACEINFO.fields_by_name['location']._options = None
    _GETTRACEINFO.fields_by_name['location']._serialized_options = b'\370\206\031\001'
    _GETTRACEINFO._options = None
    _GETTRACEINFO._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETTRACETAG.fields_by_name['trace_id']._options = None
    _SETTRACETAG.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _SETTRACETAG.fields_by_name['location_id']._options = None
    _SETTRACETAG.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _SETTRACETAG.fields_by_name['key']._options = None
    _SETTRACETAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _SETTRACETAG._options = None
    _SETTRACETAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETETRACETAG.fields_by_name['trace_id']._options = None
    _DELETETRACETAG.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _DELETETRACETAG.fields_by_name['location_id']._options = None
    _DELETETRACETAG.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _DELETETRACETAG.fields_by_name['key']._options = None
    _DELETETRACETAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _DELETETRACETAG._options = None
    _DELETETRACETAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHTRACES._options = None
    _SEARCHTRACES._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CREATETRACEUCSTORAGELOCATION.fields_by_name['sql_warehouse_id']._options = None
    _CREATETRACEUCSTORAGELOCATION.fields_by_name['sql_warehouse_id']._serialized_options = b'\370\206\031\001'
    _CREATETRACEUCSTORAGELOCATION._options = None
    _CREATETRACEUCSTORAGELOCATION._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LINKEXPERIMENTTOUCTRACELOCATION.fields_by_name['experiment_id']._options = None
    _LINKEXPERIMENTTOUCTRACELOCATION.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _LINKEXPERIMENTTOUCTRACELOCATION._options = None
    _LINKEXPERIMENTTOUCTRACELOCATION._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _UNLINKEXPERIMENTTOUCTRACELOCATION.fields_by_name['experiment_id']._options = None
    _UNLINKEXPERIMENTTOUCTRACELOCATION.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _UNLINKEXPERIMENTTOUCTRACELOCATION._options = None
    _UNLINKEXPERIMENTTOUCTRACELOCATION._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _ASSESSMENT_METADATAENTRY._options = None
    _ASSESSMENT_METADATAENTRY._serialized_options = b'8\001'
    _ASSESSMENT.fields_by_name['assessment_name']._options = None
    _ASSESSMENT.fields_by_name['assessment_name']._serialized_options = b'\370\206\031\001'
    _ASSESSMENT.fields_by_name['trace_id']._options = None
    _ASSESSMENT.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _CREATEASSESSMENT.fields_by_name['location_id']._options = None
    _CREATEASSESSMENT.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _CREATEASSESSMENT.fields_by_name['assessment']._options = None
    _CREATEASSESSMENT.fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _CREATEASSESSMENT._options = None
    _CREATEASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETASSESSMENT.fields_by_name['location_id']._options = None
    _GETASSESSMENT.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _GETASSESSMENT.fields_by_name['trace_id']._options = None
    _GETASSESSMENT.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _GETASSESSMENT.fields_by_name['assessment_id']._options = None
    _GETASSESSMENT.fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _GETASSESSMENT._options = None
    _GETASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _UPDATEASSESSMENT.fields_by_name['location_id']._options = None
    _UPDATEASSESSMENT.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _UPDATEASSESSMENT.fields_by_name['assessment']._options = None
    _UPDATEASSESSMENT.fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _UPDATEASSESSMENT.fields_by_name['update_mask']._options = None
    _UPDATEASSESSMENT.fields_by_name['update_mask']._serialized_options = b'\370\206\031\001'
    _UPDATEASSESSMENT._options = None
    _UPDATEASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETEASSESSMENT.fields_by_name['location_id']._options = None
    _DELETEASSESSMENT.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _DELETEASSESSMENT.fields_by_name['trace_id']._options = None
    _DELETEASSESSMENT.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _DELETEASSESSMENT.fields_by_name['assessment_id']._options = None
    _DELETEASSESSMENT.fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _DELETEASSESSMENT._options = None
    _DELETEASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _BATCHLINKTRACETORUN.fields_by_name['location_id']._options = None
    _BATCHLINKTRACETORUN.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _BATCHLINKTRACETORUN.fields_by_name['run_id']._options = None
    _BATCHLINKTRACETORUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _BATCHLINKTRACETORUN._options = None
    _BATCHLINKTRACETORUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _BATCHUNLINKTRACEFROMRUN.fields_by_name['location_id']._options = None
    _BATCHUNLINKTRACEFROMRUN.fields_by_name['location_id']._serialized_options = b'\370\206\031\001'
    _BATCHUNLINKTRACEFROMRUN.fields_by_name['run_id']._options = None
    _BATCHUNLINKTRACEFROMRUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _BATCHUNLINKTRACEFROMRUN._options = None
    _BATCHUNLINKTRACEFROMRUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['createTraceInfo']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['createTraceInfo']._serialized_options = b'\362\206\031_\nE\n\004POST\0227/mlflow/traces/{location_id}/{trace_info.trace_id}/info\032\004\010\004\020\000\020\003*\024Create Trace Info v4'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['batchGetTraces']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['batchGetTraces']._serialized_options = b'\362\206\031E\n2\n\003GET\022%/mlflow/traces/{location_id}/batchGet\032\004\010\004\020\000\020\003*\rGet Traces V4'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['getTraceInfo']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['getTraceInfo']._serialized_options = b'\362\206\031I\n6\n\003GET\022)/mlflow/traces/{location}/{trace_id}/info\032\004\010\004\020\000\020\003*\rGet TraceInfo'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['setTraceTag']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['setTraceTag']._serialized_options = b'\362\206\031N\n;\n\005PATCH\022,/mlflow/traces/{location_id}/{trace_id}/tags\032\004\010\004\020\000\020\003*\rSet Trace Tag'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['deleteTraceTag']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['deleteTraceTag']._serialized_options = b'\362\206\031X\nB\n\006DELETE\0222/mlflow/traces/{location_id}/{trace_id}/tags/{key}\032\004\010\004\020\000\020\003*\020Delete Trace Tag'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['searchTraces']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['searchTraces']._serialized_options = b'\362\206\0316\n#\n\004POST\022\025/mlflow/traces/search\032\004\010\004\020\000\020\003*\rSearch Traces'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['createTraceUCStorageLocation']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['createTraceUCStorageLocation']._serialized_options = b'\362\206\031K\n%\n\004POST\022\027/mlflow/traces/location\032\004\010\004\020\000\020\003* Create Trace UC Storage Location'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['linkExperimentToUCTraceLocation']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['linkExperimentToUCTraceLocation']._serialized_options = b'\362\206\031d\n:\n\004POST\022,/mlflow/traces/{experiment_id}/link-location\032\004\010\004\020\000\020\003*$Link Experiment to UC Trace Location'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['unlinkExperimentToUCTraceLocation']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['unlinkExperimentToUCTraceLocation']._serialized_options = b'\362\206\031h\n<\n\004POST\022./mlflow/traces/{experiment_id}/unlink-location\032\004\010\004\020\000\020\003*&Unlink Experiment to UC Trace Location'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['createAssessment']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['createAssessment']._serialized_options = b'\362\206\031c\nL\n\004POST\022>/mlflow/traces/{location_id}/{assessment.trace_id}/assessments\032\004\010\004\020\000\020\003*\021Create Assessment'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['getAssessment']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['getAssessment']._serialized_options = b'\362\206\031d\nP\n\003GET\022C/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\032\004\010\004\020\000\020\003*\016Get Assessment'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['updateAssessment']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['updateAssessment']._serialized_options = b'\362\206\031\177\nh\n\005PATCH\022Y/mlflow/traces/{location_id}/{assessment.trace_id}/assessments/{assessment.assessment_id}\032\004\010\004\020\000\020\003*\021Update Assessment'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['deleteAssessment']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['deleteAssessment']._serialized_options = b'\362\206\031j\nS\n\006DELETE\022C/mlflow/traces/{location_id}/{trace_id}/assessments/{assessment_id}\032\004\010\004\020\000\020\003*\021Delete Assessment'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['batchLinkTraceToRun']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['batchLinkTraceToRun']._serialized_options = b'\362\206\031Z\nB\n\004POST\0224/mlflow/traces/{location_id}/link-to-run/batchCreate\032\004\010\004\020\000\020\003*\022Link Traces to Run'
    _DATABRICKSTRACKINGSERVICE.methods_by_name['batchUnlinkTraceFromRun']._options = None
    _DATABRICKSTRACKINGSERVICE.methods_by_name['batchUnlinkTraceFromRun']._serialized_options = b'\362\206\031d\nH\n\006DELETE\0228/mlflow/traces/{location_id}/unlink-from-run/batchDelete\032\004\010\004\020\000\020\003*\026Unlink Traces from Run'
    _UCSCHEMALOCATION._serialized_start=248
    _UCSCHEMALOCATION._serialized_end=370
    _MLFLOWEXPERIMENTLOCATION._serialized_start=372
    _MLFLOWEXPERIMENTLOCATION._serialized_end=421
    _INFERENCETABLELOCATION._serialized_start=423
    _INFERENCETABLELOCATION._serialized_end=472
    _TRACELOCATION._serialized_start=475
    _TRACELOCATION._serialized_end=889
    _TRACELOCATION_TRACELOCATIONTYPE._serialized_start=760
    _TRACELOCATION_TRACELOCATIONTYPE._serialized_end=875
    _TRACEINFO._serialized_start=892
    _TRACEINFO._serialized_end=1559
    _TRACEINFO_TRACEMETADATAENTRY._serialized_start=1394
    _TRACEINFO_TRACEMETADATAENTRY._serialized_end=1446
    _TRACEINFO_TAGSENTRY._serialized_start=1448
    _TRACEINFO_TAGSENTRY._serialized_end=1491
    _TRACEINFO_STATE._serialized_start=1493
    _TRACEINFO_STATE._serialized_end=1559
    _CREATETRACEINFO._serialized_start=1562
    _CREATETRACEINFO._serialized_end=1769
    _CREATETRACEINFO_RESPONSE._serialized_start=1664
    _CREATETRACEINFO_RESPONSE._serialized_end=1724
    _TRACEPATH._serialized_start=1771
    _TRACEPATH._serialized_end=1870
    _TRACE._serialized_start=1872
    _TRACE._serialized_end=1980
    _BATCHGETTRACES._serialized_start=1983
    _BATCHGETTRACES._serialized_end=2170
    _BATCHGETTRACES_RESPONSE._serialized_start=2073
    _BATCHGETTRACES_RESPONSE._serialized_end=2125
    _GETTRACEINFO._serialized_start=2173
    _GETTRACEINFO._serialized_end=2359
    _GETTRACEINFO_RESPONSE._serialized_start=2263
    _GETTRACEINFO_RESPONSE._serialized_end=2314
    _SETTRACETAG._serialized_start=2362
    _SETTRACETAG._serialized_end=2517
    _SETTRACETAG_RESPONSE._serialized_start=1664
    _SETTRACETAG_RESPONSE._serialized_end=1674
    _DELETETRACETAG._serialized_start=2520
    _DELETETRACETAG._serialized_end=2689
    _DELETETRACETAG_RESPONSE._serialized_start=1664
    _DELETETRACETAG_RESPONSE._serialized_end=1674
    _SEARCHTRACES._serialized_start=2692
    _SEARCHTRACES._serialized_end=2998
    _SEARCHTRACES_RESPONSE._serialized_start=2867
    _SEARCHTRACES_RESPONSE._serialized_end=2953
    _CREATETRACEUCSTORAGELOCATION._serialized_start=3001
    _CREATETRACEUCSTORAGELOCATION._serialized_end=3246
    _CREATETRACEUCSTORAGELOCATION_RESPONSE._serialized_start=3123
    _CREATETRACEUCSTORAGELOCATION_RESPONSE._serialized_end=3189
    _LINKEXPERIMENTTOUCTRACELOCATION._serialized_start=3249
    _LINKEXPERIMENTTOUCTRACELOCATION._serialized_end=3438
    _LINKEXPERIMENTTOUCTRACELOCATION_RESPONSE._serialized_start=1664
    _LINKEXPERIMENTTOUCTRACELOCATION_RESPONSE._serialized_end=1674
    _UNLINKEXPERIMENTTOUCTRACELOCATION._serialized_start=3441
    _UNLINKEXPERIMENTTOUCTRACELOCATION._serialized_end=3632
    _UNLINKEXPERIMENTTOUCTRACELOCATION_RESPONSE._serialized_start=1664
    _UNLINKEXPERIMENTTOUCTRACELOCATION_RESPONSE._serialized_end=1674
    _ASSESSMENT._serialized_start=3635
    _ASSESSMENT._serialized_end=4243
    _ASSESSMENT_METADATAENTRY._serialized_start=4187
    _ASSESSMENT_METADATAENTRY._serialized_end=4234
    _CREATEASSESSMENT._serialized_start=4246
    _CREATEASSESSMENT._serialized_end=4482
    _CREATEASSESSMENT_RESPONSE._serialized_start=4376
    _CREATEASSESSMENT_RESPONSE._serialized_end=4437
    _GETASSESSMENT._serialized_start=4485
    _GETASSESSMENT._serialized_end=4714
    _GETASSESSMENT_RESPONSE._serialized_start=4376
    _GETASSESSMENT_RESPONSE._serialized_end=4437
    _UPDATEASSESSMENT._serialized_start=4717
    _UPDATEASSESSMENT._serialized_end=5008
    _UPDATEASSESSMENT_RESPONSE._serialized_start=4376
    _UPDATEASSESSMENT_RESPONSE._serialized_end=4437
    _DELETEASSESSMENT._serialized_start=5011
    _DELETEASSESSMENT._serialized_end=5192
    _DELETEASSESSMENT_RESPONSE._serialized_start=1664
    _DELETEASSESSMENT_RESPONSE._serialized_end=1674
    _BATCHLINKTRACETORUN._serialized_start=5195
    _BATCHLINKTRACETORUN._serialized_end=5341
    _BATCHLINKTRACETORUN_RESPONSE._serialized_start=1664
    _BATCHLINKTRACETORUN_RESPONSE._serialized_end=1674
    _BATCHUNLINKTRACEFROMRUN._serialized_start=5344
    _BATCHUNLINKTRACEFROMRUN._serialized_end=5494
    _BATCHUNLINKTRACEFROMRUN_RESPONSE._serialized_start=1664
    _BATCHUNLINKTRACEFROMRUN_RESPONSE._serialized_end=1674
    _DATABRICKSTRACKINGSERVICE._serialized_start=5497
    _DATABRICKSTRACKINGSERVICE._serialized_end=8617
  DatabricksTrackingService = service_reflection.GeneratedServiceType('DatabricksTrackingService', (_service.Service,), dict(
    DESCRIPTOR = _DATABRICKSTRACKINGSERVICE,
    __module__ = 'databricks_tracing_pb2'
    ))

  DatabricksTrackingService_Stub = service_reflection.GeneratedServiceStubType('DatabricksTrackingService_Stub', (DatabricksTrackingService,), dict(
    DESCRIPTOR = _DATABRICKSTRACKINGSERVICE,
    __module__ = 'databricks_tracing_pb2'
    ))


  # @@protoc_insertion_point(module_scope)

