body,button,input,option,select,textarea,thead{font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji}body,thead{box-shadow:none;font-size:13px;font-weight:400;line-height:18px}body,code,html,pre{margin:0;padding:0}body{min-height:100vh}html{overflow-y:hidden}