"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[4704],{94704:function(e,n,u){u.r(n),u.d(n,{DirectRunPage:function(){return m}});var r=u(27299),o=u(46709),t=u(56530),f=u(91105),l=u(11522),i=u(5690),a=u(60284),s=u(2724),c=u(2560),d=u(55570),p=u(91113),I=u(73408);const g=(0,c.h)((0,t.Ng)(((e,n)=>({runInfo:e.entities.runInfosByUuid[n.params.runUuid]})))((e=>{const{runUuid:n}=(0,f.g)(),[u,c]=(0,o.useState)(),d=(0,f.Zp)(),p=(0,t.wA)();return(0,o.useEffect)((()=>{c(void 0)}),[n]),(0,o.useEffect)((()=>{if(!e.runInfo&&n){const e=(0,i.aO)(n);e.payload.catch((e=>{l.A.logErrorAndNotifyUser(e),c(e)})),p(e)}}),[p,n,e.runInfo]),(0,o.useEffect)((()=>{var n;null!==(n=e.runInfo)&&void 0!==n&&n.experimentId&&d(a.Ay.getRunPageRoute(e.runInfo.experimentId,e.runInfo.runUuid),{replace:!0})}),[d,e.runInfo]),404===(null===u||void 0===u?void 0:u.status)?(0,I.Y)(s.o,{}):(0,I.Y)(r.PageWrapper,{children:(0,I.Y)(r.LegacySkeleton,{})})}))),m=(0,d.X)(p.A.mlflowServices.RUN_TRACKING,g);n.default=m}}]);