import databricks_pb2 as _databricks_pb2
import databricks_uc_registry_messages_pb2 as _databricks_uc_registry_messages_pb2
from scalapb import scalapb_pb2 as _scalapb_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class UcModelRegistryService(_service.service): ...

class UcModelRegistryService_Stub(UcModelRegistryService): ...
