"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[9563],{57184:function(e,l,t){t.r(l);var n=t(5690),s=t(46665),i=t(11573),o=t(73408);const p={setExperimentTagApi:n.EJ,getExperimentApi:n.yc,setCompareExperiments:n.I_};l.default=()=>(0,o.Y)(s.Z,{actions:p,children:(0,o.Y)(i.n,{showHeader:!1})})}}]);