"use strict";(self.webpackChunk_mlflow_mlflow=self.webpackChunk_mlflow_mlflow||[]).push([[859],{20859:function(e,l,f){f.r(l);var o=f(91105),t=f(60284),s=f(73408);l.default=()=>(0,s.Y)(o.C5,{to:t.Ay.experimentsObservatoryRoute})}}]);