
import google.protobuf
from packaging.version import Version
if Version(google.protobuf.__version__).major >= 5:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: service.proto
  # Protobuf Python Version: 5.26.0
  """Generated protocol buffer code."""
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf.internal import builder as _builder
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import assessments_pb2 as assessments__pb2
  from . import databricks_pb2 as databricks__pb2
  from . import datasets_pb2 as datasets__pb2
  from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
  from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
  from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
  from opentelemetry.proto.trace.v1 import trace_pb2 as opentelemetry_dot_proto_dot_trace_dot_v1_dot_trace__pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rservice.proto\x12\x06mlflow\x1a\x11\x61ssessments.proto\x1a\x10\x64\x61tabricks.proto\x1a\x0e\x64\x61tasets.proto\x1a\x1egoogle/protobuf/duration.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a(opentelemetry/proto/trace/v1/trace.proto\x1a\x15scalapb/scalapb.proto\"\xb0\x01\n\x06Metric\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0f\n\x04step\x18\x04 \x01(\x03:\x01\x30\x12\x1a\n\x0c\x64\x61taset_name\x18\x05 \x01(\tB\x04\xf0\x86\x19\x03\x12\x1c\n\x0e\x64\x61taset_digest\x18\x06 \x01(\tB\x04\xf0\x86\x19\x03\x12\x16\n\x08model_id\x18\x07 \x01(\tB\x04\xf0\x86\x19\x03\x12\x14\n\x06run_id\x18\x08 \x01(\tB\x04\xf0\x86\x19\x03\"#\n\x05Param\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x8b\x01\n\x03Run\x12\x1d\n\x04info\x18\x01 \x01(\x0b\x32\x0f.mlflow.RunInfo\x12\x1d\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x0f.mlflow.RunData\x12!\n\x06inputs\x18\x03 \x01(\x0b\x32\x11.mlflow.RunInputs\x12#\n\x07outputs\x18\x04 \x01(\x0b\x32\x12.mlflow.RunOutputs\"g\n\x07RunData\x12\x1f\n\x07metrics\x18\x01 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x1d\n\x06params\x18\x02 \x03(\x0b\x32\r.mlflow.Param\x12\x1c\n\x04tags\x18\x03 \x03(\x0b\x32\x0e.mlflow.RunTag\"c\n\tRunInputs\x12,\n\x0e\x64\x61taset_inputs\x18\x01 \x03(\x0b\x32\x14.mlflow.DatasetInput\x12(\n\x0cmodel_inputs\x18\x02 \x03(\x0b\x32\x12.mlflow.ModelInput\"8\n\nRunOutputs\x12*\n\rmodel_outputs\x18\x01 \x03(\x0b\x32\x13.mlflow.ModelOutput\"$\n\x06RunTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\rExperimentTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xdd\x01\n\x07RunInfo\x12\x0e\n\x06run_id\x18\x0f \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x10\n\x08run_name\x18\x03 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x0f\n\x07user_id\x18\x06 \x01(\t\x12!\n\x06status\x18\x07 \x01(\x0e\x32\x11.mlflow.RunStatus\x12\x12\n\nstart_time\x18\x08 \x01(\x03\x12\x10\n\x08\x65nd_time\x18\t \x01(\x03\x12\x14\n\x0c\x61rtifact_uri\x18\r \x01(\t\x12\x17\n\x0flifecycle_stage\x18\x0e \x01(\t\"\xbb\x01\n\nExperiment\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x19\n\x11\x61rtifact_location\x18\x03 \x01(\t\x12\x17\n\x0flifecycle_stage\x18\x04 \x01(\t\x12\x18\n\x10last_update_time\x18\x05 \x01(\x03\x12\x15\n\rcreation_time\x18\x06 \x01(\x03\x12#\n\x04tags\x18\x07 \x03(\x0b\x32\x15.mlflow.ExperimentTag\"V\n\x0c\x44\x61tasetInput\x12\x1e\n\x04tags\x18\x01 \x03(\x0b\x32\x10.mlflow.InputTag\x12&\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x0f.mlflow.DatasetB\x04\xf8\x86\x19\x01\"$\n\nModelInput\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\"2\n\x08InputTag\x12\x11\n\x03key\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\"\x85\x01\n\x07\x44\x61taset\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06\x64igest\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bsource_type\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06source\x18\x04 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x0f\n\x07profile\x18\x06 \x01(\t\"9\n\x0bModelOutput\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04step\x18\x02 \x01(\x03\x42\x04\xf8\x86\x19\x01\"\xb6\x01\n\x10\x43reateExperiment\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x11\x61rtifact_location\x18\x02 \x01(\t\x12#\n\x04tags\x18\x03 \x03(\x0b\x32\x15.mlflow.ExperimentTag\x1a!\n\x08Response\x12\x15\n\rexperiment_id\x18\x01 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xfe\x01\n\x11SearchExperiments\x12\x13\n\x0bmax_results\x18\x01 \x01(\x03\x12\x12\n\npage_token\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\t\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12#\n\tview_type\x18\x05 \x01(\x0e\x32\x10.mlflow.ViewType\x1aL\n\x08Response\x12\'\n\x0b\x65xperiments\x18\x01 \x03(\x0b\x32\x12.mlflow.Experiment\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8d\x01\n\rGetExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\nexperiment\x18\x01 \x01(\x0b\x32\x12.mlflow.Experiment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"h\n\x10\x44\x65leteExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"i\n\x11RestoreExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"z\n\x10UpdateExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x10\n\x08new_name\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xca\x01\n\tCreateRun\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x10\n\x08run_name\x18\x03 \x01(\t\x12\x12\n\nstart_time\x18\x07 \x01(\x03\x12\x1c\n\x04tags\x18\t \x03(\x0b\x32\x0e.mlflow.RunTag\x1a$\n\x08Response\x12\x18\n\x03run\x18\x01 \x01(\x0b\x32\x0b.mlflow.Run:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd0\x01\n\tUpdateRun\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12!\n\x06status\x18\x02 \x01(\x0e\x32\x11.mlflow.RunStatus\x12\x10\n\x08\x65nd_time\x18\x03 \x01(\x03\x12\x10\n\x08run_name\x18\x05 \x01(\t\x1a-\n\x08Response\x12!\n\x08run_info\x18\x01 \x01(\x0b\x32\x0f.mlflow.RunInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"Z\n\tDeleteRun\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"[\n\nRestoreRun\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8a\x02\n\tLogMetric\x12\x0e\n\x06run_id\x18\x06 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\x01\x42\x04\xf8\x86\x19\x01\x12\x17\n\ttimestamp\x18\x04 \x01(\x03\x42\x04\xf8\x86\x19\x01\x12\x0f\n\x04step\x18\x05 \x01(\x03:\x01\x30\x12\x16\n\x08model_id\x18\x07 \x01(\tB\x04\xf0\x86\x19\x03\x12\x1a\n\x0c\x64\x61taset_name\x18\x08 \x01(\tB\x04\xf0\x86\x19\x03\x12\x1c\n\x0e\x64\x61taset_digest\x18\t \x01(\tB\x04\xf0\x86\x19\x03\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8d\x01\n\x08LogParam\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x90\x01\n\x10SetExperimentTag\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"~\n\x13\x44\x65leteExperimentTag\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8b\x01\n\x06SetTag\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"m\n\tDeleteTag\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"}\n\x06GetRun\x12\x0e\n\x06run_id\x18\x02 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x1a$\n\x08Response\x12\x18\n\x03run\x18\x01 \x01(\x0b\x32\x0b.mlflow.Run:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x98\x02\n\nSearchRuns\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\x34\n\rrun_view_type\x18\x03 \x01(\x0e\x32\x10.mlflow.ViewType:\x0b\x41\x43TIVE_ONLY\x12\x19\n\x0bmax_results\x18\x05 \x01(\x05:\x04\x31\x30\x30\x30\x12\x10\n\x08order_by\x18\x06 \x03(\t\x12\x12\n\npage_token\x18\x07 \x01(\t\x1a>\n\x08Response\x12\x19\n\x04runs\x18\x01 \x03(\x0b\x32\x0b.mlflow.Run\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd8\x01\n\rListArtifacts\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x12\n\npage_token\x18\x04 \x01(\t\x1aV\n\x08Response\x12\x10\n\x08root_uri\x18\x01 \x01(\t\x12\x1f\n\x05\x66iles\x18\x02 \x03(\x0b\x32\x10.mlflow.FileInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\";\n\x08\x46ileInfo\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06is_dir\x18\x02 \x01(\x08\x12\x11\n\tfile_size\x18\x03 \x01(\x03\"\xea\x01\n\x10GetMetricHistory\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x18\n\nmetric_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\npage_token\x18\x04 \x01(\t\x12\x13\n\x0bmax_results\x18\x05 \x01(\x05\x1a\x44\n\x08Response\x12\x1f\n\x07metrics\x18\x01 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"a\n\x0fMetricWithRunId\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0f\n\x04step\x18\x04 \x01(\x03:\x01\x30\x12\x0e\n\x06run_id\x18\x05 \x01(\t\"\xe7\x01\n\x1cGetMetricHistoryBulkInterval\x12\x0f\n\x07run_ids\x18\x01 \x03(\t\x12\x18\n\nmetric_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\nstart_step\x18\x03 \x01(\x05\x12\x10\n\x08\x65nd_step\x18\x04 \x01(\x05\x12\x13\n\x0bmax_results\x18\x05 \x01(\x05\x1a\x34\n\x08Response\x12(\n\x07metrics\x18\x01 \x03(\x0b\x32\x17.mlflow.MetricWithRunId:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb1\x01\n\x08LogBatch\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x1f\n\x07metrics\x18\x02 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x1d\n\x06params\x18\x03 \x03(\x0b\x32\r.mlflow.Param\x12\x1c\n\x04tags\x18\x04 \x03(\x0b\x32\x0e.mlflow.RunTag\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"g\n\x08LogModel\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x12\n\nmodel_json\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xac\x01\n\tLogInputs\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12&\n\x08\x64\x61tasets\x18\x02 \x03(\x0b\x32\x14.mlflow.DatasetInput\x12(\n\x06models\x18\x03 \x03(\x0b\x32\x12.mlflow.ModelInputB\x04\xf0\x86\x19\x03\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x80\x01\n\nLogOutputs\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12#\n\x06models\x18\x02 \x03(\x0b\x32\x13.mlflow.ModelOutput\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x95\x01\n\x13GetExperimentByName\x12\x1d\n\x0f\x65xperiment_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\nexperiment\x18\x01 \x01(\x0b\x32\x12.mlflow.Experiment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb9\x01\n\x10\x43reateAssessment\x12\x38\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.AssessmentB\x04\xf8\x86\x19\x01\x1a>\n\x08Response\x12\x32\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf0\x01\n\x10UpdateAssessment\x12\x38\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.AssessmentB\x04\xf8\x86\x19\x01\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xf8\x86\x19\x01\x1a>\n\x08Response\x12\x32\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x80\x01\n\x10\x44\x65leteAssessment\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb8\x01\n\x14GetAssessmentRequest\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a>\n\x08Response\x12\x32\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xe4\x01\n\tTraceInfo\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x03 \x01(\x03\x12\x19\n\x11\x65xecution_time_ms\x18\x04 \x01(\x03\x12#\n\x06status\x18\x05 \x01(\x0e\x32\x13.mlflow.TraceStatus\x12\x36\n\x10request_metadata\x18\x06 \x03(\x0b\x32\x1c.mlflow.TraceRequestMetadata\x12\x1e\n\x04tags\x18\x07 \x03(\x0b\x32\x10.mlflow.TraceTag\"2\n\x14TraceRequestMetadata\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"&\n\x08TraceTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xf1\x01\n\nStartTrace\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x02 \x01(\x03\x12\x36\n\x10request_metadata\x18\x03 \x03(\x0b\x32\x1c.mlflow.TraceRequestMetadata\x12\x1e\n\x04tags\x18\x04 \x03(\x0b\x32\x10.mlflow.TraceTag\x1a\x31\n\x08Response\x12%\n\ntrace_info\x18\x01 \x01(\x0b\x32\x11.mlflow.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x91\x02\n\x08\x45ndTrace\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x02 \x01(\x03\x12#\n\x06status\x18\x03 \x01(\x0e\x32\x13.mlflow.TraceStatus\x12\x36\n\x10request_metadata\x18\x04 \x03(\x0b\x32\x1c.mlflow.TraceRequestMetadata\x12\x1e\n\x04tags\x18\x05 \x03(\x0b\x32\x10.mlflow.TraceTag\x1a\x31\n\x08Response\x12%\n\ntrace_info\x18\x01 \x01(\x0b\x32\x11.mlflow.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x82\x01\n\x0cGetTraceInfo\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x1a\x31\n\x08Response\x12%\n\ntrace_info\x18\x01 \x01(\x0b\x32\x11.mlflow.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"y\n\x0eGetTraceInfoV3\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x1a(\n\x08Response\x12\x1c\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"{\n\x0e\x42\x61tchGetTraces\x12\x11\n\ttrace_ids\x18\x01 \x03(\t\x1a)\n\x08Response\x12\x1d\n\x06traces\x18\x01 \x03(\x0b\x32\r.mlflow.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x97\x01\n\x08GetTrace\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1c\n\rallow_partial\x18\x02 \x01(\x08:\x05\x66\x61lse\x1a(\n\x08Response\x12\x1c\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xeb\x01\n\x0cSearchTraces\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x18\n\x0bmax_results\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x12\n\npage_token\x18\x05 \x01(\t\x1a\x46\n\x08Response\x12!\n\x06traces\x18\x01 \x03(\x0b\x32\x11.mlflow.TraceInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xaa\x02\n\x13SearchUnifiedTraces\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1e\n\x10sql_warehouse_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x03 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\x18\n\x0bmax_results\x18\x05 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x06 \x03(\t\x12\x12\n\npage_token\x18\x07 \x01(\t\x1a\x46\n\x08Response\x12!\n\x06traces\x18\x01 \x03(\x0b\x32\x11.mlflow.TraceInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc1\x01\n\x15GetOnlineTraceDetails\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1e\n\x10sql_warehouse_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12$\n\x16source_inference_table\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12*\n\x1csource_databricks_request_id\x18\x04 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x1e\n\x08Response\x12\x12\n\ntrace_data\x18\x01 \x01(\t\"\xc3\x01\n\x0c\x44\x65leteTraces\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1c\n\x14max_timestamp_millis\x18\x02 \x01(\x03\x12\x12\n\nmax_traces\x18\x03 \x01(\x05\x12\x13\n\x0brequest_ids\x18\x04 \x03(\t\x1a\"\n\x08Response\x12\x16\n\x0etraces_deleted\x18\x01 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc5\x01\n\x0e\x44\x65leteTracesV3\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1c\n\x14max_timestamp_millis\x18\x02 \x01(\x03\x12\x12\n\nmax_traces\x18\x03 \x01(\x05\x12\x13\n\x0brequest_ids\x18\x04 \x03(\t\x1a\"\n\x08Response\x12\x16\n\x0etraces_deleted\x18\x01 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb5\x02\n\x1f\x43\x61lculateTraceFilterCorrelation\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x16\n\x0e\x66ilter_string1\x18\x02 \x01(\t\x12\x16\n\x0e\x66ilter_string2\x18\x03 \x01(\t\x12\x13\n\x0b\x62\x61se_filter\x18\x04 \x01(\t\x1a\x87\x01\n\x08Response\x12\x0c\n\x04npmi\x18\x01 \x01(\x01\x12\x15\n\rnpmi_smoothed\x18\x02 \x01(\x01\x12\x15\n\rfilter1_count\x18\x03 \x01(\x05\x12\x15\n\rfilter2_count\x18\x04 \x01(\x05\x12\x13\n\x0bjoint_count\x18\x05 \x01(\x05\x12\x13\n\x0btotal_count\x18\x06 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"v\n\x0bSetTraceTag\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x88\x01\n\rSetTraceTagV3\x12\x10\n\x08trace_id\x18\x04 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]J\x04\x08\x01\x10\x02R\nrequest_id\"j\n\x0e\x44\x65leteTraceTag\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"|\n\x10\x44\x65leteTraceTagV3\x12\x10\n\x08trace_id\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]J\x04\x08\x01\x10\x02R\nrequest_id\"c\n\x05Trace\x12\'\n\ntrace_info\x18\x01 \x01(\x0b\x32\x13.mlflow.TraceInfoV3\x12\x31\n\x05spans\x18\x02 \x03(\x0b\x32\".opentelemetry.proto.trace.v1.Span\"\xb6\x03\n\rTraceLocation\x12\x35\n\x04type\x18\x01 \x01(\x0e\x32\'.mlflow.TraceLocation.TraceLocationType\x12K\n\x11mlflow_experiment\x18\x02 \x01(\x0b\x32..mlflow.TraceLocation.MlflowExperimentLocationH\x00\x12G\n\x0finference_table\x18\x03 \x01(\x0b\x32,.mlflow.TraceLocation.InferenceTableLocationH\x00\x1a\x31\n\x18MlflowExperimentLocation\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x1a\x31\n\x16InferenceTableLocation\x12\x17\n\x0f\x66ull_table_name\x18\x01 \x01(\t\"d\n\x11TraceLocationType\x12#\n\x1fTRACE_LOCATION_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11MLFLOW_EXPERIMENT\x10\x01\x12\x13\n\x0fINFERENCE_TABLE\x10\x02\x42\x0c\n\nidentifier\"\x9b\x05\n\x0bTraceInfoV3\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x12\x19\n\x11\x63lient_request_id\x18\x02 \x01(\t\x12-\n\x0etrace_location\x18\x03 \x01(\x0b\x32\x15.mlflow.TraceLocation\x12\x0f\n\x07request\x18\x04 \x01(\t\x12\x10\n\x08response\x18\x05 \x01(\t\x12\x17\n\x0frequest_preview\x18\x0c \x01(\t\x12\x18\n\x10response_preview\x18\r \x01(\t\x12\x30\n\x0crequest_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12\x65xecution_duration\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12(\n\x05state\x18\x08 \x01(\x0e\x32\x19.mlflow.TraceInfoV3.State\x12>\n\x0etrace_metadata\x18\t \x03(\x0b\x32&.mlflow.TraceInfoV3.TraceMetadataEntry\x12\x33\n\x0b\x61ssessments\x18\n \x03(\x0b\x32\x1e.mlflow.assessments.Assessment\x12+\n\x04tags\x18\x0b \x03(\x0b\x32\x1d.mlflow.TraceInfoV3.TagsEntry\x1a\x34\n\x12TraceMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0f\n\x0bIN_PROGRESS\x10\x03\"\\\n\x0cStartTraceV3\x12\"\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.TraceB\x04\xf8\x86\x19\x01\x1a(\n\x08Response\x12\x1c\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.Trace\"F\n\x0fLinkTracesToRun\x12\x11\n\ttrace_ids\x18\x01 \x03(\t\x12\x14\n\x06run_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response\"\xbd\x01\n\x12LinkPromptsToTrace\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x44\n\x0fprompt_versions\x18\x02 \x03(\x0b\x32+.mlflow.LinkPromptsToTrace.PromptVersionRef\x1a=\n\x10PromptVersionRef\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x15\n\x07version\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response\"h\n\x0e\x44\x61tasetSummary\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06\x64igest\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0f\n\x07\x63ontext\x18\x04 \x01(\t\"\x94\x01\n\x0eSearchDatasets\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x1a=\n\x08Response\x12\x31\n\x11\x64\x61taset_summaries\x18\x01 \x03(\x0b\x32\x16.mlflow.DatasetSummary:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9a\x02\n\x11\x43reateLoggedModel\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nmodel_type\x18\x03 \x01(\t\x12\x15\n\rsource_run_id\x18\x04 \x01(\t\x12,\n\x06params\x18\x05 \x03(\x0b\x32\x1c.mlflow.LoggedModelParameter\x12$\n\x04tags\x18\x06 \x03(\x0b\x32\x16.mlflow.LoggedModelTag\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xbb\x01\n\x13\x46inalizeLoggedModel\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12/\n\x06status\x18\x02 \x01(\x0e\x32\x19.mlflow.LoggedModelStatusB\x04\xf8\x86\x19\x01\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x85\x01\n\x0eGetLoggedModel\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"d\n\x11\x44\x65leteLoggedModel\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf7\x03\n\x12SearchLoggedModels\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x34\n\x08\x64\x61tasets\x18\x06 \x03(\x0b\x32\".mlflow.SearchLoggedModels.Dataset\x12\x17\n\x0bmax_results\x18\x03 \x01(\x05:\x02\x35\x30\x12\x34\n\x08order_by\x18\x04 \x03(\x0b\x32\".mlflow.SearchLoggedModels.OrderBy\x12\x12\n\npage_token\x18\x05 \x01(\t\x1a=\n\x07\x44\x61taset\x12\x1a\n\x0c\x64\x61taset_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x64\x61taset_digest\x18\x02 \x01(\t\x1aj\n\x07OrderBy\x12\x18\n\nfield_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x17\n\tascending\x18\x02 \x01(\x08:\x04true\x12\x14\n\x0c\x64\x61taset_name\x18\x03 \x01(\t\x12\x16\n\x0e\x64\x61taset_digest\x18\x04 \x01(\t\x1aH\n\x08Response\x12#\n\x06models\x18\x01 \x03(\x0b\x32\x13.mlflow.LoggedModel\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xaf\x01\n\x12SetLoggedModelTags\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12$\n\x04tags\x18\x02 \x03(\x0b\x32\x16.mlflow.LoggedModelTag\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"~\n\x14\x44\x65leteLoggedModelTag\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x15\n\x07tag_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xec\x01\n\x18ListLoggedModelArtifacts\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1f\n\x17\x61rtifact_directory_path\x18\x02 \x01(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1aV\n\x08Response\x12\x10\n\x08root_uri\x18\x01 \x01(\t\x12\x1f\n\x05\x66iles\x18\x02 \x03(\x0b\x32\x10.mlflow.FileInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9c\x01\n\x1bLogLoggedModelParamsRequest\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12,\n\x06params\x18\x02 \x03(\x0b\x32\x1c.mlflow.LoggedModelParameter\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"[\n\x0bLoggedModel\x12%\n\x04info\x18\x01 \x01(\x0b\x32\x17.mlflow.LoggedModelInfo\x12%\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x17.mlflow.LoggedModelData\"\x84\x03\n\x0fLoggedModelInfo\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1d\n\x15\x63reation_timestamp_ms\x18\x04 \x01(\x03\x12!\n\x19last_updated_timestamp_ms\x18\x05 \x01(\x03\x12\x14\n\x0c\x61rtifact_uri\x18\x06 \x01(\t\x12)\n\x06status\x18\x07 \x01(\x0e\x32\x19.mlflow.LoggedModelStatus\x12\x12\n\ncreator_id\x18\x08 \x01(\x03\x12\x12\n\nmodel_type\x18\t \x01(\t\x12\x15\n\rsource_run_id\x18\n \x01(\t\x12\x16\n\x0estatus_message\x18\x0b \x01(\t\x12$\n\x04tags\x18\x0c \x03(\x0b\x32\x16.mlflow.LoggedModelTag\x12:\n\rregistrations\x18\r \x03(\x0b\x32#.mlflow.LoggedModelRegistrationInfo\",\n\x0eLoggedModelTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"<\n\x1bLoggedModelRegistrationInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"`\n\x0fLoggedModelData\x12,\n\x06params\x18\x01 \x03(\x0b\x32\x1c.mlflow.LoggedModelParameter\x12\x1f\n\x07metrics\x18\x02 \x03(\x0b\x32\x0e.mlflow.Metric\"2\n\x14LoggedModelParameter\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x81\x02\n\x0eSearchTracesV3\x12(\n\tlocations\x18\x01 \x03(\x0b\x32\x15.mlflow.TraceLocation\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x18\n\x0bmax_results\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x12\n\npage_token\x18\x05 \x01(\t\x1aH\n\x08Response\x12#\n\x06traces\x18\x01 \x03(\x0b\x32\x13.mlflow.TraceInfoV3\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb8\x02\n\rCreateDataset\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x02 \x03(\t\x12\x44\n\x0bsource_type\x18\x03 \x01(\x0e\x32/.mlflow.datasets.DatasetRecordSource.SourceType\x12\x0e\n\x06source\x18\x04 \x01(\t\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x0f\n\x07profile\x18\x06 \x01(\t\x12\x12\n\ncreated_by\x18\x07 \x01(\t\x12\x0c\n\x04tags\x18\x08 \x01(\t\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb7\x01\n\nGetDataset\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\npage_token\x18\x02 \x01(\t\x1aN\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"b\n\rDeleteDataset\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x88\x02\n\x18SearchEvaluationDatasets\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x15\n\rfilter_string\x18\x02 \x01(\t\x12\x19\n\x0bmax_results\x18\x03 \x01(\x05:\x04\x31\x30\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x12\n\npage_token\x18\x05 \x01(\t\x1aO\n\x08Response\x12*\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x18.mlflow.datasets.Dataset\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa2\x01\n\x0eSetDatasetTags\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04tags\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"x\n\x10\x44\x65leteDatasetTag\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc3\x01\n\x14UpsertDatasetRecords\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x15\n\x07records\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\nupdated_by\x18\x03 \x01(\t\x1a\x39\n\x08Response\x12\x16\n\x0einserted_count\x18\x01 \x01(\x05\x12\x15\n\rupdated_count\x18\x02 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x84\x01\n\x17GetDatasetExperimentIds\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\"\n\x08Response\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xbf\x01\n\x11GetDatasetRecords\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bmax_results\x18\x02 \x01(\x05:\x04\x31\x30\x30\x30\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a\x34\n\x08Response\x12\x0f\n\x07records\x18\x01 \x01(\t\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xaf\x01\n\x17\x41\x64\x64\x44\x61tasetToExperiments\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x02 \x03(\t\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb4\x01\n\x1cRemoveDatasetFromExperiments\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x02 \x03(\t\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x85\x02\n\x0eRegisterScorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x19\n\x11serialized_scorer\x18\x03 \x01(\t\x1a\x85\x01\n\x08Response\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x11\n\tscorer_id\x18\x02 \x01(\t\x12\x15\n\rexperiment_id\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x19\n\x11serialized_scorer\x18\x05 \x01(\t\x12\x15\n\rcreation_time\x18\x06 \x01(\x03:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"~\n\x0bListScorers\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x1a+\n\x08Response\x12\x1f\n\x07scorers\x18\x01 \x03(\x0b\x32\x0e.mlflow.Scorer:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x93\x01\n\x12ListScorerVersions\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x1a+\n\x08Response\x12\x1f\n\x07scorers\x18\x01 \x03(\x0b\x32\x0e.mlflow.Scorer:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9a\x01\n\tGetScorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\x05\x1a*\n\x08Response\x12\x1e\n\x06scorer\x18\x01 \x01(\x0b\x32\x0e.mlflow.Scorer:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"}\n\x0c\x44\x65leteScorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\x05\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x91\x01\n\x06Scorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\x05\x12\x13\n\x0bscorer_name\x18\x02 \x01(\t\x12\x16\n\x0escorer_version\x18\x03 \x01(\x05\x12\x19\n\x11serialized_scorer\x18\x04 \x01(\t\x12\x15\n\rcreation_time\x18\x05 \x01(\x03\x12\x11\n\tscorer_id\x18\x06 \x01(\t\"\xba\x02\n\x11GatewaySecretInfo\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x12\x13\n\x0bsecret_name\x18\x02 \x01(\t\x12\x42\n\rmasked_values\x18\x03 \x03(\x0b\x32+.mlflow.GatewaySecretInfo.MaskedValuesEntry\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x05 \x01(\x03\x12\x10\n\x08provider\x18\x06 \x01(\t\x12\x12\n\ncreated_by\x18\x07 \x01(\t\x12\x17\n\x0flast_updated_by\x18\x08 \x01(\t\x12\x18\n\x10\x61uth_config_json\x18\t \x01(\t\x1a\x33\n\x11MaskedValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xeb\x01\n\x16GatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tsecret_id\x18\x03 \x01(\t\x12\x13\n\x0bsecret_name\x18\x04 \x01(\t\x12\x10\n\x08provider\x18\x05 \x01(\t\x12\x12\n\nmodel_name\x18\x06 \x01(\t\x12\x12\n\ncreated_at\x18\x07 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x08 \x01(\x03\x12\x12\n\ncreated_by\x18\t \x01(\t\x12\x17\n\x0flast_updated_by\x18\n \x01(\t\"\xd5\x01\n\x1bGatewayEndpointModelMapping\x12\x12\n\nmapping_id\x18\x01 \x01(\t\x12\x13\n\x0b\x65ndpoint_id\x18\x02 \x01(\t\x12\x1b\n\x13model_definition_id\x18\x03 \x01(\t\x12\x38\n\x10model_definition\x18\x04 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\x12\x0e\n\x06weight\x18\x05 \x01(\x02\x12\x12\n\ncreated_at\x18\x06 \x01(\x03\x12\x12\n\ncreated_by\x18\x07 \x01(\t\"\xf5\x01\n\x0fGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x03 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x04 \x01(\x03\x12;\n\x0emodel_mappings\x18\x05 \x03(\x0b\x32#.mlflow.GatewayEndpointModelMapping\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x17\n\x0flast_updated_by\x18\x07 \x01(\t\x12(\n\x04tags\x18\x08 \x03(\x0b\x32\x1a.mlflow.GatewayEndpointTag\"0\n\x12GatewayEndpointTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xb3\x01\n\x16GatewayEndpointBinding\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x05 \x01(\x03\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x17\n\x0flast_updated_by\x18\x07 \x01(\t\"\xb0\x02\n\x13\x43reateGatewaySecret\x12\x13\n\x0bsecret_name\x18\x01 \x01(\t\x12\x42\n\x0csecret_value\x18\x02 \x03(\x0b\x32,.mlflow.CreateGatewaySecret.SecretValueEntry\x12\x10\n\x08provider\x18\x03 \x01(\t\x12\x18\n\x10\x61uth_config_json\x18\x05 \x01(\t\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x1a\x32\n\x10SecretValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x08Response\x12)\n\x06secret\x18\x01 \x01(\x0b\x32\x19.mlflow.GatewaySecretInfoJ\x04\x08\x04\x10\x05R\x0f\x63redential_name\"u\n\x14GetGatewaySecretInfo\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x12\x13\n\x0bsecret_name\x18\x02 \x01(\t\x1a\x35\n\x08Response\x12)\n\x06secret\x18\x01 \x01(\x0b\x32\x19.mlflow.GatewaySecretInfo\"\x9c\x02\n\x13UpdateGatewaySecret\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x12\x42\n\x0csecret_value\x18\x02 \x03(\x0b\x32,.mlflow.UpdateGatewaySecret.SecretValueEntry\x12\x18\n\x10\x61uth_config_json\x18\x04 \x01(\t\x12\x12\n\nupdated_by\x18\x05 \x01(\t\x1a\x32\n\x10SecretValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x08Response\x12)\n\x06secret\x18\x01 \x01(\x0b\x32\x19.mlflow.GatewaySecretInfoJ\x04\x08\x03\x10\x04R\x0f\x63redential_name\"4\n\x13\x44\x65leteGatewaySecret\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x1a\n\n\x08Response\"b\n\x16ListGatewaySecretInfos\x12\x10\n\x08provider\x18\x01 \x01(\t\x1a\x36\n\x08Response\x12*\n\x07secrets\x18\x01 \x03(\x0b\x32\x19.mlflow.GatewaySecretInfo\"\xbf\x01\n\x1c\x43reateGatewayModelDefinition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tsecret_id\x18\x02 \x01(\t\x12\x10\n\x08provider\x18\x03 \x01(\t\x12\x12\n\nmodel_name\x18\x04 \x01(\t\x12\x12\n\ncreated_by\x18\x05 \x01(\t\x1a\x44\n\x08Response\x12\x38\n\x10model_definition\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"~\n\x19GetGatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x1a\x44\n\x08Response\x12\x38\n\x10model_definition\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"\x89\x01\n\x1bListGatewayModelDefinitions\x12\x10\n\x08provider\x18\x01 \x01(\t\x12\x11\n\tsecret_id\x18\x02 \x01(\t\x1a\x45\n\x08Response\x12\x39\n\x11model_definitions\x18\x01 \x03(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"\xdc\x01\n\x1cUpdateGatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tsecret_id\x18\x03 \x01(\t\x12\x12\n\nmodel_name\x18\x04 \x01(\t\x12\x12\n\nupdated_by\x18\x05 \x01(\t\x12\x10\n\x08provider\x18\x06 \x01(\t\x1a\x44\n\x08Response\x12\x38\n\x10model_definition\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"G\n\x1c\x44\x65leteGatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x1a\n\n\x08Response\"\x8e\x01\n\x15\x43reateGatewayEndpoint\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1c\n\x14model_definition_ids\x18\x02 \x03(\t\x12\x12\n\ncreated_by\x18\x03 \x01(\t\x1a\x35\n\x08Response\x12)\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x17.mlflow.GatewayEndpoint\"n\n\x12GetGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x1a\x35\n\x08Response\x12)\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x17.mlflow.GatewayEndpoint\"\x85\x01\n\x15UpdateGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nupdated_by\x18\x03 \x01(\t\x1a\x35\n\x08Response\x12)\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x17.mlflow.GatewayEndpoint\"8\n\x15\x44\x65leteGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x1a\n\n\x08Response\"s\n\x14ListGatewayEndpoints\x12\x10\n\x08provider\x18\x01 \x01(\t\x12\x11\n\tsecret_id\x18\x02 \x01(\t\x1a\x36\n\x08Response\x12*\n\tendpoints\x18\x01 \x03(\x0b\x32\x17.mlflow.GatewayEndpoint\"\xb6\x01\n\x1c\x41ttachModelToGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x1b\n\x13model_definition_id\x18\x02 \x01(\t\x12\x0e\n\x06weight\x18\x03 \x01(\x02\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x1a@\n\x08Response\x12\x34\n\x07mapping\x18\x01 \x01(\x0b\x32#.mlflow.GatewayEndpointModelMapping\"^\n\x1e\x44\x65tachModelFromGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x1b\n\x13model_definition_id\x18\x02 \x01(\t\x1a\n\n\x08Response\"\xb0\x01\n\x1c\x43reateGatewayEndpointBinding\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x1a;\n\x08Response\x12/\n\x07\x62inding\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayEndpointBinding\"k\n\x1c\x44\x65leteGatewayEndpointBinding\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x1a\n\n\x08Response\"\x9c\x01\n\x1bListGatewayEndpointBindings\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x1a<\n\x08Response\x12\x30\n\x08\x62indings\x18\x01 \x03(\x0b\x32\x1e.mlflow.GatewayEndpointBinding\"T\n\x15SetGatewayEndpointTag\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x1a\n\n\x08Response\"H\n\x18\x44\x65leteGatewayEndpointTag\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\n\n\x08Response\"9\n\x10GetSecretsConfig\x1a%\n\x08Response\x12\x19\n\x11secrets_available\x18\x01 \x01(\x08*6\n\x08ViewType\x12\x0f\n\x0b\x41\x43TIVE_ONLY\x10\x01\x12\x10\n\x0c\x44\x45LETED_ONLY\x10\x02\x12\x07\n\x03\x41LL\x10\x03*I\n\nSourceType\x12\x0c\n\x08NOTEBOOK\x10\x01\x12\x07\n\x03JOB\x10\x02\x12\x0b\n\x07PROJECT\x10\x03\x12\t\n\x05LOCAL\x10\x04\x12\x0c\n\x07UNKNOWN\x10\xe8\x07*M\n\tRunStatus\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tSCHEDULED\x10\x02\x12\x0c\n\x08\x46INISHED\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\n\n\x06KILLED\x10\x05*O\n\x0bTraceStatus\x12\x1c\n\x18TRACE_STATUS_UNSPECIFIED\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0f\n\x0bIN_PROGRESS\x10\x03*\x8a\x01\n\x11LoggedModelStatus\x12#\n\x1fLOGGED_MODEL_STATUS_UNSPECIFIED\x10\x00\x12\x18\n\x14LOGGED_MODEL_PENDING\x10\x01\x12\x16\n\x12LOGGED_MODEL_READY\x10\x02\x12\x1e\n\x1aLOGGED_MODEL_UPLOAD_FAILED\x10\x03\x32\x86|\n\rMlflowService\x12\xa6\x01\n\x13getExperimentByName\x12\x1b.mlflow.GetExperimentByName\x1a$.mlflow.GetExperimentByName.Response\"L\xf2\x86\x19H\n,\n\x03GET\x12\x1f/mlflow/experiments/get-by-name\x1a\x04\x08\x02\x10\x00\x10\x01*\x16Get Experiment By Name\x12\x94\x01\n\x10\x63reateExperiment\x12\x18.mlflow.CreateExperiment\x1a!.mlflow.CreateExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x11\x43reate Experiment\x12\xc1\x01\n\x11searchExperiments\x12\x19.mlflow.SearchExperiments\x1a\".mlflow.SearchExperiments.Response\"m\xf2\x86\x19i\n(\n\x04POST\x12\x1a/mlflow/experiments/search\x1a\x04\x08\x02\x10\x00\n\'\n\x03GET\x12\x1a/mlflow/experiments/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Search Experiments\x12\x88\x01\n\rgetExperiment\x12\x15.mlflow.GetExperiment\x1a\x1e.mlflow.GetExperiment.Response\"@\xf2\x86\x19\x38\n$\n\x03GET\x12\x17/mlflow/experiments/get\x1a\x04\x08\x02\x10\x00\x10\x01*\x0eGet Experiment\xba\x8c\x19\x00\x12\x94\x01\n\x10\x64\x65leteExperiment\x12\x18.mlflow.DeleteExperiment\x1a!.mlflow.DeleteExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x11\x44\x65lete Experiment\x12\x99\x01\n\x11restoreExperiment\x12\x19.mlflow.RestoreExperiment\x1a\".mlflow.RestoreExperiment.Response\"E\xf2\x86\x19\x41\n)\n\x04POST\x12\x1b/mlflow/experiments/restore\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Restore Experiment\x12\x94\x01\n\x10updateExperiment\x12\x18.mlflow.UpdateExperiment\x1a!.mlflow.UpdateExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x11Update Experiment\x12q\n\tcreateRun\x12\x11.mlflow.CreateRun\x1a\x1a.mlflow.CreateRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/create\x1a\x04\x08\x02\x10\x00\x10\x01*\nCreate Run\x12q\n\tupdateRun\x12\x11.mlflow.UpdateRun\x1a\x1a.mlflow.UpdateRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/update\x1a\x04\x08\x02\x10\x00\x10\x01*\nUpdate Run\x12q\n\tdeleteRun\x12\x11.mlflow.DeleteRun\x1a\x1a.mlflow.DeleteRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\nDelete Run\x12v\n\nrestoreRun\x12\x12.mlflow.RestoreRun\x1a\x1b.mlflow.RestoreRun.Response\"7\xf2\x86\x19\x33\n\"\n\x04POST\x12\x14/mlflow/runs/restore\x1a\x04\x08\x02\x10\x00\x10\x01*\x0bRestore Run\x12u\n\tlogMetric\x12\x11.mlflow.LogMetric\x1a\x1a.mlflow.LogMetric.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/log-metric\x1a\x04\x08\x02\x10\x00\x10\x01*\nLog Metric\x12t\n\x08logParam\x12\x10.mlflow.LogParam\x1a\x19.mlflow.LogParam.Response\";\xf2\x86\x19\x37\n(\n\x04POST\x12\x1a/mlflow/runs/log-parameter\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Param\x12\xa1\x01\n\x10setExperimentTag\x12\x18.mlflow.SetExperimentTag\x1a!.mlflow.SetExperimentTag.Response\"P\xf2\x86\x19L\n4\n\x04POST\x12&/mlflow/experiments/set-experiment-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Set Experiment Tag\x12\xb0\x01\n\x13\x64\x65leteExperimentTag\x12\x1b.mlflow.DeleteExperimentTag\x1a$.mlflow.DeleteExperimentTag.Response\"V\xf2\x86\x19R\n7\n\x04POST\x12)/mlflow/experiments/delete-experiment-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x15\x44\x65lete Experiment Tag\x12\x66\n\x06setTag\x12\x0e.mlflow.SetTag\x1a\x17.mlflow.SetTag.Response\"3\xf2\x86\x19/\n\"\n\x04POST\x12\x14/mlflow/runs/set-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x07Set Tag\x12\x88\x01\n\x0bsetTraceTag\x12\x13.mlflow.SetTraceTag\x1a\x1c.mlflow.SetTraceTag.Response\"F\xf2\x86\x19\x42\n/\n\x05PATCH\x12 /mlflow/traces/{request_id}/tags\x1a\x04\x08\x02\x10\x00\x10\x03*\rSet Trace Tag\x12\x8f\x01\n\rsetTraceTagV3\x12\x15.mlflow.SetTraceTagV3\x1a\x1e.mlflow.SetTraceTagV3.Response\"G\xf2\x86\x19\x43\n-\n\x05PATCH\x12\x1e/mlflow/traces/{trace_id}/tags\x1a\x04\x08\x03\x10\x00\x10\x03*\x10Set Trace Tag V3\x12\x95\x01\n\x0e\x64\x65leteTraceTag\x12\x16.mlflow.DeleteTraceTag\x1a\x1f.mlflow.DeleteTraceTag.Response\"J\xf2\x86\x19\x46\n0\n\x06\x44\x45LETE\x12 /mlflow/traces/{request_id}/tags\x1a\x04\x08\x02\x10\x00\x10\x03*\x10\x44\x65lete Trace Tag\x12\x9c\x01\n\x10\x64\x65leteTraceTagV3\x12\x18.mlflow.DeleteTraceTagV3\x1a!.mlflow.DeleteTraceTagV3.Response\"K\xf2\x86\x19G\n.\n\x06\x44\x45LETE\x12\x1e/mlflow/traces/{trace_id}/tags\x1a\x04\x08\x03\x10\x00\x10\x03*\x13\x44\x65lete Trace Tag V3\x12u\n\tdeleteTag\x12\x11.mlflow.DeleteTag\x1a\x1a.mlflow.DeleteTag.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\nDelete Tag\x12\x65\n\x06getRun\x12\x0e.mlflow.GetRun\x1a\x17.mlflow.GetRun.Response\"2\xf2\x86\x19*\n\x1d\n\x03GET\x12\x10/mlflow/runs/get\x1a\x04\x08\x02\x10\x00\x10\x01*\x07Get Run\xba\x8c\x19\x00\x12y\n\nsearchRuns\x12\x12.mlflow.SearchRuns\x1a\x1b.mlflow.SearchRuns.Response\":\xf2\x86\x19\x32\n!\n\x04POST\x12\x13/mlflow/runs/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x0bSearch Runs\xba\x8c\x19\x00\x12\x87\x01\n\rlistArtifacts\x12\x15.mlflow.ListArtifacts\x1a\x1e.mlflow.ListArtifacts.Response\"?\xf2\x86\x19\x37\n#\n\x03GET\x12\x16/mlflow/artifacts/list\x1a\x04\x08\x02\x10\x00\x10\x01*\x0eList Artifacts\xba\x8c\x19\x00\x12\x95\x01\n\x10getMetricHistory\x12\x18.mlflow.GetMetricHistory\x1a!.mlflow.GetMetricHistory.Response\"D\xf2\x86\x19@\n(\n\x03GET\x12\x1b/mlflow/metrics/get-history\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Get Metric History\x12\xb7\x01\n\x1cgetMetricHistoryBulkInterval\x12$.mlflow.GetMetricHistoryBulkInterval\x1a-.mlflow.GetMetricHistoryBulkInterval.Response\"B\xf2\x86\x19:\n6\n\x03GET\x12)/mlflow/metrics/get-history-bulk-interval\x1a\x04\x08\x02\x10\x0b\x10\x03\xba\x8c\x19\x00\x12p\n\x08logBatch\x12\x10.mlflow.LogBatch\x1a\x19.mlflow.LogBatch.Response\"7\xf2\x86\x19\x33\n$\n\x04POST\x12\x16/mlflow/runs/log-batch\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Batch\x12p\n\x08logModel\x12\x10.mlflow.LogModel\x1a\x19.mlflow.LogModel.Response\"7\xf2\x86\x19\x33\n$\n\x04POST\x12\x16/mlflow/runs/log-model\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Model\x12u\n\tlogInputs\x12\x11.mlflow.LogInputs\x1a\x1a.mlflow.LogInputs.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/log-inputs\x1a\x04\x08\x02\x10\x00\x10\x01*\nLog Inputs\x12v\n\nlogOutputs\x12\x12.mlflow.LogOutputs\x1a\x1b.mlflow.LogOutputs.Response\"7\xf2\x86\x19\x33\n\"\n\x04POST\x12\x14/mlflow/runs/outputs\x1a\x04\x08\x02\x10\x00\x10\x03*\x0bLog Outputs\x12\x87\x01\n\x0esearchDatasets\x12\x16.mlflow.SearchDatasets\x1a\x1f.mlflow.SearchDatasets.Response\"<\xf2\x86\x19\x34\n0\n\x04POST\x12\"mlflow/experiments/search-datasets\x1a\x04\x08\x02\x10\x00\x10\x03\xba\x8c\x19\x00\x12p\n\nstartTrace\x12\x12.mlflow.StartTrace\x1a\x1b.mlflow.StartTrace.Response\"1\xf2\x86\x19-\n\x1c\n\x04POST\x12\x0e/mlflow/traces\x1a\x04\x08\x02\x10\x00\x10\x03*\x0bStart Trace\x12v\n\x08\x65ndTrace\x12\x10.mlflow.EndTrace\x1a\x19.mlflow.EndTrace.Response\"=\xf2\x86\x19\x39\n*\n\x05PATCH\x12\x1b/mlflow/traces/{request_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\tEnd Trace\x12\x89\x01\n\x0cgetTraceInfo\x12\x14.mlflow.GetTraceInfo\x1a\x1d.mlflow.GetTraceInfo.Response\"D\xf2\x86\x19@\n-\n\x03GET\x12 /mlflow/traces/{request_id}/info\x1a\x04\x08\x02\x10\x00\x10\x03*\rGet TraceInfo\x12\x8b\x01\n\x0egetTraceInfoV3\x12\x16.mlflow.GetTraceInfoV3\x1a\x1f.mlflow.GetTraceInfoV3.Response\"@\xf2\x86\x19<\n&\n\x03GET\x12\x19/mlflow/traces/{trace_id}\x1a\x04\x08\x03\x10\x00\x10\x03*\x10Get TraceInfo v3\x12n\n\x08getTrace\x12\x10.mlflow.GetTrace\x1a\x19.mlflow.GetTrace.Response\"5\xf2\x86\x19\x31\n\x1f\n\x03GET\x12\x12/mlflow/traces/get\x1a\x04\x08\x03\x10\x00\x10\x03*\x0cGet Trace v3\x12\x83\x01\n\x0e\x62\x61tchGetTraces\x12\x16.mlflow.BatchGetTraces\x1a\x1f.mlflow.BatchGetTraces.Response\"8\xf2\x86\x19\x34\n$\n\x03GET\x12\x17/mlflow/traces/batchGet\x1a\x04\x08\x03\x10\x00\x10\x03*\nGet Traces\x12w\n\x0csearchTraces\x12\x14.mlflow.SearchTraces\x1a\x1d.mlflow.SearchTraces.Response\"2\xf2\x86\x19.\n\x1b\n\x03GET\x12\x0e/mlflow/traces\x1a\x04\x08\x02\x10\x00\x10\x03*\rSearch Traces\x12\x88\x01\n\x0esearchTracesV3\x12\x16.mlflow.SearchTracesV3\x1a\x1f.mlflow.SearchTracesV3.Response\"=\xf2\x86\x19\x39\n#\n\x04POST\x12\x15/mlflow/traces/search\x1a\x04\x08\x03\x10\x00\x10\x03*\x10Search Traces V3\x12i\n\x0cstartTraceV3\x12\x14.mlflow.StartTraceV3\x1a\x1d.mlflow.StartTraceV3.Response\"$\xf2\x86\x19 \n\x1c\n\x04POST\x12\x0e/mlflow/traces\x1a\x04\x08\x03\x10\x00\x10\x03\x12\x92\x01\n\x0flinkTracesToRun\x12\x17.mlflow.LinkTracesToRun\x1a .mlflow.LinkTracesToRun.Response\"D\xf2\x86\x19@\n(\n\x04POST\x12\x1a/mlflow/traces/link-to-run\x1a\x04\x08\x02\x10\x00\x10\x03*\x12Link Traces to Run\x12\x9f\x01\n\x12linkPromptsToTrace\x12\x1a.mlflow.LinkPromptsToTrace\x1a#.mlflow.LinkPromptsToTrace.Response\"H\xf2\x86\x19\x44\n)\n\x04POST\x12\x1b/mlflow/traces/link-prompts\x1a\x04\x08\x02\x10\x00\x10\x03*\x15Link Prompts to Trace\x12\xa2\x01\n\x19searchUnifiedTraceHandler\x12\x1b.mlflow.SearchUnifiedTraces\x1a$.mlflow.SearchUnifiedTraces.Response\"B\xf2\x86\x19>\n#\n\x03GET\x12\x16/mlflow/unified-traces\x1a\x04\x08\x02\x10\x00\x10\x03*\x15Search Unified Traces\x12\xaf\x01\n\x15getOnlineTraceDetails\x12\x1d.mlflow.GetOnlineTraceDetails\x1a&.mlflow.GetOnlineTraceDetails.Response\"O\xf2\x86\x19K\n-\n\x03GET\x12 /mlflow/get-online-trace-details\x1a\x04\x08\x02\x10\x00\x10\x03*\x18Get Online Trace Details\x12\x86\x01\n\x0c\x64\x65leteTraces\x12\x14.mlflow.DeleteTraces\x1a\x1d.mlflow.DeleteTraces.Response\"A\xf2\x86\x19=\n*\n\x04POST\x12\x1c/mlflow/traces/delete-traces\x1a\x04\x08\x02\x10\x00\x10\x03*\rDelete Traces\x12\x8f\x01\n\x0e\x64\x65leteTracesV3\x12\x16.mlflow.DeleteTracesV3\x1a\x1f.mlflow.DeleteTracesV3.Response\"D\xf2\x86\x19@\n*\n\x04POST\x12\x1c/mlflow/traces/delete-traces\x1a\x04\x08\x03\x10\x00\x10\x03*\x10\x44\x65lete Traces V3\x12\xe3\x01\n\x1f\x63\x61lculateTraceFilterCorrelation\x12\'.mlflow.CalculateTraceFilterCorrelation\x1a\x30.mlflow.CalculateTraceFilterCorrelation.Response\"e\xf2\x86\x19\x61\n9\n\x04POST\x12+/mlflow/traces/calculate-filter-correlation\x1a\x04\x08\x03\x10\x00\x10\x03*\"Calculate Trace Filter Correlation\x12\x94\x01\n\x11\x63reateLoggedModel\x12\x19.mlflow.CreateLoggedModel\x1a\".mlflow.CreateLoggedModel.Response\"@\xf2\x86\x19<\n#\n\x04POST\x12\x15/mlflow/logged-models\x1a\x04\x08\x02\x10\x00\x10\x03*\x13\x43reate Logged Model\x12\xa8\x01\n\x13\x66inalizeLoggedModel\x12\x1b.mlflow.FinalizeLoggedModel\x1a$.mlflow.FinalizeLoggedModel.Response\"N\xf2\x86\x19J\n/\n\x05PATCH\x12 /mlflow/logged-models/{model_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\x15\x46inalize Logged Model\x12\x92\x01\n\x0egetLoggedModel\x12\x16.mlflow.GetLoggedModel\x1a\x1f.mlflow.GetLoggedModel.Response\"G\xf2\x86\x19\x43\n-\n\x03GET\x12 /mlflow/logged-models/{model_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\x10Get Logged Model\x12\xa3\x01\n\x11\x64\x65leteLoggedModel\x12\x19.mlflow.DeleteLoggedModel\x1a\".mlflow.DeleteLoggedModel.Response\"O\xf2\x86\x19K\n0\n\x06\x44\x45LETE\x12 /mlflow/logged-models/{model_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\x15\x44\x65lete a Logged Model\x12\x9e\x01\n\x12searchLoggedModels\x12\x1a.mlflow.SearchLoggedModels\x1a#.mlflow.SearchLoggedModels.Response\"G\xf2\x86\x19\x43\n*\n\x04POST\x12\x1c/mlflow/logged-models/search\x1a\x04\x08\x02\x10\x00\x10\x03*\x13Search LoggedModels\x12\xa9\x01\n\x12setLoggedModelTags\x12\x1a.mlflow.SetLoggedModelTags\x1a#.mlflow.SetLoggedModelTags.Response\"R\xf2\x86\x19N\n4\n\x05PATCH\x12%/mlflow/logged-models/{model_id}/tags\x1a\x04\x08\x02\x10\x00\x10\x03*\x14Set Logged Model Tag\x12\xbd\x01\n\x14\x64\x65leteLoggedModelTag\x12\x1c.mlflow.DeleteLoggedModelTag\x1a%.mlflow.DeleteLoggedModelTag.Response\"`\xf2\x86\x19\\\n?\n\x06\x44\x45LETE\x12//mlflow/logged-models/{model_id}/tags/{tag_key}\x1a\x04\x08\x02\x10\x00\x10\x03*\x17\x44\x65lete Logged Model Tag\x12\xd6\x01\n\x18listLoggedModelArtifacts\x12 .mlflow.ListLoggedModelArtifacts\x1a).mlflow.ListLoggedModelArtifacts.Response\"m\xf2\x86\x19i\nC\n\x03GET\x12\x36/mlflow/logged-models/{model_id}/artifacts/directories\x1a\x04\x08\x02\x10\x00\x10\x03* List Artifacts for Logged Models\x12\xc1\x01\n\x14LogLoggedModelParams\x12#.mlflow.LogLoggedModelParamsRequest\x1a,.mlflow.LogLoggedModelParamsRequest.Response\"V\xf2\x86\x19R\n5\n\x04POST\x12\'/mlflow/logged-models/{model_id}/params\x1a\x04\x08\x02\x10\x00\x10\x03*\x17Log Logged Model Params\x12\xb0\x01\n\rGetAssessment\x12\x1c.mlflow.GetAssessmentRequest\x1a%.mlflow.GetAssessmentRequest.Response\"Z\xf2\x86\x19V\nB\n\x03GET\x12\x35/mlflow/traces/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x03\x10\x00\x10\x03*\x0eGet Assessment\x12\xdf\x01\n\x10\x63reateAssessment\x12\x18.mlflow.CreateAssessment\x1a!.mlflow.CreateAssessment.Response\"\x8d\x01\xf2\x86\x19\x88\x01\n>\n\x04POST\x12\x30/mlflow/traces/{assessment.trace_id}/assessments\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xee\x07\x18\x0c\x18\x01*:Create an assessment of a trace or a span within the trace\x12\xd0\x01\n\x10updateAssessment\x12\x18.mlflow.UpdateAssessment\x1a!.mlflow.UpdateAssessment.Response\"\x7f\xf2\x86\x19{\nD\n\x05PATCH\x12\x35/mlflow/traces/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xee\x07\x18\x01*)Update an existing assessment on a trace.\x12\xb1\x01\n\x10\x64\x65leteAssessment\x12\x18.mlflow.DeleteAssessment\x1a!.mlflow.DeleteAssessment.Response\"`\xf2\x86\x19\\\nE\n\x06\x44\x45LETE\x12\x35/mlflow/traces/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x03\x10\x00\x10\x03*\x11\x44\x65lete Assessment\x12\x9a\x01\n\rcreateDataset\x12\x15.mlflow.CreateDataset\x1a\x1e.mlflow.CreateDataset.Response\"R\xf2\x86\x19N\n%\n\x04POST\x12\x17/mlflow/datasets/create\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xee\x07\x18\x0c\x18\x01*\x19\x43reate Evaluation Dataset\x12\x91\x01\n\ngetDataset\x12\x12.mlflow.GetDataset\x1a\x1b.mlflow.GetDataset.Response\"R\xf2\x86\x19N\n*\n\x03GET\x12\x1d/mlflow/datasets/{dataset_id}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x16Get Evaluation Dataset\x12\xa0\x01\n\rdeleteDataset\x12\x15.mlflow.DeleteDataset\x1a\x1e.mlflow.DeleteDataset.Response\"X\xf2\x86\x19T\n-\n\x06\x44\x45LETE\x12\x1d/mlflow/datasets/{dataset_id}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x19\x44\x65lete Evaluation Dataset\x12\xdd\x01\n\x18searchEvaluationDatasets\x12 .mlflow.SearchEvaluationDatasets\x1a).mlflow.SearchEvaluationDatasets.Response\"t\xf2\x86\x19p\n%\n\x04POST\x12\x17/mlflow/datasets/search\x1a\x04\x08\x03\x10\x00\n$\n\x03GET\x12\x17/mlflow/datasets/search\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\x01*\x1aSearch Evaluation Datasets\x12\xa9\x01\n\x0esetDatasetTags\x12\x16.mlflow.SetDatasetTags\x1a\x1f.mlflow.SetDatasetTags.Response\"^\xf2\x86\x19Z\n1\n\x05PATCH\x12\"/mlflow/datasets/{dataset_id}/tags\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1bSet Evaluation Dataset Tags\x12\xb8\x01\n\x10\x64\x65leteDatasetTag\x12\x18.mlflow.DeleteDatasetTag\x1a!.mlflow.DeleteDatasetTag.Response\"g\xf2\x86\x19\x63\n8\n\x06\x44\x45LETE\x12(/mlflow/datasets/{dataset_id}/tags/{key}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1d\x44\x65lete Evaluation Dataset Tag\x12\xc3\x01\n\x14upsertDatasetRecords\x12\x1c.mlflow.UpsertDatasetRecords\x1a%.mlflow.UpsertDatasetRecords.Response\"f\xf2\x86\x19\x62\n3\n\x04POST\x12%/mlflow/datasets/{dataset_id}/records\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*!Upsert Evaluation Dataset Records\x12\xd6\x01\n\x17getDatasetExperimentIds\x12\x1f.mlflow.GetDatasetExperimentIds\x1a(.mlflow.GetDatasetExperimentIds.Response\"p\xf2\x86\x19l\n9\n\x03GET\x12,/mlflow/datasets/{dataset_id}/experiment-ids\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*%Get Evaluation Dataset Experiment IDs\x12\x8a\x01\n\x0eregisterScorer\x12\x16.mlflow.RegisterScorer\x1a\x1f.mlflow.RegisterScorer.Response\"?\xf2\x86\x19;\n&\n\x04POST\x12\x18/mlflow/scorers/register\x1a\x04\x08\x03\x10\x00\x10\x01*\x0fRegister Scorer\x12y\n\x0blistScorers\x12\x13.mlflow.ListScorers\x1a\x1c.mlflow.ListScorers.Response\"7\xf2\x86\x19\x33\n!\n\x03GET\x12\x14/mlflow/scorers/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x0cList Scorers\x12\x9a\x01\n\x12listScorerVersions\x12\x1a.mlflow.ListScorerVersions\x1a#.mlflow.ListScorerVersions.Response\"C\xf2\x86\x19?\n%\n\x03GET\x12\x18/mlflow/scorers/versions\x1a\x04\x08\x03\x10\x00\x10\x01*\x14List Scorer Versions\x12p\n\tgetScorer\x12\x11.mlflow.GetScorer\x1a\x1a.mlflow.GetScorer.Response\"4\xf2\x86\x19\x30\n \n\x03GET\x12\x13/mlflow/scorers/get\x1a\x04\x08\x03\x10\x00\x10\x01*\nGet Scorer\x12\x82\x01\n\x0c\x64\x65leteScorer\x12\x14.mlflow.DeleteScorer\x1a\x1d.mlflow.DeleteScorer.Response\"=\xf2\x86\x19\x39\n&\n\x06\x44\x45LETE\x12\x16/mlflow/scorers/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\rDelete Scorer\x12\xb6\x01\n\x11getDatasetRecords\x12\x19.mlflow.GetDatasetRecords\x1a\".mlflow.GetDatasetRecords.Response\"b\xf2\x86\x19^\n2\n\x03GET\x12%/mlflow/datasets/{dataset_id}/records\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1eGet Evaluation Dataset Records\x12\xcd\x01\n\x17\x61\x64\x64\x44\x61tasetToExperiments\x12\x1f.mlflow.AddDatasetToExperiments\x1a(.mlflow.AddDatasetToExperiments.Response\"g\xf2\x86\x19\x63\n;\n\x04POST\x12-/mlflow/datasets/{dataset_id}/add-experiments\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1a\x41\x64\x64 Dataset to Experiments\x12\xe4\x01\n\x1cremoveDatasetFromExperiments\x12$.mlflow.RemoveDatasetFromExperiments\x1a-.mlflow.RemoveDatasetFromExperiments.Response\"o\xf2\x86\x19k\n>\n\x04POST\x12\x30/mlflow/datasets/{dataset_id}/remove-experiments\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1fRemove Dataset from Experiments\x12\xa5\x01\n\x13\x63reateGatewaySecret\x12\x1b.mlflow.CreateGatewaySecret\x1a$.mlflow.CreateGatewaySecret.Response\"K\xf2\x86\x19G\n,\n\x04POST\x12\x1e/mlflow/gateway/secrets/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x15\x43reate Gateway Secret\x12\xa6\x01\n\x14getGatewaySecretInfo\x12\x1c.mlflow.GetGatewaySecretInfo\x1a%.mlflow.GetGatewaySecretInfo.Response\"I\xf2\x86\x19\x45\n(\n\x03GET\x12\x1b/mlflow/gateway/secrets/get\x1a\x04\x08\x03\x10\x00\x10\x01*\x17Get Gateway Secret Info\x12\xa5\x01\n\x13updateGatewaySecret\x12\x1b.mlflow.UpdateGatewaySecret\x1a$.mlflow.UpdateGatewaySecret.Response\"K\xf2\x86\x19G\n,\n\x04POST\x12\x1e/mlflow/gateway/secrets/update\x1a\x04\x08\x03\x10\x00\x10\x01*\x15Update Gateway Secret\x12\xa7\x01\n\x13\x64\x65leteGatewaySecret\x12\x1b.mlflow.DeleteGatewaySecret\x1a$.mlflow.DeleteGatewaySecret.Response\"M\xf2\x86\x19I\n.\n\x06\x44\x45LETE\x12\x1e/mlflow/gateway/secrets/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x15\x44\x65lete Gateway Secret\x12\xaa\x01\n\x16listGatewaySecretInfos\x12\x1e.mlflow.ListGatewaySecretInfos\x1a\'.mlflow.ListGatewaySecretInfos.Response\"G\xf2\x86\x19\x43\n)\n\x03GET\x12\x1c/mlflow/gateway/secrets/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x14List Gateway Secrets\x12\xaf\x01\n\x15\x63reateGatewayEndpoint\x12\x1d.mlflow.CreateGatewayEndpoint\x1a&.mlflow.CreateGatewayEndpoint.Response\"O\xf2\x86\x19K\n.\n\x04POST\x12 /mlflow/gateway/endpoints/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x43reate Gateway Endpoint\x12\x9f\x01\n\x12getGatewayEndpoint\x12\x1a.mlflow.GetGatewayEndpoint\x1a#.mlflow.GetGatewayEndpoint.Response\"H\xf2\x86\x19\x44\n*\n\x03GET\x12\x1d/mlflow/gateway/endpoints/get\x1a\x04\x08\x03\x10\x00\x10\x01*\x14Get Gateway Endpoint\x12\xaf\x01\n\x15updateGatewayEndpoint\x12\x1d.mlflow.UpdateGatewayEndpoint\x1a&.mlflow.UpdateGatewayEndpoint.Response\"O\xf2\x86\x19K\n.\n\x04POST\x12 /mlflow/gateway/endpoints/update\x1a\x04\x08\x03\x10\x00\x10\x01*\x17Update Gateway Endpoint\x12\xb1\x01\n\x15\x64\x65leteGatewayEndpoint\x12\x1d.mlflow.DeleteGatewayEndpoint\x1a&.mlflow.DeleteGatewayEndpoint.Response\"Q\xf2\x86\x19M\n0\n\x06\x44\x45LETE\x12 /mlflow/gateway/endpoints/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x44\x65lete Gateway Endpoint\x12\xa8\x01\n\x14listGatewayEndpoints\x12\x1c.mlflow.ListGatewayEndpoints\x1a%.mlflow.ListGatewayEndpoints.Response\"K\xf2\x86\x19G\n+\n\x03GET\x12\x1e/mlflow/gateway/endpoints/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x16List Gateway Endpoints\x12\xd4\x01\n\x1c\x63reateGatewayModelDefinition\x12$.mlflow.CreateGatewayModelDefinition\x1a-.mlflow.CreateGatewayModelDefinition.Response\"_\xf2\x86\x19[\n6\n\x04POST\x12(/mlflow/gateway/model-definitions/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x1f\x43reate Gateway Model Definition\x12\xc4\x01\n\x19getGatewayModelDefinition\x12!.mlflow.GetGatewayModelDefinition\x1a*.mlflow.GetGatewayModelDefinition.Response\"X\xf2\x86\x19T\n2\n\x03GET\x12%/mlflow/gateway/model-definitions/get\x1a\x04\x08\x03\x10\x00\x10\x01*\x1cGet Gateway Model Definition\x12\xcd\x01\n\x1blistGatewayModelDefinitions\x12#.mlflow.ListGatewayModelDefinitions\x1a,.mlflow.ListGatewayModelDefinitions.Response\"[\xf2\x86\x19W\n3\n\x03GET\x12&/mlflow/gateway/model-definitions/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x1eList Gateway Model Definitions\x12\xd4\x01\n\x1cupdateGatewayModelDefinition\x12$.mlflow.UpdateGatewayModelDefinition\x1a-.mlflow.UpdateGatewayModelDefinition.Response\"_\xf2\x86\x19[\n6\n\x04POST\x12(/mlflow/gateway/model-definitions/update\x1a\x04\x08\x03\x10\x00\x10\x01*\x1fUpdate Gateway Model Definition\x12\xd6\x01\n\x1c\x64\x65leteGatewayModelDefinition\x12$.mlflow.DeleteGatewayModelDefinition\x1a-.mlflow.DeleteGatewayModelDefinition.Response\"a\xf2\x86\x19]\n8\n\x06\x44\x45LETE\x12(/mlflow/gateway/model-definitions/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x1f\x44\x65lete Gateway Model Definition\x12\xc5\x01\n\x15\x61ttachModelToEndpoint\x12$.mlflow.AttachModelToGatewayEndpoint\x1a-.mlflow.AttachModelToGatewayEndpoint.Response\"W\xf2\x86\x19S\n5\n\x04POST\x12\'/mlflow/gateway/endpoints/models/attach\x1a\x04\x08\x03\x10\x00\x10\x01*\x18\x41ttach Model to Endpoint\x12\xcd\x01\n\x17\x64\x65tachModelFromEndpoint\x12&.mlflow.DetachModelFromGatewayEndpoint\x1a/.mlflow.DetachModelFromGatewayEndpoint.Response\"Y\xf2\x86\x19U\n5\n\x04POST\x12\'/mlflow/gateway/endpoints/models/detach\x1a\x04\x08\x03\x10\x00\x10\x01*\x1a\x44\x65tach Model from Endpoint\x12\xc6\x01\n\x15\x63reateEndpointBinding\x12$.mlflow.CreateGatewayEndpointBinding\x1a-.mlflow.CreateGatewayEndpointBinding.Response\"X\xf2\x86\x19T\n7\n\x04POST\x12)/mlflow/gateway/endpoints/bindings/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x43reate Endpoint Binding\x12\xc8\x01\n\x15\x64\x65leteEndpointBinding\x12$.mlflow.DeleteGatewayEndpointBinding\x1a-.mlflow.DeleteGatewayEndpointBinding.Response\"Z\xf2\x86\x19V\n9\n\x06\x44\x45LETE\x12)/mlflow/gateway/endpoints/bindings/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x44\x65lete Endpoint Binding\x12\xbf\x01\n\x14listEndpointBindings\x12#.mlflow.ListGatewayEndpointBindings\x1a,.mlflow.ListGatewayEndpointBindings.Response\"T\xf2\x86\x19P\n4\n\x03GET\x12\'/mlflow/gateway/endpoints/bindings/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x16List Endpoint Bindings\x12\xb1\x01\n\x15setGatewayEndpointTag\x12\x1d.mlflow.SetGatewayEndpointTag\x1a&.mlflow.SetGatewayEndpointTag.Response\"Q\xf2\x86\x19M\n/\n\x04POST\x12!/mlflow/gateway/endpoints/set-tag\x1a\x04\x08\x03\x10\x00\x10\x01*\x18Gateway Set Endpoint Tag\x12\xc2\x01\n\x18\x64\x65leteGatewayEndpointTag\x12 .mlflow.DeleteGatewayEndpointTag\x1a).mlflow.DeleteGatewayEndpointTag.Response\"Y\xf2\x86\x19U\n4\n\x06\x44\x45LETE\x12$/mlflow/gateway/endpoints/delete-tag\x1a\x04\x08\x03\x10\x00\x10\x01*\x1bGateway Delete Endpoint TagB\x1e\n\x14org.mlflow.api.proto\x90\x01\x01\xe2?\x02\x10\x01')

  _globals = globals()
  _builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
  _builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'service_pb2', _globals)
  if not _descriptor._USE_C_DESCRIPTORS:
    _globals['DESCRIPTOR']._loaded_options = None
    _globals['DESCRIPTOR']._serialized_options = b'\n\024org.mlflow.api.proto\220\001\001\342?\002\020\001'
    _globals['_METRIC'].fields_by_name['dataset_name']._loaded_options = None
    _globals['_METRIC'].fields_by_name['dataset_name']._serialized_options = b'\360\206\031\003'
    _globals['_METRIC'].fields_by_name['dataset_digest']._loaded_options = None
    _globals['_METRIC'].fields_by_name['dataset_digest']._serialized_options = b'\360\206\031\003'
    _globals['_METRIC'].fields_by_name['model_id']._loaded_options = None
    _globals['_METRIC'].fields_by_name['model_id']._serialized_options = b'\360\206\031\003'
    _globals['_METRIC'].fields_by_name['run_id']._loaded_options = None
    _globals['_METRIC'].fields_by_name['run_id']._serialized_options = b'\360\206\031\003'
    _globals['_DATASETINPUT'].fields_by_name['dataset']._loaded_options = None
    _globals['_DATASETINPUT'].fields_by_name['dataset']._serialized_options = b'\370\206\031\001'
    _globals['_MODELINPUT'].fields_by_name['model_id']._loaded_options = None
    _globals['_MODELINPUT'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_INPUTTAG'].fields_by_name['key']._loaded_options = None
    _globals['_INPUTTAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_INPUTTAG'].fields_by_name['value']._loaded_options = None
    _globals['_INPUTTAG'].fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _globals['_DATASET'].fields_by_name['name']._loaded_options = None
    _globals['_DATASET'].fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _globals['_DATASET'].fields_by_name['digest']._loaded_options = None
    _globals['_DATASET'].fields_by_name['digest']._serialized_options = b'\370\206\031\001'
    _globals['_DATASET'].fields_by_name['source_type']._loaded_options = None
    _globals['_DATASET'].fields_by_name['source_type']._serialized_options = b'\370\206\031\001'
    _globals['_DATASET'].fields_by_name['source']._loaded_options = None
    _globals['_DATASET'].fields_by_name['source']._serialized_options = b'\370\206\031\001'
    _globals['_MODELOUTPUT'].fields_by_name['model_id']._loaded_options = None
    _globals['_MODELOUTPUT'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_MODELOUTPUT'].fields_by_name['step']._loaded_options = None
    _globals['_MODELOUTPUT'].fields_by_name['step']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEEXPERIMENT'].fields_by_name['name']._loaded_options = None
    _globals['_CREATEEXPERIMENT'].fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEEXPERIMENT']._loaded_options = None
    _globals['_CREATEEXPERIMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHEXPERIMENTS']._loaded_options = None
    _globals['_SEARCHEXPERIMENTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETEXPERIMENT'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_GETEXPERIMENT'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETEXPERIMENT']._loaded_options = None
    _globals['_GETEXPERIMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETEEXPERIMENT'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_DELETEEXPERIMENT'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEEXPERIMENT']._loaded_options = None
    _globals['_DELETEEXPERIMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_RESTOREEXPERIMENT'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_RESTOREEXPERIMENT'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_RESTOREEXPERIMENT']._loaded_options = None
    _globals['_RESTOREEXPERIMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_UPDATEEXPERIMENT'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_UPDATEEXPERIMENT'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_UPDATEEXPERIMENT']._loaded_options = None
    _globals['_UPDATEEXPERIMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CREATERUN']._loaded_options = None
    _globals['_CREATERUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_UPDATERUN']._loaded_options = None
    _globals['_UPDATERUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETERUN'].fields_by_name['run_id']._loaded_options = None
    _globals['_DELETERUN'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETERUN']._loaded_options = None
    _globals['_DELETERUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_RESTORERUN'].fields_by_name['run_id']._loaded_options = None
    _globals['_RESTORERUN'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_RESTORERUN']._loaded_options = None
    _globals['_RESTORERUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGMETRIC'].fields_by_name['key']._loaded_options = None
    _globals['_LOGMETRIC'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_LOGMETRIC'].fields_by_name['value']._loaded_options = None
    _globals['_LOGMETRIC'].fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _globals['_LOGMETRIC'].fields_by_name['timestamp']._loaded_options = None
    _globals['_LOGMETRIC'].fields_by_name['timestamp']._serialized_options = b'\370\206\031\001'
    _globals['_LOGMETRIC'].fields_by_name['model_id']._loaded_options = None
    _globals['_LOGMETRIC'].fields_by_name['model_id']._serialized_options = b'\360\206\031\003'
    _globals['_LOGMETRIC'].fields_by_name['dataset_name']._loaded_options = None
    _globals['_LOGMETRIC'].fields_by_name['dataset_name']._serialized_options = b'\360\206\031\003'
    _globals['_LOGMETRIC'].fields_by_name['dataset_digest']._loaded_options = None
    _globals['_LOGMETRIC'].fields_by_name['dataset_digest']._serialized_options = b'\360\206\031\003'
    _globals['_LOGMETRIC']._loaded_options = None
    _globals['_LOGMETRIC']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGPARAM'].fields_by_name['key']._loaded_options = None
    _globals['_LOGPARAM'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_LOGPARAM'].fields_by_name['value']._loaded_options = None
    _globals['_LOGPARAM'].fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _globals['_LOGPARAM']._loaded_options = None
    _globals['_LOGPARAM']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETEXPERIMENTTAG'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_SETEXPERIMENTTAG'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_SETEXPERIMENTTAG'].fields_by_name['key']._loaded_options = None
    _globals['_SETEXPERIMENTTAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_SETEXPERIMENTTAG'].fields_by_name['value']._loaded_options = None
    _globals['_SETEXPERIMENTTAG'].fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _globals['_SETEXPERIMENTTAG']._loaded_options = None
    _globals['_SETEXPERIMENTTAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETEEXPERIMENTTAG'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_DELETEEXPERIMENTTAG'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEEXPERIMENTTAG'].fields_by_name['key']._loaded_options = None
    _globals['_DELETEEXPERIMENTTAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEEXPERIMENTTAG']._loaded_options = None
    _globals['_DELETEEXPERIMENTTAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETTAG'].fields_by_name['key']._loaded_options = None
    _globals['_SETTAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_SETTAG'].fields_by_name['value']._loaded_options = None
    _globals['_SETTAG'].fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _globals['_SETTAG']._loaded_options = None
    _globals['_SETTAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETETAG'].fields_by_name['run_id']._loaded_options = None
    _globals['_DELETETAG'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETAG'].fields_by_name['key']._loaded_options = None
    _globals['_DELETETAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETAG']._loaded_options = None
    _globals['_DELETETAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETRUN']._loaded_options = None
    _globals['_GETRUN']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHRUNS']._loaded_options = None
    _globals['_SEARCHRUNS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LISTARTIFACTS']._loaded_options = None
    _globals['_LISTARTIFACTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETMETRICHISTORY'].fields_by_name['metric_key']._loaded_options = None
    _globals['_GETMETRICHISTORY'].fields_by_name['metric_key']._serialized_options = b'\370\206\031\001'
    _globals['_GETMETRICHISTORY']._loaded_options = None
    _globals['_GETMETRICHISTORY']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETMETRICHISTORYBULKINTERVAL'].fields_by_name['metric_key']._loaded_options = None
    _globals['_GETMETRICHISTORYBULKINTERVAL'].fields_by_name['metric_key']._serialized_options = b'\370\206\031\001'
    _globals['_GETMETRICHISTORYBULKINTERVAL']._loaded_options = None
    _globals['_GETMETRICHISTORYBULKINTERVAL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGBATCH']._loaded_options = None
    _globals['_LOGBATCH']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGMODEL']._loaded_options = None
    _globals['_LOGMODEL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGINPUTS'].fields_by_name['run_id']._loaded_options = None
    _globals['_LOGINPUTS'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_LOGINPUTS'].fields_by_name['models']._loaded_options = None
    _globals['_LOGINPUTS'].fields_by_name['models']._serialized_options = b'\360\206\031\003'
    _globals['_LOGINPUTS']._loaded_options = None
    _globals['_LOGINPUTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGOUTPUTS'].fields_by_name['run_id']._loaded_options = None
    _globals['_LOGOUTPUTS'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_LOGOUTPUTS']._loaded_options = None
    _globals['_LOGOUTPUTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETEXPERIMENTBYNAME'].fields_by_name['experiment_name']._loaded_options = None
    _globals['_GETEXPERIMENTBYNAME'].fields_by_name['experiment_name']._serialized_options = b'\370\206\031\001'
    _globals['_GETEXPERIMENTBYNAME']._loaded_options = None
    _globals['_GETEXPERIMENTBYNAME']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CREATEASSESSMENT'].fields_by_name['assessment']._loaded_options = None
    _globals['_CREATEASSESSMENT'].fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEASSESSMENT']._loaded_options = None
    _globals['_CREATEASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_UPDATEASSESSMENT'].fields_by_name['assessment']._loaded_options = None
    _globals['_UPDATEASSESSMENT'].fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _globals['_UPDATEASSESSMENT'].fields_by_name['update_mask']._loaded_options = None
    _globals['_UPDATEASSESSMENT'].fields_by_name['update_mask']._serialized_options = b'\370\206\031\001'
    _globals['_UPDATEASSESSMENT']._loaded_options = None
    _globals['_UPDATEASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETEASSESSMENT'].fields_by_name['trace_id']._loaded_options = None
    _globals['_DELETEASSESSMENT'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEASSESSMENT'].fields_by_name['assessment_id']._loaded_options = None
    _globals['_DELETEASSESSMENT'].fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEASSESSMENT']._loaded_options = None
    _globals['_DELETEASSESSMENT']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETASSESSMENTREQUEST'].fields_by_name['trace_id']._loaded_options = None
    _globals['_GETASSESSMENTREQUEST'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETASSESSMENTREQUEST'].fields_by_name['assessment_id']._loaded_options = None
    _globals['_GETASSESSMENTREQUEST'].fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETASSESSMENTREQUEST']._loaded_options = None
    _globals['_GETASSESSMENTREQUEST']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_STARTTRACE']._loaded_options = None
    _globals['_STARTTRACE']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_ENDTRACE']._loaded_options = None
    _globals['_ENDTRACE']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETTRACEINFO']._loaded_options = None
    _globals['_GETTRACEINFO']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETTRACEINFOV3']._loaded_options = None
    _globals['_GETTRACEINFOV3']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_BATCHGETTRACES']._loaded_options = None
    _globals['_BATCHGETTRACES']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETTRACE'].fields_by_name['trace_id']._loaded_options = None
    _globals['_GETTRACE'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETTRACE']._loaded_options = None
    _globals['_GETTRACE']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHTRACES']._loaded_options = None
    _globals['_SEARCHTRACES']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHUNIFIEDTRACES'].fields_by_name['model_id']._loaded_options = None
    _globals['_SEARCHUNIFIEDTRACES'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_SEARCHUNIFIEDTRACES'].fields_by_name['sql_warehouse_id']._loaded_options = None
    _globals['_SEARCHUNIFIEDTRACES'].fields_by_name['sql_warehouse_id']._serialized_options = b'\370\206\031\001'
    _globals['_SEARCHUNIFIEDTRACES']._loaded_options = None
    _globals['_SEARCHUNIFIEDTRACES']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['trace_id']._loaded_options = None
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['sql_warehouse_id']._loaded_options = None
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['sql_warehouse_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['source_inference_table']._loaded_options = None
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['source_inference_table']._serialized_options = b'\370\206\031\001'
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['source_databricks_request_id']._loaded_options = None
    _globals['_GETONLINETRACEDETAILS'].fields_by_name['source_databricks_request_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETRACES'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_DELETETRACES'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETRACES']._loaded_options = None
    _globals['_DELETETRACES']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETETRACESV3'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_DELETETRACESV3'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETETRACESV3']._loaded_options = None
    _globals['_DELETETRACESV3']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CALCULATETRACEFILTERCORRELATION']._loaded_options = None
    _globals['_CALCULATETRACEFILTERCORRELATION']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETTRACETAG']._loaded_options = None
    _globals['_SETTRACETAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETTRACETAGV3']._loaded_options = None
    _globals['_SETTRACETAGV3']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETETRACETAG']._loaded_options = None
    _globals['_DELETETRACETAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETETRACETAGV3']._loaded_options = None
    _globals['_DELETETRACETAGV3']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_TRACEINFOV3_TRACEMETADATAENTRY']._loaded_options = None
    _globals['_TRACEINFOV3_TRACEMETADATAENTRY']._serialized_options = b'8\001'
    _globals['_TRACEINFOV3_TAGSENTRY']._loaded_options = None
    _globals['_TRACEINFOV3_TAGSENTRY']._serialized_options = b'8\001'
    _globals['_STARTTRACEV3'].fields_by_name['trace']._loaded_options = None
    _globals['_STARTTRACEV3'].fields_by_name['trace']._serialized_options = b'\370\206\031\001'
    _globals['_LINKTRACESTORUN'].fields_by_name['run_id']._loaded_options = None
    _globals['_LINKTRACESTORUN'].fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _globals['_LINKPROMPTSTOTRACE_PROMPTVERSIONREF'].fields_by_name['name']._loaded_options = None
    _globals['_LINKPROMPTSTOTRACE_PROMPTVERSIONREF'].fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _globals['_LINKPROMPTSTOTRACE_PROMPTVERSIONREF'].fields_by_name['version']._loaded_options = None
    _globals['_LINKPROMPTSTOTRACE_PROMPTVERSIONREF'].fields_by_name['version']._serialized_options = b'\370\206\031\001'
    _globals['_LINKPROMPTSTOTRACE'].fields_by_name['trace_id']._loaded_options = None
    _globals['_LINKPROMPTSTOTRACE'].fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _globals['_DATASETSUMMARY'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_DATASETSUMMARY'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_DATASETSUMMARY'].fields_by_name['name']._loaded_options = None
    _globals['_DATASETSUMMARY'].fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _globals['_DATASETSUMMARY'].fields_by_name['digest']._loaded_options = None
    _globals['_DATASETSUMMARY'].fields_by_name['digest']._serialized_options = b'\370\206\031\001'
    _globals['_SEARCHDATASETS']._loaded_options = None
    _globals['_SEARCHDATASETS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CREATELOGGEDMODEL'].fields_by_name['experiment_id']._loaded_options = None
    _globals['_CREATELOGGEDMODEL'].fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _globals['_CREATELOGGEDMODEL']._loaded_options = None
    _globals['_CREATELOGGEDMODEL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_FINALIZELOGGEDMODEL'].fields_by_name['model_id']._loaded_options = None
    _globals['_FINALIZELOGGEDMODEL'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_FINALIZELOGGEDMODEL'].fields_by_name['status']._loaded_options = None
    _globals['_FINALIZELOGGEDMODEL'].fields_by_name['status']._serialized_options = b'\370\206\031\001'
    _globals['_FINALIZELOGGEDMODEL']._loaded_options = None
    _globals['_FINALIZELOGGEDMODEL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETLOGGEDMODEL'].fields_by_name['model_id']._loaded_options = None
    _globals['_GETLOGGEDMODEL'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETLOGGEDMODEL']._loaded_options = None
    _globals['_GETLOGGEDMODEL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETELOGGEDMODEL'].fields_by_name['model_id']._loaded_options = None
    _globals['_DELETELOGGEDMODEL'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETELOGGEDMODEL']._loaded_options = None
    _globals['_DELETELOGGEDMODEL']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHLOGGEDMODELS_DATASET'].fields_by_name['dataset_name']._loaded_options = None
    _globals['_SEARCHLOGGEDMODELS_DATASET'].fields_by_name['dataset_name']._serialized_options = b'\370\206\031\001'
    _globals['_SEARCHLOGGEDMODELS_ORDERBY'].fields_by_name['field_name']._loaded_options = None
    _globals['_SEARCHLOGGEDMODELS_ORDERBY'].fields_by_name['field_name']._serialized_options = b'\370\206\031\001'
    _globals['_SEARCHLOGGEDMODELS']._loaded_options = None
    _globals['_SEARCHLOGGEDMODELS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETLOGGEDMODELTAGS'].fields_by_name['model_id']._loaded_options = None
    _globals['_SETLOGGEDMODELTAGS'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_SETLOGGEDMODELTAGS']._loaded_options = None
    _globals['_SETLOGGEDMODELTAGS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETELOGGEDMODELTAG'].fields_by_name['model_id']._loaded_options = None
    _globals['_DELETELOGGEDMODELTAG'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETELOGGEDMODELTAG'].fields_by_name['tag_key']._loaded_options = None
    _globals['_DELETELOGGEDMODELTAG'].fields_by_name['tag_key']._serialized_options = b'\370\206\031\001'
    _globals['_DELETELOGGEDMODELTAG']._loaded_options = None
    _globals['_DELETELOGGEDMODELTAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LISTLOGGEDMODELARTIFACTS'].fields_by_name['model_id']._loaded_options = None
    _globals['_LISTLOGGEDMODELARTIFACTS'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_LISTLOGGEDMODELARTIFACTS']._loaded_options = None
    _globals['_LISTLOGGEDMODELARTIFACTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LOGLOGGEDMODELPARAMSREQUEST'].fields_by_name['model_id']._loaded_options = None
    _globals['_LOGLOGGEDMODELPARAMSREQUEST'].fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _globals['_LOGLOGGEDMODELPARAMSREQUEST']._loaded_options = None
    _globals['_LOGLOGGEDMODELPARAMSREQUEST']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHTRACESV3']._loaded_options = None
    _globals['_SEARCHTRACESV3']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_CREATEDATASET'].fields_by_name['name']._loaded_options = None
    _globals['_CREATEDATASET'].fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _globals['_CREATEDATASET']._loaded_options = None
    _globals['_CREATEDATASET']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETDATASET'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_GETDATASET'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETDATASET']._loaded_options = None
    _globals['_GETDATASET']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETEDATASET'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_DELETEDATASET'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEDATASET']._loaded_options = None
    _globals['_DELETEDATASET']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SEARCHEVALUATIONDATASETS']._loaded_options = None
    _globals['_SEARCHEVALUATIONDATASETS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_SETDATASETTAGS'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_SETDATASETTAGS'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_SETDATASETTAGS'].fields_by_name['tags']._loaded_options = None
    _globals['_SETDATASETTAGS'].fields_by_name['tags']._serialized_options = b'\370\206\031\001'
    _globals['_SETDATASETTAGS']._loaded_options = None
    _globals['_SETDATASETTAGS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETEDATASETTAG'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_DELETEDATASETTAG'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEDATASETTAG'].fields_by_name['key']._loaded_options = None
    _globals['_DELETEDATASETTAG'].fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _globals['_DELETEDATASETTAG']._loaded_options = None
    _globals['_DELETEDATASETTAG']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_UPSERTDATASETRECORDS'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_UPSERTDATASETRECORDS'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_UPSERTDATASETRECORDS'].fields_by_name['records']._loaded_options = None
    _globals['_UPSERTDATASETRECORDS'].fields_by_name['records']._serialized_options = b'\370\206\031\001'
    _globals['_UPSERTDATASETRECORDS']._loaded_options = None
    _globals['_UPSERTDATASETRECORDS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETDATASETEXPERIMENTIDS'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_GETDATASETEXPERIMENTIDS'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETDATASETEXPERIMENTIDS']._loaded_options = None
    _globals['_GETDATASETEXPERIMENTIDS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETDATASETRECORDS'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_GETDATASETRECORDS'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_GETDATASETRECORDS']._loaded_options = None
    _globals['_GETDATASETRECORDS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_ADDDATASETTOEXPERIMENTS'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_ADDDATASETTOEXPERIMENTS'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_ADDDATASETTOEXPERIMENTS']._loaded_options = None
    _globals['_ADDDATASETTOEXPERIMENTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_REMOVEDATASETFROMEXPERIMENTS'].fields_by_name['dataset_id']._loaded_options = None
    _globals['_REMOVEDATASETFROMEXPERIMENTS'].fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _globals['_REMOVEDATASETFROMEXPERIMENTS']._loaded_options = None
    _globals['_REMOVEDATASETFROMEXPERIMENTS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_REGISTERSCORER']._loaded_options = None
    _globals['_REGISTERSCORER']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LISTSCORERS']._loaded_options = None
    _globals['_LISTSCORERS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_LISTSCORERVERSIONS']._loaded_options = None
    _globals['_LISTSCORERVERSIONS']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GETSCORER']._loaded_options = None
    _globals['_GETSCORER']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_DELETESCORER']._loaded_options = None
    _globals['_DELETESCORER']._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _globals['_GATEWAYSECRETINFO_MASKEDVALUESENTRY']._loaded_options = None
    _globals['_GATEWAYSECRETINFO_MASKEDVALUESENTRY']._serialized_options = b'8\001'
    _globals['_CREATEGATEWAYSECRET_SECRETVALUEENTRY']._loaded_options = None
    _globals['_CREATEGATEWAYSECRET_SECRETVALUEENTRY']._serialized_options = b'8\001'
    _globals['_UPDATEGATEWAYSECRET_SECRETVALUEENTRY']._loaded_options = None
    _globals['_UPDATEGATEWAYSECRET_SECRETVALUEENTRY']._serialized_options = b'8\001'
    _globals['_MLFLOWSERVICE'].methods_by_name['getExperimentByName']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getExperimentByName']._serialized_options = b'\362\206\031H\n,\n\003GET\022\037/mlflow/experiments/get-by-name\032\004\010\002\020\000\020\001*\026Get Experiment By Name'
    _globals['_MLFLOWSERVICE'].methods_by_name['createExperiment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/create\032\004\010\002\020\000\020\001*\021Create Experiment'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchExperiments']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchExperiments']._serialized_options = b'\362\206\031i\n(\n\004POST\022\032/mlflow/experiments/search\032\004\010\002\020\000\n\'\n\003GET\022\032/mlflow/experiments/search\032\004\010\002\020\000\020\001*\022Search Experiments'
    _globals['_MLFLOWSERVICE'].methods_by_name['getExperiment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getExperiment']._serialized_options = b'\362\206\0318\n$\n\003GET\022\027/mlflow/experiments/get\032\004\010\002\020\000\020\001*\016Get Experiment\272\214\031\000'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteExperiment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/delete\032\004\010\002\020\000\020\001*\021Delete Experiment'
    _globals['_MLFLOWSERVICE'].methods_by_name['restoreExperiment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['restoreExperiment']._serialized_options = b'\362\206\031A\n)\n\004POST\022\033/mlflow/experiments/restore\032\004\010\002\020\000\020\001*\022Restore Experiment'
    _globals['_MLFLOWSERVICE'].methods_by_name['updateExperiment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['updateExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/update\032\004\010\002\020\000\020\001*\021Update Experiment'
    _globals['_MLFLOWSERVICE'].methods_by_name['createRun']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/create\032\004\010\002\020\000\020\001*\nCreate Run'
    _globals['_MLFLOWSERVICE'].methods_by_name['updateRun']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['updateRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/update\032\004\010\002\020\000\020\001*\nUpdate Run'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteRun']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/delete\032\004\010\002\020\000\020\001*\nDelete Run'
    _globals['_MLFLOWSERVICE'].methods_by_name['restoreRun']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['restoreRun']._serialized_options = b'\362\206\0313\n\"\n\004POST\022\024/mlflow/runs/restore\032\004\010\002\020\000\020\001*\013Restore Run'
    _globals['_MLFLOWSERVICE'].methods_by_name['logMetric']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['logMetric']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/log-metric\032\004\010\002\020\000\020\001*\nLog Metric'
    _globals['_MLFLOWSERVICE'].methods_by_name['logParam']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['logParam']._serialized_options = b'\362\206\0317\n(\n\004POST\022\032/mlflow/runs/log-parameter\032\004\010\002\020\000\020\001*\tLog Param'
    _globals['_MLFLOWSERVICE'].methods_by_name['setExperimentTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setExperimentTag']._serialized_options = b'\362\206\031L\n4\n\004POST\022&/mlflow/experiments/set-experiment-tag\032\004\010\002\020\000\020\001*\022Set Experiment Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteExperimentTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteExperimentTag']._serialized_options = b'\362\206\031R\n7\n\004POST\022)/mlflow/experiments/delete-experiment-tag\032\004\010\002\020\000\020\001*\025Delete Experiment Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['setTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setTag']._serialized_options = b'\362\206\031/\n\"\n\004POST\022\024/mlflow/runs/set-tag\032\004\010\002\020\000\020\001*\007Set Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['setTraceTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setTraceTag']._serialized_options = b'\362\206\031B\n/\n\005PATCH\022 /mlflow/traces/{request_id}/tags\032\004\010\002\020\000\020\003*\rSet Trace Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['setTraceTagV3']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setTraceTagV3']._serialized_options = b'\362\206\031C\n-\n\005PATCH\022\036/mlflow/traces/{trace_id}/tags\032\004\010\003\020\000\020\003*\020Set Trace Tag V3'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTraceTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTraceTag']._serialized_options = b'\362\206\031F\n0\n\006DELETE\022 /mlflow/traces/{request_id}/tags\032\004\010\002\020\000\020\003*\020Delete Trace Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTraceTagV3']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTraceTagV3']._serialized_options = b'\362\206\031G\n.\n\006DELETE\022\036/mlflow/traces/{trace_id}/tags\032\004\010\003\020\000\020\003*\023Delete Trace Tag V3'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTag']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/delete-tag\032\004\010\002\020\000\020\001*\nDelete Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['getRun']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getRun']._serialized_options = b'\362\206\031*\n\035\n\003GET\022\020/mlflow/runs/get\032\004\010\002\020\000\020\001*\007Get Run\272\214\031\000'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchRuns']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchRuns']._serialized_options = b'\362\206\0312\n!\n\004POST\022\023/mlflow/runs/search\032\004\010\002\020\000\020\001*\013Search Runs\272\214\031\000'
    _globals['_MLFLOWSERVICE'].methods_by_name['listArtifacts']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listArtifacts']._serialized_options = b'\362\206\0317\n#\n\003GET\022\026/mlflow/artifacts/list\032\004\010\002\020\000\020\001*\016List Artifacts\272\214\031\000'
    _globals['_MLFLOWSERVICE'].methods_by_name['getMetricHistory']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getMetricHistory']._serialized_options = b'\362\206\031@\n(\n\003GET\022\033/mlflow/metrics/get-history\032\004\010\002\020\000\020\001*\022Get Metric History'
    _globals['_MLFLOWSERVICE'].methods_by_name['getMetricHistoryBulkInterval']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getMetricHistoryBulkInterval']._serialized_options = b'\362\206\031:\n6\n\003GET\022)/mlflow/metrics/get-history-bulk-interval\032\004\010\002\020\013\020\003\272\214\031\000'
    _globals['_MLFLOWSERVICE'].methods_by_name['logBatch']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['logBatch']._serialized_options = b'\362\206\0313\n$\n\004POST\022\026/mlflow/runs/log-batch\032\004\010\002\020\000\020\001*\tLog Batch'
    _globals['_MLFLOWSERVICE'].methods_by_name['logModel']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['logModel']._serialized_options = b'\362\206\0313\n$\n\004POST\022\026/mlflow/runs/log-model\032\004\010\002\020\000\020\001*\tLog Model'
    _globals['_MLFLOWSERVICE'].methods_by_name['logInputs']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['logInputs']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/log-inputs\032\004\010\002\020\000\020\001*\nLog Inputs'
    _globals['_MLFLOWSERVICE'].methods_by_name['logOutputs']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['logOutputs']._serialized_options = b'\362\206\0313\n\"\n\004POST\022\024/mlflow/runs/outputs\032\004\010\002\020\000\020\003*\013Log Outputs'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchDatasets']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchDatasets']._serialized_options = b'\362\206\0314\n0\n\004POST\022\"mlflow/experiments/search-datasets\032\004\010\002\020\000\020\003\272\214\031\000'
    _globals['_MLFLOWSERVICE'].methods_by_name['startTrace']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['startTrace']._serialized_options = b'\362\206\031-\n\034\n\004POST\022\016/mlflow/traces\032\004\010\002\020\000\020\003*\013Start Trace'
    _globals['_MLFLOWSERVICE'].methods_by_name['endTrace']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['endTrace']._serialized_options = b'\362\206\0319\n*\n\005PATCH\022\033/mlflow/traces/{request_id}\032\004\010\002\020\000\020\003*\tEnd Trace'
    _globals['_MLFLOWSERVICE'].methods_by_name['getTraceInfo']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getTraceInfo']._serialized_options = b'\362\206\031@\n-\n\003GET\022 /mlflow/traces/{request_id}/info\032\004\010\002\020\000\020\003*\rGet TraceInfo'
    _globals['_MLFLOWSERVICE'].methods_by_name['getTraceInfoV3']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getTraceInfoV3']._serialized_options = b'\362\206\031<\n&\n\003GET\022\031/mlflow/traces/{trace_id}\032\004\010\003\020\000\020\003*\020Get TraceInfo v3'
    _globals['_MLFLOWSERVICE'].methods_by_name['getTrace']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getTrace']._serialized_options = b'\362\206\0311\n\037\n\003GET\022\022/mlflow/traces/get\032\004\010\003\020\000\020\003*\014Get Trace v3'
    _globals['_MLFLOWSERVICE'].methods_by_name['batchGetTraces']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['batchGetTraces']._serialized_options = b'\362\206\0314\n$\n\003GET\022\027/mlflow/traces/batchGet\032\004\010\003\020\000\020\003*\nGet Traces'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchTraces']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchTraces']._serialized_options = b'\362\206\031.\n\033\n\003GET\022\016/mlflow/traces\032\004\010\002\020\000\020\003*\rSearch Traces'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchTracesV3']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchTracesV3']._serialized_options = b'\362\206\0319\n#\n\004POST\022\025/mlflow/traces/search\032\004\010\003\020\000\020\003*\020Search Traces V3'
    _globals['_MLFLOWSERVICE'].methods_by_name['startTraceV3']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['startTraceV3']._serialized_options = b'\362\206\031 \n\034\n\004POST\022\016/mlflow/traces\032\004\010\003\020\000\020\003'
    _globals['_MLFLOWSERVICE'].methods_by_name['linkTracesToRun']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['linkTracesToRun']._serialized_options = b'\362\206\031@\n(\n\004POST\022\032/mlflow/traces/link-to-run\032\004\010\002\020\000\020\003*\022Link Traces to Run'
    _globals['_MLFLOWSERVICE'].methods_by_name['linkPromptsToTrace']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['linkPromptsToTrace']._serialized_options = b'\362\206\031D\n)\n\004POST\022\033/mlflow/traces/link-prompts\032\004\010\002\020\000\020\003*\025Link Prompts to Trace'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchUnifiedTraceHandler']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchUnifiedTraceHandler']._serialized_options = b'\362\206\031>\n#\n\003GET\022\026/mlflow/unified-traces\032\004\010\002\020\000\020\003*\025Search Unified Traces'
    _globals['_MLFLOWSERVICE'].methods_by_name['getOnlineTraceDetails']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getOnlineTraceDetails']._serialized_options = b'\362\206\031K\n-\n\003GET\022 /mlflow/get-online-trace-details\032\004\010\002\020\000\020\003*\030Get Online Trace Details'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTraces']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTraces']._serialized_options = b'\362\206\031=\n*\n\004POST\022\034/mlflow/traces/delete-traces\032\004\010\002\020\000\020\003*\rDelete Traces'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTracesV3']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteTracesV3']._serialized_options = b'\362\206\031@\n*\n\004POST\022\034/mlflow/traces/delete-traces\032\004\010\003\020\000\020\003*\020Delete Traces V3'
    _globals['_MLFLOWSERVICE'].methods_by_name['calculateTraceFilterCorrelation']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['calculateTraceFilterCorrelation']._serialized_options = b'\362\206\031a\n9\n\004POST\022+/mlflow/traces/calculate-filter-correlation\032\004\010\003\020\000\020\003*\"Calculate Trace Filter Correlation'
    _globals['_MLFLOWSERVICE'].methods_by_name['createLoggedModel']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createLoggedModel']._serialized_options = b'\362\206\031<\n#\n\004POST\022\025/mlflow/logged-models\032\004\010\002\020\000\020\003*\023Create Logged Model'
    _globals['_MLFLOWSERVICE'].methods_by_name['finalizeLoggedModel']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['finalizeLoggedModel']._serialized_options = b'\362\206\031J\n/\n\005PATCH\022 /mlflow/logged-models/{model_id}\032\004\010\002\020\000\020\003*\025Finalize Logged Model'
    _globals['_MLFLOWSERVICE'].methods_by_name['getLoggedModel']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getLoggedModel']._serialized_options = b'\362\206\031C\n-\n\003GET\022 /mlflow/logged-models/{model_id}\032\004\010\002\020\000\020\003*\020Get Logged Model'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteLoggedModel']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteLoggedModel']._serialized_options = b'\362\206\031K\n0\n\006DELETE\022 /mlflow/logged-models/{model_id}\032\004\010\002\020\000\020\003*\025Delete a Logged Model'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchLoggedModels']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchLoggedModels']._serialized_options = b'\362\206\031C\n*\n\004POST\022\034/mlflow/logged-models/search\032\004\010\002\020\000\020\003*\023Search LoggedModels'
    _globals['_MLFLOWSERVICE'].methods_by_name['setLoggedModelTags']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setLoggedModelTags']._serialized_options = b'\362\206\031N\n4\n\005PATCH\022%/mlflow/logged-models/{model_id}/tags\032\004\010\002\020\000\020\003*\024Set Logged Model Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteLoggedModelTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteLoggedModelTag']._serialized_options = b'\362\206\031\\\n?\n\006DELETE\022//mlflow/logged-models/{model_id}/tags/{tag_key}\032\004\010\002\020\000\020\003*\027Delete Logged Model Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['listLoggedModelArtifacts']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listLoggedModelArtifacts']._serialized_options = b'\362\206\031i\nC\n\003GET\0226/mlflow/logged-models/{model_id}/artifacts/directories\032\004\010\002\020\000\020\003* List Artifacts for Logged Models'
    _globals['_MLFLOWSERVICE'].methods_by_name['LogLoggedModelParams']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['LogLoggedModelParams']._serialized_options = b'\362\206\031R\n5\n\004POST\022\'/mlflow/logged-models/{model_id}/params\032\004\010\002\020\000\020\003*\027Log Logged Model Params'
    _globals['_MLFLOWSERVICE'].methods_by_name['GetAssessment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['GetAssessment']._serialized_options = b'\362\206\031V\nB\n\003GET\0225/mlflow/traces/{trace_id}/assessments/{assessment_id}\032\004\010\003\020\000\020\003*\016Get Assessment'
    _globals['_MLFLOWSERVICE'].methods_by_name['createAssessment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createAssessment']._serialized_options = b'\362\206\031\210\001\n>\n\004POST\0220/mlflow/traces/{assessment.trace_id}/assessments\032\004\010\003\020\000\020\003\030\350\007\030\356\007\030\014\030\001*:Create an assessment of a trace or a span within the trace'
    _globals['_MLFLOWSERVICE'].methods_by_name['updateAssessment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['updateAssessment']._serialized_options = b'\362\206\031{\nD\n\005PATCH\0225/mlflow/traces/{trace_id}/assessments/{assessment_id}\032\004\010\003\020\000\020\003\030\350\007\030\356\007\030\001*)Update an existing assessment on a trace.'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteAssessment']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteAssessment']._serialized_options = b'\362\206\031\\\nE\n\006DELETE\0225/mlflow/traces/{trace_id}/assessments/{assessment_id}\032\004\010\003\020\000\020\003*\021Delete Assessment'
    _globals['_MLFLOWSERVICE'].methods_by_name['createDataset']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createDataset']._serialized_options = b'\362\206\031N\n%\n\004POST\022\027/mlflow/datasets/create\032\004\010\003\020\000\020\003\030\350\007\030\356\007\030\014\030\001*\031Create Evaluation Dataset'
    _globals['_MLFLOWSERVICE'].methods_by_name['getDataset']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getDataset']._serialized_options = b'\362\206\031N\n*\n\003GET\022\035/mlflow/datasets/{dataset_id}\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\026Get Evaluation Dataset'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteDataset']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteDataset']._serialized_options = b'\362\206\031T\n-\n\006DELETE\022\035/mlflow/datasets/{dataset_id}\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\031Delete Evaluation Dataset'
    _globals['_MLFLOWSERVICE'].methods_by_name['searchEvaluationDatasets']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['searchEvaluationDatasets']._serialized_options = b'\362\206\031p\n%\n\004POST\022\027/mlflow/datasets/search\032\004\010\003\020\000\n$\n\003GET\022\027/mlflow/datasets/search\032\004\010\003\020\000\020\003\030\350\007\030\001*\032Search Evaluation Datasets'
    _globals['_MLFLOWSERVICE'].methods_by_name['setDatasetTags']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setDatasetTags']._serialized_options = b'\362\206\031Z\n1\n\005PATCH\022\"/mlflow/datasets/{dataset_id}/tags\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\033Set Evaluation Dataset Tags'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteDatasetTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteDatasetTag']._serialized_options = b'\362\206\031c\n8\n\006DELETE\022(/mlflow/datasets/{dataset_id}/tags/{key}\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\035Delete Evaluation Dataset Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['upsertDatasetRecords']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['upsertDatasetRecords']._serialized_options = b'\362\206\031b\n3\n\004POST\022%/mlflow/datasets/{dataset_id}/records\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*!Upsert Evaluation Dataset Records'
    _globals['_MLFLOWSERVICE'].methods_by_name['getDatasetExperimentIds']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getDatasetExperimentIds']._serialized_options = b'\362\206\031l\n9\n\003GET\022,/mlflow/datasets/{dataset_id}/experiment-ids\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*%Get Evaluation Dataset Experiment IDs'
    _globals['_MLFLOWSERVICE'].methods_by_name['registerScorer']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['registerScorer']._serialized_options = b'\362\206\031;\n&\n\004POST\022\030/mlflow/scorers/register\032\004\010\003\020\000\020\001*\017Register Scorer'
    _globals['_MLFLOWSERVICE'].methods_by_name['listScorers']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listScorers']._serialized_options = b'\362\206\0313\n!\n\003GET\022\024/mlflow/scorers/list\032\004\010\003\020\000\020\001*\014List Scorers'
    _globals['_MLFLOWSERVICE'].methods_by_name['listScorerVersions']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listScorerVersions']._serialized_options = b'\362\206\031?\n%\n\003GET\022\030/mlflow/scorers/versions\032\004\010\003\020\000\020\001*\024List Scorer Versions'
    _globals['_MLFLOWSERVICE'].methods_by_name['getScorer']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getScorer']._serialized_options = b'\362\206\0310\n \n\003GET\022\023/mlflow/scorers/get\032\004\010\003\020\000\020\001*\nGet Scorer'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteScorer']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteScorer']._serialized_options = b'\362\206\0319\n&\n\006DELETE\022\026/mlflow/scorers/delete\032\004\010\003\020\000\020\001*\rDelete Scorer'
    _globals['_MLFLOWSERVICE'].methods_by_name['getDatasetRecords']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getDatasetRecords']._serialized_options = b'\362\206\031^\n2\n\003GET\022%/mlflow/datasets/{dataset_id}/records\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\036Get Evaluation Dataset Records'
    _globals['_MLFLOWSERVICE'].methods_by_name['addDatasetToExperiments']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['addDatasetToExperiments']._serialized_options = b'\362\206\031c\n;\n\004POST\022-/mlflow/datasets/{dataset_id}/add-experiments\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\032Add Dataset to Experiments'
    _globals['_MLFLOWSERVICE'].methods_by_name['removeDatasetFromExperiments']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['removeDatasetFromExperiments']._serialized_options = b'\362\206\031k\n>\n\004POST\0220/mlflow/datasets/{dataset_id}/remove-experiments\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\037Remove Dataset from Experiments'
    _globals['_MLFLOWSERVICE'].methods_by_name['createGatewaySecret']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createGatewaySecret']._serialized_options = b'\362\206\031G\n,\n\004POST\022\036/mlflow/gateway/secrets/create\032\004\010\003\020\000\020\001*\025Create Gateway Secret'
    _globals['_MLFLOWSERVICE'].methods_by_name['getGatewaySecretInfo']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getGatewaySecretInfo']._serialized_options = b'\362\206\031E\n(\n\003GET\022\033/mlflow/gateway/secrets/get\032\004\010\003\020\000\020\001*\027Get Gateway Secret Info'
    _globals['_MLFLOWSERVICE'].methods_by_name['updateGatewaySecret']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['updateGatewaySecret']._serialized_options = b'\362\206\031G\n,\n\004POST\022\036/mlflow/gateway/secrets/update\032\004\010\003\020\000\020\001*\025Update Gateway Secret'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewaySecret']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewaySecret']._serialized_options = b'\362\206\031I\n.\n\006DELETE\022\036/mlflow/gateway/secrets/delete\032\004\010\003\020\000\020\001*\025Delete Gateway Secret'
    _globals['_MLFLOWSERVICE'].methods_by_name['listGatewaySecretInfos']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listGatewaySecretInfos']._serialized_options = b'\362\206\031C\n)\n\003GET\022\034/mlflow/gateway/secrets/list\032\004\010\003\020\000\020\001*\024List Gateway Secrets'
    _globals['_MLFLOWSERVICE'].methods_by_name['createGatewayEndpoint']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createGatewayEndpoint']._serialized_options = b'\362\206\031K\n.\n\004POST\022 /mlflow/gateway/endpoints/create\032\004\010\003\020\000\020\001*\027Create Gateway Endpoint'
    _globals['_MLFLOWSERVICE'].methods_by_name['getGatewayEndpoint']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getGatewayEndpoint']._serialized_options = b'\362\206\031D\n*\n\003GET\022\035/mlflow/gateway/endpoints/get\032\004\010\003\020\000\020\001*\024Get Gateway Endpoint'
    _globals['_MLFLOWSERVICE'].methods_by_name['updateGatewayEndpoint']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['updateGatewayEndpoint']._serialized_options = b'\362\206\031K\n.\n\004POST\022 /mlflow/gateway/endpoints/update\032\004\010\003\020\000\020\001*\027Update Gateway Endpoint'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewayEndpoint']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewayEndpoint']._serialized_options = b'\362\206\031M\n0\n\006DELETE\022 /mlflow/gateway/endpoints/delete\032\004\010\003\020\000\020\001*\027Delete Gateway Endpoint'
    _globals['_MLFLOWSERVICE'].methods_by_name['listGatewayEndpoints']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listGatewayEndpoints']._serialized_options = b'\362\206\031G\n+\n\003GET\022\036/mlflow/gateway/endpoints/list\032\004\010\003\020\000\020\001*\026List Gateway Endpoints'
    _globals['_MLFLOWSERVICE'].methods_by_name['createGatewayModelDefinition']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createGatewayModelDefinition']._serialized_options = b'\362\206\031[\n6\n\004POST\022(/mlflow/gateway/model-definitions/create\032\004\010\003\020\000\020\001*\037Create Gateway Model Definition'
    _globals['_MLFLOWSERVICE'].methods_by_name['getGatewayModelDefinition']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['getGatewayModelDefinition']._serialized_options = b'\362\206\031T\n2\n\003GET\022%/mlflow/gateway/model-definitions/get\032\004\010\003\020\000\020\001*\034Get Gateway Model Definition'
    _globals['_MLFLOWSERVICE'].methods_by_name['listGatewayModelDefinitions']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listGatewayModelDefinitions']._serialized_options = b'\362\206\031W\n3\n\003GET\022&/mlflow/gateway/model-definitions/list\032\004\010\003\020\000\020\001*\036List Gateway Model Definitions'
    _globals['_MLFLOWSERVICE'].methods_by_name['updateGatewayModelDefinition']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['updateGatewayModelDefinition']._serialized_options = b'\362\206\031[\n6\n\004POST\022(/mlflow/gateway/model-definitions/update\032\004\010\003\020\000\020\001*\037Update Gateway Model Definition'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewayModelDefinition']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewayModelDefinition']._serialized_options = b'\362\206\031]\n8\n\006DELETE\022(/mlflow/gateway/model-definitions/delete\032\004\010\003\020\000\020\001*\037Delete Gateway Model Definition'
    _globals['_MLFLOWSERVICE'].methods_by_name['attachModelToEndpoint']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['attachModelToEndpoint']._serialized_options = b'\362\206\031S\n5\n\004POST\022\'/mlflow/gateway/endpoints/models/attach\032\004\010\003\020\000\020\001*\030Attach Model to Endpoint'
    _globals['_MLFLOWSERVICE'].methods_by_name['detachModelFromEndpoint']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['detachModelFromEndpoint']._serialized_options = b'\362\206\031U\n5\n\004POST\022\'/mlflow/gateway/endpoints/models/detach\032\004\010\003\020\000\020\001*\032Detach Model from Endpoint'
    _globals['_MLFLOWSERVICE'].methods_by_name['createEndpointBinding']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['createEndpointBinding']._serialized_options = b'\362\206\031T\n7\n\004POST\022)/mlflow/gateway/endpoints/bindings/create\032\004\010\003\020\000\020\001*\027Create Endpoint Binding'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteEndpointBinding']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteEndpointBinding']._serialized_options = b'\362\206\031V\n9\n\006DELETE\022)/mlflow/gateway/endpoints/bindings/delete\032\004\010\003\020\000\020\001*\027Delete Endpoint Binding'
    _globals['_MLFLOWSERVICE'].methods_by_name['listEndpointBindings']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['listEndpointBindings']._serialized_options = b'\362\206\031P\n4\n\003GET\022\'/mlflow/gateway/endpoints/bindings/list\032\004\010\003\020\000\020\001*\026List Endpoint Bindings'
    _globals['_MLFLOWSERVICE'].methods_by_name['setGatewayEndpointTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['setGatewayEndpointTag']._serialized_options = b'\362\206\031M\n/\n\004POST\022!/mlflow/gateway/endpoints/set-tag\032\004\010\003\020\000\020\001*\030Gateway Set Endpoint Tag'
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewayEndpointTag']._loaded_options = None
    _globals['_MLFLOWSERVICE'].methods_by_name['deleteGatewayEndpointTag']._serialized_options = b'\362\206\031U\n4\n\006DELETE\022$/mlflow/gateway/endpoints/delete-tag\032\004\010\003\020\000\020\001*\033Gateway Delete Endpoint Tag'
    _globals['_VIEWTYPE']._serialized_start=22427
    _globals['_VIEWTYPE']._serialized_end=22481
    _globals['_SOURCETYPE']._serialized_start=22483
    _globals['_SOURCETYPE']._serialized_end=22556
    _globals['_RUNSTATUS']._serialized_start=22558
    _globals['_RUNSTATUS']._serialized_end=22635
    _globals['_TRACESTATUS']._serialized_start=22637
    _globals['_TRACESTATUS']._serialized_end=22716
    _globals['_LOGGEDMODELSTATUS']._serialized_start=22719
    _globals['_LOGGEDMODELSTATUS']._serialized_end=22857
    _globals['_METRIC']._serialized_start=243
    _globals['_METRIC']._serialized_end=419
    _globals['_PARAM']._serialized_start=421
    _globals['_PARAM']._serialized_end=456
    _globals['_RUN']._serialized_start=459
    _globals['_RUN']._serialized_end=598
    _globals['_RUNDATA']._serialized_start=600
    _globals['_RUNDATA']._serialized_end=703
    _globals['_RUNINPUTS']._serialized_start=705
    _globals['_RUNINPUTS']._serialized_end=804
    _globals['_RUNOUTPUTS']._serialized_start=806
    _globals['_RUNOUTPUTS']._serialized_end=862
    _globals['_RUNTAG']._serialized_start=864
    _globals['_RUNTAG']._serialized_end=900
    _globals['_EXPERIMENTTAG']._serialized_start=902
    _globals['_EXPERIMENTTAG']._serialized_end=945
    _globals['_RUNINFO']._serialized_start=948
    _globals['_RUNINFO']._serialized_end=1169
    _globals['_EXPERIMENT']._serialized_start=1172
    _globals['_EXPERIMENT']._serialized_end=1359
    _globals['_DATASETINPUT']._serialized_start=1361
    _globals['_DATASETINPUT']._serialized_end=1447
    _globals['_MODELINPUT']._serialized_start=1449
    _globals['_MODELINPUT']._serialized_end=1485
    _globals['_INPUTTAG']._serialized_start=1487
    _globals['_INPUTTAG']._serialized_end=1537
    _globals['_DATASET']._serialized_start=1540
    _globals['_DATASET']._serialized_end=1673
    _globals['_MODELOUTPUT']._serialized_start=1675
    _globals['_MODELOUTPUT']._serialized_end=1732
    _globals['_CREATEEXPERIMENT']._serialized_start=1735
    _globals['_CREATEEXPERIMENT']._serialized_end=1917
    _globals['_CREATEEXPERIMENT_RESPONSE']._serialized_start=1839
    _globals['_CREATEEXPERIMENT_RESPONSE']._serialized_end=1872
    _globals['_SEARCHEXPERIMENTS']._serialized_start=1920
    _globals['_SEARCHEXPERIMENTS']._serialized_end=2174
    _globals['_SEARCHEXPERIMENTS_RESPONSE']._serialized_start=2053
    _globals['_SEARCHEXPERIMENTS_RESPONSE']._serialized_end=2129
    _globals['_GETEXPERIMENT']._serialized_start=2177
    _globals['_GETEXPERIMENT']._serialized_end=2318
    _globals['_GETEXPERIMENT_RESPONSE']._serialized_start=2223
    _globals['_GETEXPERIMENT_RESPONSE']._serialized_end=2273
    _globals['_DELETEEXPERIMENT']._serialized_start=2320
    _globals['_DELETEEXPERIMENT']._serialized_end=2424
    _globals['_DELETEEXPERIMENT_RESPONSE']._serialized_start=1839
    _globals['_DELETEEXPERIMENT_RESPONSE']._serialized_end=1849
    _globals['_RESTOREEXPERIMENT']._serialized_start=2426
    _globals['_RESTOREEXPERIMENT']._serialized_end=2531
    _globals['_RESTOREEXPERIMENT_RESPONSE']._serialized_start=1839
    _globals['_RESTOREEXPERIMENT_RESPONSE']._serialized_end=1849
    _globals['_UPDATEEXPERIMENT']._serialized_start=2533
    _globals['_UPDATEEXPERIMENT']._serialized_end=2655
    _globals['_UPDATEEXPERIMENT_RESPONSE']._serialized_start=1839
    _globals['_UPDATEEXPERIMENT_RESPONSE']._serialized_end=1849
    _globals['_CREATERUN']._serialized_start=2658
    _globals['_CREATERUN']._serialized_end=2860
    _globals['_CREATERUN_RESPONSE']._serialized_start=2779
    _globals['_CREATERUN_RESPONSE']._serialized_end=2815
    _globals['_UPDATERUN']._serialized_start=2863
    _globals['_UPDATERUN']._serialized_end=3071
    _globals['_UPDATERUN_RESPONSE']._serialized_start=2981
    _globals['_UPDATERUN_RESPONSE']._serialized_end=3026
    _globals['_DELETERUN']._serialized_start=3073
    _globals['_DELETERUN']._serialized_end=3163
    _globals['_DELETERUN_RESPONSE']._serialized_start=1839
    _globals['_DELETERUN_RESPONSE']._serialized_end=1849
    _globals['_RESTORERUN']._serialized_start=3165
    _globals['_RESTORERUN']._serialized_end=3256
    _globals['_RESTORERUN_RESPONSE']._serialized_start=1839
    _globals['_RESTORERUN_RESPONSE']._serialized_end=1849
    _globals['_LOGMETRIC']._serialized_start=3259
    _globals['_LOGMETRIC']._serialized_end=3525
    _globals['_LOGMETRIC_RESPONSE']._serialized_start=1839
    _globals['_LOGMETRIC_RESPONSE']._serialized_end=1849
    _globals['_LOGPARAM']._serialized_start=3528
    _globals['_LOGPARAM']._serialized_end=3669
    _globals['_LOGPARAM_RESPONSE']._serialized_start=1839
    _globals['_LOGPARAM_RESPONSE']._serialized_end=1849
    _globals['_SETEXPERIMENTTAG']._serialized_start=3672
    _globals['_SETEXPERIMENTTAG']._serialized_end=3816
    _globals['_SETEXPERIMENTTAG_RESPONSE']._serialized_start=1839
    _globals['_SETEXPERIMENTTAG_RESPONSE']._serialized_end=1849
    _globals['_DELETEEXPERIMENTTAG']._serialized_start=3818
    _globals['_DELETEEXPERIMENTTAG']._serialized_end=3944
    _globals['_DELETEEXPERIMENTTAG_RESPONSE']._serialized_start=1839
    _globals['_DELETEEXPERIMENTTAG_RESPONSE']._serialized_end=1849
    _globals['_SETTAG']._serialized_start=3947
    _globals['_SETTAG']._serialized_end=4086
    _globals['_SETTAG_RESPONSE']._serialized_start=1839
    _globals['_SETTAG_RESPONSE']._serialized_end=1849
    _globals['_DELETETAG']._serialized_start=4088
    _globals['_DELETETAG']._serialized_end=4197
    _globals['_DELETETAG_RESPONSE']._serialized_start=1839
    _globals['_DELETETAG_RESPONSE']._serialized_end=1849
    _globals['_GETRUN']._serialized_start=4199
    _globals['_GETRUN']._serialized_end=4324
    _globals['_GETRUN_RESPONSE']._serialized_start=2779
    _globals['_GETRUN_RESPONSE']._serialized_end=2815
    _globals['_SEARCHRUNS']._serialized_start=4327
    _globals['_SEARCHRUNS']._serialized_end=4607
    _globals['_SEARCHRUNS_RESPONSE']._serialized_start=4500
    _globals['_SEARCHRUNS_RESPONSE']._serialized_end=4562
    _globals['_LISTARTIFACTS']._serialized_start=4610
    _globals['_LISTARTIFACTS']._serialized_end=4826
    _globals['_LISTARTIFACTS_RESPONSE']._serialized_start=4695
    _globals['_LISTARTIFACTS_RESPONSE']._serialized_end=4781
    _globals['_FILEINFO']._serialized_start=4828
    _globals['_FILEINFO']._serialized_end=4887
    _globals['_GETMETRICHISTORY']._serialized_start=4890
    _globals['_GETMETRICHISTORY']._serialized_end=5124
    _globals['_GETMETRICHISTORY_RESPONSE']._serialized_start=5011
    _globals['_GETMETRICHISTORY_RESPONSE']._serialized_end=5079
    _globals['_METRICWITHRUNID']._serialized_start=5126
    _globals['_METRICWITHRUNID']._serialized_end=5223
    _globals['_GETMETRICHISTORYBULKINTERVAL']._serialized_start=5226
    _globals['_GETMETRICHISTORYBULKINTERVAL']._serialized_end=5457
    _globals['_GETMETRICHISTORYBULKINTERVAL_RESPONSE']._serialized_start=5360
    _globals['_GETMETRICHISTORYBULKINTERVAL_RESPONSE']._serialized_end=5412
    _globals['_LOGBATCH']._serialized_start=5460
    _globals['_LOGBATCH']._serialized_end=5637
    _globals['_LOGBATCH_RESPONSE']._serialized_start=1839
    _globals['_LOGBATCH_RESPONSE']._serialized_end=1849
    _globals['_LOGMODEL']._serialized_start=5639
    _globals['_LOGMODEL']._serialized_end=5742
    _globals['_LOGMODEL_RESPONSE']._serialized_start=1839
    _globals['_LOGMODEL_RESPONSE']._serialized_end=1849
    _globals['_LOGINPUTS']._serialized_start=5745
    _globals['_LOGINPUTS']._serialized_end=5917
    _globals['_LOGINPUTS_RESPONSE']._serialized_start=1839
    _globals['_LOGINPUTS_RESPONSE']._serialized_end=1849
    _globals['_LOGOUTPUTS']._serialized_start=5920
    _globals['_LOGOUTPUTS']._serialized_end=6048
    _globals['_LOGOUTPUTS_RESPONSE']._serialized_start=1839
    _globals['_LOGOUTPUTS_RESPONSE']._serialized_end=1849
    _globals['_GETEXPERIMENTBYNAME']._serialized_start=6051
    _globals['_GETEXPERIMENTBYNAME']._serialized_end=6200
    _globals['_GETEXPERIMENTBYNAME_RESPONSE']._serialized_start=2223
    _globals['_GETEXPERIMENTBYNAME_RESPONSE']._serialized_end=2273
    _globals['_CREATEASSESSMENT']._serialized_start=6203
    _globals['_CREATEASSESSMENT']._serialized_end=6388
    _globals['_CREATEASSESSMENT_RESPONSE']._serialized_start=6281
    _globals['_CREATEASSESSMENT_RESPONSE']._serialized_end=6343
    _globals['_UPDATEASSESSMENT']._serialized_start=6391
    _globals['_UPDATEASSESSMENT']._serialized_end=6631
    _globals['_UPDATEASSESSMENT_RESPONSE']._serialized_start=6281
    _globals['_UPDATEASSESSMENT_RESPONSE']._serialized_end=6343
    _globals['_DELETEASSESSMENT']._serialized_start=6634
    _globals['_DELETEASSESSMENT']._serialized_end=6762
    _globals['_DELETEASSESSMENT_RESPONSE']._serialized_start=1839
    _globals['_DELETEASSESSMENT_RESPONSE']._serialized_end=1849
    _globals['_GETASSESSMENTREQUEST']._serialized_start=6765
    _globals['_GETASSESSMENTREQUEST']._serialized_end=6949
    _globals['_GETASSESSMENTREQUEST_RESPONSE']._serialized_start=6281
    _globals['_GETASSESSMENTREQUEST_RESPONSE']._serialized_end=6343
    _globals['_TRACEINFO']._serialized_start=6952
    _globals['_TRACEINFO']._serialized_end=7180
    _globals['_TRACEREQUESTMETADATA']._serialized_start=7182
    _globals['_TRACEREQUESTMETADATA']._serialized_end=7232
    _globals['_TRACETAG']._serialized_start=7234
    _globals['_TRACETAG']._serialized_end=7272
    _globals['_STARTTRACE']._serialized_start=7275
    _globals['_STARTTRACE']._serialized_end=7516
    _globals['_STARTTRACE_RESPONSE']._serialized_start=7422
    _globals['_STARTTRACE_RESPONSE']._serialized_end=7471
    _globals['_ENDTRACE']._serialized_start=7519
    _globals['_ENDTRACE']._serialized_end=7792
    _globals['_ENDTRACE_RESPONSE']._serialized_start=7422
    _globals['_ENDTRACE_RESPONSE']._serialized_end=7471
    _globals['_GETTRACEINFO']._serialized_start=7795
    _globals['_GETTRACEINFO']._serialized_end=7925
    _globals['_GETTRACEINFO_RESPONSE']._serialized_start=7422
    _globals['_GETTRACEINFO_RESPONSE']._serialized_end=7471
    _globals['_GETTRACEINFOV3']._serialized_start=7927
    _globals['_GETTRACEINFOV3']._serialized_end=8048
    _globals['_GETTRACEINFOV3_RESPONSE']._serialized_start=7963
    _globals['_GETTRACEINFOV3_RESPONSE']._serialized_end=8003
    _globals['_BATCHGETTRACES']._serialized_start=8050
    _globals['_BATCHGETTRACES']._serialized_end=8173
    _globals['_BATCHGETTRACES_RESPONSE']._serialized_start=8087
    _globals['_BATCHGETTRACES_RESPONSE']._serialized_end=8128
    _globals['_GETTRACE']._serialized_start=8176
    _globals['_GETTRACE']._serialized_end=8327
    _globals['_GETTRACE_RESPONSE']._serialized_start=7963
    _globals['_GETTRACE_RESPONSE']._serialized_end=8003
    _globals['_SEARCHTRACES']._serialized_start=8330
    _globals['_SEARCHTRACES']._serialized_end=8565
    _globals['_SEARCHTRACES_RESPONSE']._serialized_start=8450
    _globals['_SEARCHTRACES_RESPONSE']._serialized_end=8520
    _globals['_SEARCHUNIFIEDTRACES']._serialized_start=8568
    _globals['_SEARCHUNIFIEDTRACES']._serialized_end=8866
    _globals['_SEARCHUNIFIEDTRACES_RESPONSE']._serialized_start=8450
    _globals['_SEARCHUNIFIEDTRACES_RESPONSE']._serialized_end=8520
    _globals['_GETONLINETRACEDETAILS']._serialized_start=8869
    _globals['_GETONLINETRACEDETAILS']._serialized_end=9062
    _globals['_GETONLINETRACEDETAILS_RESPONSE']._serialized_start=9032
    _globals['_GETONLINETRACEDETAILS_RESPONSE']._serialized_end=9062
    _globals['_DELETETRACES']._serialized_start=9065
    _globals['_DELETETRACES']._serialized_end=9260
    _globals['_DELETETRACES_RESPONSE']._serialized_start=9181
    _globals['_DELETETRACES_RESPONSE']._serialized_end=9215
    _globals['_DELETETRACESV3']._serialized_start=9263
    _globals['_DELETETRACESV3']._serialized_end=9460
    _globals['_DELETETRACESV3_RESPONSE']._serialized_start=9181
    _globals['_DELETETRACESV3_RESPONSE']._serialized_end=9215
    _globals['_CALCULATETRACEFILTERCORRELATION']._serialized_start=9463
    _globals['_CALCULATETRACEFILTERCORRELATION']._serialized_end=9772
    _globals['_CALCULATETRACEFILTERCORRELATION_RESPONSE']._serialized_start=9592
    _globals['_CALCULATETRACEFILTERCORRELATION_RESPONSE']._serialized_end=9727
    _globals['_SETTRACETAG']._serialized_start=9774
    _globals['_SETTRACETAG']._serialized_end=9892
    _globals['_SETTRACETAG_RESPONSE']._serialized_start=1839
    _globals['_SETTRACETAG_RESPONSE']._serialized_end=1849
    _globals['_SETTRACETAGV3']._serialized_start=9895
    _globals['_SETTRACETAGV3']._serialized_end=10031
    _globals['_SETTRACETAGV3_RESPONSE']._serialized_start=1839
    _globals['_SETTRACETAGV3_RESPONSE']._serialized_end=1849
    _globals['_DELETETRACETAG']._serialized_start=10033
    _globals['_DELETETRACETAG']._serialized_end=10139
    _globals['_DELETETRACETAG_RESPONSE']._serialized_start=1839
    _globals['_DELETETRACETAG_RESPONSE']._serialized_end=1849
    _globals['_DELETETRACETAGV3']._serialized_start=10141
    _globals['_DELETETRACETAGV3']._serialized_end=10265
    _globals['_DELETETRACETAGV3_RESPONSE']._serialized_start=1839
    _globals['_DELETETRACETAGV3_RESPONSE']._serialized_end=1849
    _globals['_TRACE']._serialized_start=10267
    _globals['_TRACE']._serialized_end=10366
    _globals['_TRACELOCATION']._serialized_start=10369
    _globals['_TRACELOCATION']._serialized_end=10807
    _globals['_TRACELOCATION_MLFLOWEXPERIMENTLOCATION']._serialized_start=10591
    _globals['_TRACELOCATION_MLFLOWEXPERIMENTLOCATION']._serialized_end=10640
    _globals['_TRACELOCATION_INFERENCETABLELOCATION']._serialized_start=10642
    _globals['_TRACELOCATION_INFERENCETABLELOCATION']._serialized_end=10691
    _globals['_TRACELOCATION_TRACELOCATIONTYPE']._serialized_start=10693
    _globals['_TRACELOCATION_TRACELOCATIONTYPE']._serialized_end=10793
    _globals['_TRACEINFOV3']._serialized_start=10810
    _globals['_TRACEINFOV3']._serialized_end=11477
    _globals['_TRACEINFOV3_TRACEMETADATAENTRY']._serialized_start=11312
    _globals['_TRACEINFOV3_TRACEMETADATAENTRY']._serialized_end=11364
    _globals['_TRACEINFOV3_TAGSENTRY']._serialized_start=11366
    _globals['_TRACEINFOV3_TAGSENTRY']._serialized_end=11409
    _globals['_TRACEINFOV3_STATE']._serialized_start=11411
    _globals['_TRACEINFOV3_STATE']._serialized_end=11477
    _globals['_STARTTRACEV3']._serialized_start=11479
    _globals['_STARTTRACEV3']._serialized_end=11571
    _globals['_STARTTRACEV3_RESPONSE']._serialized_start=7963
    _globals['_STARTTRACEV3_RESPONSE']._serialized_end=8003
    _globals['_LINKTRACESTORUN']._serialized_start=11573
    _globals['_LINKTRACESTORUN']._serialized_end=11643
    _globals['_LINKTRACESTORUN_RESPONSE']._serialized_start=1839
    _globals['_LINKTRACESTORUN_RESPONSE']._serialized_end=1849
    _globals['_LINKPROMPTSTOTRACE']._serialized_start=11646
    _globals['_LINKPROMPTSTOTRACE']._serialized_end=11835
    _globals['_LINKPROMPTSTOTRACE_PROMPTVERSIONREF']._serialized_start=11762
    _globals['_LINKPROMPTSTOTRACE_PROMPTVERSIONREF']._serialized_end=11823
    _globals['_LINKPROMPTSTOTRACE_RESPONSE']._serialized_start=1839
    _globals['_LINKPROMPTSTOTRACE_RESPONSE']._serialized_end=1849
    _globals['_DATASETSUMMARY']._serialized_start=11837
    _globals['_DATASETSUMMARY']._serialized_end=11941
    _globals['_SEARCHDATASETS']._serialized_start=11944
    _globals['_SEARCHDATASETS']._serialized_end=12092
    _globals['_SEARCHDATASETS_RESPONSE']._serialized_start=11986
    _globals['_SEARCHDATASETS_RESPONSE']._serialized_end=12047
    _globals['_CREATELOGGEDMODEL']._serialized_start=12095
    _globals['_CREATELOGGEDMODEL']._serialized_end=12377
    _globals['_CREATELOGGEDMODEL_RESPONSE']._serialized_start=12286
    _globals['_CREATELOGGEDMODEL_RESPONSE']._serialized_end=12332
    _globals['_FINALIZELOGGEDMODEL']._serialized_start=12380
    _globals['_FINALIZELOGGEDMODEL']._serialized_end=12567
    _globals['_FINALIZELOGGEDMODEL_RESPONSE']._serialized_start=12286
    _globals['_FINALIZELOGGEDMODEL_RESPONSE']._serialized_end=12332
    _globals['_GETLOGGEDMODEL']._serialized_start=12570
    _globals['_GETLOGGEDMODEL']._serialized_end=12703
    _globals['_GETLOGGEDMODEL_RESPONSE']._serialized_start=12286
    _globals['_GETLOGGEDMODEL_RESPONSE']._serialized_end=12332
    _globals['_DELETELOGGEDMODEL']._serialized_start=12705
    _globals['_DELETELOGGEDMODEL']._serialized_end=12805
    _globals['_DELETELOGGEDMODEL_RESPONSE']._serialized_start=1839
    _globals['_DELETELOGGEDMODEL_RESPONSE']._serialized_end=1849
    _globals['_SEARCHLOGGEDMODELS']._serialized_start=12808
    _globals['_SEARCHLOGGEDMODELS']._serialized_end=13311
    _globals['_SEARCHLOGGEDMODELS_DATASET']._serialized_start=13023
    _globals['_SEARCHLOGGEDMODELS_DATASET']._serialized_end=13084
    _globals['_SEARCHLOGGEDMODELS_ORDERBY']._serialized_start=13086
    _globals['_SEARCHLOGGEDMODELS_ORDERBY']._serialized_end=13192
    _globals['_SEARCHLOGGEDMODELS_RESPONSE']._serialized_start=13194
    _globals['_SEARCHLOGGEDMODELS_RESPONSE']._serialized_end=13266
    _globals['_SETLOGGEDMODELTAGS']._serialized_start=13314
    _globals['_SETLOGGEDMODELTAGS']._serialized_end=13489
    _globals['_SETLOGGEDMODELTAGS_RESPONSE']._serialized_start=12286
    _globals['_SETLOGGEDMODELTAGS_RESPONSE']._serialized_end=12332
    _globals['_DELETELOGGEDMODELTAG']._serialized_start=13491
    _globals['_DELETELOGGEDMODELTAG']._serialized_end=13617
    _globals['_DELETELOGGEDMODELTAG_RESPONSE']._serialized_start=1839
    _globals['_DELETELOGGEDMODELTAG_RESPONSE']._serialized_end=1849
    _globals['_LISTLOGGEDMODELARTIFACTS']._serialized_start=13620
    _globals['_LISTLOGGEDMODELARTIFACTS']._serialized_end=13856
    _globals['_LISTLOGGEDMODELARTIFACTS_RESPONSE']._serialized_start=4695
    _globals['_LISTLOGGEDMODELARTIFACTS_RESPONSE']._serialized_end=4781
    _globals['_LOGLOGGEDMODELPARAMSREQUEST']._serialized_start=13859
    _globals['_LOGLOGGEDMODELPARAMSREQUEST']._serialized_end=14015
    _globals['_LOGLOGGEDMODELPARAMSREQUEST_RESPONSE']._serialized_start=1839
    _globals['_LOGLOGGEDMODELPARAMSREQUEST_RESPONSE']._serialized_end=1849
    _globals['_LOGGEDMODEL']._serialized_start=14017
    _globals['_LOGGEDMODEL']._serialized_end=14108
    _globals['_LOGGEDMODELINFO']._serialized_start=14111
    _globals['_LOGGEDMODELINFO']._serialized_end=14499
    _globals['_LOGGEDMODELTAG']._serialized_start=14501
    _globals['_LOGGEDMODELTAG']._serialized_end=14545
    _globals['_LOGGEDMODELREGISTRATIONINFO']._serialized_start=14547
    _globals['_LOGGEDMODELREGISTRATIONINFO']._serialized_end=14607
    _globals['_LOGGEDMODELDATA']._serialized_start=14609
    _globals['_LOGGEDMODELDATA']._serialized_end=14705
    _globals['_LOGGEDMODELPARAMETER']._serialized_start=14707
    _globals['_LOGGEDMODELPARAMETER']._serialized_end=14757
    _globals['_SEARCHTRACESV3']._serialized_start=14760
    _globals['_SEARCHTRACESV3']._serialized_end=15017
    _globals['_SEARCHTRACESV3_RESPONSE']._serialized_start=14900
    _globals['_SEARCHTRACESV3_RESPONSE']._serialized_end=14972
    _globals['_CREATEDATASET']._serialized_start=15020
    _globals['_CREATEDATASET']._serialized_end=15332
    _globals['_CREATEDATASET_RESPONSE']._serialized_start=15234
    _globals['_CREATEDATASET_RESPONSE']._serialized_end=15287
    _globals['_GETDATASET']._serialized_start=15335
    _globals['_GETDATASET']._serialized_end=15518
    _globals['_GETDATASET_RESPONSE']._serialized_start=15395
    _globals['_GETDATASET_RESPONSE']._serialized_end=15473
    _globals['_DELETEDATASET']._serialized_start=15520
    _globals['_DELETEDATASET']._serialized_end=15618
    _globals['_DELETEDATASET_RESPONSE']._serialized_start=1839
    _globals['_DELETEDATASET_RESPONSE']._serialized_end=1849
    _globals['_SEARCHEVALUATIONDATASETS']._serialized_start=15621
    _globals['_SEARCHEVALUATIONDATASETS']._serialized_end=15885
    _globals['_SEARCHEVALUATIONDATASETS_RESPONSE']._serialized_start=15761
    _globals['_SEARCHEVALUATIONDATASETS_RESPONSE']._serialized_end=15840
    _globals['_SETDATASETTAGS']._serialized_start=15888
    _globals['_SETDATASETTAGS']._serialized_end=16050
    _globals['_SETDATASETTAGS_RESPONSE']._serialized_start=15234
    _globals['_SETDATASETTAGS_RESPONSE']._serialized_end=15287
    _globals['_DELETEDATASETTAG']._serialized_start=16052
    _globals['_DELETEDATASETTAG']._serialized_end=16172
    _globals['_DELETEDATASETTAG_RESPONSE']._serialized_start=1839
    _globals['_DELETEDATASETTAG_RESPONSE']._serialized_end=1849
    _globals['_UPSERTDATASETRECORDS']._serialized_start=16175
    _globals['_UPSERTDATASETRECORDS']._serialized_end=16370
    _globals['_UPSERTDATASETRECORDS_RESPONSE']._serialized_start=16268
    _globals['_UPSERTDATASETRECORDS_RESPONSE']._serialized_end=16325
    _globals['_GETDATASETEXPERIMENTIDS']._serialized_start=16373
    _globals['_GETDATASETEXPERIMENTIDS']._serialized_end=16505
    _globals['_GETDATASETEXPERIMENTIDS_RESPONSE']._serialized_start=16426
    _globals['_GETDATASETEXPERIMENTIDS_RESPONSE']._serialized_end=16460
    _globals['_GETDATASETRECORDS']._serialized_start=16508
    _globals['_GETDATASETRECORDS']._serialized_end=16699
    _globals['_GETDATASETRECORDS_RESPONSE']._serialized_start=16602
    _globals['_GETDATASETRECORDS_RESPONSE']._serialized_end=16654
    _globals['_ADDDATASETTOEXPERIMENTS']._serialized_start=16702
    _globals['_ADDDATASETTOEXPERIMENTS']._serialized_end=16877
    _globals['_ADDDATASETTOEXPERIMENTS_RESPONSE']._serialized_start=15234
    _globals['_ADDDATASETTOEXPERIMENTS_RESPONSE']._serialized_end=15287
    _globals['_REMOVEDATASETFROMEXPERIMENTS']._serialized_start=16880
    _globals['_REMOVEDATASETFROMEXPERIMENTS']._serialized_end=17060
    _globals['_REMOVEDATASETFROMEXPERIMENTS_RESPONSE']._serialized_start=15234
    _globals['_REMOVEDATASETFROMEXPERIMENTS_RESPONSE']._serialized_end=15287
    _globals['_REGISTERSCORER']._serialized_start=17063
    _globals['_REGISTERSCORER']._serialized_end=17324
    _globals['_REGISTERSCORER_RESPONSE']._serialized_start=17146
    _globals['_REGISTERSCORER_RESPONSE']._serialized_end=17279
    _globals['_LISTSCORERS']._serialized_start=17326
    _globals['_LISTSCORERS']._serialized_end=17452
    _globals['_LISTSCORERS_RESPONSE']._serialized_start=17364
    _globals['_LISTSCORERS_RESPONSE']._serialized_end=17407
    _globals['_LISTSCORERVERSIONS']._serialized_start=17455
    _globals['_LISTSCORERVERSIONS']._serialized_end=17602
    _globals['_LISTSCORERVERSIONS_RESPONSE']._serialized_start=17364
    _globals['_LISTSCORERVERSIONS_RESPONSE']._serialized_end=17407
    _globals['_GETSCORER']._serialized_start=17605
    _globals['_GETSCORER']._serialized_end=17759
    _globals['_GETSCORER_RESPONSE']._serialized_start=17672
    _globals['_GETSCORER_RESPONSE']._serialized_end=17714
    _globals['_DELETESCORER']._serialized_start=17761
    _globals['_DELETESCORER']._serialized_end=17886
    _globals['_DELETESCORER_RESPONSE']._serialized_start=1839
    _globals['_DELETESCORER_RESPONSE']._serialized_end=1849
    _globals['_SCORER']._serialized_start=17889
    _globals['_SCORER']._serialized_end=18034
    _globals['_GATEWAYSECRETINFO']._serialized_start=18037
    _globals['_GATEWAYSECRETINFO']._serialized_end=18351
    _globals['_GATEWAYSECRETINFO_MASKEDVALUESENTRY']._serialized_start=18300
    _globals['_GATEWAYSECRETINFO_MASKEDVALUESENTRY']._serialized_end=18351
    _globals['_GATEWAYMODELDEFINITION']._serialized_start=18354
    _globals['_GATEWAYMODELDEFINITION']._serialized_end=18589
    _globals['_GATEWAYENDPOINTMODELMAPPING']._serialized_start=18592
    _globals['_GATEWAYENDPOINTMODELMAPPING']._serialized_end=18805
    _globals['_GATEWAYENDPOINT']._serialized_start=18808
    _globals['_GATEWAYENDPOINT']._serialized_end=19053
    _globals['_GATEWAYENDPOINTTAG']._serialized_start=19055
    _globals['_GATEWAYENDPOINTTAG']._serialized_end=19103
    _globals['_GATEWAYENDPOINTBINDING']._serialized_start=19106
    _globals['_GATEWAYENDPOINTBINDING']._serialized_end=19285
    _globals['_CREATEGATEWAYSECRET']._serialized_start=19288
    _globals['_CREATEGATEWAYSECRET']._serialized_end=19592
    _globals['_CREATEGATEWAYSECRET_SECRETVALUEENTRY']._serialized_start=19464
    _globals['_CREATEGATEWAYSECRET_SECRETVALUEENTRY']._serialized_end=19514
    _globals['_CREATEGATEWAYSECRET_RESPONSE']._serialized_start=19516
    _globals['_CREATEGATEWAYSECRET_RESPONSE']._serialized_end=19569
    _globals['_GETGATEWAYSECRETINFO']._serialized_start=19594
    _globals['_GETGATEWAYSECRETINFO']._serialized_end=19711
    _globals['_GETGATEWAYSECRETINFO_RESPONSE']._serialized_start=19516
    _globals['_GETGATEWAYSECRETINFO_RESPONSE']._serialized_end=19569
    _globals['_UPDATEGATEWAYSECRET']._serialized_start=19714
    _globals['_UPDATEGATEWAYSECRET']._serialized_end=19998
    _globals['_UPDATEGATEWAYSECRET_SECRETVALUEENTRY']._serialized_start=19464
    _globals['_UPDATEGATEWAYSECRET_SECRETVALUEENTRY']._serialized_end=19514
    _globals['_UPDATEGATEWAYSECRET_RESPONSE']._serialized_start=19516
    _globals['_UPDATEGATEWAYSECRET_RESPONSE']._serialized_end=19569
    _globals['_DELETEGATEWAYSECRET']._serialized_start=20000
    _globals['_DELETEGATEWAYSECRET']._serialized_end=20052
    _globals['_DELETEGATEWAYSECRET_RESPONSE']._serialized_start=1839
    _globals['_DELETEGATEWAYSECRET_RESPONSE']._serialized_end=1849
    _globals['_LISTGATEWAYSECRETINFOS']._serialized_start=20054
    _globals['_LISTGATEWAYSECRETINFOS']._serialized_end=20152
    _globals['_LISTGATEWAYSECRETINFOS_RESPONSE']._serialized_start=20098
    _globals['_LISTGATEWAYSECRETINFOS_RESPONSE']._serialized_end=20152
    _globals['_CREATEGATEWAYMODELDEFINITION']._serialized_start=20155
    _globals['_CREATEGATEWAYMODELDEFINITION']._serialized_end=20346
    _globals['_CREATEGATEWAYMODELDEFINITION_RESPONSE']._serialized_start=20278
    _globals['_CREATEGATEWAYMODELDEFINITION_RESPONSE']._serialized_end=20346
    _globals['_GETGATEWAYMODELDEFINITION']._serialized_start=20348
    _globals['_GETGATEWAYMODELDEFINITION']._serialized_end=20474
    _globals['_GETGATEWAYMODELDEFINITION_RESPONSE']._serialized_start=20278
    _globals['_GETGATEWAYMODELDEFINITION_RESPONSE']._serialized_end=20346
    _globals['_LISTGATEWAYMODELDEFINITIONS']._serialized_start=20477
    _globals['_LISTGATEWAYMODELDEFINITIONS']._serialized_end=20614
    _globals['_LISTGATEWAYMODELDEFINITIONS_RESPONSE']._serialized_start=20545
    _globals['_LISTGATEWAYMODELDEFINITIONS_RESPONSE']._serialized_end=20614
    _globals['_UPDATEGATEWAYMODELDEFINITION']._serialized_start=20617
    _globals['_UPDATEGATEWAYMODELDEFINITION']._serialized_end=20837
    _globals['_UPDATEGATEWAYMODELDEFINITION_RESPONSE']._serialized_start=20278
    _globals['_UPDATEGATEWAYMODELDEFINITION_RESPONSE']._serialized_end=20346
    _globals['_DELETEGATEWAYMODELDEFINITION']._serialized_start=20839
    _globals['_DELETEGATEWAYMODELDEFINITION']._serialized_end=20910
    _globals['_DELETEGATEWAYMODELDEFINITION_RESPONSE']._serialized_start=1839
    _globals['_DELETEGATEWAYMODELDEFINITION_RESPONSE']._serialized_end=1849
    _globals['_CREATEGATEWAYENDPOINT']._serialized_start=20913
    _globals['_CREATEGATEWAYENDPOINT']._serialized_end=21055
    _globals['_CREATEGATEWAYENDPOINT_RESPONSE']._serialized_start=21002
    _globals['_CREATEGATEWAYENDPOINT_RESPONSE']._serialized_end=21055
    _globals['_GETGATEWAYENDPOINT']._serialized_start=21057
    _globals['_GETGATEWAYENDPOINT']._serialized_end=21167
    _globals['_GETGATEWAYENDPOINT_RESPONSE']._serialized_start=21002
    _globals['_GETGATEWAYENDPOINT_RESPONSE']._serialized_end=21055
    _globals['_UPDATEGATEWAYENDPOINT']._serialized_start=21170
    _globals['_UPDATEGATEWAYENDPOINT']._serialized_end=21303
    _globals['_UPDATEGATEWAYENDPOINT_RESPONSE']._serialized_start=21002
    _globals['_UPDATEGATEWAYENDPOINT_RESPONSE']._serialized_end=21055
    _globals['_DELETEGATEWAYENDPOINT']._serialized_start=21305
    _globals['_DELETEGATEWAYENDPOINT']._serialized_end=21361
    _globals['_DELETEGATEWAYENDPOINT_RESPONSE']._serialized_start=1839
    _globals['_DELETEGATEWAYENDPOINT_RESPONSE']._serialized_end=1849
    _globals['_LISTGATEWAYENDPOINTS']._serialized_start=21363
    _globals['_LISTGATEWAYENDPOINTS']._serialized_end=21478
    _globals['_LISTGATEWAYENDPOINTS_RESPONSE']._serialized_start=21424
    _globals['_LISTGATEWAYENDPOINTS_RESPONSE']._serialized_end=21478
    _globals['_ATTACHMODELTOGATEWAYENDPOINT']._serialized_start=21481
    _globals['_ATTACHMODELTOGATEWAYENDPOINT']._serialized_end=21663
    _globals['_ATTACHMODELTOGATEWAYENDPOINT_RESPONSE']._serialized_start=21599
    _globals['_ATTACHMODELTOGATEWAYENDPOINT_RESPONSE']._serialized_end=21663
    _globals['_DETACHMODELFROMGATEWAYENDPOINT']._serialized_start=21665
    _globals['_DETACHMODELFROMGATEWAYENDPOINT']._serialized_end=21759
    _globals['_DETACHMODELFROMGATEWAYENDPOINT_RESPONSE']._serialized_start=1839
    _globals['_DETACHMODELFROMGATEWAYENDPOINT_RESPONSE']._serialized_end=1849
    _globals['_CREATEGATEWAYENDPOINTBINDING']._serialized_start=21762
    _globals['_CREATEGATEWAYENDPOINTBINDING']._serialized_end=21938
    _globals['_CREATEGATEWAYENDPOINTBINDING_RESPONSE']._serialized_start=21879
    _globals['_CREATEGATEWAYENDPOINTBINDING_RESPONSE']._serialized_end=21938
    _globals['_DELETEGATEWAYENDPOINTBINDING']._serialized_start=21940
    _globals['_DELETEGATEWAYENDPOINTBINDING']._serialized_end=22047
    _globals['_DELETEGATEWAYENDPOINTBINDING_RESPONSE']._serialized_start=1839
    _globals['_DELETEGATEWAYENDPOINTBINDING_RESPONSE']._serialized_end=1849
    _globals['_LISTGATEWAYENDPOINTBINDINGS']._serialized_start=22050
    _globals['_LISTGATEWAYENDPOINTBINDINGS']._serialized_end=22206
    _globals['_LISTGATEWAYENDPOINTBINDINGS_RESPONSE']._serialized_start=22146
    _globals['_LISTGATEWAYENDPOINTBINDINGS_RESPONSE']._serialized_end=22206
    _globals['_SETGATEWAYENDPOINTTAG']._serialized_start=22208
    _globals['_SETGATEWAYENDPOINTTAG']._serialized_end=22292
    _globals['_SETGATEWAYENDPOINTTAG_RESPONSE']._serialized_start=1839
    _globals['_SETGATEWAYENDPOINTTAG_RESPONSE']._serialized_end=1849
    _globals['_DELETEGATEWAYENDPOINTTAG']._serialized_start=22294
    _globals['_DELETEGATEWAYENDPOINTTAG']._serialized_end=22366
    _globals['_DELETEGATEWAYENDPOINTTAG_RESPONSE']._serialized_start=1839
    _globals['_DELETEGATEWAYENDPOINTTAG_RESPONSE']._serialized_end=1849
    _globals['_GETSECRETSCONFIG']._serialized_start=22368
    _globals['_GETSECRETSCONFIG']._serialized_end=22425
    _globals['_GETSECRETSCONFIG_RESPONSE']._serialized_start=22388
    _globals['_GETSECRETSCONFIG_RESPONSE']._serialized_end=22425
    _globals['_MLFLOWSERVICE']._serialized_start=22860
    _globals['_MLFLOWSERVICE']._serialized_end=38738
  _builder.BuildServices(DESCRIPTOR, 'service_pb2', _globals)
  # @@protoc_insertion_point(module_scope)

else:
  # -*- coding: utf-8 -*-
  # Generated by the protocol buffer compiler.  DO NOT EDIT!
  # source: service.proto
  """Generated protocol buffer code."""
  from google.protobuf.internal import enum_type_wrapper
  from google.protobuf import descriptor as _descriptor
  from google.protobuf import descriptor_pool as _descriptor_pool
  from google.protobuf import message as _message
  from google.protobuf import reflection as _reflection
  from google.protobuf import symbol_database as _symbol_database
  from google.protobuf import service as _service
  from google.protobuf import service_reflection
  # @@protoc_insertion_point(imports)

  _sym_db = _symbol_database.Default()


  from . import assessments_pb2 as assessments__pb2
  from . import databricks_pb2 as databricks__pb2
  from . import datasets_pb2 as datasets__pb2
  from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
  from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
  from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
  from opentelemetry.proto.trace.v1 import trace_pb2 as opentelemetry_dot_proto_dot_trace_dot_v1_dot_trace__pb2
  from .scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


  DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rservice.proto\x12\x06mlflow\x1a\x11\x61ssessments.proto\x1a\x10\x64\x61tabricks.proto\x1a\x0e\x64\x61tasets.proto\x1a\x1egoogle/protobuf/duration.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a(opentelemetry/proto/trace/v1/trace.proto\x1a\x15scalapb/scalapb.proto\"\xb0\x01\n\x06Metric\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0f\n\x04step\x18\x04 \x01(\x03:\x01\x30\x12\x1a\n\x0c\x64\x61taset_name\x18\x05 \x01(\tB\x04\xf0\x86\x19\x03\x12\x1c\n\x0e\x64\x61taset_digest\x18\x06 \x01(\tB\x04\xf0\x86\x19\x03\x12\x16\n\x08model_id\x18\x07 \x01(\tB\x04\xf0\x86\x19\x03\x12\x14\n\x06run_id\x18\x08 \x01(\tB\x04\xf0\x86\x19\x03\"#\n\x05Param\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x8b\x01\n\x03Run\x12\x1d\n\x04info\x18\x01 \x01(\x0b\x32\x0f.mlflow.RunInfo\x12\x1d\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x0f.mlflow.RunData\x12!\n\x06inputs\x18\x03 \x01(\x0b\x32\x11.mlflow.RunInputs\x12#\n\x07outputs\x18\x04 \x01(\x0b\x32\x12.mlflow.RunOutputs\"g\n\x07RunData\x12\x1f\n\x07metrics\x18\x01 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x1d\n\x06params\x18\x02 \x03(\x0b\x32\r.mlflow.Param\x12\x1c\n\x04tags\x18\x03 \x03(\x0b\x32\x0e.mlflow.RunTag\"c\n\tRunInputs\x12,\n\x0e\x64\x61taset_inputs\x18\x01 \x03(\x0b\x32\x14.mlflow.DatasetInput\x12(\n\x0cmodel_inputs\x18\x02 \x03(\x0b\x32\x12.mlflow.ModelInput\"8\n\nRunOutputs\x12*\n\rmodel_outputs\x18\x01 \x03(\x0b\x32\x13.mlflow.ModelOutput\"$\n\x06RunTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\rExperimentTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xdd\x01\n\x07RunInfo\x12\x0e\n\x06run_id\x18\x0f \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x10\n\x08run_name\x18\x03 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x0f\n\x07user_id\x18\x06 \x01(\t\x12!\n\x06status\x18\x07 \x01(\x0e\x32\x11.mlflow.RunStatus\x12\x12\n\nstart_time\x18\x08 \x01(\x03\x12\x10\n\x08\x65nd_time\x18\t \x01(\x03\x12\x14\n\x0c\x61rtifact_uri\x18\r \x01(\t\x12\x17\n\x0flifecycle_stage\x18\x0e \x01(\t\"\xbb\x01\n\nExperiment\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x19\n\x11\x61rtifact_location\x18\x03 \x01(\t\x12\x17\n\x0flifecycle_stage\x18\x04 \x01(\t\x12\x18\n\x10last_update_time\x18\x05 \x01(\x03\x12\x15\n\rcreation_time\x18\x06 \x01(\x03\x12#\n\x04tags\x18\x07 \x03(\x0b\x32\x15.mlflow.ExperimentTag\"V\n\x0c\x44\x61tasetInput\x12\x1e\n\x04tags\x18\x01 \x03(\x0b\x32\x10.mlflow.InputTag\x12&\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x0f.mlflow.DatasetB\x04\xf8\x86\x19\x01\"$\n\nModelInput\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\"2\n\x08InputTag\x12\x11\n\x03key\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\"\x85\x01\n\x07\x44\x61taset\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06\x64igest\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bsource_type\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06source\x18\x04 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x0f\n\x07profile\x18\x06 \x01(\t\"9\n\x0bModelOutput\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04step\x18\x02 \x01(\x03\x42\x04\xf8\x86\x19\x01\"\xb6\x01\n\x10\x43reateExperiment\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x11\x61rtifact_location\x18\x02 \x01(\t\x12#\n\x04tags\x18\x03 \x03(\x0b\x32\x15.mlflow.ExperimentTag\x1a!\n\x08Response\x12\x15\n\rexperiment_id\x18\x01 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xfe\x01\n\x11SearchExperiments\x12\x13\n\x0bmax_results\x18\x01 \x01(\x03\x12\x12\n\npage_token\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\t\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12#\n\tview_type\x18\x05 \x01(\x0e\x32\x10.mlflow.ViewType\x1aL\n\x08Response\x12\'\n\x0b\x65xperiments\x18\x01 \x03(\x0b\x32\x12.mlflow.Experiment\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8d\x01\n\rGetExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\nexperiment\x18\x01 \x01(\x0b\x32\x12.mlflow.Experiment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"h\n\x10\x44\x65leteExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"i\n\x11RestoreExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"z\n\x10UpdateExperiment\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x10\n\x08new_name\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xca\x01\n\tCreateRun\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x10\n\x08run_name\x18\x03 \x01(\t\x12\x12\n\nstart_time\x18\x07 \x01(\x03\x12\x1c\n\x04tags\x18\t \x03(\x0b\x32\x0e.mlflow.RunTag\x1a$\n\x08Response\x12\x18\n\x03run\x18\x01 \x01(\x0b\x32\x0b.mlflow.Run:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd0\x01\n\tUpdateRun\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12!\n\x06status\x18\x02 \x01(\x0e\x32\x11.mlflow.RunStatus\x12\x10\n\x08\x65nd_time\x18\x03 \x01(\x03\x12\x10\n\x08run_name\x18\x05 \x01(\t\x1a-\n\x08Response\x12!\n\x08run_info\x18\x01 \x01(\x0b\x32\x0f.mlflow.RunInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"Z\n\tDeleteRun\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"[\n\nRestoreRun\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8a\x02\n\tLogMetric\x12\x0e\n\x06run_id\x18\x06 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\x01\x42\x04\xf8\x86\x19\x01\x12\x17\n\ttimestamp\x18\x04 \x01(\x03\x42\x04\xf8\x86\x19\x01\x12\x0f\n\x04step\x18\x05 \x01(\x03:\x01\x30\x12\x16\n\x08model_id\x18\x07 \x01(\tB\x04\xf0\x86\x19\x03\x12\x1a\n\x0c\x64\x61taset_name\x18\x08 \x01(\tB\x04\xf0\x86\x19\x03\x12\x1c\n\x0e\x64\x61taset_digest\x18\t \x01(\tB\x04\xf0\x86\x19\x03\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8d\x01\n\x08LogParam\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x90\x01\n\x10SetExperimentTag\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"~\n\x13\x44\x65leteExperimentTag\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x8b\x01\n\x06SetTag\x12\x0e\n\x06run_id\x18\x04 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x13\n\x05value\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"m\n\tDeleteTag\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"}\n\x06GetRun\x12\x0e\n\x06run_id\x18\x02 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x1a$\n\x08Response\x12\x18\n\x03run\x18\x01 \x01(\x0b\x32\x0b.mlflow.Run:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x98\x02\n\nSearchRuns\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\x34\n\rrun_view_type\x18\x03 \x01(\x0e\x32\x10.mlflow.ViewType:\x0b\x41\x43TIVE_ONLY\x12\x19\n\x0bmax_results\x18\x05 \x01(\x05:\x04\x31\x30\x30\x30\x12\x10\n\x08order_by\x18\x06 \x03(\t\x12\x12\n\npage_token\x18\x07 \x01(\t\x1a>\n\x08Response\x12\x19\n\x04runs\x18\x01 \x03(\x0b\x32\x0b.mlflow.Run\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xd8\x01\n\rListArtifacts\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x12\n\npage_token\x18\x04 \x01(\t\x1aV\n\x08Response\x12\x10\n\x08root_uri\x18\x01 \x01(\t\x12\x1f\n\x05\x66iles\x18\x02 \x03(\x0b\x32\x10.mlflow.FileInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\";\n\x08\x46ileInfo\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06is_dir\x18\x02 \x01(\x08\x12\x11\n\tfile_size\x18\x03 \x01(\x03\"\xea\x01\n\x10GetMetricHistory\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x10\n\x08run_uuid\x18\x01 \x01(\t\x12\x18\n\nmetric_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\npage_token\x18\x04 \x01(\t\x12\x13\n\x0bmax_results\x18\x05 \x01(\x05\x1a\x44\n\x08Response\x12\x1f\n\x07metrics\x18\x01 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"a\n\x0fMetricWithRunId\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x11\n\ttimestamp\x18\x03 \x01(\x03\x12\x0f\n\x04step\x18\x04 \x01(\x03:\x01\x30\x12\x0e\n\x06run_id\x18\x05 \x01(\t\"\xe7\x01\n\x1cGetMetricHistoryBulkInterval\x12\x0f\n\x07run_ids\x18\x01 \x03(\t\x12\x18\n\nmetric_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\nstart_step\x18\x03 \x01(\x05\x12\x10\n\x08\x65nd_step\x18\x04 \x01(\x05\x12\x13\n\x0bmax_results\x18\x05 \x01(\x05\x1a\x34\n\x08Response\x12(\n\x07metrics\x18\x01 \x03(\x0b\x32\x17.mlflow.MetricWithRunId:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb1\x01\n\x08LogBatch\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x1f\n\x07metrics\x18\x02 \x03(\x0b\x32\x0e.mlflow.Metric\x12\x1d\n\x06params\x18\x03 \x03(\x0b\x32\r.mlflow.Param\x12\x1c\n\x04tags\x18\x04 \x03(\x0b\x32\x0e.mlflow.RunTag\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"g\n\x08LogModel\x12\x0e\n\x06run_id\x18\x01 \x01(\t\x12\x12\n\nmodel_json\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xac\x01\n\tLogInputs\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12&\n\x08\x64\x61tasets\x18\x02 \x03(\x0b\x32\x14.mlflow.DatasetInput\x12(\n\x06models\x18\x03 \x03(\x0b\x32\x12.mlflow.ModelInputB\x04\xf0\x86\x19\x03\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x80\x01\n\nLogOutputs\x12\x14\n\x06run_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12#\n\x06models\x18\x02 \x03(\x0b\x32\x13.mlflow.ModelOutput\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x95\x01\n\x13GetExperimentByName\x12\x1d\n\x0f\x65xperiment_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x32\n\x08Response\x12&\n\nexperiment\x18\x01 \x01(\x0b\x32\x12.mlflow.Experiment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb9\x01\n\x10\x43reateAssessment\x12\x38\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.AssessmentB\x04\xf8\x86\x19\x01\x1a>\n\x08Response\x12\x32\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf0\x01\n\x10UpdateAssessment\x12\x38\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.AssessmentB\x04\xf8\x86\x19\x01\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xf8\x86\x19\x01\x1a>\n\x08Response\x12\x32\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x80\x01\n\x10\x44\x65leteAssessment\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb8\x01\n\x14GetAssessmentRequest\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1b\n\rassessment_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a>\n\x08Response\x12\x32\n\nassessment\x18\x01 \x01(\x0b\x32\x1e.mlflow.assessments.Assessment:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xe4\x01\n\tTraceInfo\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x03 \x01(\x03\x12\x19\n\x11\x65xecution_time_ms\x18\x04 \x01(\x03\x12#\n\x06status\x18\x05 \x01(\x0e\x32\x13.mlflow.TraceStatus\x12\x36\n\x10request_metadata\x18\x06 \x03(\x0b\x32\x1c.mlflow.TraceRequestMetadata\x12\x1e\n\x04tags\x18\x07 \x03(\x0b\x32\x10.mlflow.TraceTag\"2\n\x14TraceRequestMetadata\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"&\n\x08TraceTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xf1\x01\n\nStartTrace\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x02 \x01(\x03\x12\x36\n\x10request_metadata\x18\x03 \x03(\x0b\x32\x1c.mlflow.TraceRequestMetadata\x12\x1e\n\x04tags\x18\x04 \x03(\x0b\x32\x10.mlflow.TraceTag\x1a\x31\n\x08Response\x12%\n\ntrace_info\x18\x01 \x01(\x0b\x32\x11.mlflow.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x91\x02\n\x08\x45ndTrace\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x14\n\x0ctimestamp_ms\x18\x02 \x01(\x03\x12#\n\x06status\x18\x03 \x01(\x0e\x32\x13.mlflow.TraceStatus\x12\x36\n\x10request_metadata\x18\x04 \x03(\x0b\x32\x1c.mlflow.TraceRequestMetadata\x12\x1e\n\x04tags\x18\x05 \x03(\x0b\x32\x10.mlflow.TraceTag\x1a\x31\n\x08Response\x12%\n\ntrace_info\x18\x01 \x01(\x0b\x32\x11.mlflow.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x82\x01\n\x0cGetTraceInfo\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x1a\x31\n\x08Response\x12%\n\ntrace_info\x18\x01 \x01(\x0b\x32\x11.mlflow.TraceInfo:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"y\n\x0eGetTraceInfoV3\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x1a(\n\x08Response\x12\x1c\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"{\n\x0e\x42\x61tchGetTraces\x12\x11\n\ttrace_ids\x18\x01 \x03(\t\x1a)\n\x08Response\x12\x1d\n\x06traces\x18\x01 \x03(\x0b\x32\r.mlflow.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x97\x01\n\x08GetTrace\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1c\n\rallow_partial\x18\x02 \x01(\x08:\x05\x66\x61lse\x1a(\n\x08Response\x12\x1c\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.Trace:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xeb\x01\n\x0cSearchTraces\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x18\n\x0bmax_results\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x12\n\npage_token\x18\x05 \x01(\t\x1a\x46\n\x08Response\x12!\n\x06traces\x18\x01 \x03(\x0b\x32\x11.mlflow.TraceInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xaa\x02\n\x13SearchUnifiedTraces\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1e\n\x10sql_warehouse_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x03 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\x18\n\x0bmax_results\x18\x05 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x06 \x03(\t\x12\x12\n\npage_token\x18\x07 \x01(\t\x1a\x46\n\x08Response\x12!\n\x06traces\x18\x01 \x03(\x0b\x32\x11.mlflow.TraceInfo\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc1\x01\n\x15GetOnlineTraceDetails\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1e\n\x10sql_warehouse_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12$\n\x16source_inference_table\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12*\n\x1csource_databricks_request_id\x18\x04 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x1e\n\x08Response\x12\x12\n\ntrace_data\x18\x01 \x01(\t\"\xc3\x01\n\x0c\x44\x65leteTraces\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1c\n\x14max_timestamp_millis\x18\x02 \x01(\x03\x12\x12\n\nmax_traces\x18\x03 \x01(\x05\x12\x13\n\x0brequest_ids\x18\x04 \x03(\t\x1a\"\n\x08Response\x12\x16\n\x0etraces_deleted\x18\x01 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc5\x01\n\x0e\x44\x65leteTracesV3\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1c\n\x14max_timestamp_millis\x18\x02 \x01(\x03\x12\x12\n\nmax_traces\x18\x03 \x01(\x05\x12\x13\n\x0brequest_ids\x18\x04 \x03(\t\x1a\"\n\x08Response\x12\x16\n\x0etraces_deleted\x18\x01 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb5\x02\n\x1f\x43\x61lculateTraceFilterCorrelation\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x16\n\x0e\x66ilter_string1\x18\x02 \x01(\t\x12\x16\n\x0e\x66ilter_string2\x18\x03 \x01(\t\x12\x13\n\x0b\x62\x61se_filter\x18\x04 \x01(\t\x1a\x87\x01\n\x08Response\x12\x0c\n\x04npmi\x18\x01 \x01(\x01\x12\x15\n\rnpmi_smoothed\x18\x02 \x01(\x01\x12\x15\n\rfilter1_count\x18\x03 \x01(\x05\x12\x15\n\rfilter2_count\x18\x04 \x01(\x05\x12\x13\n\x0bjoint_count\x18\x05 \x01(\x05\x12\x13\n\x0btotal_count\x18\x06 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"v\n\x0bSetTraceTag\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x88\x01\n\rSetTraceTagV3\x12\x10\n\x08trace_id\x18\x04 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]J\x04\x08\x01\x10\x02R\nrequest_id\"j\n\x0e\x44\x65leteTraceTag\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"|\n\x10\x44\x65leteTraceTagV3\x12\x10\n\x08trace_id\x18\x03 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]J\x04\x08\x01\x10\x02R\nrequest_id\"c\n\x05Trace\x12\'\n\ntrace_info\x18\x01 \x01(\x0b\x32\x13.mlflow.TraceInfoV3\x12\x31\n\x05spans\x18\x02 \x03(\x0b\x32\".opentelemetry.proto.trace.v1.Span\"\xb6\x03\n\rTraceLocation\x12\x35\n\x04type\x18\x01 \x01(\x0e\x32\'.mlflow.TraceLocation.TraceLocationType\x12K\n\x11mlflow_experiment\x18\x02 \x01(\x0b\x32..mlflow.TraceLocation.MlflowExperimentLocationH\x00\x12G\n\x0finference_table\x18\x03 \x01(\x0b\x32,.mlflow.TraceLocation.InferenceTableLocationH\x00\x1a\x31\n\x18MlflowExperimentLocation\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x1a\x31\n\x16InferenceTableLocation\x12\x17\n\x0f\x66ull_table_name\x18\x01 \x01(\t\"d\n\x11TraceLocationType\x12#\n\x1fTRACE_LOCATION_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11MLFLOW_EXPERIMENT\x10\x01\x12\x13\n\x0fINFERENCE_TABLE\x10\x02\x42\x0c\n\nidentifier\"\x9b\x05\n\x0bTraceInfoV3\x12\x10\n\x08trace_id\x18\x01 \x01(\t\x12\x19\n\x11\x63lient_request_id\x18\x02 \x01(\t\x12-\n\x0etrace_location\x18\x03 \x01(\x0b\x32\x15.mlflow.TraceLocation\x12\x0f\n\x07request\x18\x04 \x01(\t\x12\x10\n\x08response\x18\x05 \x01(\t\x12\x17\n\x0frequest_preview\x18\x0c \x01(\t\x12\x18\n\x10response_preview\x18\r \x01(\t\x12\x30\n\x0crequest_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x35\n\x12\x65xecution_duration\x18\x07 \x01(\x0b\x32\x19.google.protobuf.Duration\x12(\n\x05state\x18\x08 \x01(\x0e\x32\x19.mlflow.TraceInfoV3.State\x12>\n\x0etrace_metadata\x18\t \x03(\x0b\x32&.mlflow.TraceInfoV3.TraceMetadataEntry\x12\x33\n\x0b\x61ssessments\x18\n \x03(\x0b\x32\x1e.mlflow.assessments.Assessment\x12+\n\x04tags\x18\x0b \x03(\x0b\x32\x1d.mlflow.TraceInfoV3.TagsEntry\x1a\x34\n\x12TraceMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"B\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0f\n\x0bIN_PROGRESS\x10\x03\"\\\n\x0cStartTraceV3\x12\"\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.TraceB\x04\xf8\x86\x19\x01\x1a(\n\x08Response\x12\x1c\n\x05trace\x18\x01 \x01(\x0b\x32\r.mlflow.Trace\"F\n\x0fLinkTracesToRun\x12\x11\n\ttrace_ids\x18\x01 \x03(\t\x12\x14\n\x06run_id\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response\"\xbd\x01\n\x12LinkPromptsToTrace\x12\x16\n\x08trace_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x44\n\x0fprompt_versions\x18\x02 \x03(\x0b\x32+.mlflow.LinkPromptsToTrace.PromptVersionRef\x1a=\n\x10PromptVersionRef\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x15\n\x07version\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response\"h\n\x0e\x44\x61tasetSummary\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04name\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x14\n\x06\x64igest\x18\x03 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0f\n\x07\x63ontext\x18\x04 \x01(\t\"\x94\x01\n\x0eSearchDatasets\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x1a=\n\x08Response\x12\x31\n\x11\x64\x61taset_summaries\x18\x01 \x03(\x0b\x32\x16.mlflow.DatasetSummary:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9a\x02\n\x11\x43reateLoggedModel\x12\x1b\n\rexperiment_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nmodel_type\x18\x03 \x01(\t\x12\x15\n\rsource_run_id\x18\x04 \x01(\t\x12,\n\x06params\x18\x05 \x03(\x0b\x32\x1c.mlflow.LoggedModelParameter\x12$\n\x04tags\x18\x06 \x03(\x0b\x32\x16.mlflow.LoggedModelTag\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xbb\x01\n\x13\x46inalizeLoggedModel\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12/\n\x06status\x18\x02 \x01(\x0e\x32\x19.mlflow.LoggedModelStatusB\x04\xf8\x86\x19\x01\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x85\x01\n\x0eGetLoggedModel\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"d\n\x11\x44\x65leteLoggedModel\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xf7\x03\n\x12SearchLoggedModels\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x34\n\x08\x64\x61tasets\x18\x06 \x03(\x0b\x32\".mlflow.SearchLoggedModels.Dataset\x12\x17\n\x0bmax_results\x18\x03 \x01(\x05:\x02\x35\x30\x12\x34\n\x08order_by\x18\x04 \x03(\x0b\x32\".mlflow.SearchLoggedModels.OrderBy\x12\x12\n\npage_token\x18\x05 \x01(\t\x1a=\n\x07\x44\x61taset\x12\x1a\n\x0c\x64\x61taset_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x64\x61taset_digest\x18\x02 \x01(\t\x1aj\n\x07OrderBy\x12\x18\n\nfield_name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x17\n\tascending\x18\x02 \x01(\x08:\x04true\x12\x14\n\x0c\x64\x61taset_name\x18\x03 \x01(\t\x12\x16\n\x0e\x64\x61taset_digest\x18\x04 \x01(\t\x1aH\n\x08Response\x12#\n\x06models\x18\x01 \x03(\x0b\x32\x13.mlflow.LoggedModel\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xaf\x01\n\x12SetLoggedModelTags\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12$\n\x04tags\x18\x02 \x03(\x0b\x32\x16.mlflow.LoggedModelTag\x1a.\n\x08Response\x12\"\n\x05model\x18\x01 \x01(\x0b\x32\x13.mlflow.LoggedModel:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"~\n\x14\x44\x65leteLoggedModelTag\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x15\n\x07tag_key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xec\x01\n\x18ListLoggedModelArtifacts\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x1f\n\x17\x61rtifact_directory_path\x18\x02 \x01(\t\x12\x12\n\npage_token\x18\x03 \x01(\t\x1aV\n\x08Response\x12\x10\n\x08root_uri\x18\x01 \x01(\t\x12\x1f\n\x05\x66iles\x18\x02 \x03(\x0b\x32\x10.mlflow.FileInfo\x12\x17\n\x0fnext_page_token\x18\x03 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9c\x01\n\x1bLogLoggedModelParamsRequest\x12\x16\n\x08model_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12,\n\x06params\x18\x02 \x03(\x0b\x32\x1c.mlflow.LoggedModelParameter\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"[\n\x0bLoggedModel\x12%\n\x04info\x18\x01 \x01(\x0b\x32\x17.mlflow.LoggedModelInfo\x12%\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x17.mlflow.LoggedModelData\"\x84\x03\n\x0fLoggedModelInfo\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1d\n\x15\x63reation_timestamp_ms\x18\x04 \x01(\x03\x12!\n\x19last_updated_timestamp_ms\x18\x05 \x01(\x03\x12\x14\n\x0c\x61rtifact_uri\x18\x06 \x01(\t\x12)\n\x06status\x18\x07 \x01(\x0e\x32\x19.mlflow.LoggedModelStatus\x12\x12\n\ncreator_id\x18\x08 \x01(\x03\x12\x12\n\nmodel_type\x18\t \x01(\t\x12\x15\n\rsource_run_id\x18\n \x01(\t\x12\x16\n\x0estatus_message\x18\x0b \x01(\t\x12$\n\x04tags\x18\x0c \x03(\x0b\x32\x16.mlflow.LoggedModelTag\x12:\n\rregistrations\x18\r \x03(\x0b\x32#.mlflow.LoggedModelRegistrationInfo\",\n\x0eLoggedModelTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"<\n\x1bLoggedModelRegistrationInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"`\n\x0fLoggedModelData\x12,\n\x06params\x18\x01 \x03(\x0b\x32\x1c.mlflow.LoggedModelParameter\x12\x1f\n\x07metrics\x18\x02 \x03(\x0b\x32\x0e.mlflow.Metric\"2\n\x14LoggedModelParameter\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x81\x02\n\x0eSearchTracesV3\x12(\n\tlocations\x18\x01 \x03(\x0b\x32\x15.mlflow.TraceLocation\x12\x0e\n\x06\x66ilter\x18\x02 \x01(\t\x12\x18\n\x0bmax_results\x18\x03 \x01(\x05:\x03\x31\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x12\n\npage_token\x18\x05 \x01(\t\x1aH\n\x08Response\x12#\n\x06traces\x18\x01 \x03(\x0b\x32\x13.mlflow.TraceInfoV3\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb8\x02\n\rCreateDataset\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x02 \x03(\t\x12\x44\n\x0bsource_type\x18\x03 \x01(\x0e\x32/.mlflow.datasets.DatasetRecordSource.SourceType\x12\x0e\n\x06source\x18\x04 \x01(\t\x12\x0e\n\x06schema\x18\x05 \x01(\t\x12\x0f\n\x07profile\x18\x06 \x01(\t\x12\x12\n\ncreated_by\x18\x07 \x01(\t\x12\x0c\n\x04tags\x18\x08 \x01(\t\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb7\x01\n\nGetDataset\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\npage_token\x18\x02 \x01(\t\x1aN\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"b\n\rDeleteDataset\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x88\x02\n\x18SearchEvaluationDatasets\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x15\n\rfilter_string\x18\x02 \x01(\t\x12\x19\n\x0bmax_results\x18\x03 \x01(\x05:\x04\x31\x30\x30\x30\x12\x10\n\x08order_by\x18\x04 \x03(\t\x12\x12\n\npage_token\x18\x05 \x01(\t\x1aO\n\x08Response\x12*\n\x08\x64\x61tasets\x18\x01 \x03(\x0b\x32\x18.mlflow.datasets.Dataset\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xa2\x01\n\x0eSetDatasetTags\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\x04tags\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"x\n\x10\x44\x65leteDatasetTag\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x11\n\x03key\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xc3\x01\n\x14UpsertDatasetRecords\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x15\n\x07records\x18\x02 \x01(\tB\x04\xf8\x86\x19\x01\x12\x12\n\nupdated_by\x18\x03 \x01(\t\x1a\x39\n\x08Response\x12\x16\n\x0einserted_count\x18\x01 \x01(\x05\x12\x15\n\rupdated_count\x18\x02 \x01(\x05:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x84\x01\n\x17GetDatasetExperimentIds\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x1a\"\n\x08Response\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xbf\x01\n\x11GetDatasetRecords\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x19\n\x0bmax_results\x18\x02 \x01(\x05:\x04\x31\x30\x30\x30\x12\x12\n\npage_token\x18\x03 \x01(\t\x1a\x34\n\x08Response\x12\x0f\n\x07records\x18\x01 \x01(\t\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xaf\x01\n\x17\x41\x64\x64\x44\x61tasetToExperiments\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x02 \x03(\t\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\xb4\x01\n\x1cRemoveDatasetFromExperiments\x12\x18\n\ndataset_id\x18\x01 \x01(\tB\x04\xf8\x86\x19\x01\x12\x16\n\x0e\x65xperiment_ids\x18\x02 \x03(\t\x1a\x35\n\x08Response\x12)\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x18.mlflow.datasets.Dataset:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x85\x02\n\x0eRegisterScorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x19\n\x11serialized_scorer\x18\x03 \x01(\t\x1a\x85\x01\n\x08Response\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x11\n\tscorer_id\x18\x02 \x01(\t\x12\x15\n\rexperiment_id\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x19\n\x11serialized_scorer\x18\x05 \x01(\t\x12\x15\n\rcreation_time\x18\x06 \x01(\x03:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"~\n\x0bListScorers\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x1a+\n\x08Response\x12\x1f\n\x07scorers\x18\x01 \x03(\x0b\x32\x0e.mlflow.Scorer:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x93\x01\n\x12ListScorerVersions\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x1a+\n\x08Response\x12\x1f\n\x07scorers\x18\x01 \x03(\x0b\x32\x0e.mlflow.Scorer:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x9a\x01\n\tGetScorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\x05\x1a*\n\x08Response\x12\x1e\n\x06scorer\x18\x01 \x01(\x0b\x32\x0e.mlflow.Scorer:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"}\n\x0c\x44\x65leteScorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\x05\x1a\n\n\x08Response:+\xe2?(\n&com.databricks.rpc.RPC[$this.Response]\"\x91\x01\n\x06Scorer\x12\x15\n\rexperiment_id\x18\x01 \x01(\x05\x12\x13\n\x0bscorer_name\x18\x02 \x01(\t\x12\x16\n\x0escorer_version\x18\x03 \x01(\x05\x12\x19\n\x11serialized_scorer\x18\x04 \x01(\t\x12\x15\n\rcreation_time\x18\x05 \x01(\x03\x12\x11\n\tscorer_id\x18\x06 \x01(\t\"\xba\x02\n\x11GatewaySecretInfo\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x12\x13\n\x0bsecret_name\x18\x02 \x01(\t\x12\x42\n\rmasked_values\x18\x03 \x03(\x0b\x32+.mlflow.GatewaySecretInfo.MaskedValuesEntry\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x05 \x01(\x03\x12\x10\n\x08provider\x18\x06 \x01(\t\x12\x12\n\ncreated_by\x18\x07 \x01(\t\x12\x17\n\x0flast_updated_by\x18\x08 \x01(\t\x12\x18\n\x10\x61uth_config_json\x18\t \x01(\t\x1a\x33\n\x11MaskedValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xeb\x01\n\x16GatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tsecret_id\x18\x03 \x01(\t\x12\x13\n\x0bsecret_name\x18\x04 \x01(\t\x12\x10\n\x08provider\x18\x05 \x01(\t\x12\x12\n\nmodel_name\x18\x06 \x01(\t\x12\x12\n\ncreated_at\x18\x07 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x08 \x01(\x03\x12\x12\n\ncreated_by\x18\t \x01(\t\x12\x17\n\x0flast_updated_by\x18\n \x01(\t\"\xd5\x01\n\x1bGatewayEndpointModelMapping\x12\x12\n\nmapping_id\x18\x01 \x01(\t\x12\x13\n\x0b\x65ndpoint_id\x18\x02 \x01(\t\x12\x1b\n\x13model_definition_id\x18\x03 \x01(\t\x12\x38\n\x10model_definition\x18\x04 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\x12\x0e\n\x06weight\x18\x05 \x01(\x02\x12\x12\n\ncreated_at\x18\x06 \x01(\x03\x12\x12\n\ncreated_by\x18\x07 \x01(\t\"\xf5\x01\n\x0fGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x03 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x04 \x01(\x03\x12;\n\x0emodel_mappings\x18\x05 \x03(\x0b\x32#.mlflow.GatewayEndpointModelMapping\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x17\n\x0flast_updated_by\x18\x07 \x01(\t\x12(\n\x04tags\x18\x08 \x03(\x0b\x32\x1a.mlflow.GatewayEndpointTag\"0\n\x12GatewayEndpointTag\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xb3\x01\n\x16GatewayEndpointBinding\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\x17\n\x0flast_updated_at\x18\x05 \x01(\x03\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x17\n\x0flast_updated_by\x18\x07 \x01(\t\"\xb0\x02\n\x13\x43reateGatewaySecret\x12\x13\n\x0bsecret_name\x18\x01 \x01(\t\x12\x42\n\x0csecret_value\x18\x02 \x03(\x0b\x32,.mlflow.CreateGatewaySecret.SecretValueEntry\x12\x10\n\x08provider\x18\x03 \x01(\t\x12\x18\n\x10\x61uth_config_json\x18\x05 \x01(\t\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x1a\x32\n\x10SecretValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x08Response\x12)\n\x06secret\x18\x01 \x01(\x0b\x32\x19.mlflow.GatewaySecretInfoJ\x04\x08\x04\x10\x05R\x0f\x63redential_name\"u\n\x14GetGatewaySecretInfo\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x12\x13\n\x0bsecret_name\x18\x02 \x01(\t\x1a\x35\n\x08Response\x12)\n\x06secret\x18\x01 \x01(\x0b\x32\x19.mlflow.GatewaySecretInfo\"\x9c\x02\n\x13UpdateGatewaySecret\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x12\x42\n\x0csecret_value\x18\x02 \x03(\x0b\x32,.mlflow.UpdateGatewaySecret.SecretValueEntry\x12\x18\n\x10\x61uth_config_json\x18\x04 \x01(\t\x12\x12\n\nupdated_by\x18\x05 \x01(\t\x1a\x32\n\x10SecretValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x08Response\x12)\n\x06secret\x18\x01 \x01(\x0b\x32\x19.mlflow.GatewaySecretInfoJ\x04\x08\x03\x10\x04R\x0f\x63redential_name\"4\n\x13\x44\x65leteGatewaySecret\x12\x11\n\tsecret_id\x18\x01 \x01(\t\x1a\n\n\x08Response\"b\n\x16ListGatewaySecretInfos\x12\x10\n\x08provider\x18\x01 \x01(\t\x1a\x36\n\x08Response\x12*\n\x07secrets\x18\x01 \x03(\x0b\x32\x19.mlflow.GatewaySecretInfo\"\xbf\x01\n\x1c\x43reateGatewayModelDefinition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tsecret_id\x18\x02 \x01(\t\x12\x10\n\x08provider\x18\x03 \x01(\t\x12\x12\n\nmodel_name\x18\x04 \x01(\t\x12\x12\n\ncreated_by\x18\x05 \x01(\t\x1a\x44\n\x08Response\x12\x38\n\x10model_definition\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"~\n\x19GetGatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x1a\x44\n\x08Response\x12\x38\n\x10model_definition\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"\x89\x01\n\x1bListGatewayModelDefinitions\x12\x10\n\x08provider\x18\x01 \x01(\t\x12\x11\n\tsecret_id\x18\x02 \x01(\t\x1a\x45\n\x08Response\x12\x39\n\x11model_definitions\x18\x01 \x03(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"\xdc\x01\n\x1cUpdateGatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x11\n\tsecret_id\x18\x03 \x01(\t\x12\x12\n\nmodel_name\x18\x04 \x01(\t\x12\x12\n\nupdated_by\x18\x05 \x01(\t\x12\x10\n\x08provider\x18\x06 \x01(\t\x1a\x44\n\x08Response\x12\x38\n\x10model_definition\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayModelDefinition\"G\n\x1c\x44\x65leteGatewayModelDefinition\x12\x1b\n\x13model_definition_id\x18\x01 \x01(\t\x1a\n\n\x08Response\"\x8e\x01\n\x15\x43reateGatewayEndpoint\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1c\n\x14model_definition_ids\x18\x02 \x03(\t\x12\x12\n\ncreated_by\x18\x03 \x01(\t\x1a\x35\n\x08Response\x12)\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x17.mlflow.GatewayEndpoint\"n\n\x12GetGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x1a\x35\n\x08Response\x12)\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x17.mlflow.GatewayEndpoint\"\x85\x01\n\x15UpdateGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\nupdated_by\x18\x03 \x01(\t\x1a\x35\n\x08Response\x12)\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\x17.mlflow.GatewayEndpoint\"8\n\x15\x44\x65leteGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x1a\n\n\x08Response\"s\n\x14ListGatewayEndpoints\x12\x10\n\x08provider\x18\x01 \x01(\t\x12\x11\n\tsecret_id\x18\x02 \x01(\t\x1a\x36\n\x08Response\x12*\n\tendpoints\x18\x01 \x03(\x0b\x32\x17.mlflow.GatewayEndpoint\"\xb6\x01\n\x1c\x41ttachModelToGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x1b\n\x13model_definition_id\x18\x02 \x01(\t\x12\x0e\n\x06weight\x18\x03 \x01(\x02\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x1a@\n\x08Response\x12\x34\n\x07mapping\x18\x01 \x01(\x0b\x32#.mlflow.GatewayEndpointModelMapping\"^\n\x1e\x44\x65tachModelFromGatewayEndpoint\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x1b\n\x13model_definition_id\x18\x02 \x01(\t\x1a\n\n\x08Response\"\xb0\x01\n\x1c\x43reateGatewayEndpointBinding\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x12\x12\n\ncreated_by\x18\x04 \x01(\t\x1a;\n\x08Response\x12/\n\x07\x62inding\x18\x01 \x01(\x0b\x32\x1e.mlflow.GatewayEndpointBinding\"k\n\x1c\x44\x65leteGatewayEndpointBinding\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x1a\n\n\x08Response\"\x9c\x01\n\x1bListGatewayEndpointBindings\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x15\n\rresource_type\x18\x02 \x01(\t\x12\x13\n\x0bresource_id\x18\x03 \x01(\t\x1a<\n\x08Response\x12\x30\n\x08\x62indings\x18\x01 \x03(\x0b\x32\x1e.mlflow.GatewayEndpointBinding\"T\n\x15SetGatewayEndpointTag\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x1a\n\n\x08Response\"H\n\x18\x44\x65leteGatewayEndpointTag\x12\x13\n\x0b\x65ndpoint_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x1a\n\n\x08Response\"9\n\x10GetSecretsConfig\x1a%\n\x08Response\x12\x19\n\x11secrets_available\x18\x01 \x01(\x08*6\n\x08ViewType\x12\x0f\n\x0b\x41\x43TIVE_ONLY\x10\x01\x12\x10\n\x0c\x44\x45LETED_ONLY\x10\x02\x12\x07\n\x03\x41LL\x10\x03*I\n\nSourceType\x12\x0c\n\x08NOTEBOOK\x10\x01\x12\x07\n\x03JOB\x10\x02\x12\x0b\n\x07PROJECT\x10\x03\x12\t\n\x05LOCAL\x10\x04\x12\x0c\n\x07UNKNOWN\x10\xe8\x07*M\n\tRunStatus\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tSCHEDULED\x10\x02\x12\x0c\n\x08\x46INISHED\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\n\n\x06KILLED\x10\x05*O\n\x0bTraceStatus\x12\x1c\n\x18TRACE_STATUS_UNSPECIFIED\x10\x00\x12\x06\n\x02OK\x10\x01\x12\t\n\x05\x45RROR\x10\x02\x12\x0f\n\x0bIN_PROGRESS\x10\x03*\x8a\x01\n\x11LoggedModelStatus\x12#\n\x1fLOGGED_MODEL_STATUS_UNSPECIFIED\x10\x00\x12\x18\n\x14LOGGED_MODEL_PENDING\x10\x01\x12\x16\n\x12LOGGED_MODEL_READY\x10\x02\x12\x1e\n\x1aLOGGED_MODEL_UPLOAD_FAILED\x10\x03\x32\x86|\n\rMlflowService\x12\xa6\x01\n\x13getExperimentByName\x12\x1b.mlflow.GetExperimentByName\x1a$.mlflow.GetExperimentByName.Response\"L\xf2\x86\x19H\n,\n\x03GET\x12\x1f/mlflow/experiments/get-by-name\x1a\x04\x08\x02\x10\x00\x10\x01*\x16Get Experiment By Name\x12\x94\x01\n\x10\x63reateExperiment\x12\x18.mlflow.CreateExperiment\x1a!.mlflow.CreateExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/create\x1a\x04\x08\x02\x10\x00\x10\x01*\x11\x43reate Experiment\x12\xc1\x01\n\x11searchExperiments\x12\x19.mlflow.SearchExperiments\x1a\".mlflow.SearchExperiments.Response\"m\xf2\x86\x19i\n(\n\x04POST\x12\x1a/mlflow/experiments/search\x1a\x04\x08\x02\x10\x00\n\'\n\x03GET\x12\x1a/mlflow/experiments/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Search Experiments\x12\x88\x01\n\rgetExperiment\x12\x15.mlflow.GetExperiment\x1a\x1e.mlflow.GetExperiment.Response\"@\xf2\x86\x19\x38\n$\n\x03GET\x12\x17/mlflow/experiments/get\x1a\x04\x08\x02\x10\x00\x10\x01*\x0eGet Experiment\xba\x8c\x19\x00\x12\x94\x01\n\x10\x64\x65leteExperiment\x12\x18.mlflow.DeleteExperiment\x1a!.mlflow.DeleteExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\x11\x44\x65lete Experiment\x12\x99\x01\n\x11restoreExperiment\x12\x19.mlflow.RestoreExperiment\x1a\".mlflow.RestoreExperiment.Response\"E\xf2\x86\x19\x41\n)\n\x04POST\x12\x1b/mlflow/experiments/restore\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Restore Experiment\x12\x94\x01\n\x10updateExperiment\x12\x18.mlflow.UpdateExperiment\x1a!.mlflow.UpdateExperiment.Response\"C\xf2\x86\x19?\n(\n\x04POST\x12\x1a/mlflow/experiments/update\x1a\x04\x08\x02\x10\x00\x10\x01*\x11Update Experiment\x12q\n\tcreateRun\x12\x11.mlflow.CreateRun\x1a\x1a.mlflow.CreateRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/create\x1a\x04\x08\x02\x10\x00\x10\x01*\nCreate Run\x12q\n\tupdateRun\x12\x11.mlflow.UpdateRun\x1a\x1a.mlflow.UpdateRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/update\x1a\x04\x08\x02\x10\x00\x10\x01*\nUpdate Run\x12q\n\tdeleteRun\x12\x11.mlflow.DeleteRun\x1a\x1a.mlflow.DeleteRun.Response\"5\xf2\x86\x19\x31\n!\n\x04POST\x12\x13/mlflow/runs/delete\x1a\x04\x08\x02\x10\x00\x10\x01*\nDelete Run\x12v\n\nrestoreRun\x12\x12.mlflow.RestoreRun\x1a\x1b.mlflow.RestoreRun.Response\"7\xf2\x86\x19\x33\n\"\n\x04POST\x12\x14/mlflow/runs/restore\x1a\x04\x08\x02\x10\x00\x10\x01*\x0bRestore Run\x12u\n\tlogMetric\x12\x11.mlflow.LogMetric\x1a\x1a.mlflow.LogMetric.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/log-metric\x1a\x04\x08\x02\x10\x00\x10\x01*\nLog Metric\x12t\n\x08logParam\x12\x10.mlflow.LogParam\x1a\x19.mlflow.LogParam.Response\";\xf2\x86\x19\x37\n(\n\x04POST\x12\x1a/mlflow/runs/log-parameter\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Param\x12\xa1\x01\n\x10setExperimentTag\x12\x18.mlflow.SetExperimentTag\x1a!.mlflow.SetExperimentTag.Response\"P\xf2\x86\x19L\n4\n\x04POST\x12&/mlflow/experiments/set-experiment-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Set Experiment Tag\x12\xb0\x01\n\x13\x64\x65leteExperimentTag\x12\x1b.mlflow.DeleteExperimentTag\x1a$.mlflow.DeleteExperimentTag.Response\"V\xf2\x86\x19R\n7\n\x04POST\x12)/mlflow/experiments/delete-experiment-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x15\x44\x65lete Experiment Tag\x12\x66\n\x06setTag\x12\x0e.mlflow.SetTag\x1a\x17.mlflow.SetTag.Response\"3\xf2\x86\x19/\n\"\n\x04POST\x12\x14/mlflow/runs/set-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\x07Set Tag\x12\x88\x01\n\x0bsetTraceTag\x12\x13.mlflow.SetTraceTag\x1a\x1c.mlflow.SetTraceTag.Response\"F\xf2\x86\x19\x42\n/\n\x05PATCH\x12 /mlflow/traces/{request_id}/tags\x1a\x04\x08\x02\x10\x00\x10\x03*\rSet Trace Tag\x12\x8f\x01\n\rsetTraceTagV3\x12\x15.mlflow.SetTraceTagV3\x1a\x1e.mlflow.SetTraceTagV3.Response\"G\xf2\x86\x19\x43\n-\n\x05PATCH\x12\x1e/mlflow/traces/{trace_id}/tags\x1a\x04\x08\x03\x10\x00\x10\x03*\x10Set Trace Tag V3\x12\x95\x01\n\x0e\x64\x65leteTraceTag\x12\x16.mlflow.DeleteTraceTag\x1a\x1f.mlflow.DeleteTraceTag.Response\"J\xf2\x86\x19\x46\n0\n\x06\x44\x45LETE\x12 /mlflow/traces/{request_id}/tags\x1a\x04\x08\x02\x10\x00\x10\x03*\x10\x44\x65lete Trace Tag\x12\x9c\x01\n\x10\x64\x65leteTraceTagV3\x12\x18.mlflow.DeleteTraceTagV3\x1a!.mlflow.DeleteTraceTagV3.Response\"K\xf2\x86\x19G\n.\n\x06\x44\x45LETE\x12\x1e/mlflow/traces/{trace_id}/tags\x1a\x04\x08\x03\x10\x00\x10\x03*\x13\x44\x65lete Trace Tag V3\x12u\n\tdeleteTag\x12\x11.mlflow.DeleteTag\x1a\x1a.mlflow.DeleteTag.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/delete-tag\x1a\x04\x08\x02\x10\x00\x10\x01*\nDelete Tag\x12\x65\n\x06getRun\x12\x0e.mlflow.GetRun\x1a\x17.mlflow.GetRun.Response\"2\xf2\x86\x19*\n\x1d\n\x03GET\x12\x10/mlflow/runs/get\x1a\x04\x08\x02\x10\x00\x10\x01*\x07Get Run\xba\x8c\x19\x00\x12y\n\nsearchRuns\x12\x12.mlflow.SearchRuns\x1a\x1b.mlflow.SearchRuns.Response\":\xf2\x86\x19\x32\n!\n\x04POST\x12\x13/mlflow/runs/search\x1a\x04\x08\x02\x10\x00\x10\x01*\x0bSearch Runs\xba\x8c\x19\x00\x12\x87\x01\n\rlistArtifacts\x12\x15.mlflow.ListArtifacts\x1a\x1e.mlflow.ListArtifacts.Response\"?\xf2\x86\x19\x37\n#\n\x03GET\x12\x16/mlflow/artifacts/list\x1a\x04\x08\x02\x10\x00\x10\x01*\x0eList Artifacts\xba\x8c\x19\x00\x12\x95\x01\n\x10getMetricHistory\x12\x18.mlflow.GetMetricHistory\x1a!.mlflow.GetMetricHistory.Response\"D\xf2\x86\x19@\n(\n\x03GET\x12\x1b/mlflow/metrics/get-history\x1a\x04\x08\x02\x10\x00\x10\x01*\x12Get Metric History\x12\xb7\x01\n\x1cgetMetricHistoryBulkInterval\x12$.mlflow.GetMetricHistoryBulkInterval\x1a-.mlflow.GetMetricHistoryBulkInterval.Response\"B\xf2\x86\x19:\n6\n\x03GET\x12)/mlflow/metrics/get-history-bulk-interval\x1a\x04\x08\x02\x10\x0b\x10\x03\xba\x8c\x19\x00\x12p\n\x08logBatch\x12\x10.mlflow.LogBatch\x1a\x19.mlflow.LogBatch.Response\"7\xf2\x86\x19\x33\n$\n\x04POST\x12\x16/mlflow/runs/log-batch\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Batch\x12p\n\x08logModel\x12\x10.mlflow.LogModel\x1a\x19.mlflow.LogModel.Response\"7\xf2\x86\x19\x33\n$\n\x04POST\x12\x16/mlflow/runs/log-model\x1a\x04\x08\x02\x10\x00\x10\x01*\tLog Model\x12u\n\tlogInputs\x12\x11.mlflow.LogInputs\x1a\x1a.mlflow.LogInputs.Response\"9\xf2\x86\x19\x35\n%\n\x04POST\x12\x17/mlflow/runs/log-inputs\x1a\x04\x08\x02\x10\x00\x10\x01*\nLog Inputs\x12v\n\nlogOutputs\x12\x12.mlflow.LogOutputs\x1a\x1b.mlflow.LogOutputs.Response\"7\xf2\x86\x19\x33\n\"\n\x04POST\x12\x14/mlflow/runs/outputs\x1a\x04\x08\x02\x10\x00\x10\x03*\x0bLog Outputs\x12\x87\x01\n\x0esearchDatasets\x12\x16.mlflow.SearchDatasets\x1a\x1f.mlflow.SearchDatasets.Response\"<\xf2\x86\x19\x34\n0\n\x04POST\x12\"mlflow/experiments/search-datasets\x1a\x04\x08\x02\x10\x00\x10\x03\xba\x8c\x19\x00\x12p\n\nstartTrace\x12\x12.mlflow.StartTrace\x1a\x1b.mlflow.StartTrace.Response\"1\xf2\x86\x19-\n\x1c\n\x04POST\x12\x0e/mlflow/traces\x1a\x04\x08\x02\x10\x00\x10\x03*\x0bStart Trace\x12v\n\x08\x65ndTrace\x12\x10.mlflow.EndTrace\x1a\x19.mlflow.EndTrace.Response\"=\xf2\x86\x19\x39\n*\n\x05PATCH\x12\x1b/mlflow/traces/{request_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\tEnd Trace\x12\x89\x01\n\x0cgetTraceInfo\x12\x14.mlflow.GetTraceInfo\x1a\x1d.mlflow.GetTraceInfo.Response\"D\xf2\x86\x19@\n-\n\x03GET\x12 /mlflow/traces/{request_id}/info\x1a\x04\x08\x02\x10\x00\x10\x03*\rGet TraceInfo\x12\x8b\x01\n\x0egetTraceInfoV3\x12\x16.mlflow.GetTraceInfoV3\x1a\x1f.mlflow.GetTraceInfoV3.Response\"@\xf2\x86\x19<\n&\n\x03GET\x12\x19/mlflow/traces/{trace_id}\x1a\x04\x08\x03\x10\x00\x10\x03*\x10Get TraceInfo v3\x12n\n\x08getTrace\x12\x10.mlflow.GetTrace\x1a\x19.mlflow.GetTrace.Response\"5\xf2\x86\x19\x31\n\x1f\n\x03GET\x12\x12/mlflow/traces/get\x1a\x04\x08\x03\x10\x00\x10\x03*\x0cGet Trace v3\x12\x83\x01\n\x0e\x62\x61tchGetTraces\x12\x16.mlflow.BatchGetTraces\x1a\x1f.mlflow.BatchGetTraces.Response\"8\xf2\x86\x19\x34\n$\n\x03GET\x12\x17/mlflow/traces/batchGet\x1a\x04\x08\x03\x10\x00\x10\x03*\nGet Traces\x12w\n\x0csearchTraces\x12\x14.mlflow.SearchTraces\x1a\x1d.mlflow.SearchTraces.Response\"2\xf2\x86\x19.\n\x1b\n\x03GET\x12\x0e/mlflow/traces\x1a\x04\x08\x02\x10\x00\x10\x03*\rSearch Traces\x12\x88\x01\n\x0esearchTracesV3\x12\x16.mlflow.SearchTracesV3\x1a\x1f.mlflow.SearchTracesV3.Response\"=\xf2\x86\x19\x39\n#\n\x04POST\x12\x15/mlflow/traces/search\x1a\x04\x08\x03\x10\x00\x10\x03*\x10Search Traces V3\x12i\n\x0cstartTraceV3\x12\x14.mlflow.StartTraceV3\x1a\x1d.mlflow.StartTraceV3.Response\"$\xf2\x86\x19 \n\x1c\n\x04POST\x12\x0e/mlflow/traces\x1a\x04\x08\x03\x10\x00\x10\x03\x12\x92\x01\n\x0flinkTracesToRun\x12\x17.mlflow.LinkTracesToRun\x1a .mlflow.LinkTracesToRun.Response\"D\xf2\x86\x19@\n(\n\x04POST\x12\x1a/mlflow/traces/link-to-run\x1a\x04\x08\x02\x10\x00\x10\x03*\x12Link Traces to Run\x12\x9f\x01\n\x12linkPromptsToTrace\x12\x1a.mlflow.LinkPromptsToTrace\x1a#.mlflow.LinkPromptsToTrace.Response\"H\xf2\x86\x19\x44\n)\n\x04POST\x12\x1b/mlflow/traces/link-prompts\x1a\x04\x08\x02\x10\x00\x10\x03*\x15Link Prompts to Trace\x12\xa2\x01\n\x19searchUnifiedTraceHandler\x12\x1b.mlflow.SearchUnifiedTraces\x1a$.mlflow.SearchUnifiedTraces.Response\"B\xf2\x86\x19>\n#\n\x03GET\x12\x16/mlflow/unified-traces\x1a\x04\x08\x02\x10\x00\x10\x03*\x15Search Unified Traces\x12\xaf\x01\n\x15getOnlineTraceDetails\x12\x1d.mlflow.GetOnlineTraceDetails\x1a&.mlflow.GetOnlineTraceDetails.Response\"O\xf2\x86\x19K\n-\n\x03GET\x12 /mlflow/get-online-trace-details\x1a\x04\x08\x02\x10\x00\x10\x03*\x18Get Online Trace Details\x12\x86\x01\n\x0c\x64\x65leteTraces\x12\x14.mlflow.DeleteTraces\x1a\x1d.mlflow.DeleteTraces.Response\"A\xf2\x86\x19=\n*\n\x04POST\x12\x1c/mlflow/traces/delete-traces\x1a\x04\x08\x02\x10\x00\x10\x03*\rDelete Traces\x12\x8f\x01\n\x0e\x64\x65leteTracesV3\x12\x16.mlflow.DeleteTracesV3\x1a\x1f.mlflow.DeleteTracesV3.Response\"D\xf2\x86\x19@\n*\n\x04POST\x12\x1c/mlflow/traces/delete-traces\x1a\x04\x08\x03\x10\x00\x10\x03*\x10\x44\x65lete Traces V3\x12\xe3\x01\n\x1f\x63\x61lculateTraceFilterCorrelation\x12\'.mlflow.CalculateTraceFilterCorrelation\x1a\x30.mlflow.CalculateTraceFilterCorrelation.Response\"e\xf2\x86\x19\x61\n9\n\x04POST\x12+/mlflow/traces/calculate-filter-correlation\x1a\x04\x08\x03\x10\x00\x10\x03*\"Calculate Trace Filter Correlation\x12\x94\x01\n\x11\x63reateLoggedModel\x12\x19.mlflow.CreateLoggedModel\x1a\".mlflow.CreateLoggedModel.Response\"@\xf2\x86\x19<\n#\n\x04POST\x12\x15/mlflow/logged-models\x1a\x04\x08\x02\x10\x00\x10\x03*\x13\x43reate Logged Model\x12\xa8\x01\n\x13\x66inalizeLoggedModel\x12\x1b.mlflow.FinalizeLoggedModel\x1a$.mlflow.FinalizeLoggedModel.Response\"N\xf2\x86\x19J\n/\n\x05PATCH\x12 /mlflow/logged-models/{model_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\x15\x46inalize Logged Model\x12\x92\x01\n\x0egetLoggedModel\x12\x16.mlflow.GetLoggedModel\x1a\x1f.mlflow.GetLoggedModel.Response\"G\xf2\x86\x19\x43\n-\n\x03GET\x12 /mlflow/logged-models/{model_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\x10Get Logged Model\x12\xa3\x01\n\x11\x64\x65leteLoggedModel\x12\x19.mlflow.DeleteLoggedModel\x1a\".mlflow.DeleteLoggedModel.Response\"O\xf2\x86\x19K\n0\n\x06\x44\x45LETE\x12 /mlflow/logged-models/{model_id}\x1a\x04\x08\x02\x10\x00\x10\x03*\x15\x44\x65lete a Logged Model\x12\x9e\x01\n\x12searchLoggedModels\x12\x1a.mlflow.SearchLoggedModels\x1a#.mlflow.SearchLoggedModels.Response\"G\xf2\x86\x19\x43\n*\n\x04POST\x12\x1c/mlflow/logged-models/search\x1a\x04\x08\x02\x10\x00\x10\x03*\x13Search LoggedModels\x12\xa9\x01\n\x12setLoggedModelTags\x12\x1a.mlflow.SetLoggedModelTags\x1a#.mlflow.SetLoggedModelTags.Response\"R\xf2\x86\x19N\n4\n\x05PATCH\x12%/mlflow/logged-models/{model_id}/tags\x1a\x04\x08\x02\x10\x00\x10\x03*\x14Set Logged Model Tag\x12\xbd\x01\n\x14\x64\x65leteLoggedModelTag\x12\x1c.mlflow.DeleteLoggedModelTag\x1a%.mlflow.DeleteLoggedModelTag.Response\"`\xf2\x86\x19\\\n?\n\x06\x44\x45LETE\x12//mlflow/logged-models/{model_id}/tags/{tag_key}\x1a\x04\x08\x02\x10\x00\x10\x03*\x17\x44\x65lete Logged Model Tag\x12\xd6\x01\n\x18listLoggedModelArtifacts\x12 .mlflow.ListLoggedModelArtifacts\x1a).mlflow.ListLoggedModelArtifacts.Response\"m\xf2\x86\x19i\nC\n\x03GET\x12\x36/mlflow/logged-models/{model_id}/artifacts/directories\x1a\x04\x08\x02\x10\x00\x10\x03* List Artifacts for Logged Models\x12\xc1\x01\n\x14LogLoggedModelParams\x12#.mlflow.LogLoggedModelParamsRequest\x1a,.mlflow.LogLoggedModelParamsRequest.Response\"V\xf2\x86\x19R\n5\n\x04POST\x12\'/mlflow/logged-models/{model_id}/params\x1a\x04\x08\x02\x10\x00\x10\x03*\x17Log Logged Model Params\x12\xb0\x01\n\rGetAssessment\x12\x1c.mlflow.GetAssessmentRequest\x1a%.mlflow.GetAssessmentRequest.Response\"Z\xf2\x86\x19V\nB\n\x03GET\x12\x35/mlflow/traces/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x03\x10\x00\x10\x03*\x0eGet Assessment\x12\xdf\x01\n\x10\x63reateAssessment\x12\x18.mlflow.CreateAssessment\x1a!.mlflow.CreateAssessment.Response\"\x8d\x01\xf2\x86\x19\x88\x01\n>\n\x04POST\x12\x30/mlflow/traces/{assessment.trace_id}/assessments\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xee\x07\x18\x0c\x18\x01*:Create an assessment of a trace or a span within the trace\x12\xd0\x01\n\x10updateAssessment\x12\x18.mlflow.UpdateAssessment\x1a!.mlflow.UpdateAssessment.Response\"\x7f\xf2\x86\x19{\nD\n\x05PATCH\x12\x35/mlflow/traces/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xee\x07\x18\x01*)Update an existing assessment on a trace.\x12\xb1\x01\n\x10\x64\x65leteAssessment\x12\x18.mlflow.DeleteAssessment\x1a!.mlflow.DeleteAssessment.Response\"`\xf2\x86\x19\\\nE\n\x06\x44\x45LETE\x12\x35/mlflow/traces/{trace_id}/assessments/{assessment_id}\x1a\x04\x08\x03\x10\x00\x10\x03*\x11\x44\x65lete Assessment\x12\x9a\x01\n\rcreateDataset\x12\x15.mlflow.CreateDataset\x1a\x1e.mlflow.CreateDataset.Response\"R\xf2\x86\x19N\n%\n\x04POST\x12\x17/mlflow/datasets/create\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xee\x07\x18\x0c\x18\x01*\x19\x43reate Evaluation Dataset\x12\x91\x01\n\ngetDataset\x12\x12.mlflow.GetDataset\x1a\x1b.mlflow.GetDataset.Response\"R\xf2\x86\x19N\n*\n\x03GET\x12\x1d/mlflow/datasets/{dataset_id}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x16Get Evaluation Dataset\x12\xa0\x01\n\rdeleteDataset\x12\x15.mlflow.DeleteDataset\x1a\x1e.mlflow.DeleteDataset.Response\"X\xf2\x86\x19T\n-\n\x06\x44\x45LETE\x12\x1d/mlflow/datasets/{dataset_id}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x19\x44\x65lete Evaluation Dataset\x12\xdd\x01\n\x18searchEvaluationDatasets\x12 .mlflow.SearchEvaluationDatasets\x1a).mlflow.SearchEvaluationDatasets.Response\"t\xf2\x86\x19p\n%\n\x04POST\x12\x17/mlflow/datasets/search\x1a\x04\x08\x03\x10\x00\n$\n\x03GET\x12\x17/mlflow/datasets/search\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\x01*\x1aSearch Evaluation Datasets\x12\xa9\x01\n\x0esetDatasetTags\x12\x16.mlflow.SetDatasetTags\x1a\x1f.mlflow.SetDatasetTags.Response\"^\xf2\x86\x19Z\n1\n\x05PATCH\x12\"/mlflow/datasets/{dataset_id}/tags\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1bSet Evaluation Dataset Tags\x12\xb8\x01\n\x10\x64\x65leteDatasetTag\x12\x18.mlflow.DeleteDatasetTag\x1a!.mlflow.DeleteDatasetTag.Response\"g\xf2\x86\x19\x63\n8\n\x06\x44\x45LETE\x12(/mlflow/datasets/{dataset_id}/tags/{key}\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1d\x44\x65lete Evaluation Dataset Tag\x12\xc3\x01\n\x14upsertDatasetRecords\x12\x1c.mlflow.UpsertDatasetRecords\x1a%.mlflow.UpsertDatasetRecords.Response\"f\xf2\x86\x19\x62\n3\n\x04POST\x12%/mlflow/datasets/{dataset_id}/records\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*!Upsert Evaluation Dataset Records\x12\xd6\x01\n\x17getDatasetExperimentIds\x12\x1f.mlflow.GetDatasetExperimentIds\x1a(.mlflow.GetDatasetExperimentIds.Response\"p\xf2\x86\x19l\n9\n\x03GET\x12,/mlflow/datasets/{dataset_id}/experiment-ids\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*%Get Evaluation Dataset Experiment IDs\x12\x8a\x01\n\x0eregisterScorer\x12\x16.mlflow.RegisterScorer\x1a\x1f.mlflow.RegisterScorer.Response\"?\xf2\x86\x19;\n&\n\x04POST\x12\x18/mlflow/scorers/register\x1a\x04\x08\x03\x10\x00\x10\x01*\x0fRegister Scorer\x12y\n\x0blistScorers\x12\x13.mlflow.ListScorers\x1a\x1c.mlflow.ListScorers.Response\"7\xf2\x86\x19\x33\n!\n\x03GET\x12\x14/mlflow/scorers/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x0cList Scorers\x12\x9a\x01\n\x12listScorerVersions\x12\x1a.mlflow.ListScorerVersions\x1a#.mlflow.ListScorerVersions.Response\"C\xf2\x86\x19?\n%\n\x03GET\x12\x18/mlflow/scorers/versions\x1a\x04\x08\x03\x10\x00\x10\x01*\x14List Scorer Versions\x12p\n\tgetScorer\x12\x11.mlflow.GetScorer\x1a\x1a.mlflow.GetScorer.Response\"4\xf2\x86\x19\x30\n \n\x03GET\x12\x13/mlflow/scorers/get\x1a\x04\x08\x03\x10\x00\x10\x01*\nGet Scorer\x12\x82\x01\n\x0c\x64\x65leteScorer\x12\x14.mlflow.DeleteScorer\x1a\x1d.mlflow.DeleteScorer.Response\"=\xf2\x86\x19\x39\n&\n\x06\x44\x45LETE\x12\x16/mlflow/scorers/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\rDelete Scorer\x12\xb6\x01\n\x11getDatasetRecords\x12\x19.mlflow.GetDatasetRecords\x1a\".mlflow.GetDatasetRecords.Response\"b\xf2\x86\x19^\n2\n\x03GET\x12%/mlflow/datasets/{dataset_id}/records\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1eGet Evaluation Dataset Records\x12\xcd\x01\n\x17\x61\x64\x64\x44\x61tasetToExperiments\x12\x1f.mlflow.AddDatasetToExperiments\x1a(.mlflow.AddDatasetToExperiments.Response\"g\xf2\x86\x19\x63\n;\n\x04POST\x12-/mlflow/datasets/{dataset_id}/add-experiments\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1a\x41\x64\x64 Dataset to Experiments\x12\xe4\x01\n\x1cremoveDatasetFromExperiments\x12$.mlflow.RemoveDatasetFromExperiments\x1a-.mlflow.RemoveDatasetFromExperiments.Response\"o\xf2\x86\x19k\n>\n\x04POST\x12\x30/mlflow/datasets/{dataset_id}/remove-experiments\x1a\x04\x08\x03\x10\x00\x10\x03\x18\xe8\x07\x18\xba\x17\x18\x01*\x1fRemove Dataset from Experiments\x12\xa5\x01\n\x13\x63reateGatewaySecret\x12\x1b.mlflow.CreateGatewaySecret\x1a$.mlflow.CreateGatewaySecret.Response\"K\xf2\x86\x19G\n,\n\x04POST\x12\x1e/mlflow/gateway/secrets/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x15\x43reate Gateway Secret\x12\xa6\x01\n\x14getGatewaySecretInfo\x12\x1c.mlflow.GetGatewaySecretInfo\x1a%.mlflow.GetGatewaySecretInfo.Response\"I\xf2\x86\x19\x45\n(\n\x03GET\x12\x1b/mlflow/gateway/secrets/get\x1a\x04\x08\x03\x10\x00\x10\x01*\x17Get Gateway Secret Info\x12\xa5\x01\n\x13updateGatewaySecret\x12\x1b.mlflow.UpdateGatewaySecret\x1a$.mlflow.UpdateGatewaySecret.Response\"K\xf2\x86\x19G\n,\n\x04POST\x12\x1e/mlflow/gateway/secrets/update\x1a\x04\x08\x03\x10\x00\x10\x01*\x15Update Gateway Secret\x12\xa7\x01\n\x13\x64\x65leteGatewaySecret\x12\x1b.mlflow.DeleteGatewaySecret\x1a$.mlflow.DeleteGatewaySecret.Response\"M\xf2\x86\x19I\n.\n\x06\x44\x45LETE\x12\x1e/mlflow/gateway/secrets/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x15\x44\x65lete Gateway Secret\x12\xaa\x01\n\x16listGatewaySecretInfos\x12\x1e.mlflow.ListGatewaySecretInfos\x1a\'.mlflow.ListGatewaySecretInfos.Response\"G\xf2\x86\x19\x43\n)\n\x03GET\x12\x1c/mlflow/gateway/secrets/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x14List Gateway Secrets\x12\xaf\x01\n\x15\x63reateGatewayEndpoint\x12\x1d.mlflow.CreateGatewayEndpoint\x1a&.mlflow.CreateGatewayEndpoint.Response\"O\xf2\x86\x19K\n.\n\x04POST\x12 /mlflow/gateway/endpoints/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x43reate Gateway Endpoint\x12\x9f\x01\n\x12getGatewayEndpoint\x12\x1a.mlflow.GetGatewayEndpoint\x1a#.mlflow.GetGatewayEndpoint.Response\"H\xf2\x86\x19\x44\n*\n\x03GET\x12\x1d/mlflow/gateway/endpoints/get\x1a\x04\x08\x03\x10\x00\x10\x01*\x14Get Gateway Endpoint\x12\xaf\x01\n\x15updateGatewayEndpoint\x12\x1d.mlflow.UpdateGatewayEndpoint\x1a&.mlflow.UpdateGatewayEndpoint.Response\"O\xf2\x86\x19K\n.\n\x04POST\x12 /mlflow/gateway/endpoints/update\x1a\x04\x08\x03\x10\x00\x10\x01*\x17Update Gateway Endpoint\x12\xb1\x01\n\x15\x64\x65leteGatewayEndpoint\x12\x1d.mlflow.DeleteGatewayEndpoint\x1a&.mlflow.DeleteGatewayEndpoint.Response\"Q\xf2\x86\x19M\n0\n\x06\x44\x45LETE\x12 /mlflow/gateway/endpoints/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x44\x65lete Gateway Endpoint\x12\xa8\x01\n\x14listGatewayEndpoints\x12\x1c.mlflow.ListGatewayEndpoints\x1a%.mlflow.ListGatewayEndpoints.Response\"K\xf2\x86\x19G\n+\n\x03GET\x12\x1e/mlflow/gateway/endpoints/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x16List Gateway Endpoints\x12\xd4\x01\n\x1c\x63reateGatewayModelDefinition\x12$.mlflow.CreateGatewayModelDefinition\x1a-.mlflow.CreateGatewayModelDefinition.Response\"_\xf2\x86\x19[\n6\n\x04POST\x12(/mlflow/gateway/model-definitions/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x1f\x43reate Gateway Model Definition\x12\xc4\x01\n\x19getGatewayModelDefinition\x12!.mlflow.GetGatewayModelDefinition\x1a*.mlflow.GetGatewayModelDefinition.Response\"X\xf2\x86\x19T\n2\n\x03GET\x12%/mlflow/gateway/model-definitions/get\x1a\x04\x08\x03\x10\x00\x10\x01*\x1cGet Gateway Model Definition\x12\xcd\x01\n\x1blistGatewayModelDefinitions\x12#.mlflow.ListGatewayModelDefinitions\x1a,.mlflow.ListGatewayModelDefinitions.Response\"[\xf2\x86\x19W\n3\n\x03GET\x12&/mlflow/gateway/model-definitions/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x1eList Gateway Model Definitions\x12\xd4\x01\n\x1cupdateGatewayModelDefinition\x12$.mlflow.UpdateGatewayModelDefinition\x1a-.mlflow.UpdateGatewayModelDefinition.Response\"_\xf2\x86\x19[\n6\n\x04POST\x12(/mlflow/gateway/model-definitions/update\x1a\x04\x08\x03\x10\x00\x10\x01*\x1fUpdate Gateway Model Definition\x12\xd6\x01\n\x1c\x64\x65leteGatewayModelDefinition\x12$.mlflow.DeleteGatewayModelDefinition\x1a-.mlflow.DeleteGatewayModelDefinition.Response\"a\xf2\x86\x19]\n8\n\x06\x44\x45LETE\x12(/mlflow/gateway/model-definitions/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x1f\x44\x65lete Gateway Model Definition\x12\xc5\x01\n\x15\x61ttachModelToEndpoint\x12$.mlflow.AttachModelToGatewayEndpoint\x1a-.mlflow.AttachModelToGatewayEndpoint.Response\"W\xf2\x86\x19S\n5\n\x04POST\x12\'/mlflow/gateway/endpoints/models/attach\x1a\x04\x08\x03\x10\x00\x10\x01*\x18\x41ttach Model to Endpoint\x12\xcd\x01\n\x17\x64\x65tachModelFromEndpoint\x12&.mlflow.DetachModelFromGatewayEndpoint\x1a/.mlflow.DetachModelFromGatewayEndpoint.Response\"Y\xf2\x86\x19U\n5\n\x04POST\x12\'/mlflow/gateway/endpoints/models/detach\x1a\x04\x08\x03\x10\x00\x10\x01*\x1a\x44\x65tach Model from Endpoint\x12\xc6\x01\n\x15\x63reateEndpointBinding\x12$.mlflow.CreateGatewayEndpointBinding\x1a-.mlflow.CreateGatewayEndpointBinding.Response\"X\xf2\x86\x19T\n7\n\x04POST\x12)/mlflow/gateway/endpoints/bindings/create\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x43reate Endpoint Binding\x12\xc8\x01\n\x15\x64\x65leteEndpointBinding\x12$.mlflow.DeleteGatewayEndpointBinding\x1a-.mlflow.DeleteGatewayEndpointBinding.Response\"Z\xf2\x86\x19V\n9\n\x06\x44\x45LETE\x12)/mlflow/gateway/endpoints/bindings/delete\x1a\x04\x08\x03\x10\x00\x10\x01*\x17\x44\x65lete Endpoint Binding\x12\xbf\x01\n\x14listEndpointBindings\x12#.mlflow.ListGatewayEndpointBindings\x1a,.mlflow.ListGatewayEndpointBindings.Response\"T\xf2\x86\x19P\n4\n\x03GET\x12\'/mlflow/gateway/endpoints/bindings/list\x1a\x04\x08\x03\x10\x00\x10\x01*\x16List Endpoint Bindings\x12\xb1\x01\n\x15setGatewayEndpointTag\x12\x1d.mlflow.SetGatewayEndpointTag\x1a&.mlflow.SetGatewayEndpointTag.Response\"Q\xf2\x86\x19M\n/\n\x04POST\x12!/mlflow/gateway/endpoints/set-tag\x1a\x04\x08\x03\x10\x00\x10\x01*\x18Gateway Set Endpoint Tag\x12\xc2\x01\n\x18\x64\x65leteGatewayEndpointTag\x12 .mlflow.DeleteGatewayEndpointTag\x1a).mlflow.DeleteGatewayEndpointTag.Response\"Y\xf2\x86\x19U\n4\n\x06\x44\x45LETE\x12$/mlflow/gateway/endpoints/delete-tag\x1a\x04\x08\x03\x10\x00\x10\x01*\x1bGateway Delete Endpoint TagB\x1e\n\x14org.mlflow.api.proto\x90\x01\x01\xe2?\x02\x10\x01')

  _VIEWTYPE = DESCRIPTOR.enum_types_by_name['ViewType']
  ViewType = enum_type_wrapper.EnumTypeWrapper(_VIEWTYPE)
  _SOURCETYPE = DESCRIPTOR.enum_types_by_name['SourceType']
  SourceType = enum_type_wrapper.EnumTypeWrapper(_SOURCETYPE)
  _RUNSTATUS = DESCRIPTOR.enum_types_by_name['RunStatus']
  RunStatus = enum_type_wrapper.EnumTypeWrapper(_RUNSTATUS)
  _TRACESTATUS = DESCRIPTOR.enum_types_by_name['TraceStatus']
  TraceStatus = enum_type_wrapper.EnumTypeWrapper(_TRACESTATUS)
  _LOGGEDMODELSTATUS = DESCRIPTOR.enum_types_by_name['LoggedModelStatus']
  LoggedModelStatus = enum_type_wrapper.EnumTypeWrapper(_LOGGEDMODELSTATUS)
  ACTIVE_ONLY = 1
  DELETED_ONLY = 2
  ALL = 3
  NOTEBOOK = 1
  JOB = 2
  PROJECT = 3
  LOCAL = 4
  UNKNOWN = 1000
  RUNNING = 1
  SCHEDULED = 2
  FINISHED = 3
  FAILED = 4
  KILLED = 5
  TRACE_STATUS_UNSPECIFIED = 0
  OK = 1
  ERROR = 2
  IN_PROGRESS = 3
  LOGGED_MODEL_STATUS_UNSPECIFIED = 0
  LOGGED_MODEL_PENDING = 1
  LOGGED_MODEL_READY = 2
  LOGGED_MODEL_UPLOAD_FAILED = 3


  _METRIC = DESCRIPTOR.message_types_by_name['Metric']
  _PARAM = DESCRIPTOR.message_types_by_name['Param']
  _RUN = DESCRIPTOR.message_types_by_name['Run']
  _RUNDATA = DESCRIPTOR.message_types_by_name['RunData']
  _RUNINPUTS = DESCRIPTOR.message_types_by_name['RunInputs']
  _RUNOUTPUTS = DESCRIPTOR.message_types_by_name['RunOutputs']
  _RUNTAG = DESCRIPTOR.message_types_by_name['RunTag']
  _EXPERIMENTTAG = DESCRIPTOR.message_types_by_name['ExperimentTag']
  _RUNINFO = DESCRIPTOR.message_types_by_name['RunInfo']
  _EXPERIMENT = DESCRIPTOR.message_types_by_name['Experiment']
  _DATASETINPUT = DESCRIPTOR.message_types_by_name['DatasetInput']
  _MODELINPUT = DESCRIPTOR.message_types_by_name['ModelInput']
  _INPUTTAG = DESCRIPTOR.message_types_by_name['InputTag']
  _DATASET = DESCRIPTOR.message_types_by_name['Dataset']
  _MODELOUTPUT = DESCRIPTOR.message_types_by_name['ModelOutput']
  _CREATEEXPERIMENT = DESCRIPTOR.message_types_by_name['CreateExperiment']
  _CREATEEXPERIMENT_RESPONSE = _CREATEEXPERIMENT.nested_types_by_name['Response']
  _SEARCHEXPERIMENTS = DESCRIPTOR.message_types_by_name['SearchExperiments']
  _SEARCHEXPERIMENTS_RESPONSE = _SEARCHEXPERIMENTS.nested_types_by_name['Response']
  _GETEXPERIMENT = DESCRIPTOR.message_types_by_name['GetExperiment']
  _GETEXPERIMENT_RESPONSE = _GETEXPERIMENT.nested_types_by_name['Response']
  _DELETEEXPERIMENT = DESCRIPTOR.message_types_by_name['DeleteExperiment']
  _DELETEEXPERIMENT_RESPONSE = _DELETEEXPERIMENT.nested_types_by_name['Response']
  _RESTOREEXPERIMENT = DESCRIPTOR.message_types_by_name['RestoreExperiment']
  _RESTOREEXPERIMENT_RESPONSE = _RESTOREEXPERIMENT.nested_types_by_name['Response']
  _UPDATEEXPERIMENT = DESCRIPTOR.message_types_by_name['UpdateExperiment']
  _UPDATEEXPERIMENT_RESPONSE = _UPDATEEXPERIMENT.nested_types_by_name['Response']
  _CREATERUN = DESCRIPTOR.message_types_by_name['CreateRun']
  _CREATERUN_RESPONSE = _CREATERUN.nested_types_by_name['Response']
  _UPDATERUN = DESCRIPTOR.message_types_by_name['UpdateRun']
  _UPDATERUN_RESPONSE = _UPDATERUN.nested_types_by_name['Response']
  _DELETERUN = DESCRIPTOR.message_types_by_name['DeleteRun']
  _DELETERUN_RESPONSE = _DELETERUN.nested_types_by_name['Response']
  _RESTORERUN = DESCRIPTOR.message_types_by_name['RestoreRun']
  _RESTORERUN_RESPONSE = _RESTORERUN.nested_types_by_name['Response']
  _LOGMETRIC = DESCRIPTOR.message_types_by_name['LogMetric']
  _LOGMETRIC_RESPONSE = _LOGMETRIC.nested_types_by_name['Response']
  _LOGPARAM = DESCRIPTOR.message_types_by_name['LogParam']
  _LOGPARAM_RESPONSE = _LOGPARAM.nested_types_by_name['Response']
  _SETEXPERIMENTTAG = DESCRIPTOR.message_types_by_name['SetExperimentTag']
  _SETEXPERIMENTTAG_RESPONSE = _SETEXPERIMENTTAG.nested_types_by_name['Response']
  _DELETEEXPERIMENTTAG = DESCRIPTOR.message_types_by_name['DeleteExperimentTag']
  _DELETEEXPERIMENTTAG_RESPONSE = _DELETEEXPERIMENTTAG.nested_types_by_name['Response']
  _SETTAG = DESCRIPTOR.message_types_by_name['SetTag']
  _SETTAG_RESPONSE = _SETTAG.nested_types_by_name['Response']
  _DELETETAG = DESCRIPTOR.message_types_by_name['DeleteTag']
  _DELETETAG_RESPONSE = _DELETETAG.nested_types_by_name['Response']
  _GETRUN = DESCRIPTOR.message_types_by_name['GetRun']
  _GETRUN_RESPONSE = _GETRUN.nested_types_by_name['Response']
  _SEARCHRUNS = DESCRIPTOR.message_types_by_name['SearchRuns']
  _SEARCHRUNS_RESPONSE = _SEARCHRUNS.nested_types_by_name['Response']
  _LISTARTIFACTS = DESCRIPTOR.message_types_by_name['ListArtifacts']
  _LISTARTIFACTS_RESPONSE = _LISTARTIFACTS.nested_types_by_name['Response']
  _FILEINFO = DESCRIPTOR.message_types_by_name['FileInfo']
  _GETMETRICHISTORY = DESCRIPTOR.message_types_by_name['GetMetricHistory']
  _GETMETRICHISTORY_RESPONSE = _GETMETRICHISTORY.nested_types_by_name['Response']
  _METRICWITHRUNID = DESCRIPTOR.message_types_by_name['MetricWithRunId']
  _GETMETRICHISTORYBULKINTERVAL = DESCRIPTOR.message_types_by_name['GetMetricHistoryBulkInterval']
  _GETMETRICHISTORYBULKINTERVAL_RESPONSE = _GETMETRICHISTORYBULKINTERVAL.nested_types_by_name['Response']
  _LOGBATCH = DESCRIPTOR.message_types_by_name['LogBatch']
  _LOGBATCH_RESPONSE = _LOGBATCH.nested_types_by_name['Response']
  _LOGMODEL = DESCRIPTOR.message_types_by_name['LogModel']
  _LOGMODEL_RESPONSE = _LOGMODEL.nested_types_by_name['Response']
  _LOGINPUTS = DESCRIPTOR.message_types_by_name['LogInputs']
  _LOGINPUTS_RESPONSE = _LOGINPUTS.nested_types_by_name['Response']
  _LOGOUTPUTS = DESCRIPTOR.message_types_by_name['LogOutputs']
  _LOGOUTPUTS_RESPONSE = _LOGOUTPUTS.nested_types_by_name['Response']
  _GETEXPERIMENTBYNAME = DESCRIPTOR.message_types_by_name['GetExperimentByName']
  _GETEXPERIMENTBYNAME_RESPONSE = _GETEXPERIMENTBYNAME.nested_types_by_name['Response']
  _CREATEASSESSMENT = DESCRIPTOR.message_types_by_name['CreateAssessment']
  _CREATEASSESSMENT_RESPONSE = _CREATEASSESSMENT.nested_types_by_name['Response']
  _UPDATEASSESSMENT = DESCRIPTOR.message_types_by_name['UpdateAssessment']
  _UPDATEASSESSMENT_RESPONSE = _UPDATEASSESSMENT.nested_types_by_name['Response']
  _DELETEASSESSMENT = DESCRIPTOR.message_types_by_name['DeleteAssessment']
  _DELETEASSESSMENT_RESPONSE = _DELETEASSESSMENT.nested_types_by_name['Response']
  _GETASSESSMENTREQUEST = DESCRIPTOR.message_types_by_name['GetAssessmentRequest']
  _GETASSESSMENTREQUEST_RESPONSE = _GETASSESSMENTREQUEST.nested_types_by_name['Response']
  _TRACEINFO = DESCRIPTOR.message_types_by_name['TraceInfo']
  _TRACEREQUESTMETADATA = DESCRIPTOR.message_types_by_name['TraceRequestMetadata']
  _TRACETAG = DESCRIPTOR.message_types_by_name['TraceTag']
  _STARTTRACE = DESCRIPTOR.message_types_by_name['StartTrace']
  _STARTTRACE_RESPONSE = _STARTTRACE.nested_types_by_name['Response']
  _ENDTRACE = DESCRIPTOR.message_types_by_name['EndTrace']
  _ENDTRACE_RESPONSE = _ENDTRACE.nested_types_by_name['Response']
  _GETTRACEINFO = DESCRIPTOR.message_types_by_name['GetTraceInfo']
  _GETTRACEINFO_RESPONSE = _GETTRACEINFO.nested_types_by_name['Response']
  _GETTRACEINFOV3 = DESCRIPTOR.message_types_by_name['GetTraceInfoV3']
  _GETTRACEINFOV3_RESPONSE = _GETTRACEINFOV3.nested_types_by_name['Response']
  _BATCHGETTRACES = DESCRIPTOR.message_types_by_name['BatchGetTraces']
  _BATCHGETTRACES_RESPONSE = _BATCHGETTRACES.nested_types_by_name['Response']
  _GETTRACE = DESCRIPTOR.message_types_by_name['GetTrace']
  _GETTRACE_RESPONSE = _GETTRACE.nested_types_by_name['Response']
  _SEARCHTRACES = DESCRIPTOR.message_types_by_name['SearchTraces']
  _SEARCHTRACES_RESPONSE = _SEARCHTRACES.nested_types_by_name['Response']
  _SEARCHUNIFIEDTRACES = DESCRIPTOR.message_types_by_name['SearchUnifiedTraces']
  _SEARCHUNIFIEDTRACES_RESPONSE = _SEARCHUNIFIEDTRACES.nested_types_by_name['Response']
  _GETONLINETRACEDETAILS = DESCRIPTOR.message_types_by_name['GetOnlineTraceDetails']
  _GETONLINETRACEDETAILS_RESPONSE = _GETONLINETRACEDETAILS.nested_types_by_name['Response']
  _DELETETRACES = DESCRIPTOR.message_types_by_name['DeleteTraces']
  _DELETETRACES_RESPONSE = _DELETETRACES.nested_types_by_name['Response']
  _DELETETRACESV3 = DESCRIPTOR.message_types_by_name['DeleteTracesV3']
  _DELETETRACESV3_RESPONSE = _DELETETRACESV3.nested_types_by_name['Response']
  _CALCULATETRACEFILTERCORRELATION = DESCRIPTOR.message_types_by_name['CalculateTraceFilterCorrelation']
  _CALCULATETRACEFILTERCORRELATION_RESPONSE = _CALCULATETRACEFILTERCORRELATION.nested_types_by_name['Response']
  _SETTRACETAG = DESCRIPTOR.message_types_by_name['SetTraceTag']
  _SETTRACETAG_RESPONSE = _SETTRACETAG.nested_types_by_name['Response']
  _SETTRACETAGV3 = DESCRIPTOR.message_types_by_name['SetTraceTagV3']
  _SETTRACETAGV3_RESPONSE = _SETTRACETAGV3.nested_types_by_name['Response']
  _DELETETRACETAG = DESCRIPTOR.message_types_by_name['DeleteTraceTag']
  _DELETETRACETAG_RESPONSE = _DELETETRACETAG.nested_types_by_name['Response']
  _DELETETRACETAGV3 = DESCRIPTOR.message_types_by_name['DeleteTraceTagV3']
  _DELETETRACETAGV3_RESPONSE = _DELETETRACETAGV3.nested_types_by_name['Response']
  _TRACE = DESCRIPTOR.message_types_by_name['Trace']
  _TRACELOCATION = DESCRIPTOR.message_types_by_name['TraceLocation']
  _TRACELOCATION_MLFLOWEXPERIMENTLOCATION = _TRACELOCATION.nested_types_by_name['MlflowExperimentLocation']
  _TRACELOCATION_INFERENCETABLELOCATION = _TRACELOCATION.nested_types_by_name['InferenceTableLocation']
  _TRACEINFOV3 = DESCRIPTOR.message_types_by_name['TraceInfoV3']
  _TRACEINFOV3_TRACEMETADATAENTRY = _TRACEINFOV3.nested_types_by_name['TraceMetadataEntry']
  _TRACEINFOV3_TAGSENTRY = _TRACEINFOV3.nested_types_by_name['TagsEntry']
  _STARTTRACEV3 = DESCRIPTOR.message_types_by_name['StartTraceV3']
  _STARTTRACEV3_RESPONSE = _STARTTRACEV3.nested_types_by_name['Response']
  _LINKTRACESTORUN = DESCRIPTOR.message_types_by_name['LinkTracesToRun']
  _LINKTRACESTORUN_RESPONSE = _LINKTRACESTORUN.nested_types_by_name['Response']
  _LINKPROMPTSTOTRACE = DESCRIPTOR.message_types_by_name['LinkPromptsToTrace']
  _LINKPROMPTSTOTRACE_PROMPTVERSIONREF = _LINKPROMPTSTOTRACE.nested_types_by_name['PromptVersionRef']
  _LINKPROMPTSTOTRACE_RESPONSE = _LINKPROMPTSTOTRACE.nested_types_by_name['Response']
  _DATASETSUMMARY = DESCRIPTOR.message_types_by_name['DatasetSummary']
  _SEARCHDATASETS = DESCRIPTOR.message_types_by_name['SearchDatasets']
  _SEARCHDATASETS_RESPONSE = _SEARCHDATASETS.nested_types_by_name['Response']
  _CREATELOGGEDMODEL = DESCRIPTOR.message_types_by_name['CreateLoggedModel']
  _CREATELOGGEDMODEL_RESPONSE = _CREATELOGGEDMODEL.nested_types_by_name['Response']
  _FINALIZELOGGEDMODEL = DESCRIPTOR.message_types_by_name['FinalizeLoggedModel']
  _FINALIZELOGGEDMODEL_RESPONSE = _FINALIZELOGGEDMODEL.nested_types_by_name['Response']
  _GETLOGGEDMODEL = DESCRIPTOR.message_types_by_name['GetLoggedModel']
  _GETLOGGEDMODEL_RESPONSE = _GETLOGGEDMODEL.nested_types_by_name['Response']
  _DELETELOGGEDMODEL = DESCRIPTOR.message_types_by_name['DeleteLoggedModel']
  _DELETELOGGEDMODEL_RESPONSE = _DELETELOGGEDMODEL.nested_types_by_name['Response']
  _SEARCHLOGGEDMODELS = DESCRIPTOR.message_types_by_name['SearchLoggedModels']
  _SEARCHLOGGEDMODELS_DATASET = _SEARCHLOGGEDMODELS.nested_types_by_name['Dataset']
  _SEARCHLOGGEDMODELS_ORDERBY = _SEARCHLOGGEDMODELS.nested_types_by_name['OrderBy']
  _SEARCHLOGGEDMODELS_RESPONSE = _SEARCHLOGGEDMODELS.nested_types_by_name['Response']
  _SETLOGGEDMODELTAGS = DESCRIPTOR.message_types_by_name['SetLoggedModelTags']
  _SETLOGGEDMODELTAGS_RESPONSE = _SETLOGGEDMODELTAGS.nested_types_by_name['Response']
  _DELETELOGGEDMODELTAG = DESCRIPTOR.message_types_by_name['DeleteLoggedModelTag']
  _DELETELOGGEDMODELTAG_RESPONSE = _DELETELOGGEDMODELTAG.nested_types_by_name['Response']
  _LISTLOGGEDMODELARTIFACTS = DESCRIPTOR.message_types_by_name['ListLoggedModelArtifacts']
  _LISTLOGGEDMODELARTIFACTS_RESPONSE = _LISTLOGGEDMODELARTIFACTS.nested_types_by_name['Response']
  _LOGLOGGEDMODELPARAMSREQUEST = DESCRIPTOR.message_types_by_name['LogLoggedModelParamsRequest']
  _LOGLOGGEDMODELPARAMSREQUEST_RESPONSE = _LOGLOGGEDMODELPARAMSREQUEST.nested_types_by_name['Response']
  _LOGGEDMODEL = DESCRIPTOR.message_types_by_name['LoggedModel']
  _LOGGEDMODELINFO = DESCRIPTOR.message_types_by_name['LoggedModelInfo']
  _LOGGEDMODELTAG = DESCRIPTOR.message_types_by_name['LoggedModelTag']
  _LOGGEDMODELREGISTRATIONINFO = DESCRIPTOR.message_types_by_name['LoggedModelRegistrationInfo']
  _LOGGEDMODELDATA = DESCRIPTOR.message_types_by_name['LoggedModelData']
  _LOGGEDMODELPARAMETER = DESCRIPTOR.message_types_by_name['LoggedModelParameter']
  _SEARCHTRACESV3 = DESCRIPTOR.message_types_by_name['SearchTracesV3']
  _SEARCHTRACESV3_RESPONSE = _SEARCHTRACESV3.nested_types_by_name['Response']
  _CREATEDATASET = DESCRIPTOR.message_types_by_name['CreateDataset']
  _CREATEDATASET_RESPONSE = _CREATEDATASET.nested_types_by_name['Response']
  _GETDATASET = DESCRIPTOR.message_types_by_name['GetDataset']
  _GETDATASET_RESPONSE = _GETDATASET.nested_types_by_name['Response']
  _DELETEDATASET = DESCRIPTOR.message_types_by_name['DeleteDataset']
  _DELETEDATASET_RESPONSE = _DELETEDATASET.nested_types_by_name['Response']
  _SEARCHEVALUATIONDATASETS = DESCRIPTOR.message_types_by_name['SearchEvaluationDatasets']
  _SEARCHEVALUATIONDATASETS_RESPONSE = _SEARCHEVALUATIONDATASETS.nested_types_by_name['Response']
  _SETDATASETTAGS = DESCRIPTOR.message_types_by_name['SetDatasetTags']
  _SETDATASETTAGS_RESPONSE = _SETDATASETTAGS.nested_types_by_name['Response']
  _DELETEDATASETTAG = DESCRIPTOR.message_types_by_name['DeleteDatasetTag']
  _DELETEDATASETTAG_RESPONSE = _DELETEDATASETTAG.nested_types_by_name['Response']
  _UPSERTDATASETRECORDS = DESCRIPTOR.message_types_by_name['UpsertDatasetRecords']
  _UPSERTDATASETRECORDS_RESPONSE = _UPSERTDATASETRECORDS.nested_types_by_name['Response']
  _GETDATASETEXPERIMENTIDS = DESCRIPTOR.message_types_by_name['GetDatasetExperimentIds']
  _GETDATASETEXPERIMENTIDS_RESPONSE = _GETDATASETEXPERIMENTIDS.nested_types_by_name['Response']
  _GETDATASETRECORDS = DESCRIPTOR.message_types_by_name['GetDatasetRecords']
  _GETDATASETRECORDS_RESPONSE = _GETDATASETRECORDS.nested_types_by_name['Response']
  _ADDDATASETTOEXPERIMENTS = DESCRIPTOR.message_types_by_name['AddDatasetToExperiments']
  _ADDDATASETTOEXPERIMENTS_RESPONSE = _ADDDATASETTOEXPERIMENTS.nested_types_by_name['Response']
  _REMOVEDATASETFROMEXPERIMENTS = DESCRIPTOR.message_types_by_name['RemoveDatasetFromExperiments']
  _REMOVEDATASETFROMEXPERIMENTS_RESPONSE = _REMOVEDATASETFROMEXPERIMENTS.nested_types_by_name['Response']
  _REGISTERSCORER = DESCRIPTOR.message_types_by_name['RegisterScorer']
  _REGISTERSCORER_RESPONSE = _REGISTERSCORER.nested_types_by_name['Response']
  _LISTSCORERS = DESCRIPTOR.message_types_by_name['ListScorers']
  _LISTSCORERS_RESPONSE = _LISTSCORERS.nested_types_by_name['Response']
  _LISTSCORERVERSIONS = DESCRIPTOR.message_types_by_name['ListScorerVersions']
  _LISTSCORERVERSIONS_RESPONSE = _LISTSCORERVERSIONS.nested_types_by_name['Response']
  _GETSCORER = DESCRIPTOR.message_types_by_name['GetScorer']
  _GETSCORER_RESPONSE = _GETSCORER.nested_types_by_name['Response']
  _DELETESCORER = DESCRIPTOR.message_types_by_name['DeleteScorer']
  _DELETESCORER_RESPONSE = _DELETESCORER.nested_types_by_name['Response']
  _SCORER = DESCRIPTOR.message_types_by_name['Scorer']
  _GATEWAYSECRETINFO = DESCRIPTOR.message_types_by_name['GatewaySecretInfo']
  _GATEWAYSECRETINFO_MASKEDVALUESENTRY = _GATEWAYSECRETINFO.nested_types_by_name['MaskedValuesEntry']
  _GATEWAYMODELDEFINITION = DESCRIPTOR.message_types_by_name['GatewayModelDefinition']
  _GATEWAYENDPOINTMODELMAPPING = DESCRIPTOR.message_types_by_name['GatewayEndpointModelMapping']
  _GATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['GatewayEndpoint']
  _GATEWAYENDPOINTTAG = DESCRIPTOR.message_types_by_name['GatewayEndpointTag']
  _GATEWAYENDPOINTBINDING = DESCRIPTOR.message_types_by_name['GatewayEndpointBinding']
  _CREATEGATEWAYSECRET = DESCRIPTOR.message_types_by_name['CreateGatewaySecret']
  _CREATEGATEWAYSECRET_SECRETVALUEENTRY = _CREATEGATEWAYSECRET.nested_types_by_name['SecretValueEntry']
  _CREATEGATEWAYSECRET_RESPONSE = _CREATEGATEWAYSECRET.nested_types_by_name['Response']
  _GETGATEWAYSECRETINFO = DESCRIPTOR.message_types_by_name['GetGatewaySecretInfo']
  _GETGATEWAYSECRETINFO_RESPONSE = _GETGATEWAYSECRETINFO.nested_types_by_name['Response']
  _UPDATEGATEWAYSECRET = DESCRIPTOR.message_types_by_name['UpdateGatewaySecret']
  _UPDATEGATEWAYSECRET_SECRETVALUEENTRY = _UPDATEGATEWAYSECRET.nested_types_by_name['SecretValueEntry']
  _UPDATEGATEWAYSECRET_RESPONSE = _UPDATEGATEWAYSECRET.nested_types_by_name['Response']
  _DELETEGATEWAYSECRET = DESCRIPTOR.message_types_by_name['DeleteGatewaySecret']
  _DELETEGATEWAYSECRET_RESPONSE = _DELETEGATEWAYSECRET.nested_types_by_name['Response']
  _LISTGATEWAYSECRETINFOS = DESCRIPTOR.message_types_by_name['ListGatewaySecretInfos']
  _LISTGATEWAYSECRETINFOS_RESPONSE = _LISTGATEWAYSECRETINFOS.nested_types_by_name['Response']
  _CREATEGATEWAYMODELDEFINITION = DESCRIPTOR.message_types_by_name['CreateGatewayModelDefinition']
  _CREATEGATEWAYMODELDEFINITION_RESPONSE = _CREATEGATEWAYMODELDEFINITION.nested_types_by_name['Response']
  _GETGATEWAYMODELDEFINITION = DESCRIPTOR.message_types_by_name['GetGatewayModelDefinition']
  _GETGATEWAYMODELDEFINITION_RESPONSE = _GETGATEWAYMODELDEFINITION.nested_types_by_name['Response']
  _LISTGATEWAYMODELDEFINITIONS = DESCRIPTOR.message_types_by_name['ListGatewayModelDefinitions']
  _LISTGATEWAYMODELDEFINITIONS_RESPONSE = _LISTGATEWAYMODELDEFINITIONS.nested_types_by_name['Response']
  _UPDATEGATEWAYMODELDEFINITION = DESCRIPTOR.message_types_by_name['UpdateGatewayModelDefinition']
  _UPDATEGATEWAYMODELDEFINITION_RESPONSE = _UPDATEGATEWAYMODELDEFINITION.nested_types_by_name['Response']
  _DELETEGATEWAYMODELDEFINITION = DESCRIPTOR.message_types_by_name['DeleteGatewayModelDefinition']
  _DELETEGATEWAYMODELDEFINITION_RESPONSE = _DELETEGATEWAYMODELDEFINITION.nested_types_by_name['Response']
  _CREATEGATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['CreateGatewayEndpoint']
  _CREATEGATEWAYENDPOINT_RESPONSE = _CREATEGATEWAYENDPOINT.nested_types_by_name['Response']
  _GETGATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['GetGatewayEndpoint']
  _GETGATEWAYENDPOINT_RESPONSE = _GETGATEWAYENDPOINT.nested_types_by_name['Response']
  _UPDATEGATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['UpdateGatewayEndpoint']
  _UPDATEGATEWAYENDPOINT_RESPONSE = _UPDATEGATEWAYENDPOINT.nested_types_by_name['Response']
  _DELETEGATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['DeleteGatewayEndpoint']
  _DELETEGATEWAYENDPOINT_RESPONSE = _DELETEGATEWAYENDPOINT.nested_types_by_name['Response']
  _LISTGATEWAYENDPOINTS = DESCRIPTOR.message_types_by_name['ListGatewayEndpoints']
  _LISTGATEWAYENDPOINTS_RESPONSE = _LISTGATEWAYENDPOINTS.nested_types_by_name['Response']
  _ATTACHMODELTOGATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['AttachModelToGatewayEndpoint']
  _ATTACHMODELTOGATEWAYENDPOINT_RESPONSE = _ATTACHMODELTOGATEWAYENDPOINT.nested_types_by_name['Response']
  _DETACHMODELFROMGATEWAYENDPOINT = DESCRIPTOR.message_types_by_name['DetachModelFromGatewayEndpoint']
  _DETACHMODELFROMGATEWAYENDPOINT_RESPONSE = _DETACHMODELFROMGATEWAYENDPOINT.nested_types_by_name['Response']
  _CREATEGATEWAYENDPOINTBINDING = DESCRIPTOR.message_types_by_name['CreateGatewayEndpointBinding']
  _CREATEGATEWAYENDPOINTBINDING_RESPONSE = _CREATEGATEWAYENDPOINTBINDING.nested_types_by_name['Response']
  _DELETEGATEWAYENDPOINTBINDING = DESCRIPTOR.message_types_by_name['DeleteGatewayEndpointBinding']
  _DELETEGATEWAYENDPOINTBINDING_RESPONSE = _DELETEGATEWAYENDPOINTBINDING.nested_types_by_name['Response']
  _LISTGATEWAYENDPOINTBINDINGS = DESCRIPTOR.message_types_by_name['ListGatewayEndpointBindings']
  _LISTGATEWAYENDPOINTBINDINGS_RESPONSE = _LISTGATEWAYENDPOINTBINDINGS.nested_types_by_name['Response']
  _SETGATEWAYENDPOINTTAG = DESCRIPTOR.message_types_by_name['SetGatewayEndpointTag']
  _SETGATEWAYENDPOINTTAG_RESPONSE = _SETGATEWAYENDPOINTTAG.nested_types_by_name['Response']
  _DELETEGATEWAYENDPOINTTAG = DESCRIPTOR.message_types_by_name['DeleteGatewayEndpointTag']
  _DELETEGATEWAYENDPOINTTAG_RESPONSE = _DELETEGATEWAYENDPOINTTAG.nested_types_by_name['Response']
  _GETSECRETSCONFIG = DESCRIPTOR.message_types_by_name['GetSecretsConfig']
  _GETSECRETSCONFIG_RESPONSE = _GETSECRETSCONFIG.nested_types_by_name['Response']
  _TRACELOCATION_TRACELOCATIONTYPE = _TRACELOCATION.enum_types_by_name['TraceLocationType']
  _TRACEINFOV3_STATE = _TRACEINFOV3.enum_types_by_name['State']
  Metric = _reflection.GeneratedProtocolMessageType('Metric', (_message.Message,), {
    'DESCRIPTOR' : _METRIC,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Metric)
    })
  _sym_db.RegisterMessage(Metric)

  Param = _reflection.GeneratedProtocolMessageType('Param', (_message.Message,), {
    'DESCRIPTOR' : _PARAM,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Param)
    })
  _sym_db.RegisterMessage(Param)

  Run = _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
    'DESCRIPTOR' : _RUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Run)
    })
  _sym_db.RegisterMessage(Run)

  RunData = _reflection.GeneratedProtocolMessageType('RunData', (_message.Message,), {
    'DESCRIPTOR' : _RUNDATA,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RunData)
    })
  _sym_db.RegisterMessage(RunData)

  RunInputs = _reflection.GeneratedProtocolMessageType('RunInputs', (_message.Message,), {
    'DESCRIPTOR' : _RUNINPUTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RunInputs)
    })
  _sym_db.RegisterMessage(RunInputs)

  RunOutputs = _reflection.GeneratedProtocolMessageType('RunOutputs', (_message.Message,), {
    'DESCRIPTOR' : _RUNOUTPUTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RunOutputs)
    })
  _sym_db.RegisterMessage(RunOutputs)

  RunTag = _reflection.GeneratedProtocolMessageType('RunTag', (_message.Message,), {
    'DESCRIPTOR' : _RUNTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RunTag)
    })
  _sym_db.RegisterMessage(RunTag)

  ExperimentTag = _reflection.GeneratedProtocolMessageType('ExperimentTag', (_message.Message,), {
    'DESCRIPTOR' : _EXPERIMENTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ExperimentTag)
    })
  _sym_db.RegisterMessage(ExperimentTag)

  RunInfo = _reflection.GeneratedProtocolMessageType('RunInfo', (_message.Message,), {
    'DESCRIPTOR' : _RUNINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RunInfo)
    })
  _sym_db.RegisterMessage(RunInfo)

  Experiment = _reflection.GeneratedProtocolMessageType('Experiment', (_message.Message,), {
    'DESCRIPTOR' : _EXPERIMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Experiment)
    })
  _sym_db.RegisterMessage(Experiment)

  DatasetInput = _reflection.GeneratedProtocolMessageType('DatasetInput', (_message.Message,), {
    'DESCRIPTOR' : _DATASETINPUT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DatasetInput)
    })
  _sym_db.RegisterMessage(DatasetInput)

  ModelInput = _reflection.GeneratedProtocolMessageType('ModelInput', (_message.Message,), {
    'DESCRIPTOR' : _MODELINPUT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ModelInput)
    })
  _sym_db.RegisterMessage(ModelInput)

  InputTag = _reflection.GeneratedProtocolMessageType('InputTag', (_message.Message,), {
    'DESCRIPTOR' : _INPUTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.InputTag)
    })
  _sym_db.RegisterMessage(InputTag)

  Dataset = _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), {
    'DESCRIPTOR' : _DATASET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Dataset)
    })
  _sym_db.RegisterMessage(Dataset)

  ModelOutput = _reflection.GeneratedProtocolMessageType('ModelOutput', (_message.Message,), {
    'DESCRIPTOR' : _MODELOUTPUT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ModelOutput)
    })
  _sym_db.RegisterMessage(ModelOutput)

  CreateExperiment = _reflection.GeneratedProtocolMessageType('CreateExperiment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEEXPERIMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateExperiment.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEEXPERIMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateExperiment)
    })
  _sym_db.RegisterMessage(CreateExperiment)
  _sym_db.RegisterMessage(CreateExperiment.Response)

  SearchExperiments = _reflection.GeneratedProtocolMessageType('SearchExperiments', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHEXPERIMENTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchExperiments.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHEXPERIMENTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchExperiments)
    })
  _sym_db.RegisterMessage(SearchExperiments)
  _sym_db.RegisterMessage(SearchExperiments.Response)

  GetExperiment = _reflection.GeneratedProtocolMessageType('GetExperiment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETEXPERIMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetExperiment.Response)
      })
    ,
    'DESCRIPTOR' : _GETEXPERIMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetExperiment)
    })
  _sym_db.RegisterMessage(GetExperiment)
  _sym_db.RegisterMessage(GetExperiment.Response)

  DeleteExperiment = _reflection.GeneratedProtocolMessageType('DeleteExperiment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEEXPERIMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteExperiment.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEEXPERIMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteExperiment)
    })
  _sym_db.RegisterMessage(DeleteExperiment)
  _sym_db.RegisterMessage(DeleteExperiment.Response)

  RestoreExperiment = _reflection.GeneratedProtocolMessageType('RestoreExperiment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _RESTOREEXPERIMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.RestoreExperiment.Response)
      })
    ,
    'DESCRIPTOR' : _RESTOREEXPERIMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RestoreExperiment)
    })
  _sym_db.RegisterMessage(RestoreExperiment)
  _sym_db.RegisterMessage(RestoreExperiment.Response)

  UpdateExperiment = _reflection.GeneratedProtocolMessageType('UpdateExperiment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEEXPERIMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateExperiment.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEEXPERIMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateExperiment)
    })
  _sym_db.RegisterMessage(UpdateExperiment)
  _sym_db.RegisterMessage(UpdateExperiment.Response)

  CreateRun = _reflection.GeneratedProtocolMessageType('CreateRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATERUN_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateRun.Response)
      })
    ,
    'DESCRIPTOR' : _CREATERUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateRun)
    })
  _sym_db.RegisterMessage(CreateRun)
  _sym_db.RegisterMessage(CreateRun.Response)

  UpdateRun = _reflection.GeneratedProtocolMessageType('UpdateRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATERUN_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateRun.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATERUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateRun)
    })
  _sym_db.RegisterMessage(UpdateRun)
  _sym_db.RegisterMessage(UpdateRun.Response)

  DeleteRun = _reflection.GeneratedProtocolMessageType('DeleteRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETERUN_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteRun.Response)
      })
    ,
    'DESCRIPTOR' : _DELETERUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteRun)
    })
  _sym_db.RegisterMessage(DeleteRun)
  _sym_db.RegisterMessage(DeleteRun.Response)

  RestoreRun = _reflection.GeneratedProtocolMessageType('RestoreRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _RESTORERUN_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.RestoreRun.Response)
      })
    ,
    'DESCRIPTOR' : _RESTORERUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RestoreRun)
    })
  _sym_db.RegisterMessage(RestoreRun)
  _sym_db.RegisterMessage(RestoreRun.Response)

  LogMetric = _reflection.GeneratedProtocolMessageType('LogMetric', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGMETRIC_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogMetric.Response)
      })
    ,
    'DESCRIPTOR' : _LOGMETRIC,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogMetric)
    })
  _sym_db.RegisterMessage(LogMetric)
  _sym_db.RegisterMessage(LogMetric.Response)

  LogParam = _reflection.GeneratedProtocolMessageType('LogParam', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGPARAM_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogParam.Response)
      })
    ,
    'DESCRIPTOR' : _LOGPARAM,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogParam)
    })
  _sym_db.RegisterMessage(LogParam)
  _sym_db.RegisterMessage(LogParam.Response)

  SetExperimentTag = _reflection.GeneratedProtocolMessageType('SetExperimentTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETEXPERIMENTTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetExperimentTag.Response)
      })
    ,
    'DESCRIPTOR' : _SETEXPERIMENTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetExperimentTag)
    })
  _sym_db.RegisterMessage(SetExperimentTag)
  _sym_db.RegisterMessage(SetExperimentTag.Response)

  DeleteExperimentTag = _reflection.GeneratedProtocolMessageType('DeleteExperimentTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEEXPERIMENTTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteExperimentTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEEXPERIMENTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteExperimentTag)
    })
  _sym_db.RegisterMessage(DeleteExperimentTag)
  _sym_db.RegisterMessage(DeleteExperimentTag.Response)

  SetTag = _reflection.GeneratedProtocolMessageType('SetTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetTag.Response)
      })
    ,
    'DESCRIPTOR' : _SETTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetTag)
    })
  _sym_db.RegisterMessage(SetTag)
  _sym_db.RegisterMessage(SetTag.Response)

  DeleteTag = _reflection.GeneratedProtocolMessageType('DeleteTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETETAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETETAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteTag)
    })
  _sym_db.RegisterMessage(DeleteTag)
  _sym_db.RegisterMessage(DeleteTag.Response)

  GetRun = _reflection.GeneratedProtocolMessageType('GetRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETRUN_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetRun.Response)
      })
    ,
    'DESCRIPTOR' : _GETRUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetRun)
    })
  _sym_db.RegisterMessage(GetRun)
  _sym_db.RegisterMessage(GetRun.Response)

  SearchRuns = _reflection.GeneratedProtocolMessageType('SearchRuns', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHRUNS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchRuns.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHRUNS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchRuns)
    })
  _sym_db.RegisterMessage(SearchRuns)
  _sym_db.RegisterMessage(SearchRuns.Response)

  ListArtifacts = _reflection.GeneratedProtocolMessageType('ListArtifacts', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTARTIFACTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListArtifacts.Response)
      })
    ,
    'DESCRIPTOR' : _LISTARTIFACTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListArtifacts)
    })
  _sym_db.RegisterMessage(ListArtifacts)
  _sym_db.RegisterMessage(ListArtifacts.Response)

  FileInfo = _reflection.GeneratedProtocolMessageType('FileInfo', (_message.Message,), {
    'DESCRIPTOR' : _FILEINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.FileInfo)
    })
  _sym_db.RegisterMessage(FileInfo)

  GetMetricHistory = _reflection.GeneratedProtocolMessageType('GetMetricHistory', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETMETRICHISTORY_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistory.Response)
      })
    ,
    'DESCRIPTOR' : _GETMETRICHISTORY,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistory)
    })
  _sym_db.RegisterMessage(GetMetricHistory)
  _sym_db.RegisterMessage(GetMetricHistory.Response)

  MetricWithRunId = _reflection.GeneratedProtocolMessageType('MetricWithRunId', (_message.Message,), {
    'DESCRIPTOR' : _METRICWITHRUNID,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.MetricWithRunId)
    })
  _sym_db.RegisterMessage(MetricWithRunId)

  GetMetricHistoryBulkInterval = _reflection.GeneratedProtocolMessageType('GetMetricHistoryBulkInterval', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETMETRICHISTORYBULKINTERVAL_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistoryBulkInterval.Response)
      })
    ,
    'DESCRIPTOR' : _GETMETRICHISTORYBULKINTERVAL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetMetricHistoryBulkInterval)
    })
  _sym_db.RegisterMessage(GetMetricHistoryBulkInterval)
  _sym_db.RegisterMessage(GetMetricHistoryBulkInterval.Response)

  LogBatch = _reflection.GeneratedProtocolMessageType('LogBatch', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGBATCH_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogBatch.Response)
      })
    ,
    'DESCRIPTOR' : _LOGBATCH,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogBatch)
    })
  _sym_db.RegisterMessage(LogBatch)
  _sym_db.RegisterMessage(LogBatch.Response)

  LogModel = _reflection.GeneratedProtocolMessageType('LogModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGMODEL_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogModel.Response)
      })
    ,
    'DESCRIPTOR' : _LOGMODEL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogModel)
    })
  _sym_db.RegisterMessage(LogModel)
  _sym_db.RegisterMessage(LogModel.Response)

  LogInputs = _reflection.GeneratedProtocolMessageType('LogInputs', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGINPUTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogInputs.Response)
      })
    ,
    'DESCRIPTOR' : _LOGINPUTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogInputs)
    })
  _sym_db.RegisterMessage(LogInputs)
  _sym_db.RegisterMessage(LogInputs.Response)

  LogOutputs = _reflection.GeneratedProtocolMessageType('LogOutputs', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGOUTPUTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogOutputs.Response)
      })
    ,
    'DESCRIPTOR' : _LOGOUTPUTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogOutputs)
    })
  _sym_db.RegisterMessage(LogOutputs)
  _sym_db.RegisterMessage(LogOutputs.Response)

  GetExperimentByName = _reflection.GeneratedProtocolMessageType('GetExperimentByName', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETEXPERIMENTBYNAME_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetExperimentByName.Response)
      })
    ,
    'DESCRIPTOR' : _GETEXPERIMENTBYNAME,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetExperimentByName)
    })
  _sym_db.RegisterMessage(GetExperimentByName)
  _sym_db.RegisterMessage(GetExperimentByName.Response)

  CreateAssessment = _reflection.GeneratedProtocolMessageType('CreateAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEASSESSMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEASSESSMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateAssessment)
    })
  _sym_db.RegisterMessage(CreateAssessment)
  _sym_db.RegisterMessage(CreateAssessment.Response)

  UpdateAssessment = _reflection.GeneratedProtocolMessageType('UpdateAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEASSESSMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEASSESSMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateAssessment)
    })
  _sym_db.RegisterMessage(UpdateAssessment)
  _sym_db.RegisterMessage(UpdateAssessment.Response)

  DeleteAssessment = _reflection.GeneratedProtocolMessageType('DeleteAssessment', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEASSESSMENT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteAssessment.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEASSESSMENT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteAssessment)
    })
  _sym_db.RegisterMessage(DeleteAssessment)
  _sym_db.RegisterMessage(DeleteAssessment.Response)

  GetAssessmentRequest = _reflection.GeneratedProtocolMessageType('GetAssessmentRequest', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETASSESSMENTREQUEST_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetAssessmentRequest.Response)
      })
    ,
    'DESCRIPTOR' : _GETASSESSMENTREQUEST,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetAssessmentRequest)
    })
  _sym_db.RegisterMessage(GetAssessmentRequest)
  _sym_db.RegisterMessage(GetAssessmentRequest.Response)

  TraceInfo = _reflection.GeneratedProtocolMessageType('TraceInfo', (_message.Message,), {
    'DESCRIPTOR' : _TRACEINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.TraceInfo)
    })
  _sym_db.RegisterMessage(TraceInfo)

  TraceRequestMetadata = _reflection.GeneratedProtocolMessageType('TraceRequestMetadata', (_message.Message,), {
    'DESCRIPTOR' : _TRACEREQUESTMETADATA,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.TraceRequestMetadata)
    })
  _sym_db.RegisterMessage(TraceRequestMetadata)

  TraceTag = _reflection.GeneratedProtocolMessageType('TraceTag', (_message.Message,), {
    'DESCRIPTOR' : _TRACETAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.TraceTag)
    })
  _sym_db.RegisterMessage(TraceTag)

  StartTrace = _reflection.GeneratedProtocolMessageType('StartTrace', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _STARTTRACE_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.StartTrace.Response)
      })
    ,
    'DESCRIPTOR' : _STARTTRACE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.StartTrace)
    })
  _sym_db.RegisterMessage(StartTrace)
  _sym_db.RegisterMessage(StartTrace.Response)

  EndTrace = _reflection.GeneratedProtocolMessageType('EndTrace', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _ENDTRACE_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.EndTrace.Response)
      })
    ,
    'DESCRIPTOR' : _ENDTRACE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.EndTrace)
    })
  _sym_db.RegisterMessage(EndTrace)
  _sym_db.RegisterMessage(EndTrace.Response)

  GetTraceInfo = _reflection.GeneratedProtocolMessageType('GetTraceInfo', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETTRACEINFO_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetTraceInfo.Response)
      })
    ,
    'DESCRIPTOR' : _GETTRACEINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetTraceInfo)
    })
  _sym_db.RegisterMessage(GetTraceInfo)
  _sym_db.RegisterMessage(GetTraceInfo.Response)

  GetTraceInfoV3 = _reflection.GeneratedProtocolMessageType('GetTraceInfoV3', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETTRACEINFOV3_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetTraceInfoV3.Response)
      })
    ,
    'DESCRIPTOR' : _GETTRACEINFOV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetTraceInfoV3)
    })
  _sym_db.RegisterMessage(GetTraceInfoV3)
  _sym_db.RegisterMessage(GetTraceInfoV3.Response)

  BatchGetTraces = _reflection.GeneratedProtocolMessageType('BatchGetTraces', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _BATCHGETTRACES_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.BatchGetTraces.Response)
      })
    ,
    'DESCRIPTOR' : _BATCHGETTRACES,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.BatchGetTraces)
    })
  _sym_db.RegisterMessage(BatchGetTraces)
  _sym_db.RegisterMessage(BatchGetTraces.Response)

  GetTrace = _reflection.GeneratedProtocolMessageType('GetTrace', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETTRACE_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetTrace.Response)
      })
    ,
    'DESCRIPTOR' : _GETTRACE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetTrace)
    })
  _sym_db.RegisterMessage(GetTrace)
  _sym_db.RegisterMessage(GetTrace.Response)

  SearchTraces = _reflection.GeneratedProtocolMessageType('SearchTraces', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHTRACES_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchTraces.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHTRACES,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchTraces)
    })
  _sym_db.RegisterMessage(SearchTraces)
  _sym_db.RegisterMessage(SearchTraces.Response)

  SearchUnifiedTraces = _reflection.GeneratedProtocolMessageType('SearchUnifiedTraces', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHUNIFIEDTRACES_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchUnifiedTraces.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHUNIFIEDTRACES,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchUnifiedTraces)
    })
  _sym_db.RegisterMessage(SearchUnifiedTraces)
  _sym_db.RegisterMessage(SearchUnifiedTraces.Response)

  GetOnlineTraceDetails = _reflection.GeneratedProtocolMessageType('GetOnlineTraceDetails', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETONLINETRACEDETAILS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetOnlineTraceDetails.Response)
      })
    ,
    'DESCRIPTOR' : _GETONLINETRACEDETAILS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetOnlineTraceDetails)
    })
  _sym_db.RegisterMessage(GetOnlineTraceDetails)
  _sym_db.RegisterMessage(GetOnlineTraceDetails.Response)

  DeleteTraces = _reflection.GeneratedProtocolMessageType('DeleteTraces', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETETRACES_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteTraces.Response)
      })
    ,
    'DESCRIPTOR' : _DELETETRACES,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteTraces)
    })
  _sym_db.RegisterMessage(DeleteTraces)
  _sym_db.RegisterMessage(DeleteTraces.Response)

  DeleteTracesV3 = _reflection.GeneratedProtocolMessageType('DeleteTracesV3', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETETRACESV3_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteTracesV3.Response)
      })
    ,
    'DESCRIPTOR' : _DELETETRACESV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteTracesV3)
    })
  _sym_db.RegisterMessage(DeleteTracesV3)
  _sym_db.RegisterMessage(DeleteTracesV3.Response)

  CalculateTraceFilterCorrelation = _reflection.GeneratedProtocolMessageType('CalculateTraceFilterCorrelation', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CALCULATETRACEFILTERCORRELATION_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CalculateTraceFilterCorrelation.Response)
      })
    ,
    'DESCRIPTOR' : _CALCULATETRACEFILTERCORRELATION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CalculateTraceFilterCorrelation)
    })
  _sym_db.RegisterMessage(CalculateTraceFilterCorrelation)
  _sym_db.RegisterMessage(CalculateTraceFilterCorrelation.Response)

  SetTraceTag = _reflection.GeneratedProtocolMessageType('SetTraceTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETTRACETAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetTraceTag.Response)
      })
    ,
    'DESCRIPTOR' : _SETTRACETAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetTraceTag)
    })
  _sym_db.RegisterMessage(SetTraceTag)
  _sym_db.RegisterMessage(SetTraceTag.Response)

  SetTraceTagV3 = _reflection.GeneratedProtocolMessageType('SetTraceTagV3', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETTRACETAGV3_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetTraceTagV3.Response)
      })
    ,
    'DESCRIPTOR' : _SETTRACETAGV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetTraceTagV3)
    })
  _sym_db.RegisterMessage(SetTraceTagV3)
  _sym_db.RegisterMessage(SetTraceTagV3.Response)

  DeleteTraceTag = _reflection.GeneratedProtocolMessageType('DeleteTraceTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETETRACETAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteTraceTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETETRACETAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteTraceTag)
    })
  _sym_db.RegisterMessage(DeleteTraceTag)
  _sym_db.RegisterMessage(DeleteTraceTag.Response)

  DeleteTraceTagV3 = _reflection.GeneratedProtocolMessageType('DeleteTraceTagV3', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETETRACETAGV3_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteTraceTagV3.Response)
      })
    ,
    'DESCRIPTOR' : _DELETETRACETAGV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteTraceTagV3)
    })
  _sym_db.RegisterMessage(DeleteTraceTagV3)
  _sym_db.RegisterMessage(DeleteTraceTagV3.Response)

  Trace = _reflection.GeneratedProtocolMessageType('Trace', (_message.Message,), {
    'DESCRIPTOR' : _TRACE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Trace)
    })
  _sym_db.RegisterMessage(Trace)

  TraceLocation = _reflection.GeneratedProtocolMessageType('TraceLocation', (_message.Message,), {

    'MlflowExperimentLocation' : _reflection.GeneratedProtocolMessageType('MlflowExperimentLocation', (_message.Message,), {
      'DESCRIPTOR' : _TRACELOCATION_MLFLOWEXPERIMENTLOCATION,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.TraceLocation.MlflowExperimentLocation)
      })
    ,

    'InferenceTableLocation' : _reflection.GeneratedProtocolMessageType('InferenceTableLocation', (_message.Message,), {
      'DESCRIPTOR' : _TRACELOCATION_INFERENCETABLELOCATION,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.TraceLocation.InferenceTableLocation)
      })
    ,
    'DESCRIPTOR' : _TRACELOCATION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.TraceLocation)
    })
  _sym_db.RegisterMessage(TraceLocation)
  _sym_db.RegisterMessage(TraceLocation.MlflowExperimentLocation)
  _sym_db.RegisterMessage(TraceLocation.InferenceTableLocation)

  TraceInfoV3 = _reflection.GeneratedProtocolMessageType('TraceInfoV3', (_message.Message,), {

    'TraceMetadataEntry' : _reflection.GeneratedProtocolMessageType('TraceMetadataEntry', (_message.Message,), {
      'DESCRIPTOR' : _TRACEINFOV3_TRACEMETADATAENTRY,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.TraceInfoV3.TraceMetadataEntry)
      })
    ,

    'TagsEntry' : _reflection.GeneratedProtocolMessageType('TagsEntry', (_message.Message,), {
      'DESCRIPTOR' : _TRACEINFOV3_TAGSENTRY,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.TraceInfoV3.TagsEntry)
      })
    ,
    'DESCRIPTOR' : _TRACEINFOV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.TraceInfoV3)
    })
  _sym_db.RegisterMessage(TraceInfoV3)
  _sym_db.RegisterMessage(TraceInfoV3.TraceMetadataEntry)
  _sym_db.RegisterMessage(TraceInfoV3.TagsEntry)

  StartTraceV3 = _reflection.GeneratedProtocolMessageType('StartTraceV3', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _STARTTRACEV3_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.StartTraceV3.Response)
      })
    ,
    'DESCRIPTOR' : _STARTTRACEV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.StartTraceV3)
    })
  _sym_db.RegisterMessage(StartTraceV3)
  _sym_db.RegisterMessage(StartTraceV3.Response)

  LinkTracesToRun = _reflection.GeneratedProtocolMessageType('LinkTracesToRun', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LINKTRACESTORUN_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LinkTracesToRun.Response)
      })
    ,
    'DESCRIPTOR' : _LINKTRACESTORUN,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LinkTracesToRun)
    })
  _sym_db.RegisterMessage(LinkTracesToRun)
  _sym_db.RegisterMessage(LinkTracesToRun.Response)

  LinkPromptsToTrace = _reflection.GeneratedProtocolMessageType('LinkPromptsToTrace', (_message.Message,), {

    'PromptVersionRef' : _reflection.GeneratedProtocolMessageType('PromptVersionRef', (_message.Message,), {
      'DESCRIPTOR' : _LINKPROMPTSTOTRACE_PROMPTVERSIONREF,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LinkPromptsToTrace.PromptVersionRef)
      })
    ,

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LINKPROMPTSTOTRACE_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LinkPromptsToTrace.Response)
      })
    ,
    'DESCRIPTOR' : _LINKPROMPTSTOTRACE,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LinkPromptsToTrace)
    })
  _sym_db.RegisterMessage(LinkPromptsToTrace)
  _sym_db.RegisterMessage(LinkPromptsToTrace.PromptVersionRef)
  _sym_db.RegisterMessage(LinkPromptsToTrace.Response)

  DatasetSummary = _reflection.GeneratedProtocolMessageType('DatasetSummary', (_message.Message,), {
    'DESCRIPTOR' : _DATASETSUMMARY,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DatasetSummary)
    })
  _sym_db.RegisterMessage(DatasetSummary)

  SearchDatasets = _reflection.GeneratedProtocolMessageType('SearchDatasets', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHDATASETS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchDatasets.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHDATASETS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchDatasets)
    })
  _sym_db.RegisterMessage(SearchDatasets)
  _sym_db.RegisterMessage(SearchDatasets.Response)

  CreateLoggedModel = _reflection.GeneratedProtocolMessageType('CreateLoggedModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATELOGGEDMODEL_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateLoggedModel.Response)
      })
    ,
    'DESCRIPTOR' : _CREATELOGGEDMODEL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateLoggedModel)
    })
  _sym_db.RegisterMessage(CreateLoggedModel)
  _sym_db.RegisterMessage(CreateLoggedModel.Response)

  FinalizeLoggedModel = _reflection.GeneratedProtocolMessageType('FinalizeLoggedModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _FINALIZELOGGEDMODEL_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.FinalizeLoggedModel.Response)
      })
    ,
    'DESCRIPTOR' : _FINALIZELOGGEDMODEL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.FinalizeLoggedModel)
    })
  _sym_db.RegisterMessage(FinalizeLoggedModel)
  _sym_db.RegisterMessage(FinalizeLoggedModel.Response)

  GetLoggedModel = _reflection.GeneratedProtocolMessageType('GetLoggedModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETLOGGEDMODEL_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetLoggedModel.Response)
      })
    ,
    'DESCRIPTOR' : _GETLOGGEDMODEL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetLoggedModel)
    })
  _sym_db.RegisterMessage(GetLoggedModel)
  _sym_db.RegisterMessage(GetLoggedModel.Response)

  DeleteLoggedModel = _reflection.GeneratedProtocolMessageType('DeleteLoggedModel', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETELOGGEDMODEL_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteLoggedModel.Response)
      })
    ,
    'DESCRIPTOR' : _DELETELOGGEDMODEL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteLoggedModel)
    })
  _sym_db.RegisterMessage(DeleteLoggedModel)
  _sym_db.RegisterMessage(DeleteLoggedModel.Response)

  SearchLoggedModels = _reflection.GeneratedProtocolMessageType('SearchLoggedModels', (_message.Message,), {

    'Dataset' : _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHLOGGEDMODELS_DATASET,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchLoggedModels.Dataset)
      })
    ,

    'OrderBy' : _reflection.GeneratedProtocolMessageType('OrderBy', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHLOGGEDMODELS_ORDERBY,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchLoggedModels.OrderBy)
      })
    ,

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHLOGGEDMODELS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchLoggedModels.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHLOGGEDMODELS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchLoggedModels)
    })
  _sym_db.RegisterMessage(SearchLoggedModels)
  _sym_db.RegisterMessage(SearchLoggedModels.Dataset)
  _sym_db.RegisterMessage(SearchLoggedModels.OrderBy)
  _sym_db.RegisterMessage(SearchLoggedModels.Response)

  SetLoggedModelTags = _reflection.GeneratedProtocolMessageType('SetLoggedModelTags', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETLOGGEDMODELTAGS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetLoggedModelTags.Response)
      })
    ,
    'DESCRIPTOR' : _SETLOGGEDMODELTAGS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetLoggedModelTags)
    })
  _sym_db.RegisterMessage(SetLoggedModelTags)
  _sym_db.RegisterMessage(SetLoggedModelTags.Response)

  DeleteLoggedModelTag = _reflection.GeneratedProtocolMessageType('DeleteLoggedModelTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETELOGGEDMODELTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteLoggedModelTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETELOGGEDMODELTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteLoggedModelTag)
    })
  _sym_db.RegisterMessage(DeleteLoggedModelTag)
  _sym_db.RegisterMessage(DeleteLoggedModelTag.Response)

  ListLoggedModelArtifacts = _reflection.GeneratedProtocolMessageType('ListLoggedModelArtifacts', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTLOGGEDMODELARTIFACTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListLoggedModelArtifacts.Response)
      })
    ,
    'DESCRIPTOR' : _LISTLOGGEDMODELARTIFACTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListLoggedModelArtifacts)
    })
  _sym_db.RegisterMessage(ListLoggedModelArtifacts)
  _sym_db.RegisterMessage(ListLoggedModelArtifacts.Response)

  LogLoggedModelParamsRequest = _reflection.GeneratedProtocolMessageType('LogLoggedModelParamsRequest', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LOGLOGGEDMODELPARAMSREQUEST_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.LogLoggedModelParamsRequest.Response)
      })
    ,
    'DESCRIPTOR' : _LOGLOGGEDMODELPARAMSREQUEST,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LogLoggedModelParamsRequest)
    })
  _sym_db.RegisterMessage(LogLoggedModelParamsRequest)
  _sym_db.RegisterMessage(LogLoggedModelParamsRequest.Response)

  LoggedModel = _reflection.GeneratedProtocolMessageType('LoggedModel', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODEL,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModel)
    })
  _sym_db.RegisterMessage(LoggedModel)

  LoggedModelInfo = _reflection.GeneratedProtocolMessageType('LoggedModelInfo', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODELINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModelInfo)
    })
  _sym_db.RegisterMessage(LoggedModelInfo)

  LoggedModelTag = _reflection.GeneratedProtocolMessageType('LoggedModelTag', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODELTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModelTag)
    })
  _sym_db.RegisterMessage(LoggedModelTag)

  LoggedModelRegistrationInfo = _reflection.GeneratedProtocolMessageType('LoggedModelRegistrationInfo', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODELREGISTRATIONINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModelRegistrationInfo)
    })
  _sym_db.RegisterMessage(LoggedModelRegistrationInfo)

  LoggedModelData = _reflection.GeneratedProtocolMessageType('LoggedModelData', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODELDATA,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModelData)
    })
  _sym_db.RegisterMessage(LoggedModelData)

  LoggedModelParameter = _reflection.GeneratedProtocolMessageType('LoggedModelParameter', (_message.Message,), {
    'DESCRIPTOR' : _LOGGEDMODELPARAMETER,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.LoggedModelParameter)
    })
  _sym_db.RegisterMessage(LoggedModelParameter)

  SearchTracesV3 = _reflection.GeneratedProtocolMessageType('SearchTracesV3', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHTRACESV3_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchTracesV3.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHTRACESV3,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchTracesV3)
    })
  _sym_db.RegisterMessage(SearchTracesV3)
  _sym_db.RegisterMessage(SearchTracesV3.Response)

  CreateDataset = _reflection.GeneratedProtocolMessageType('CreateDataset', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEDATASET_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateDataset.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEDATASET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateDataset)
    })
  _sym_db.RegisterMessage(CreateDataset)
  _sym_db.RegisterMessage(CreateDataset.Response)

  GetDataset = _reflection.GeneratedProtocolMessageType('GetDataset', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETDATASET_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetDataset.Response)
      })
    ,
    'DESCRIPTOR' : _GETDATASET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetDataset)
    })
  _sym_db.RegisterMessage(GetDataset)
  _sym_db.RegisterMessage(GetDataset.Response)

  DeleteDataset = _reflection.GeneratedProtocolMessageType('DeleteDataset', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEDATASET_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteDataset.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEDATASET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteDataset)
    })
  _sym_db.RegisterMessage(DeleteDataset)
  _sym_db.RegisterMessage(DeleteDataset.Response)

  SearchEvaluationDatasets = _reflection.GeneratedProtocolMessageType('SearchEvaluationDatasets', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SEARCHEVALUATIONDATASETS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SearchEvaluationDatasets.Response)
      })
    ,
    'DESCRIPTOR' : _SEARCHEVALUATIONDATASETS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SearchEvaluationDatasets)
    })
  _sym_db.RegisterMessage(SearchEvaluationDatasets)
  _sym_db.RegisterMessage(SearchEvaluationDatasets.Response)

  SetDatasetTags = _reflection.GeneratedProtocolMessageType('SetDatasetTags', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETDATASETTAGS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetDatasetTags.Response)
      })
    ,
    'DESCRIPTOR' : _SETDATASETTAGS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetDatasetTags)
    })
  _sym_db.RegisterMessage(SetDatasetTags)
  _sym_db.RegisterMessage(SetDatasetTags.Response)

  DeleteDatasetTag = _reflection.GeneratedProtocolMessageType('DeleteDatasetTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEDATASETTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteDatasetTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEDATASETTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteDatasetTag)
    })
  _sym_db.RegisterMessage(DeleteDatasetTag)
  _sym_db.RegisterMessage(DeleteDatasetTag.Response)

  UpsertDatasetRecords = _reflection.GeneratedProtocolMessageType('UpsertDatasetRecords', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPSERTDATASETRECORDS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpsertDatasetRecords.Response)
      })
    ,
    'DESCRIPTOR' : _UPSERTDATASETRECORDS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpsertDatasetRecords)
    })
  _sym_db.RegisterMessage(UpsertDatasetRecords)
  _sym_db.RegisterMessage(UpsertDatasetRecords.Response)

  GetDatasetExperimentIds = _reflection.GeneratedProtocolMessageType('GetDatasetExperimentIds', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETDATASETEXPERIMENTIDS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetDatasetExperimentIds.Response)
      })
    ,
    'DESCRIPTOR' : _GETDATASETEXPERIMENTIDS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetDatasetExperimentIds)
    })
  _sym_db.RegisterMessage(GetDatasetExperimentIds)
  _sym_db.RegisterMessage(GetDatasetExperimentIds.Response)

  GetDatasetRecords = _reflection.GeneratedProtocolMessageType('GetDatasetRecords', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETDATASETRECORDS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetDatasetRecords.Response)
      })
    ,
    'DESCRIPTOR' : _GETDATASETRECORDS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetDatasetRecords)
    })
  _sym_db.RegisterMessage(GetDatasetRecords)
  _sym_db.RegisterMessage(GetDatasetRecords.Response)

  AddDatasetToExperiments = _reflection.GeneratedProtocolMessageType('AddDatasetToExperiments', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _ADDDATASETTOEXPERIMENTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.AddDatasetToExperiments.Response)
      })
    ,
    'DESCRIPTOR' : _ADDDATASETTOEXPERIMENTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.AddDatasetToExperiments)
    })
  _sym_db.RegisterMessage(AddDatasetToExperiments)
  _sym_db.RegisterMessage(AddDatasetToExperiments.Response)

  RemoveDatasetFromExperiments = _reflection.GeneratedProtocolMessageType('RemoveDatasetFromExperiments', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _REMOVEDATASETFROMEXPERIMENTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.RemoveDatasetFromExperiments.Response)
      })
    ,
    'DESCRIPTOR' : _REMOVEDATASETFROMEXPERIMENTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RemoveDatasetFromExperiments)
    })
  _sym_db.RegisterMessage(RemoveDatasetFromExperiments)
  _sym_db.RegisterMessage(RemoveDatasetFromExperiments.Response)

  RegisterScorer = _reflection.GeneratedProtocolMessageType('RegisterScorer', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _REGISTERSCORER_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.RegisterScorer.Response)
      })
    ,
    'DESCRIPTOR' : _REGISTERSCORER,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.RegisterScorer)
    })
  _sym_db.RegisterMessage(RegisterScorer)
  _sym_db.RegisterMessage(RegisterScorer.Response)

  ListScorers = _reflection.GeneratedProtocolMessageType('ListScorers', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTSCORERS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListScorers.Response)
      })
    ,
    'DESCRIPTOR' : _LISTSCORERS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListScorers)
    })
  _sym_db.RegisterMessage(ListScorers)
  _sym_db.RegisterMessage(ListScorers.Response)

  ListScorerVersions = _reflection.GeneratedProtocolMessageType('ListScorerVersions', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTSCORERVERSIONS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListScorerVersions.Response)
      })
    ,
    'DESCRIPTOR' : _LISTSCORERVERSIONS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListScorerVersions)
    })
  _sym_db.RegisterMessage(ListScorerVersions)
  _sym_db.RegisterMessage(ListScorerVersions.Response)

  GetScorer = _reflection.GeneratedProtocolMessageType('GetScorer', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETSCORER_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetScorer.Response)
      })
    ,
    'DESCRIPTOR' : _GETSCORER,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetScorer)
    })
  _sym_db.RegisterMessage(GetScorer)
  _sym_db.RegisterMessage(GetScorer.Response)

  DeleteScorer = _reflection.GeneratedProtocolMessageType('DeleteScorer', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETESCORER_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteScorer.Response)
      })
    ,
    'DESCRIPTOR' : _DELETESCORER,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteScorer)
    })
  _sym_db.RegisterMessage(DeleteScorer)
  _sym_db.RegisterMessage(DeleteScorer.Response)

  Scorer = _reflection.GeneratedProtocolMessageType('Scorer', (_message.Message,), {
    'DESCRIPTOR' : _SCORER,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.Scorer)
    })
  _sym_db.RegisterMessage(Scorer)

  GatewaySecretInfo = _reflection.GeneratedProtocolMessageType('GatewaySecretInfo', (_message.Message,), {

    'MaskedValuesEntry' : _reflection.GeneratedProtocolMessageType('MaskedValuesEntry', (_message.Message,), {
      'DESCRIPTOR' : _GATEWAYSECRETINFO_MASKEDVALUESENTRY,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GatewaySecretInfo.MaskedValuesEntry)
      })
    ,
    'DESCRIPTOR' : _GATEWAYSECRETINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GatewaySecretInfo)
    })
  _sym_db.RegisterMessage(GatewaySecretInfo)
  _sym_db.RegisterMessage(GatewaySecretInfo.MaskedValuesEntry)

  GatewayModelDefinition = _reflection.GeneratedProtocolMessageType('GatewayModelDefinition', (_message.Message,), {
    'DESCRIPTOR' : _GATEWAYMODELDEFINITION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GatewayModelDefinition)
    })
  _sym_db.RegisterMessage(GatewayModelDefinition)

  GatewayEndpointModelMapping = _reflection.GeneratedProtocolMessageType('GatewayEndpointModelMapping', (_message.Message,), {
    'DESCRIPTOR' : _GATEWAYENDPOINTMODELMAPPING,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GatewayEndpointModelMapping)
    })
  _sym_db.RegisterMessage(GatewayEndpointModelMapping)

  GatewayEndpoint = _reflection.GeneratedProtocolMessageType('GatewayEndpoint', (_message.Message,), {
    'DESCRIPTOR' : _GATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GatewayEndpoint)
    })
  _sym_db.RegisterMessage(GatewayEndpoint)

  GatewayEndpointTag = _reflection.GeneratedProtocolMessageType('GatewayEndpointTag', (_message.Message,), {
    'DESCRIPTOR' : _GATEWAYENDPOINTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GatewayEndpointTag)
    })
  _sym_db.RegisterMessage(GatewayEndpointTag)

  GatewayEndpointBinding = _reflection.GeneratedProtocolMessageType('GatewayEndpointBinding', (_message.Message,), {
    'DESCRIPTOR' : _GATEWAYENDPOINTBINDING,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GatewayEndpointBinding)
    })
  _sym_db.RegisterMessage(GatewayEndpointBinding)

  CreateGatewaySecret = _reflection.GeneratedProtocolMessageType('CreateGatewaySecret', (_message.Message,), {

    'SecretValueEntry' : _reflection.GeneratedProtocolMessageType('SecretValueEntry', (_message.Message,), {
      'DESCRIPTOR' : _CREATEGATEWAYSECRET_SECRETVALUEENTRY,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateGatewaySecret.SecretValueEntry)
      })
    ,

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEGATEWAYSECRET_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateGatewaySecret.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEGATEWAYSECRET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateGatewaySecret)
    })
  _sym_db.RegisterMessage(CreateGatewaySecret)
  _sym_db.RegisterMessage(CreateGatewaySecret.SecretValueEntry)
  _sym_db.RegisterMessage(CreateGatewaySecret.Response)

  GetGatewaySecretInfo = _reflection.GeneratedProtocolMessageType('GetGatewaySecretInfo', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETGATEWAYSECRETINFO_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetGatewaySecretInfo.Response)
      })
    ,
    'DESCRIPTOR' : _GETGATEWAYSECRETINFO,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetGatewaySecretInfo)
    })
  _sym_db.RegisterMessage(GetGatewaySecretInfo)
  _sym_db.RegisterMessage(GetGatewaySecretInfo.Response)

  UpdateGatewaySecret = _reflection.GeneratedProtocolMessageType('UpdateGatewaySecret', (_message.Message,), {

    'SecretValueEntry' : _reflection.GeneratedProtocolMessageType('SecretValueEntry', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEGATEWAYSECRET_SECRETVALUEENTRY,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewaySecret.SecretValueEntry)
      })
    ,

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEGATEWAYSECRET_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewaySecret.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEGATEWAYSECRET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewaySecret)
    })
  _sym_db.RegisterMessage(UpdateGatewaySecret)
  _sym_db.RegisterMessage(UpdateGatewaySecret.SecretValueEntry)
  _sym_db.RegisterMessage(UpdateGatewaySecret.Response)

  DeleteGatewaySecret = _reflection.GeneratedProtocolMessageType('DeleteGatewaySecret', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEGATEWAYSECRET_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewaySecret.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEGATEWAYSECRET,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewaySecret)
    })
  _sym_db.RegisterMessage(DeleteGatewaySecret)
  _sym_db.RegisterMessage(DeleteGatewaySecret.Response)

  ListGatewaySecretInfos = _reflection.GeneratedProtocolMessageType('ListGatewaySecretInfos', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTGATEWAYSECRETINFOS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListGatewaySecretInfos.Response)
      })
    ,
    'DESCRIPTOR' : _LISTGATEWAYSECRETINFOS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListGatewaySecretInfos)
    })
  _sym_db.RegisterMessage(ListGatewaySecretInfos)
  _sym_db.RegisterMessage(ListGatewaySecretInfos.Response)

  CreateGatewayModelDefinition = _reflection.GeneratedProtocolMessageType('CreateGatewayModelDefinition', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEGATEWAYMODELDEFINITION_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateGatewayModelDefinition.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEGATEWAYMODELDEFINITION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateGatewayModelDefinition)
    })
  _sym_db.RegisterMessage(CreateGatewayModelDefinition)
  _sym_db.RegisterMessage(CreateGatewayModelDefinition.Response)

  GetGatewayModelDefinition = _reflection.GeneratedProtocolMessageType('GetGatewayModelDefinition', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETGATEWAYMODELDEFINITION_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetGatewayModelDefinition.Response)
      })
    ,
    'DESCRIPTOR' : _GETGATEWAYMODELDEFINITION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetGatewayModelDefinition)
    })
  _sym_db.RegisterMessage(GetGatewayModelDefinition)
  _sym_db.RegisterMessage(GetGatewayModelDefinition.Response)

  ListGatewayModelDefinitions = _reflection.GeneratedProtocolMessageType('ListGatewayModelDefinitions', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTGATEWAYMODELDEFINITIONS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListGatewayModelDefinitions.Response)
      })
    ,
    'DESCRIPTOR' : _LISTGATEWAYMODELDEFINITIONS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListGatewayModelDefinitions)
    })
  _sym_db.RegisterMessage(ListGatewayModelDefinitions)
  _sym_db.RegisterMessage(ListGatewayModelDefinitions.Response)

  UpdateGatewayModelDefinition = _reflection.GeneratedProtocolMessageType('UpdateGatewayModelDefinition', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEGATEWAYMODELDEFINITION_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewayModelDefinition.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEGATEWAYMODELDEFINITION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewayModelDefinition)
    })
  _sym_db.RegisterMessage(UpdateGatewayModelDefinition)
  _sym_db.RegisterMessage(UpdateGatewayModelDefinition.Response)

  DeleteGatewayModelDefinition = _reflection.GeneratedProtocolMessageType('DeleteGatewayModelDefinition', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEGATEWAYMODELDEFINITION_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayModelDefinition.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEGATEWAYMODELDEFINITION,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayModelDefinition)
    })
  _sym_db.RegisterMessage(DeleteGatewayModelDefinition)
  _sym_db.RegisterMessage(DeleteGatewayModelDefinition.Response)

  CreateGatewayEndpoint = _reflection.GeneratedProtocolMessageType('CreateGatewayEndpoint', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEGATEWAYENDPOINT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateGatewayEndpoint.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEGATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateGatewayEndpoint)
    })
  _sym_db.RegisterMessage(CreateGatewayEndpoint)
  _sym_db.RegisterMessage(CreateGatewayEndpoint.Response)

  GetGatewayEndpoint = _reflection.GeneratedProtocolMessageType('GetGatewayEndpoint', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETGATEWAYENDPOINT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetGatewayEndpoint.Response)
      })
    ,
    'DESCRIPTOR' : _GETGATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetGatewayEndpoint)
    })
  _sym_db.RegisterMessage(GetGatewayEndpoint)
  _sym_db.RegisterMessage(GetGatewayEndpoint.Response)

  UpdateGatewayEndpoint = _reflection.GeneratedProtocolMessageType('UpdateGatewayEndpoint', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _UPDATEGATEWAYENDPOINT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewayEndpoint.Response)
      })
    ,
    'DESCRIPTOR' : _UPDATEGATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.UpdateGatewayEndpoint)
    })
  _sym_db.RegisterMessage(UpdateGatewayEndpoint)
  _sym_db.RegisterMessage(UpdateGatewayEndpoint.Response)

  DeleteGatewayEndpoint = _reflection.GeneratedProtocolMessageType('DeleteGatewayEndpoint', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEGATEWAYENDPOINT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayEndpoint.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEGATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayEndpoint)
    })
  _sym_db.RegisterMessage(DeleteGatewayEndpoint)
  _sym_db.RegisterMessage(DeleteGatewayEndpoint.Response)

  ListGatewayEndpoints = _reflection.GeneratedProtocolMessageType('ListGatewayEndpoints', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTGATEWAYENDPOINTS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListGatewayEndpoints.Response)
      })
    ,
    'DESCRIPTOR' : _LISTGATEWAYENDPOINTS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListGatewayEndpoints)
    })
  _sym_db.RegisterMessage(ListGatewayEndpoints)
  _sym_db.RegisterMessage(ListGatewayEndpoints.Response)

  AttachModelToGatewayEndpoint = _reflection.GeneratedProtocolMessageType('AttachModelToGatewayEndpoint', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _ATTACHMODELTOGATEWAYENDPOINT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.AttachModelToGatewayEndpoint.Response)
      })
    ,
    'DESCRIPTOR' : _ATTACHMODELTOGATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.AttachModelToGatewayEndpoint)
    })
  _sym_db.RegisterMessage(AttachModelToGatewayEndpoint)
  _sym_db.RegisterMessage(AttachModelToGatewayEndpoint.Response)

  DetachModelFromGatewayEndpoint = _reflection.GeneratedProtocolMessageType('DetachModelFromGatewayEndpoint', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DETACHMODELFROMGATEWAYENDPOINT_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DetachModelFromGatewayEndpoint.Response)
      })
    ,
    'DESCRIPTOR' : _DETACHMODELFROMGATEWAYENDPOINT,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DetachModelFromGatewayEndpoint)
    })
  _sym_db.RegisterMessage(DetachModelFromGatewayEndpoint)
  _sym_db.RegisterMessage(DetachModelFromGatewayEndpoint.Response)

  CreateGatewayEndpointBinding = _reflection.GeneratedProtocolMessageType('CreateGatewayEndpointBinding', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _CREATEGATEWAYENDPOINTBINDING_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.CreateGatewayEndpointBinding.Response)
      })
    ,
    'DESCRIPTOR' : _CREATEGATEWAYENDPOINTBINDING,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.CreateGatewayEndpointBinding)
    })
  _sym_db.RegisterMessage(CreateGatewayEndpointBinding)
  _sym_db.RegisterMessage(CreateGatewayEndpointBinding.Response)

  DeleteGatewayEndpointBinding = _reflection.GeneratedProtocolMessageType('DeleteGatewayEndpointBinding', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEGATEWAYENDPOINTBINDING_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayEndpointBinding.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEGATEWAYENDPOINTBINDING,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayEndpointBinding)
    })
  _sym_db.RegisterMessage(DeleteGatewayEndpointBinding)
  _sym_db.RegisterMessage(DeleteGatewayEndpointBinding.Response)

  ListGatewayEndpointBindings = _reflection.GeneratedProtocolMessageType('ListGatewayEndpointBindings', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _LISTGATEWAYENDPOINTBINDINGS_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.ListGatewayEndpointBindings.Response)
      })
    ,
    'DESCRIPTOR' : _LISTGATEWAYENDPOINTBINDINGS,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.ListGatewayEndpointBindings)
    })
  _sym_db.RegisterMessage(ListGatewayEndpointBindings)
  _sym_db.RegisterMessage(ListGatewayEndpointBindings.Response)

  SetGatewayEndpointTag = _reflection.GeneratedProtocolMessageType('SetGatewayEndpointTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _SETGATEWAYENDPOINTTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.SetGatewayEndpointTag.Response)
      })
    ,
    'DESCRIPTOR' : _SETGATEWAYENDPOINTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.SetGatewayEndpointTag)
    })
  _sym_db.RegisterMessage(SetGatewayEndpointTag)
  _sym_db.RegisterMessage(SetGatewayEndpointTag.Response)

  DeleteGatewayEndpointTag = _reflection.GeneratedProtocolMessageType('DeleteGatewayEndpointTag', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _DELETEGATEWAYENDPOINTTAG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayEndpointTag.Response)
      })
    ,
    'DESCRIPTOR' : _DELETEGATEWAYENDPOINTTAG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.DeleteGatewayEndpointTag)
    })
  _sym_db.RegisterMessage(DeleteGatewayEndpointTag)
  _sym_db.RegisterMessage(DeleteGatewayEndpointTag.Response)

  GetSecretsConfig = _reflection.GeneratedProtocolMessageType('GetSecretsConfig', (_message.Message,), {

    'Response' : _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
      'DESCRIPTOR' : _GETSECRETSCONFIG_RESPONSE,
      '__module__' : 'service_pb2'
      # @@protoc_insertion_point(class_scope:mlflow.GetSecretsConfig.Response)
      })
    ,
    'DESCRIPTOR' : _GETSECRETSCONFIG,
    '__module__' : 'service_pb2'
    # @@protoc_insertion_point(class_scope:mlflow.GetSecretsConfig)
    })
  _sym_db.RegisterMessage(GetSecretsConfig)
  _sym_db.RegisterMessage(GetSecretsConfig.Response)

  _MLFLOWSERVICE = DESCRIPTOR.services_by_name['MlflowService']
  if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'\n\024org.mlflow.api.proto\220\001\001\342?\002\020\001'
    _METRIC.fields_by_name['dataset_name']._options = None
    _METRIC.fields_by_name['dataset_name']._serialized_options = b'\360\206\031\003'
    _METRIC.fields_by_name['dataset_digest']._options = None
    _METRIC.fields_by_name['dataset_digest']._serialized_options = b'\360\206\031\003'
    _METRIC.fields_by_name['model_id']._options = None
    _METRIC.fields_by_name['model_id']._serialized_options = b'\360\206\031\003'
    _METRIC.fields_by_name['run_id']._options = None
    _METRIC.fields_by_name['run_id']._serialized_options = b'\360\206\031\003'
    _DATASETINPUT.fields_by_name['dataset']._options = None
    _DATASETINPUT.fields_by_name['dataset']._serialized_options = b'\370\206\031\001'
    _MODELINPUT.fields_by_name['model_id']._options = None
    _MODELINPUT.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _INPUTTAG.fields_by_name['key']._options = None
    _INPUTTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _INPUTTAG.fields_by_name['value']._options = None
    _INPUTTAG.fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _DATASET.fields_by_name['name']._options = None
    _DATASET.fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _DATASET.fields_by_name['digest']._options = None
    _DATASET.fields_by_name['digest']._serialized_options = b'\370\206\031\001'
    _DATASET.fields_by_name['source_type']._options = None
    _DATASET.fields_by_name['source_type']._serialized_options = b'\370\206\031\001'
    _DATASET.fields_by_name['source']._options = None
    _DATASET.fields_by_name['source']._serialized_options = b'\370\206\031\001'
    _MODELOUTPUT.fields_by_name['model_id']._options = None
    _MODELOUTPUT.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _MODELOUTPUT.fields_by_name['step']._options = None
    _MODELOUTPUT.fields_by_name['step']._serialized_options = b'\370\206\031\001'
    _CREATEEXPERIMENT.fields_by_name['name']._options = None
    _CREATEEXPERIMENT.fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _CREATEEXPERIMENT._options = None
    _CREATEEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHEXPERIMENTS._options = None
    _SEARCHEXPERIMENTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETEXPERIMENT.fields_by_name['experiment_id']._options = None
    _GETEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _GETEXPERIMENT._options = None
    _GETEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETEEXPERIMENT.fields_by_name['experiment_id']._options = None
    _DELETEEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _DELETEEXPERIMENT._options = None
    _DELETEEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _RESTOREEXPERIMENT.fields_by_name['experiment_id']._options = None
    _RESTOREEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _RESTOREEXPERIMENT._options = None
    _RESTOREEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _UPDATEEXPERIMENT.fields_by_name['experiment_id']._options = None
    _UPDATEEXPERIMENT.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _UPDATEEXPERIMENT._options = None
    _UPDATEEXPERIMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CREATERUN._options = None
    _CREATERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _UPDATERUN._options = None
    _UPDATERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETERUN.fields_by_name['run_id']._options = None
    _DELETERUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _DELETERUN._options = None
    _DELETERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _RESTORERUN.fields_by_name['run_id']._options = None
    _RESTORERUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _RESTORERUN._options = None
    _RESTORERUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGMETRIC.fields_by_name['key']._options = None
    _LOGMETRIC.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _LOGMETRIC.fields_by_name['value']._options = None
    _LOGMETRIC.fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _LOGMETRIC.fields_by_name['timestamp']._options = None
    _LOGMETRIC.fields_by_name['timestamp']._serialized_options = b'\370\206\031\001'
    _LOGMETRIC.fields_by_name['model_id']._options = None
    _LOGMETRIC.fields_by_name['model_id']._serialized_options = b'\360\206\031\003'
    _LOGMETRIC.fields_by_name['dataset_name']._options = None
    _LOGMETRIC.fields_by_name['dataset_name']._serialized_options = b'\360\206\031\003'
    _LOGMETRIC.fields_by_name['dataset_digest']._options = None
    _LOGMETRIC.fields_by_name['dataset_digest']._serialized_options = b'\360\206\031\003'
    _LOGMETRIC._options = None
    _LOGMETRIC._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGPARAM.fields_by_name['key']._options = None
    _LOGPARAM.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _LOGPARAM.fields_by_name['value']._options = None
    _LOGPARAM.fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _LOGPARAM._options = None
    _LOGPARAM._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETEXPERIMENTTAG.fields_by_name['experiment_id']._options = None
    _SETEXPERIMENTTAG.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _SETEXPERIMENTTAG.fields_by_name['key']._options = None
    _SETEXPERIMENTTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _SETEXPERIMENTTAG.fields_by_name['value']._options = None
    _SETEXPERIMENTTAG.fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _SETEXPERIMENTTAG._options = None
    _SETEXPERIMENTTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETEEXPERIMENTTAG.fields_by_name['experiment_id']._options = None
    _DELETEEXPERIMENTTAG.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _DELETEEXPERIMENTTAG.fields_by_name['key']._options = None
    _DELETEEXPERIMENTTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _DELETEEXPERIMENTTAG._options = None
    _DELETEEXPERIMENTTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETTAG.fields_by_name['key']._options = None
    _SETTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _SETTAG.fields_by_name['value']._options = None
    _SETTAG.fields_by_name['value']._serialized_options = b'\370\206\031\001'
    _SETTAG._options = None
    _SETTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETETAG.fields_by_name['run_id']._options = None
    _DELETETAG.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _DELETETAG.fields_by_name['key']._options = None
    _DELETETAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _DELETETAG._options = None
    _DELETETAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETRUN._options = None
    _GETRUN._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHRUNS._options = None
    _SEARCHRUNS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LISTARTIFACTS._options = None
    _LISTARTIFACTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETMETRICHISTORY.fields_by_name['metric_key']._options = None
    _GETMETRICHISTORY.fields_by_name['metric_key']._serialized_options = b'\370\206\031\001'
    _GETMETRICHISTORY._options = None
    _GETMETRICHISTORY._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETMETRICHISTORYBULKINTERVAL.fields_by_name['metric_key']._options = None
    _GETMETRICHISTORYBULKINTERVAL.fields_by_name['metric_key']._serialized_options = b'\370\206\031\001'
    _GETMETRICHISTORYBULKINTERVAL._options = None
    _GETMETRICHISTORYBULKINTERVAL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGBATCH._options = None
    _LOGBATCH._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGMODEL._options = None
    _LOGMODEL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGINPUTS.fields_by_name['run_id']._options = None
    _LOGINPUTS.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _LOGINPUTS.fields_by_name['models']._options = None
    _LOGINPUTS.fields_by_name['models']._serialized_options = b'\360\206\031\003'
    _LOGINPUTS._options = None
    _LOGINPUTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGOUTPUTS.fields_by_name['run_id']._options = None
    _LOGOUTPUTS.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _LOGOUTPUTS._options = None
    _LOGOUTPUTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETEXPERIMENTBYNAME.fields_by_name['experiment_name']._options = None
    _GETEXPERIMENTBYNAME.fields_by_name['experiment_name']._serialized_options = b'\370\206\031\001'
    _GETEXPERIMENTBYNAME._options = None
    _GETEXPERIMENTBYNAME._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CREATEASSESSMENT.fields_by_name['assessment']._options = None
    _CREATEASSESSMENT.fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _CREATEASSESSMENT._options = None
    _CREATEASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _UPDATEASSESSMENT.fields_by_name['assessment']._options = None
    _UPDATEASSESSMENT.fields_by_name['assessment']._serialized_options = b'\370\206\031\001'
    _UPDATEASSESSMENT.fields_by_name['update_mask']._options = None
    _UPDATEASSESSMENT.fields_by_name['update_mask']._serialized_options = b'\370\206\031\001'
    _UPDATEASSESSMENT._options = None
    _UPDATEASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETEASSESSMENT.fields_by_name['trace_id']._options = None
    _DELETEASSESSMENT.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _DELETEASSESSMENT.fields_by_name['assessment_id']._options = None
    _DELETEASSESSMENT.fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _DELETEASSESSMENT._options = None
    _DELETEASSESSMENT._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETASSESSMENTREQUEST.fields_by_name['trace_id']._options = None
    _GETASSESSMENTREQUEST.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _GETASSESSMENTREQUEST.fields_by_name['assessment_id']._options = None
    _GETASSESSMENTREQUEST.fields_by_name['assessment_id']._serialized_options = b'\370\206\031\001'
    _GETASSESSMENTREQUEST._options = None
    _GETASSESSMENTREQUEST._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _STARTTRACE._options = None
    _STARTTRACE._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _ENDTRACE._options = None
    _ENDTRACE._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETTRACEINFO._options = None
    _GETTRACEINFO._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETTRACEINFOV3._options = None
    _GETTRACEINFOV3._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _BATCHGETTRACES._options = None
    _BATCHGETTRACES._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETTRACE.fields_by_name['trace_id']._options = None
    _GETTRACE.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _GETTRACE._options = None
    _GETTRACE._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHTRACES._options = None
    _SEARCHTRACES._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHUNIFIEDTRACES.fields_by_name['model_id']._options = None
    _SEARCHUNIFIEDTRACES.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _SEARCHUNIFIEDTRACES.fields_by_name['sql_warehouse_id']._options = None
    _SEARCHUNIFIEDTRACES.fields_by_name['sql_warehouse_id']._serialized_options = b'\370\206\031\001'
    _SEARCHUNIFIEDTRACES._options = None
    _SEARCHUNIFIEDTRACES._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETONLINETRACEDETAILS.fields_by_name['trace_id']._options = None
    _GETONLINETRACEDETAILS.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _GETONLINETRACEDETAILS.fields_by_name['sql_warehouse_id']._options = None
    _GETONLINETRACEDETAILS.fields_by_name['sql_warehouse_id']._serialized_options = b'\370\206\031\001'
    _GETONLINETRACEDETAILS.fields_by_name['source_inference_table']._options = None
    _GETONLINETRACEDETAILS.fields_by_name['source_inference_table']._serialized_options = b'\370\206\031\001'
    _GETONLINETRACEDETAILS.fields_by_name['source_databricks_request_id']._options = None
    _GETONLINETRACEDETAILS.fields_by_name['source_databricks_request_id']._serialized_options = b'\370\206\031\001'
    _DELETETRACES.fields_by_name['experiment_id']._options = None
    _DELETETRACES.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _DELETETRACES._options = None
    _DELETETRACES._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETETRACESV3.fields_by_name['experiment_id']._options = None
    _DELETETRACESV3.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _DELETETRACESV3._options = None
    _DELETETRACESV3._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CALCULATETRACEFILTERCORRELATION._options = None
    _CALCULATETRACEFILTERCORRELATION._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETTRACETAG._options = None
    _SETTRACETAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETTRACETAGV3._options = None
    _SETTRACETAGV3._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETETRACETAG._options = None
    _DELETETRACETAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETETRACETAGV3._options = None
    _DELETETRACETAGV3._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _TRACEINFOV3_TRACEMETADATAENTRY._options = None
    _TRACEINFOV3_TRACEMETADATAENTRY._serialized_options = b'8\001'
    _TRACEINFOV3_TAGSENTRY._options = None
    _TRACEINFOV3_TAGSENTRY._serialized_options = b'8\001'
    _STARTTRACEV3.fields_by_name['trace']._options = None
    _STARTTRACEV3.fields_by_name['trace']._serialized_options = b'\370\206\031\001'
    _LINKTRACESTORUN.fields_by_name['run_id']._options = None
    _LINKTRACESTORUN.fields_by_name['run_id']._serialized_options = b'\370\206\031\001'
    _LINKPROMPTSTOTRACE_PROMPTVERSIONREF.fields_by_name['name']._options = None
    _LINKPROMPTSTOTRACE_PROMPTVERSIONREF.fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _LINKPROMPTSTOTRACE_PROMPTVERSIONREF.fields_by_name['version']._options = None
    _LINKPROMPTSTOTRACE_PROMPTVERSIONREF.fields_by_name['version']._serialized_options = b'\370\206\031\001'
    _LINKPROMPTSTOTRACE.fields_by_name['trace_id']._options = None
    _LINKPROMPTSTOTRACE.fields_by_name['trace_id']._serialized_options = b'\370\206\031\001'
    _DATASETSUMMARY.fields_by_name['experiment_id']._options = None
    _DATASETSUMMARY.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _DATASETSUMMARY.fields_by_name['name']._options = None
    _DATASETSUMMARY.fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _DATASETSUMMARY.fields_by_name['digest']._options = None
    _DATASETSUMMARY.fields_by_name['digest']._serialized_options = b'\370\206\031\001'
    _SEARCHDATASETS._options = None
    _SEARCHDATASETS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CREATELOGGEDMODEL.fields_by_name['experiment_id']._options = None
    _CREATELOGGEDMODEL.fields_by_name['experiment_id']._serialized_options = b'\370\206\031\001'
    _CREATELOGGEDMODEL._options = None
    _CREATELOGGEDMODEL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _FINALIZELOGGEDMODEL.fields_by_name['model_id']._options = None
    _FINALIZELOGGEDMODEL.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _FINALIZELOGGEDMODEL.fields_by_name['status']._options = None
    _FINALIZELOGGEDMODEL.fields_by_name['status']._serialized_options = b'\370\206\031\001'
    _FINALIZELOGGEDMODEL._options = None
    _FINALIZELOGGEDMODEL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETLOGGEDMODEL.fields_by_name['model_id']._options = None
    _GETLOGGEDMODEL.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _GETLOGGEDMODEL._options = None
    _GETLOGGEDMODEL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETELOGGEDMODEL.fields_by_name['model_id']._options = None
    _DELETELOGGEDMODEL.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _DELETELOGGEDMODEL._options = None
    _DELETELOGGEDMODEL._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHLOGGEDMODELS_DATASET.fields_by_name['dataset_name']._options = None
    _SEARCHLOGGEDMODELS_DATASET.fields_by_name['dataset_name']._serialized_options = b'\370\206\031\001'
    _SEARCHLOGGEDMODELS_ORDERBY.fields_by_name['field_name']._options = None
    _SEARCHLOGGEDMODELS_ORDERBY.fields_by_name['field_name']._serialized_options = b'\370\206\031\001'
    _SEARCHLOGGEDMODELS._options = None
    _SEARCHLOGGEDMODELS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETLOGGEDMODELTAGS.fields_by_name['model_id']._options = None
    _SETLOGGEDMODELTAGS.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _SETLOGGEDMODELTAGS._options = None
    _SETLOGGEDMODELTAGS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETELOGGEDMODELTAG.fields_by_name['model_id']._options = None
    _DELETELOGGEDMODELTAG.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _DELETELOGGEDMODELTAG.fields_by_name['tag_key']._options = None
    _DELETELOGGEDMODELTAG.fields_by_name['tag_key']._serialized_options = b'\370\206\031\001'
    _DELETELOGGEDMODELTAG._options = None
    _DELETELOGGEDMODELTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LISTLOGGEDMODELARTIFACTS.fields_by_name['model_id']._options = None
    _LISTLOGGEDMODELARTIFACTS.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _LISTLOGGEDMODELARTIFACTS._options = None
    _LISTLOGGEDMODELARTIFACTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LOGLOGGEDMODELPARAMSREQUEST.fields_by_name['model_id']._options = None
    _LOGLOGGEDMODELPARAMSREQUEST.fields_by_name['model_id']._serialized_options = b'\370\206\031\001'
    _LOGLOGGEDMODELPARAMSREQUEST._options = None
    _LOGLOGGEDMODELPARAMSREQUEST._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHTRACESV3._options = None
    _SEARCHTRACESV3._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _CREATEDATASET.fields_by_name['name']._options = None
    _CREATEDATASET.fields_by_name['name']._serialized_options = b'\370\206\031\001'
    _CREATEDATASET._options = None
    _CREATEDATASET._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETDATASET.fields_by_name['dataset_id']._options = None
    _GETDATASET.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _GETDATASET._options = None
    _GETDATASET._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETEDATASET.fields_by_name['dataset_id']._options = None
    _DELETEDATASET.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _DELETEDATASET._options = None
    _DELETEDATASET._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SEARCHEVALUATIONDATASETS._options = None
    _SEARCHEVALUATIONDATASETS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _SETDATASETTAGS.fields_by_name['dataset_id']._options = None
    _SETDATASETTAGS.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _SETDATASETTAGS.fields_by_name['tags']._options = None
    _SETDATASETTAGS.fields_by_name['tags']._serialized_options = b'\370\206\031\001'
    _SETDATASETTAGS._options = None
    _SETDATASETTAGS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETEDATASETTAG.fields_by_name['dataset_id']._options = None
    _DELETEDATASETTAG.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _DELETEDATASETTAG.fields_by_name['key']._options = None
    _DELETEDATASETTAG.fields_by_name['key']._serialized_options = b'\370\206\031\001'
    _DELETEDATASETTAG._options = None
    _DELETEDATASETTAG._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _UPSERTDATASETRECORDS.fields_by_name['dataset_id']._options = None
    _UPSERTDATASETRECORDS.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _UPSERTDATASETRECORDS.fields_by_name['records']._options = None
    _UPSERTDATASETRECORDS.fields_by_name['records']._serialized_options = b'\370\206\031\001'
    _UPSERTDATASETRECORDS._options = None
    _UPSERTDATASETRECORDS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETDATASETEXPERIMENTIDS.fields_by_name['dataset_id']._options = None
    _GETDATASETEXPERIMENTIDS.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _GETDATASETEXPERIMENTIDS._options = None
    _GETDATASETEXPERIMENTIDS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETDATASETRECORDS.fields_by_name['dataset_id']._options = None
    _GETDATASETRECORDS.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _GETDATASETRECORDS._options = None
    _GETDATASETRECORDS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _ADDDATASETTOEXPERIMENTS.fields_by_name['dataset_id']._options = None
    _ADDDATASETTOEXPERIMENTS.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _ADDDATASETTOEXPERIMENTS._options = None
    _ADDDATASETTOEXPERIMENTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _REMOVEDATASETFROMEXPERIMENTS.fields_by_name['dataset_id']._options = None
    _REMOVEDATASETFROMEXPERIMENTS.fields_by_name['dataset_id']._serialized_options = b'\370\206\031\001'
    _REMOVEDATASETFROMEXPERIMENTS._options = None
    _REMOVEDATASETFROMEXPERIMENTS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _REGISTERSCORER._options = None
    _REGISTERSCORER._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LISTSCORERS._options = None
    _LISTSCORERS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _LISTSCORERVERSIONS._options = None
    _LISTSCORERVERSIONS._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GETSCORER._options = None
    _GETSCORER._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _DELETESCORER._options = None
    _DELETESCORER._serialized_options = b'\342?(\n&com.databricks.rpc.RPC[$this.Response]'
    _GATEWAYSECRETINFO_MASKEDVALUESENTRY._options = None
    _GATEWAYSECRETINFO_MASKEDVALUESENTRY._serialized_options = b'8\001'
    _CREATEGATEWAYSECRET_SECRETVALUEENTRY._options = None
    _CREATEGATEWAYSECRET_SECRETVALUEENTRY._serialized_options = b'8\001'
    _UPDATEGATEWAYSECRET_SECRETVALUEENTRY._options = None
    _UPDATEGATEWAYSECRET_SECRETVALUEENTRY._serialized_options = b'8\001'
    _MLFLOWSERVICE.methods_by_name['getExperimentByName']._options = None
    _MLFLOWSERVICE.methods_by_name['getExperimentByName']._serialized_options = b'\362\206\031H\n,\n\003GET\022\037/mlflow/experiments/get-by-name\032\004\010\002\020\000\020\001*\026Get Experiment By Name'
    _MLFLOWSERVICE.methods_by_name['createExperiment']._options = None
    _MLFLOWSERVICE.methods_by_name['createExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/create\032\004\010\002\020\000\020\001*\021Create Experiment'
    _MLFLOWSERVICE.methods_by_name['searchExperiments']._options = None
    _MLFLOWSERVICE.methods_by_name['searchExperiments']._serialized_options = b'\362\206\031i\n(\n\004POST\022\032/mlflow/experiments/search\032\004\010\002\020\000\n\'\n\003GET\022\032/mlflow/experiments/search\032\004\010\002\020\000\020\001*\022Search Experiments'
    _MLFLOWSERVICE.methods_by_name['getExperiment']._options = None
    _MLFLOWSERVICE.methods_by_name['getExperiment']._serialized_options = b'\362\206\0318\n$\n\003GET\022\027/mlflow/experiments/get\032\004\010\002\020\000\020\001*\016Get Experiment\272\214\031\000'
    _MLFLOWSERVICE.methods_by_name['deleteExperiment']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/delete\032\004\010\002\020\000\020\001*\021Delete Experiment'
    _MLFLOWSERVICE.methods_by_name['restoreExperiment']._options = None
    _MLFLOWSERVICE.methods_by_name['restoreExperiment']._serialized_options = b'\362\206\031A\n)\n\004POST\022\033/mlflow/experiments/restore\032\004\010\002\020\000\020\001*\022Restore Experiment'
    _MLFLOWSERVICE.methods_by_name['updateExperiment']._options = None
    _MLFLOWSERVICE.methods_by_name['updateExperiment']._serialized_options = b'\362\206\031?\n(\n\004POST\022\032/mlflow/experiments/update\032\004\010\002\020\000\020\001*\021Update Experiment'
    _MLFLOWSERVICE.methods_by_name['createRun']._options = None
    _MLFLOWSERVICE.methods_by_name['createRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/create\032\004\010\002\020\000\020\001*\nCreate Run'
    _MLFLOWSERVICE.methods_by_name['updateRun']._options = None
    _MLFLOWSERVICE.methods_by_name['updateRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/update\032\004\010\002\020\000\020\001*\nUpdate Run'
    _MLFLOWSERVICE.methods_by_name['deleteRun']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteRun']._serialized_options = b'\362\206\0311\n!\n\004POST\022\023/mlflow/runs/delete\032\004\010\002\020\000\020\001*\nDelete Run'
    _MLFLOWSERVICE.methods_by_name['restoreRun']._options = None
    _MLFLOWSERVICE.methods_by_name['restoreRun']._serialized_options = b'\362\206\0313\n\"\n\004POST\022\024/mlflow/runs/restore\032\004\010\002\020\000\020\001*\013Restore Run'
    _MLFLOWSERVICE.methods_by_name['logMetric']._options = None
    _MLFLOWSERVICE.methods_by_name['logMetric']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/log-metric\032\004\010\002\020\000\020\001*\nLog Metric'
    _MLFLOWSERVICE.methods_by_name['logParam']._options = None
    _MLFLOWSERVICE.methods_by_name['logParam']._serialized_options = b'\362\206\0317\n(\n\004POST\022\032/mlflow/runs/log-parameter\032\004\010\002\020\000\020\001*\tLog Param'
    _MLFLOWSERVICE.methods_by_name['setExperimentTag']._options = None
    _MLFLOWSERVICE.methods_by_name['setExperimentTag']._serialized_options = b'\362\206\031L\n4\n\004POST\022&/mlflow/experiments/set-experiment-tag\032\004\010\002\020\000\020\001*\022Set Experiment Tag'
    _MLFLOWSERVICE.methods_by_name['deleteExperimentTag']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteExperimentTag']._serialized_options = b'\362\206\031R\n7\n\004POST\022)/mlflow/experiments/delete-experiment-tag\032\004\010\002\020\000\020\001*\025Delete Experiment Tag'
    _MLFLOWSERVICE.methods_by_name['setTag']._options = None
    _MLFLOWSERVICE.methods_by_name['setTag']._serialized_options = b'\362\206\031/\n\"\n\004POST\022\024/mlflow/runs/set-tag\032\004\010\002\020\000\020\001*\007Set Tag'
    _MLFLOWSERVICE.methods_by_name['setTraceTag']._options = None
    _MLFLOWSERVICE.methods_by_name['setTraceTag']._serialized_options = b'\362\206\031B\n/\n\005PATCH\022 /mlflow/traces/{request_id}/tags\032\004\010\002\020\000\020\003*\rSet Trace Tag'
    _MLFLOWSERVICE.methods_by_name['setTraceTagV3']._options = None
    _MLFLOWSERVICE.methods_by_name['setTraceTagV3']._serialized_options = b'\362\206\031C\n-\n\005PATCH\022\036/mlflow/traces/{trace_id}/tags\032\004\010\003\020\000\020\003*\020Set Trace Tag V3'
    _MLFLOWSERVICE.methods_by_name['deleteTraceTag']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteTraceTag']._serialized_options = b'\362\206\031F\n0\n\006DELETE\022 /mlflow/traces/{request_id}/tags\032\004\010\002\020\000\020\003*\020Delete Trace Tag'
    _MLFLOWSERVICE.methods_by_name['deleteTraceTagV3']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteTraceTagV3']._serialized_options = b'\362\206\031G\n.\n\006DELETE\022\036/mlflow/traces/{trace_id}/tags\032\004\010\003\020\000\020\003*\023Delete Trace Tag V3'
    _MLFLOWSERVICE.methods_by_name['deleteTag']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteTag']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/delete-tag\032\004\010\002\020\000\020\001*\nDelete Tag'
    _MLFLOWSERVICE.methods_by_name['getRun']._options = None
    _MLFLOWSERVICE.methods_by_name['getRun']._serialized_options = b'\362\206\031*\n\035\n\003GET\022\020/mlflow/runs/get\032\004\010\002\020\000\020\001*\007Get Run\272\214\031\000'
    _MLFLOWSERVICE.methods_by_name['searchRuns']._options = None
    _MLFLOWSERVICE.methods_by_name['searchRuns']._serialized_options = b'\362\206\0312\n!\n\004POST\022\023/mlflow/runs/search\032\004\010\002\020\000\020\001*\013Search Runs\272\214\031\000'
    _MLFLOWSERVICE.methods_by_name['listArtifacts']._options = None
    _MLFLOWSERVICE.methods_by_name['listArtifacts']._serialized_options = b'\362\206\0317\n#\n\003GET\022\026/mlflow/artifacts/list\032\004\010\002\020\000\020\001*\016List Artifacts\272\214\031\000'
    _MLFLOWSERVICE.methods_by_name['getMetricHistory']._options = None
    _MLFLOWSERVICE.methods_by_name['getMetricHistory']._serialized_options = b'\362\206\031@\n(\n\003GET\022\033/mlflow/metrics/get-history\032\004\010\002\020\000\020\001*\022Get Metric History'
    _MLFLOWSERVICE.methods_by_name['getMetricHistoryBulkInterval']._options = None
    _MLFLOWSERVICE.methods_by_name['getMetricHistoryBulkInterval']._serialized_options = b'\362\206\031:\n6\n\003GET\022)/mlflow/metrics/get-history-bulk-interval\032\004\010\002\020\013\020\003\272\214\031\000'
    _MLFLOWSERVICE.methods_by_name['logBatch']._options = None
    _MLFLOWSERVICE.methods_by_name['logBatch']._serialized_options = b'\362\206\0313\n$\n\004POST\022\026/mlflow/runs/log-batch\032\004\010\002\020\000\020\001*\tLog Batch'
    _MLFLOWSERVICE.methods_by_name['logModel']._options = None
    _MLFLOWSERVICE.methods_by_name['logModel']._serialized_options = b'\362\206\0313\n$\n\004POST\022\026/mlflow/runs/log-model\032\004\010\002\020\000\020\001*\tLog Model'
    _MLFLOWSERVICE.methods_by_name['logInputs']._options = None
    _MLFLOWSERVICE.methods_by_name['logInputs']._serialized_options = b'\362\206\0315\n%\n\004POST\022\027/mlflow/runs/log-inputs\032\004\010\002\020\000\020\001*\nLog Inputs'
    _MLFLOWSERVICE.methods_by_name['logOutputs']._options = None
    _MLFLOWSERVICE.methods_by_name['logOutputs']._serialized_options = b'\362\206\0313\n\"\n\004POST\022\024/mlflow/runs/outputs\032\004\010\002\020\000\020\003*\013Log Outputs'
    _MLFLOWSERVICE.methods_by_name['searchDatasets']._options = None
    _MLFLOWSERVICE.methods_by_name['searchDatasets']._serialized_options = b'\362\206\0314\n0\n\004POST\022\"mlflow/experiments/search-datasets\032\004\010\002\020\000\020\003\272\214\031\000'
    _MLFLOWSERVICE.methods_by_name['startTrace']._options = None
    _MLFLOWSERVICE.methods_by_name['startTrace']._serialized_options = b'\362\206\031-\n\034\n\004POST\022\016/mlflow/traces\032\004\010\002\020\000\020\003*\013Start Trace'
    _MLFLOWSERVICE.methods_by_name['endTrace']._options = None
    _MLFLOWSERVICE.methods_by_name['endTrace']._serialized_options = b'\362\206\0319\n*\n\005PATCH\022\033/mlflow/traces/{request_id}\032\004\010\002\020\000\020\003*\tEnd Trace'
    _MLFLOWSERVICE.methods_by_name['getTraceInfo']._options = None
    _MLFLOWSERVICE.methods_by_name['getTraceInfo']._serialized_options = b'\362\206\031@\n-\n\003GET\022 /mlflow/traces/{request_id}/info\032\004\010\002\020\000\020\003*\rGet TraceInfo'
    _MLFLOWSERVICE.methods_by_name['getTraceInfoV3']._options = None
    _MLFLOWSERVICE.methods_by_name['getTraceInfoV3']._serialized_options = b'\362\206\031<\n&\n\003GET\022\031/mlflow/traces/{trace_id}\032\004\010\003\020\000\020\003*\020Get TraceInfo v3'
    _MLFLOWSERVICE.methods_by_name['getTrace']._options = None
    _MLFLOWSERVICE.methods_by_name['getTrace']._serialized_options = b'\362\206\0311\n\037\n\003GET\022\022/mlflow/traces/get\032\004\010\003\020\000\020\003*\014Get Trace v3'
    _MLFLOWSERVICE.methods_by_name['batchGetTraces']._options = None
    _MLFLOWSERVICE.methods_by_name['batchGetTraces']._serialized_options = b'\362\206\0314\n$\n\003GET\022\027/mlflow/traces/batchGet\032\004\010\003\020\000\020\003*\nGet Traces'
    _MLFLOWSERVICE.methods_by_name['searchTraces']._options = None
    _MLFLOWSERVICE.methods_by_name['searchTraces']._serialized_options = b'\362\206\031.\n\033\n\003GET\022\016/mlflow/traces\032\004\010\002\020\000\020\003*\rSearch Traces'
    _MLFLOWSERVICE.methods_by_name['searchTracesV3']._options = None
    _MLFLOWSERVICE.methods_by_name['searchTracesV3']._serialized_options = b'\362\206\0319\n#\n\004POST\022\025/mlflow/traces/search\032\004\010\003\020\000\020\003*\020Search Traces V3'
    _MLFLOWSERVICE.methods_by_name['startTraceV3']._options = None
    _MLFLOWSERVICE.methods_by_name['startTraceV3']._serialized_options = b'\362\206\031 \n\034\n\004POST\022\016/mlflow/traces\032\004\010\003\020\000\020\003'
    _MLFLOWSERVICE.methods_by_name['linkTracesToRun']._options = None
    _MLFLOWSERVICE.methods_by_name['linkTracesToRun']._serialized_options = b'\362\206\031@\n(\n\004POST\022\032/mlflow/traces/link-to-run\032\004\010\002\020\000\020\003*\022Link Traces to Run'
    _MLFLOWSERVICE.methods_by_name['linkPromptsToTrace']._options = None
    _MLFLOWSERVICE.methods_by_name['linkPromptsToTrace']._serialized_options = b'\362\206\031D\n)\n\004POST\022\033/mlflow/traces/link-prompts\032\004\010\002\020\000\020\003*\025Link Prompts to Trace'
    _MLFLOWSERVICE.methods_by_name['searchUnifiedTraceHandler']._options = None
    _MLFLOWSERVICE.methods_by_name['searchUnifiedTraceHandler']._serialized_options = b'\362\206\031>\n#\n\003GET\022\026/mlflow/unified-traces\032\004\010\002\020\000\020\003*\025Search Unified Traces'
    _MLFLOWSERVICE.methods_by_name['getOnlineTraceDetails']._options = None
    _MLFLOWSERVICE.methods_by_name['getOnlineTraceDetails']._serialized_options = b'\362\206\031K\n-\n\003GET\022 /mlflow/get-online-trace-details\032\004\010\002\020\000\020\003*\030Get Online Trace Details'
    _MLFLOWSERVICE.methods_by_name['deleteTraces']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteTraces']._serialized_options = b'\362\206\031=\n*\n\004POST\022\034/mlflow/traces/delete-traces\032\004\010\002\020\000\020\003*\rDelete Traces'
    _MLFLOWSERVICE.methods_by_name['deleteTracesV3']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteTracesV3']._serialized_options = b'\362\206\031@\n*\n\004POST\022\034/mlflow/traces/delete-traces\032\004\010\003\020\000\020\003*\020Delete Traces V3'
    _MLFLOWSERVICE.methods_by_name['calculateTraceFilterCorrelation']._options = None
    _MLFLOWSERVICE.methods_by_name['calculateTraceFilterCorrelation']._serialized_options = b'\362\206\031a\n9\n\004POST\022+/mlflow/traces/calculate-filter-correlation\032\004\010\003\020\000\020\003*\"Calculate Trace Filter Correlation'
    _MLFLOWSERVICE.methods_by_name['createLoggedModel']._options = None
    _MLFLOWSERVICE.methods_by_name['createLoggedModel']._serialized_options = b'\362\206\031<\n#\n\004POST\022\025/mlflow/logged-models\032\004\010\002\020\000\020\003*\023Create Logged Model'
    _MLFLOWSERVICE.methods_by_name['finalizeLoggedModel']._options = None
    _MLFLOWSERVICE.methods_by_name['finalizeLoggedModel']._serialized_options = b'\362\206\031J\n/\n\005PATCH\022 /mlflow/logged-models/{model_id}\032\004\010\002\020\000\020\003*\025Finalize Logged Model'
    _MLFLOWSERVICE.methods_by_name['getLoggedModel']._options = None
    _MLFLOWSERVICE.methods_by_name['getLoggedModel']._serialized_options = b'\362\206\031C\n-\n\003GET\022 /mlflow/logged-models/{model_id}\032\004\010\002\020\000\020\003*\020Get Logged Model'
    _MLFLOWSERVICE.methods_by_name['deleteLoggedModel']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteLoggedModel']._serialized_options = b'\362\206\031K\n0\n\006DELETE\022 /mlflow/logged-models/{model_id}\032\004\010\002\020\000\020\003*\025Delete a Logged Model'
    _MLFLOWSERVICE.methods_by_name['searchLoggedModels']._options = None
    _MLFLOWSERVICE.methods_by_name['searchLoggedModels']._serialized_options = b'\362\206\031C\n*\n\004POST\022\034/mlflow/logged-models/search\032\004\010\002\020\000\020\003*\023Search LoggedModels'
    _MLFLOWSERVICE.methods_by_name['setLoggedModelTags']._options = None
    _MLFLOWSERVICE.methods_by_name['setLoggedModelTags']._serialized_options = b'\362\206\031N\n4\n\005PATCH\022%/mlflow/logged-models/{model_id}/tags\032\004\010\002\020\000\020\003*\024Set Logged Model Tag'
    _MLFLOWSERVICE.methods_by_name['deleteLoggedModelTag']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteLoggedModelTag']._serialized_options = b'\362\206\031\\\n?\n\006DELETE\022//mlflow/logged-models/{model_id}/tags/{tag_key}\032\004\010\002\020\000\020\003*\027Delete Logged Model Tag'
    _MLFLOWSERVICE.methods_by_name['listLoggedModelArtifacts']._options = None
    _MLFLOWSERVICE.methods_by_name['listLoggedModelArtifacts']._serialized_options = b'\362\206\031i\nC\n\003GET\0226/mlflow/logged-models/{model_id}/artifacts/directories\032\004\010\002\020\000\020\003* List Artifacts for Logged Models'
    _MLFLOWSERVICE.methods_by_name['LogLoggedModelParams']._options = None
    _MLFLOWSERVICE.methods_by_name['LogLoggedModelParams']._serialized_options = b'\362\206\031R\n5\n\004POST\022\'/mlflow/logged-models/{model_id}/params\032\004\010\002\020\000\020\003*\027Log Logged Model Params'
    _MLFLOWSERVICE.methods_by_name['GetAssessment']._options = None
    _MLFLOWSERVICE.methods_by_name['GetAssessment']._serialized_options = b'\362\206\031V\nB\n\003GET\0225/mlflow/traces/{trace_id}/assessments/{assessment_id}\032\004\010\003\020\000\020\003*\016Get Assessment'
    _MLFLOWSERVICE.methods_by_name['createAssessment']._options = None
    _MLFLOWSERVICE.methods_by_name['createAssessment']._serialized_options = b'\362\206\031\210\001\n>\n\004POST\0220/mlflow/traces/{assessment.trace_id}/assessments\032\004\010\003\020\000\020\003\030\350\007\030\356\007\030\014\030\001*:Create an assessment of a trace or a span within the trace'
    _MLFLOWSERVICE.methods_by_name['updateAssessment']._options = None
    _MLFLOWSERVICE.methods_by_name['updateAssessment']._serialized_options = b'\362\206\031{\nD\n\005PATCH\0225/mlflow/traces/{trace_id}/assessments/{assessment_id}\032\004\010\003\020\000\020\003\030\350\007\030\356\007\030\001*)Update an existing assessment on a trace.'
    _MLFLOWSERVICE.methods_by_name['deleteAssessment']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteAssessment']._serialized_options = b'\362\206\031\\\nE\n\006DELETE\0225/mlflow/traces/{trace_id}/assessments/{assessment_id}\032\004\010\003\020\000\020\003*\021Delete Assessment'
    _MLFLOWSERVICE.methods_by_name['createDataset']._options = None
    _MLFLOWSERVICE.methods_by_name['createDataset']._serialized_options = b'\362\206\031N\n%\n\004POST\022\027/mlflow/datasets/create\032\004\010\003\020\000\020\003\030\350\007\030\356\007\030\014\030\001*\031Create Evaluation Dataset'
    _MLFLOWSERVICE.methods_by_name['getDataset']._options = None
    _MLFLOWSERVICE.methods_by_name['getDataset']._serialized_options = b'\362\206\031N\n*\n\003GET\022\035/mlflow/datasets/{dataset_id}\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\026Get Evaluation Dataset'
    _MLFLOWSERVICE.methods_by_name['deleteDataset']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteDataset']._serialized_options = b'\362\206\031T\n-\n\006DELETE\022\035/mlflow/datasets/{dataset_id}\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\031Delete Evaluation Dataset'
    _MLFLOWSERVICE.methods_by_name['searchEvaluationDatasets']._options = None
    _MLFLOWSERVICE.methods_by_name['searchEvaluationDatasets']._serialized_options = b'\362\206\031p\n%\n\004POST\022\027/mlflow/datasets/search\032\004\010\003\020\000\n$\n\003GET\022\027/mlflow/datasets/search\032\004\010\003\020\000\020\003\030\350\007\030\001*\032Search Evaluation Datasets'
    _MLFLOWSERVICE.methods_by_name['setDatasetTags']._options = None
    _MLFLOWSERVICE.methods_by_name['setDatasetTags']._serialized_options = b'\362\206\031Z\n1\n\005PATCH\022\"/mlflow/datasets/{dataset_id}/tags\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\033Set Evaluation Dataset Tags'
    _MLFLOWSERVICE.methods_by_name['deleteDatasetTag']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteDatasetTag']._serialized_options = b'\362\206\031c\n8\n\006DELETE\022(/mlflow/datasets/{dataset_id}/tags/{key}\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\035Delete Evaluation Dataset Tag'
    _MLFLOWSERVICE.methods_by_name['upsertDatasetRecords']._options = None
    _MLFLOWSERVICE.methods_by_name['upsertDatasetRecords']._serialized_options = b'\362\206\031b\n3\n\004POST\022%/mlflow/datasets/{dataset_id}/records\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*!Upsert Evaluation Dataset Records'
    _MLFLOWSERVICE.methods_by_name['getDatasetExperimentIds']._options = None
    _MLFLOWSERVICE.methods_by_name['getDatasetExperimentIds']._serialized_options = b'\362\206\031l\n9\n\003GET\022,/mlflow/datasets/{dataset_id}/experiment-ids\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*%Get Evaluation Dataset Experiment IDs'
    _MLFLOWSERVICE.methods_by_name['registerScorer']._options = None
    _MLFLOWSERVICE.methods_by_name['registerScorer']._serialized_options = b'\362\206\031;\n&\n\004POST\022\030/mlflow/scorers/register\032\004\010\003\020\000\020\001*\017Register Scorer'
    _MLFLOWSERVICE.methods_by_name['listScorers']._options = None
    _MLFLOWSERVICE.methods_by_name['listScorers']._serialized_options = b'\362\206\0313\n!\n\003GET\022\024/mlflow/scorers/list\032\004\010\003\020\000\020\001*\014List Scorers'
    _MLFLOWSERVICE.methods_by_name['listScorerVersions']._options = None
    _MLFLOWSERVICE.methods_by_name['listScorerVersions']._serialized_options = b'\362\206\031?\n%\n\003GET\022\030/mlflow/scorers/versions\032\004\010\003\020\000\020\001*\024List Scorer Versions'
    _MLFLOWSERVICE.methods_by_name['getScorer']._options = None
    _MLFLOWSERVICE.methods_by_name['getScorer']._serialized_options = b'\362\206\0310\n \n\003GET\022\023/mlflow/scorers/get\032\004\010\003\020\000\020\001*\nGet Scorer'
    _MLFLOWSERVICE.methods_by_name['deleteScorer']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteScorer']._serialized_options = b'\362\206\0319\n&\n\006DELETE\022\026/mlflow/scorers/delete\032\004\010\003\020\000\020\001*\rDelete Scorer'
    _MLFLOWSERVICE.methods_by_name['getDatasetRecords']._options = None
    _MLFLOWSERVICE.methods_by_name['getDatasetRecords']._serialized_options = b'\362\206\031^\n2\n\003GET\022%/mlflow/datasets/{dataset_id}/records\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\036Get Evaluation Dataset Records'
    _MLFLOWSERVICE.methods_by_name['addDatasetToExperiments']._options = None
    _MLFLOWSERVICE.methods_by_name['addDatasetToExperiments']._serialized_options = b'\362\206\031c\n;\n\004POST\022-/mlflow/datasets/{dataset_id}/add-experiments\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\032Add Dataset to Experiments'
    _MLFLOWSERVICE.methods_by_name['removeDatasetFromExperiments']._options = None
    _MLFLOWSERVICE.methods_by_name['removeDatasetFromExperiments']._serialized_options = b'\362\206\031k\n>\n\004POST\0220/mlflow/datasets/{dataset_id}/remove-experiments\032\004\010\003\020\000\020\003\030\350\007\030\272\027\030\001*\037Remove Dataset from Experiments'
    _MLFLOWSERVICE.methods_by_name['createGatewaySecret']._options = None
    _MLFLOWSERVICE.methods_by_name['createGatewaySecret']._serialized_options = b'\362\206\031G\n,\n\004POST\022\036/mlflow/gateway/secrets/create\032\004\010\003\020\000\020\001*\025Create Gateway Secret'
    _MLFLOWSERVICE.methods_by_name['getGatewaySecretInfo']._options = None
    _MLFLOWSERVICE.methods_by_name['getGatewaySecretInfo']._serialized_options = b'\362\206\031E\n(\n\003GET\022\033/mlflow/gateway/secrets/get\032\004\010\003\020\000\020\001*\027Get Gateway Secret Info'
    _MLFLOWSERVICE.methods_by_name['updateGatewaySecret']._options = None
    _MLFLOWSERVICE.methods_by_name['updateGatewaySecret']._serialized_options = b'\362\206\031G\n,\n\004POST\022\036/mlflow/gateway/secrets/update\032\004\010\003\020\000\020\001*\025Update Gateway Secret'
    _MLFLOWSERVICE.methods_by_name['deleteGatewaySecret']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteGatewaySecret']._serialized_options = b'\362\206\031I\n.\n\006DELETE\022\036/mlflow/gateway/secrets/delete\032\004\010\003\020\000\020\001*\025Delete Gateway Secret'
    _MLFLOWSERVICE.methods_by_name['listGatewaySecretInfos']._options = None
    _MLFLOWSERVICE.methods_by_name['listGatewaySecretInfos']._serialized_options = b'\362\206\031C\n)\n\003GET\022\034/mlflow/gateway/secrets/list\032\004\010\003\020\000\020\001*\024List Gateway Secrets'
    _MLFLOWSERVICE.methods_by_name['createGatewayEndpoint']._options = None
    _MLFLOWSERVICE.methods_by_name['createGatewayEndpoint']._serialized_options = b'\362\206\031K\n.\n\004POST\022 /mlflow/gateway/endpoints/create\032\004\010\003\020\000\020\001*\027Create Gateway Endpoint'
    _MLFLOWSERVICE.methods_by_name['getGatewayEndpoint']._options = None
    _MLFLOWSERVICE.methods_by_name['getGatewayEndpoint']._serialized_options = b'\362\206\031D\n*\n\003GET\022\035/mlflow/gateway/endpoints/get\032\004\010\003\020\000\020\001*\024Get Gateway Endpoint'
    _MLFLOWSERVICE.methods_by_name['updateGatewayEndpoint']._options = None
    _MLFLOWSERVICE.methods_by_name['updateGatewayEndpoint']._serialized_options = b'\362\206\031K\n.\n\004POST\022 /mlflow/gateway/endpoints/update\032\004\010\003\020\000\020\001*\027Update Gateway Endpoint'
    _MLFLOWSERVICE.methods_by_name['deleteGatewayEndpoint']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteGatewayEndpoint']._serialized_options = b'\362\206\031M\n0\n\006DELETE\022 /mlflow/gateway/endpoints/delete\032\004\010\003\020\000\020\001*\027Delete Gateway Endpoint'
    _MLFLOWSERVICE.methods_by_name['listGatewayEndpoints']._options = None
    _MLFLOWSERVICE.methods_by_name['listGatewayEndpoints']._serialized_options = b'\362\206\031G\n+\n\003GET\022\036/mlflow/gateway/endpoints/list\032\004\010\003\020\000\020\001*\026List Gateway Endpoints'
    _MLFLOWSERVICE.methods_by_name['createGatewayModelDefinition']._options = None
    _MLFLOWSERVICE.methods_by_name['createGatewayModelDefinition']._serialized_options = b'\362\206\031[\n6\n\004POST\022(/mlflow/gateway/model-definitions/create\032\004\010\003\020\000\020\001*\037Create Gateway Model Definition'
    _MLFLOWSERVICE.methods_by_name['getGatewayModelDefinition']._options = None
    _MLFLOWSERVICE.methods_by_name['getGatewayModelDefinition']._serialized_options = b'\362\206\031T\n2\n\003GET\022%/mlflow/gateway/model-definitions/get\032\004\010\003\020\000\020\001*\034Get Gateway Model Definition'
    _MLFLOWSERVICE.methods_by_name['listGatewayModelDefinitions']._options = None
    _MLFLOWSERVICE.methods_by_name['listGatewayModelDefinitions']._serialized_options = b'\362\206\031W\n3\n\003GET\022&/mlflow/gateway/model-definitions/list\032\004\010\003\020\000\020\001*\036List Gateway Model Definitions'
    _MLFLOWSERVICE.methods_by_name['updateGatewayModelDefinition']._options = None
    _MLFLOWSERVICE.methods_by_name['updateGatewayModelDefinition']._serialized_options = b'\362\206\031[\n6\n\004POST\022(/mlflow/gateway/model-definitions/update\032\004\010\003\020\000\020\001*\037Update Gateway Model Definition'
    _MLFLOWSERVICE.methods_by_name['deleteGatewayModelDefinition']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteGatewayModelDefinition']._serialized_options = b'\362\206\031]\n8\n\006DELETE\022(/mlflow/gateway/model-definitions/delete\032\004\010\003\020\000\020\001*\037Delete Gateway Model Definition'
    _MLFLOWSERVICE.methods_by_name['attachModelToEndpoint']._options = None
    _MLFLOWSERVICE.methods_by_name['attachModelToEndpoint']._serialized_options = b'\362\206\031S\n5\n\004POST\022\'/mlflow/gateway/endpoints/models/attach\032\004\010\003\020\000\020\001*\030Attach Model to Endpoint'
    _MLFLOWSERVICE.methods_by_name['detachModelFromEndpoint']._options = None
    _MLFLOWSERVICE.methods_by_name['detachModelFromEndpoint']._serialized_options = b'\362\206\031U\n5\n\004POST\022\'/mlflow/gateway/endpoints/models/detach\032\004\010\003\020\000\020\001*\032Detach Model from Endpoint'
    _MLFLOWSERVICE.methods_by_name['createEndpointBinding']._options = None
    _MLFLOWSERVICE.methods_by_name['createEndpointBinding']._serialized_options = b'\362\206\031T\n7\n\004POST\022)/mlflow/gateway/endpoints/bindings/create\032\004\010\003\020\000\020\001*\027Create Endpoint Binding'
    _MLFLOWSERVICE.methods_by_name['deleteEndpointBinding']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteEndpointBinding']._serialized_options = b'\362\206\031V\n9\n\006DELETE\022)/mlflow/gateway/endpoints/bindings/delete\032\004\010\003\020\000\020\001*\027Delete Endpoint Binding'
    _MLFLOWSERVICE.methods_by_name['listEndpointBindings']._options = None
    _MLFLOWSERVICE.methods_by_name['listEndpointBindings']._serialized_options = b'\362\206\031P\n4\n\003GET\022\'/mlflow/gateway/endpoints/bindings/list\032\004\010\003\020\000\020\001*\026List Endpoint Bindings'
    _MLFLOWSERVICE.methods_by_name['setGatewayEndpointTag']._options = None
    _MLFLOWSERVICE.methods_by_name['setGatewayEndpointTag']._serialized_options = b'\362\206\031M\n/\n\004POST\022!/mlflow/gateway/endpoints/set-tag\032\004\010\003\020\000\020\001*\030Gateway Set Endpoint Tag'
    _MLFLOWSERVICE.methods_by_name['deleteGatewayEndpointTag']._options = None
    _MLFLOWSERVICE.methods_by_name['deleteGatewayEndpointTag']._serialized_options = b'\362\206\031U\n4\n\006DELETE\022$/mlflow/gateway/endpoints/delete-tag\032\004\010\003\020\000\020\001*\033Gateway Delete Endpoint Tag'
    _VIEWTYPE._serialized_start=22427
    _VIEWTYPE._serialized_end=22481
    _SOURCETYPE._serialized_start=22483
    _SOURCETYPE._serialized_end=22556
    _RUNSTATUS._serialized_start=22558
    _RUNSTATUS._serialized_end=22635
    _TRACESTATUS._serialized_start=22637
    _TRACESTATUS._serialized_end=22716
    _LOGGEDMODELSTATUS._serialized_start=22719
    _LOGGEDMODELSTATUS._serialized_end=22857
    _METRIC._serialized_start=243
    _METRIC._serialized_end=419
    _PARAM._serialized_start=421
    _PARAM._serialized_end=456
    _RUN._serialized_start=459
    _RUN._serialized_end=598
    _RUNDATA._serialized_start=600
    _RUNDATA._serialized_end=703
    _RUNINPUTS._serialized_start=705
    _RUNINPUTS._serialized_end=804
    _RUNOUTPUTS._serialized_start=806
    _RUNOUTPUTS._serialized_end=862
    _RUNTAG._serialized_start=864
    _RUNTAG._serialized_end=900
    _EXPERIMENTTAG._serialized_start=902
    _EXPERIMENTTAG._serialized_end=945
    _RUNINFO._serialized_start=948
    _RUNINFO._serialized_end=1169
    _EXPERIMENT._serialized_start=1172
    _EXPERIMENT._serialized_end=1359
    _DATASETINPUT._serialized_start=1361
    _DATASETINPUT._serialized_end=1447
    _MODELINPUT._serialized_start=1449
    _MODELINPUT._serialized_end=1485
    _INPUTTAG._serialized_start=1487
    _INPUTTAG._serialized_end=1537
    _DATASET._serialized_start=1540
    _DATASET._serialized_end=1673
    _MODELOUTPUT._serialized_start=1675
    _MODELOUTPUT._serialized_end=1732
    _CREATEEXPERIMENT._serialized_start=1735
    _CREATEEXPERIMENT._serialized_end=1917
    _CREATEEXPERIMENT_RESPONSE._serialized_start=1839
    _CREATEEXPERIMENT_RESPONSE._serialized_end=1872
    _SEARCHEXPERIMENTS._serialized_start=1920
    _SEARCHEXPERIMENTS._serialized_end=2174
    _SEARCHEXPERIMENTS_RESPONSE._serialized_start=2053
    _SEARCHEXPERIMENTS_RESPONSE._serialized_end=2129
    _GETEXPERIMENT._serialized_start=2177
    _GETEXPERIMENT._serialized_end=2318
    _GETEXPERIMENT_RESPONSE._serialized_start=2223
    _GETEXPERIMENT_RESPONSE._serialized_end=2273
    _DELETEEXPERIMENT._serialized_start=2320
    _DELETEEXPERIMENT._serialized_end=2424
    _DELETEEXPERIMENT_RESPONSE._serialized_start=1839
    _DELETEEXPERIMENT_RESPONSE._serialized_end=1849
    _RESTOREEXPERIMENT._serialized_start=2426
    _RESTOREEXPERIMENT._serialized_end=2531
    _RESTOREEXPERIMENT_RESPONSE._serialized_start=1839
    _RESTOREEXPERIMENT_RESPONSE._serialized_end=1849
    _UPDATEEXPERIMENT._serialized_start=2533
    _UPDATEEXPERIMENT._serialized_end=2655
    _UPDATEEXPERIMENT_RESPONSE._serialized_start=1839
    _UPDATEEXPERIMENT_RESPONSE._serialized_end=1849
    _CREATERUN._serialized_start=2658
    _CREATERUN._serialized_end=2860
    _CREATERUN_RESPONSE._serialized_start=2779
    _CREATERUN_RESPONSE._serialized_end=2815
    _UPDATERUN._serialized_start=2863
    _UPDATERUN._serialized_end=3071
    _UPDATERUN_RESPONSE._serialized_start=2981
    _UPDATERUN_RESPONSE._serialized_end=3026
    _DELETERUN._serialized_start=3073
    _DELETERUN._serialized_end=3163
    _DELETERUN_RESPONSE._serialized_start=1839
    _DELETERUN_RESPONSE._serialized_end=1849
    _RESTORERUN._serialized_start=3165
    _RESTORERUN._serialized_end=3256
    _RESTORERUN_RESPONSE._serialized_start=1839
    _RESTORERUN_RESPONSE._serialized_end=1849
    _LOGMETRIC._serialized_start=3259
    _LOGMETRIC._serialized_end=3525
    _LOGMETRIC_RESPONSE._serialized_start=1839
    _LOGMETRIC_RESPONSE._serialized_end=1849
    _LOGPARAM._serialized_start=3528
    _LOGPARAM._serialized_end=3669
    _LOGPARAM_RESPONSE._serialized_start=1839
    _LOGPARAM_RESPONSE._serialized_end=1849
    _SETEXPERIMENTTAG._serialized_start=3672
    _SETEXPERIMENTTAG._serialized_end=3816
    _SETEXPERIMENTTAG_RESPONSE._serialized_start=1839
    _SETEXPERIMENTTAG_RESPONSE._serialized_end=1849
    _DELETEEXPERIMENTTAG._serialized_start=3818
    _DELETEEXPERIMENTTAG._serialized_end=3944
    _DELETEEXPERIMENTTAG_RESPONSE._serialized_start=1839
    _DELETEEXPERIMENTTAG_RESPONSE._serialized_end=1849
    _SETTAG._serialized_start=3947
    _SETTAG._serialized_end=4086
    _SETTAG_RESPONSE._serialized_start=1839
    _SETTAG_RESPONSE._serialized_end=1849
    _DELETETAG._serialized_start=4088
    _DELETETAG._serialized_end=4197
    _DELETETAG_RESPONSE._serialized_start=1839
    _DELETETAG_RESPONSE._serialized_end=1849
    _GETRUN._serialized_start=4199
    _GETRUN._serialized_end=4324
    _GETRUN_RESPONSE._serialized_start=2779
    _GETRUN_RESPONSE._serialized_end=2815
    _SEARCHRUNS._serialized_start=4327
    _SEARCHRUNS._serialized_end=4607
    _SEARCHRUNS_RESPONSE._serialized_start=4500
    _SEARCHRUNS_RESPONSE._serialized_end=4562
    _LISTARTIFACTS._serialized_start=4610
    _LISTARTIFACTS._serialized_end=4826
    _LISTARTIFACTS_RESPONSE._serialized_start=4695
    _LISTARTIFACTS_RESPONSE._serialized_end=4781
    _FILEINFO._serialized_start=4828
    _FILEINFO._serialized_end=4887
    _GETMETRICHISTORY._serialized_start=4890
    _GETMETRICHISTORY._serialized_end=5124
    _GETMETRICHISTORY_RESPONSE._serialized_start=5011
    _GETMETRICHISTORY_RESPONSE._serialized_end=5079
    _METRICWITHRUNID._serialized_start=5126
    _METRICWITHRUNID._serialized_end=5223
    _GETMETRICHISTORYBULKINTERVAL._serialized_start=5226
    _GETMETRICHISTORYBULKINTERVAL._serialized_end=5457
    _GETMETRICHISTORYBULKINTERVAL_RESPONSE._serialized_start=5360
    _GETMETRICHISTORYBULKINTERVAL_RESPONSE._serialized_end=5412
    _LOGBATCH._serialized_start=5460
    _LOGBATCH._serialized_end=5637
    _LOGBATCH_RESPONSE._serialized_start=1839
    _LOGBATCH_RESPONSE._serialized_end=1849
    _LOGMODEL._serialized_start=5639
    _LOGMODEL._serialized_end=5742
    _LOGMODEL_RESPONSE._serialized_start=1839
    _LOGMODEL_RESPONSE._serialized_end=1849
    _LOGINPUTS._serialized_start=5745
    _LOGINPUTS._serialized_end=5917
    _LOGINPUTS_RESPONSE._serialized_start=1839
    _LOGINPUTS_RESPONSE._serialized_end=1849
    _LOGOUTPUTS._serialized_start=5920
    _LOGOUTPUTS._serialized_end=6048
    _LOGOUTPUTS_RESPONSE._serialized_start=1839
    _LOGOUTPUTS_RESPONSE._serialized_end=1849
    _GETEXPERIMENTBYNAME._serialized_start=6051
    _GETEXPERIMENTBYNAME._serialized_end=6200
    _GETEXPERIMENTBYNAME_RESPONSE._serialized_start=2223
    _GETEXPERIMENTBYNAME_RESPONSE._serialized_end=2273
    _CREATEASSESSMENT._serialized_start=6203
    _CREATEASSESSMENT._serialized_end=6388
    _CREATEASSESSMENT_RESPONSE._serialized_start=6281
    _CREATEASSESSMENT_RESPONSE._serialized_end=6343
    _UPDATEASSESSMENT._serialized_start=6391
    _UPDATEASSESSMENT._serialized_end=6631
    _UPDATEASSESSMENT_RESPONSE._serialized_start=6281
    _UPDATEASSESSMENT_RESPONSE._serialized_end=6343
    _DELETEASSESSMENT._serialized_start=6634
    _DELETEASSESSMENT._serialized_end=6762
    _DELETEASSESSMENT_RESPONSE._serialized_start=1839
    _DELETEASSESSMENT_RESPONSE._serialized_end=1849
    _GETASSESSMENTREQUEST._serialized_start=6765
    _GETASSESSMENTREQUEST._serialized_end=6949
    _GETASSESSMENTREQUEST_RESPONSE._serialized_start=6281
    _GETASSESSMENTREQUEST_RESPONSE._serialized_end=6343
    _TRACEINFO._serialized_start=6952
    _TRACEINFO._serialized_end=7180
    _TRACEREQUESTMETADATA._serialized_start=7182
    _TRACEREQUESTMETADATA._serialized_end=7232
    _TRACETAG._serialized_start=7234
    _TRACETAG._serialized_end=7272
    _STARTTRACE._serialized_start=7275
    _STARTTRACE._serialized_end=7516
    _STARTTRACE_RESPONSE._serialized_start=7422
    _STARTTRACE_RESPONSE._serialized_end=7471
    _ENDTRACE._serialized_start=7519
    _ENDTRACE._serialized_end=7792
    _ENDTRACE_RESPONSE._serialized_start=7422
    _ENDTRACE_RESPONSE._serialized_end=7471
    _GETTRACEINFO._serialized_start=7795
    _GETTRACEINFO._serialized_end=7925
    _GETTRACEINFO_RESPONSE._serialized_start=7422
    _GETTRACEINFO_RESPONSE._serialized_end=7471
    _GETTRACEINFOV3._serialized_start=7927
    _GETTRACEINFOV3._serialized_end=8048
    _GETTRACEINFOV3_RESPONSE._serialized_start=7963
    _GETTRACEINFOV3_RESPONSE._serialized_end=8003
    _BATCHGETTRACES._serialized_start=8050
    _BATCHGETTRACES._serialized_end=8173
    _BATCHGETTRACES_RESPONSE._serialized_start=8087
    _BATCHGETTRACES_RESPONSE._serialized_end=8128
    _GETTRACE._serialized_start=8176
    _GETTRACE._serialized_end=8327
    _GETTRACE_RESPONSE._serialized_start=7963
    _GETTRACE_RESPONSE._serialized_end=8003
    _SEARCHTRACES._serialized_start=8330
    _SEARCHTRACES._serialized_end=8565
    _SEARCHTRACES_RESPONSE._serialized_start=8450
    _SEARCHTRACES_RESPONSE._serialized_end=8520
    _SEARCHUNIFIEDTRACES._serialized_start=8568
    _SEARCHUNIFIEDTRACES._serialized_end=8866
    _SEARCHUNIFIEDTRACES_RESPONSE._serialized_start=8450
    _SEARCHUNIFIEDTRACES_RESPONSE._serialized_end=8520
    _GETONLINETRACEDETAILS._serialized_start=8869
    _GETONLINETRACEDETAILS._serialized_end=9062
    _GETONLINETRACEDETAILS_RESPONSE._serialized_start=9032
    _GETONLINETRACEDETAILS_RESPONSE._serialized_end=9062
    _DELETETRACES._serialized_start=9065
    _DELETETRACES._serialized_end=9260
    _DELETETRACES_RESPONSE._serialized_start=9181
    _DELETETRACES_RESPONSE._serialized_end=9215
    _DELETETRACESV3._serialized_start=9263
    _DELETETRACESV3._serialized_end=9460
    _DELETETRACESV3_RESPONSE._serialized_start=9181
    _DELETETRACESV3_RESPONSE._serialized_end=9215
    _CALCULATETRACEFILTERCORRELATION._serialized_start=9463
    _CALCULATETRACEFILTERCORRELATION._serialized_end=9772
    _CALCULATETRACEFILTERCORRELATION_RESPONSE._serialized_start=9592
    _CALCULATETRACEFILTERCORRELATION_RESPONSE._serialized_end=9727
    _SETTRACETAG._serialized_start=9774
    _SETTRACETAG._serialized_end=9892
    _SETTRACETAG_RESPONSE._serialized_start=1839
    _SETTRACETAG_RESPONSE._serialized_end=1849
    _SETTRACETAGV3._serialized_start=9895
    _SETTRACETAGV3._serialized_end=10031
    _SETTRACETAGV3_RESPONSE._serialized_start=1839
    _SETTRACETAGV3_RESPONSE._serialized_end=1849
    _DELETETRACETAG._serialized_start=10033
    _DELETETRACETAG._serialized_end=10139
    _DELETETRACETAG_RESPONSE._serialized_start=1839
    _DELETETRACETAG_RESPONSE._serialized_end=1849
    _DELETETRACETAGV3._serialized_start=10141
    _DELETETRACETAGV3._serialized_end=10265
    _DELETETRACETAGV3_RESPONSE._serialized_start=1839
    _DELETETRACETAGV3_RESPONSE._serialized_end=1849
    _TRACE._serialized_start=10267
    _TRACE._serialized_end=10366
    _TRACELOCATION._serialized_start=10369
    _TRACELOCATION._serialized_end=10807
    _TRACELOCATION_MLFLOWEXPERIMENTLOCATION._serialized_start=10591
    _TRACELOCATION_MLFLOWEXPERIMENTLOCATION._serialized_end=10640
    _TRACELOCATION_INFERENCETABLELOCATION._serialized_start=10642
    _TRACELOCATION_INFERENCETABLELOCATION._serialized_end=10691
    _TRACELOCATION_TRACELOCATIONTYPE._serialized_start=10693
    _TRACELOCATION_TRACELOCATIONTYPE._serialized_end=10793
    _TRACEINFOV3._serialized_start=10810
    _TRACEINFOV3._serialized_end=11477
    _TRACEINFOV3_TRACEMETADATAENTRY._serialized_start=11312
    _TRACEINFOV3_TRACEMETADATAENTRY._serialized_end=11364
    _TRACEINFOV3_TAGSENTRY._serialized_start=11366
    _TRACEINFOV3_TAGSENTRY._serialized_end=11409
    _TRACEINFOV3_STATE._serialized_start=11411
    _TRACEINFOV3_STATE._serialized_end=11477
    _STARTTRACEV3._serialized_start=11479
    _STARTTRACEV3._serialized_end=11571
    _STARTTRACEV3_RESPONSE._serialized_start=7963
    _STARTTRACEV3_RESPONSE._serialized_end=8003
    _LINKTRACESTORUN._serialized_start=11573
    _LINKTRACESTORUN._serialized_end=11643
    _LINKTRACESTORUN_RESPONSE._serialized_start=1839
    _LINKTRACESTORUN_RESPONSE._serialized_end=1849
    _LINKPROMPTSTOTRACE._serialized_start=11646
    _LINKPROMPTSTOTRACE._serialized_end=11835
    _LINKPROMPTSTOTRACE_PROMPTVERSIONREF._serialized_start=11762
    _LINKPROMPTSTOTRACE_PROMPTVERSIONREF._serialized_end=11823
    _LINKPROMPTSTOTRACE_RESPONSE._serialized_start=1839
    _LINKPROMPTSTOTRACE_RESPONSE._serialized_end=1849
    _DATASETSUMMARY._serialized_start=11837
    _DATASETSUMMARY._serialized_end=11941
    _SEARCHDATASETS._serialized_start=11944
    _SEARCHDATASETS._serialized_end=12092
    _SEARCHDATASETS_RESPONSE._serialized_start=11986
    _SEARCHDATASETS_RESPONSE._serialized_end=12047
    _CREATELOGGEDMODEL._serialized_start=12095
    _CREATELOGGEDMODEL._serialized_end=12377
    _CREATELOGGEDMODEL_RESPONSE._serialized_start=12286
    _CREATELOGGEDMODEL_RESPONSE._serialized_end=12332
    _FINALIZELOGGEDMODEL._serialized_start=12380
    _FINALIZELOGGEDMODEL._serialized_end=12567
    _FINALIZELOGGEDMODEL_RESPONSE._serialized_start=12286
    _FINALIZELOGGEDMODEL_RESPONSE._serialized_end=12332
    _GETLOGGEDMODEL._serialized_start=12570
    _GETLOGGEDMODEL._serialized_end=12703
    _GETLOGGEDMODEL_RESPONSE._serialized_start=12286
    _GETLOGGEDMODEL_RESPONSE._serialized_end=12332
    _DELETELOGGEDMODEL._serialized_start=12705
    _DELETELOGGEDMODEL._serialized_end=12805
    _DELETELOGGEDMODEL_RESPONSE._serialized_start=1839
    _DELETELOGGEDMODEL_RESPONSE._serialized_end=1849
    _SEARCHLOGGEDMODELS._serialized_start=12808
    _SEARCHLOGGEDMODELS._serialized_end=13311
    _SEARCHLOGGEDMODELS_DATASET._serialized_start=13023
    _SEARCHLOGGEDMODELS_DATASET._serialized_end=13084
    _SEARCHLOGGEDMODELS_ORDERBY._serialized_start=13086
    _SEARCHLOGGEDMODELS_ORDERBY._serialized_end=13192
    _SEARCHLOGGEDMODELS_RESPONSE._serialized_start=13194
    _SEARCHLOGGEDMODELS_RESPONSE._serialized_end=13266
    _SETLOGGEDMODELTAGS._serialized_start=13314
    _SETLOGGEDMODELTAGS._serialized_end=13489
    _SETLOGGEDMODELTAGS_RESPONSE._serialized_start=12286
    _SETLOGGEDMODELTAGS_RESPONSE._serialized_end=12332
    _DELETELOGGEDMODELTAG._serialized_start=13491
    _DELETELOGGEDMODELTAG._serialized_end=13617
    _DELETELOGGEDMODELTAG_RESPONSE._serialized_start=1839
    _DELETELOGGEDMODELTAG_RESPONSE._serialized_end=1849
    _LISTLOGGEDMODELARTIFACTS._serialized_start=13620
    _LISTLOGGEDMODELARTIFACTS._serialized_end=13856
    _LISTLOGGEDMODELARTIFACTS_RESPONSE._serialized_start=4695
    _LISTLOGGEDMODELARTIFACTS_RESPONSE._serialized_end=4781
    _LOGLOGGEDMODELPARAMSREQUEST._serialized_start=13859
    _LOGLOGGEDMODELPARAMSREQUEST._serialized_end=14015
    _LOGLOGGEDMODELPARAMSREQUEST_RESPONSE._serialized_start=1839
    _LOGLOGGEDMODELPARAMSREQUEST_RESPONSE._serialized_end=1849
    _LOGGEDMODEL._serialized_start=14017
    _LOGGEDMODEL._serialized_end=14108
    _LOGGEDMODELINFO._serialized_start=14111
    _LOGGEDMODELINFO._serialized_end=14499
    _LOGGEDMODELTAG._serialized_start=14501
    _LOGGEDMODELTAG._serialized_end=14545
    _LOGGEDMODELREGISTRATIONINFO._serialized_start=14547
    _LOGGEDMODELREGISTRATIONINFO._serialized_end=14607
    _LOGGEDMODELDATA._serialized_start=14609
    _LOGGEDMODELDATA._serialized_end=14705
    _LOGGEDMODELPARAMETER._serialized_start=14707
    _LOGGEDMODELPARAMETER._serialized_end=14757
    _SEARCHTRACESV3._serialized_start=14760
    _SEARCHTRACESV3._serialized_end=15017
    _SEARCHTRACESV3_RESPONSE._serialized_start=14900
    _SEARCHTRACESV3_RESPONSE._serialized_end=14972
    _CREATEDATASET._serialized_start=15020
    _CREATEDATASET._serialized_end=15332
    _CREATEDATASET_RESPONSE._serialized_start=15234
    _CREATEDATASET_RESPONSE._serialized_end=15287
    _GETDATASET._serialized_start=15335
    _GETDATASET._serialized_end=15518
    _GETDATASET_RESPONSE._serialized_start=15395
    _GETDATASET_RESPONSE._serialized_end=15473
    _DELETEDATASET._serialized_start=15520
    _DELETEDATASET._serialized_end=15618
    _DELETEDATASET_RESPONSE._serialized_start=1839
    _DELETEDATASET_RESPONSE._serialized_end=1849
    _SEARCHEVALUATIONDATASETS._serialized_start=15621
    _SEARCHEVALUATIONDATASETS._serialized_end=15885
    _SEARCHEVALUATIONDATASETS_RESPONSE._serialized_start=15761
    _SEARCHEVALUATIONDATASETS_RESPONSE._serialized_end=15840
    _SETDATASETTAGS._serialized_start=15888
    _SETDATASETTAGS._serialized_end=16050
    _SETDATASETTAGS_RESPONSE._serialized_start=15234
    _SETDATASETTAGS_RESPONSE._serialized_end=15287
    _DELETEDATASETTAG._serialized_start=16052
    _DELETEDATASETTAG._serialized_end=16172
    _DELETEDATASETTAG_RESPONSE._serialized_start=1839
    _DELETEDATASETTAG_RESPONSE._serialized_end=1849
    _UPSERTDATASETRECORDS._serialized_start=16175
    _UPSERTDATASETRECORDS._serialized_end=16370
    _UPSERTDATASETRECORDS_RESPONSE._serialized_start=16268
    _UPSERTDATASETRECORDS_RESPONSE._serialized_end=16325
    _GETDATASETEXPERIMENTIDS._serialized_start=16373
    _GETDATASETEXPERIMENTIDS._serialized_end=16505
    _GETDATASETEXPERIMENTIDS_RESPONSE._serialized_start=16426
    _GETDATASETEXPERIMENTIDS_RESPONSE._serialized_end=16460
    _GETDATASETRECORDS._serialized_start=16508
    _GETDATASETRECORDS._serialized_end=16699
    _GETDATASETRECORDS_RESPONSE._serialized_start=16602
    _GETDATASETRECORDS_RESPONSE._serialized_end=16654
    _ADDDATASETTOEXPERIMENTS._serialized_start=16702
    _ADDDATASETTOEXPERIMENTS._serialized_end=16877
    _ADDDATASETTOEXPERIMENTS_RESPONSE._serialized_start=15234
    _ADDDATASETTOEXPERIMENTS_RESPONSE._serialized_end=15287
    _REMOVEDATASETFROMEXPERIMENTS._serialized_start=16880
    _REMOVEDATASETFROMEXPERIMENTS._serialized_end=17060
    _REMOVEDATASETFROMEXPERIMENTS_RESPONSE._serialized_start=15234
    _REMOVEDATASETFROMEXPERIMENTS_RESPONSE._serialized_end=15287
    _REGISTERSCORER._serialized_start=17063
    _REGISTERSCORER._serialized_end=17324
    _REGISTERSCORER_RESPONSE._serialized_start=17146
    _REGISTERSCORER_RESPONSE._serialized_end=17279
    _LISTSCORERS._serialized_start=17326
    _LISTSCORERS._serialized_end=17452
    _LISTSCORERS_RESPONSE._serialized_start=17364
    _LISTSCORERS_RESPONSE._serialized_end=17407
    _LISTSCORERVERSIONS._serialized_start=17455
    _LISTSCORERVERSIONS._serialized_end=17602
    _LISTSCORERVERSIONS_RESPONSE._serialized_start=17364
    _LISTSCORERVERSIONS_RESPONSE._serialized_end=17407
    _GETSCORER._serialized_start=17605
    _GETSCORER._serialized_end=17759
    _GETSCORER_RESPONSE._serialized_start=17672
    _GETSCORER_RESPONSE._serialized_end=17714
    _DELETESCORER._serialized_start=17761
    _DELETESCORER._serialized_end=17886
    _DELETESCORER_RESPONSE._serialized_start=1839
    _DELETESCORER_RESPONSE._serialized_end=1849
    _SCORER._serialized_start=17889
    _SCORER._serialized_end=18034
    _GATEWAYSECRETINFO._serialized_start=18037
    _GATEWAYSECRETINFO._serialized_end=18351
    _GATEWAYSECRETINFO_MASKEDVALUESENTRY._serialized_start=18300
    _GATEWAYSECRETINFO_MASKEDVALUESENTRY._serialized_end=18351
    _GATEWAYMODELDEFINITION._serialized_start=18354
    _GATEWAYMODELDEFINITION._serialized_end=18589
    _GATEWAYENDPOINTMODELMAPPING._serialized_start=18592
    _GATEWAYENDPOINTMODELMAPPING._serialized_end=18805
    _GATEWAYENDPOINT._serialized_start=18808
    _GATEWAYENDPOINT._serialized_end=19053
    _GATEWAYENDPOINTTAG._serialized_start=19055
    _GATEWAYENDPOINTTAG._serialized_end=19103
    _GATEWAYENDPOINTBINDING._serialized_start=19106
    _GATEWAYENDPOINTBINDING._serialized_end=19285
    _CREATEGATEWAYSECRET._serialized_start=19288
    _CREATEGATEWAYSECRET._serialized_end=19592
    _CREATEGATEWAYSECRET_SECRETVALUEENTRY._serialized_start=19464
    _CREATEGATEWAYSECRET_SECRETVALUEENTRY._serialized_end=19514
    _CREATEGATEWAYSECRET_RESPONSE._serialized_start=19516
    _CREATEGATEWAYSECRET_RESPONSE._serialized_end=19569
    _GETGATEWAYSECRETINFO._serialized_start=19594
    _GETGATEWAYSECRETINFO._serialized_end=19711
    _GETGATEWAYSECRETINFO_RESPONSE._serialized_start=19516
    _GETGATEWAYSECRETINFO_RESPONSE._serialized_end=19569
    _UPDATEGATEWAYSECRET._serialized_start=19714
    _UPDATEGATEWAYSECRET._serialized_end=19998
    _UPDATEGATEWAYSECRET_SECRETVALUEENTRY._serialized_start=19464
    _UPDATEGATEWAYSECRET_SECRETVALUEENTRY._serialized_end=19514
    _UPDATEGATEWAYSECRET_RESPONSE._serialized_start=19516
    _UPDATEGATEWAYSECRET_RESPONSE._serialized_end=19569
    _DELETEGATEWAYSECRET._serialized_start=20000
    _DELETEGATEWAYSECRET._serialized_end=20052
    _DELETEGATEWAYSECRET_RESPONSE._serialized_start=1839
    _DELETEGATEWAYSECRET_RESPONSE._serialized_end=1849
    _LISTGATEWAYSECRETINFOS._serialized_start=20054
    _LISTGATEWAYSECRETINFOS._serialized_end=20152
    _LISTGATEWAYSECRETINFOS_RESPONSE._serialized_start=20098
    _LISTGATEWAYSECRETINFOS_RESPONSE._serialized_end=20152
    _CREATEGATEWAYMODELDEFINITION._serialized_start=20155
    _CREATEGATEWAYMODELDEFINITION._serialized_end=20346
    _CREATEGATEWAYMODELDEFINITION_RESPONSE._serialized_start=20278
    _CREATEGATEWAYMODELDEFINITION_RESPONSE._serialized_end=20346
    _GETGATEWAYMODELDEFINITION._serialized_start=20348
    _GETGATEWAYMODELDEFINITION._serialized_end=20474
    _GETGATEWAYMODELDEFINITION_RESPONSE._serialized_start=20278
    _GETGATEWAYMODELDEFINITION_RESPONSE._serialized_end=20346
    _LISTGATEWAYMODELDEFINITIONS._serialized_start=20477
    _LISTGATEWAYMODELDEFINITIONS._serialized_end=20614
    _LISTGATEWAYMODELDEFINITIONS_RESPONSE._serialized_start=20545
    _LISTGATEWAYMODELDEFINITIONS_RESPONSE._serialized_end=20614
    _UPDATEGATEWAYMODELDEFINITION._serialized_start=20617
    _UPDATEGATEWAYMODELDEFINITION._serialized_end=20837
    _UPDATEGATEWAYMODELDEFINITION_RESPONSE._serialized_start=20278
    _UPDATEGATEWAYMODELDEFINITION_RESPONSE._serialized_end=20346
    _DELETEGATEWAYMODELDEFINITION._serialized_start=20839
    _DELETEGATEWAYMODELDEFINITION._serialized_end=20910
    _DELETEGATEWAYMODELDEFINITION_RESPONSE._serialized_start=1839
    _DELETEGATEWAYMODELDEFINITION_RESPONSE._serialized_end=1849
    _CREATEGATEWAYENDPOINT._serialized_start=20913
    _CREATEGATEWAYENDPOINT._serialized_end=21055
    _CREATEGATEWAYENDPOINT_RESPONSE._serialized_start=21002
    _CREATEGATEWAYENDPOINT_RESPONSE._serialized_end=21055
    _GETGATEWAYENDPOINT._serialized_start=21057
    _GETGATEWAYENDPOINT._serialized_end=21167
    _GETGATEWAYENDPOINT_RESPONSE._serialized_start=21002
    _GETGATEWAYENDPOINT_RESPONSE._serialized_end=21055
    _UPDATEGATEWAYENDPOINT._serialized_start=21170
    _UPDATEGATEWAYENDPOINT._serialized_end=21303
    _UPDATEGATEWAYENDPOINT_RESPONSE._serialized_start=21002
    _UPDATEGATEWAYENDPOINT_RESPONSE._serialized_end=21055
    _DELETEGATEWAYENDPOINT._serialized_start=21305
    _DELETEGATEWAYENDPOINT._serialized_end=21361
    _DELETEGATEWAYENDPOINT_RESPONSE._serialized_start=1839
    _DELETEGATEWAYENDPOINT_RESPONSE._serialized_end=1849
    _LISTGATEWAYENDPOINTS._serialized_start=21363
    _LISTGATEWAYENDPOINTS._serialized_end=21478
    _LISTGATEWAYENDPOINTS_RESPONSE._serialized_start=21424
    _LISTGATEWAYENDPOINTS_RESPONSE._serialized_end=21478
    _ATTACHMODELTOGATEWAYENDPOINT._serialized_start=21481
    _ATTACHMODELTOGATEWAYENDPOINT._serialized_end=21663
    _ATTACHMODELTOGATEWAYENDPOINT_RESPONSE._serialized_start=21599
    _ATTACHMODELTOGATEWAYENDPOINT_RESPONSE._serialized_end=21663
    _DETACHMODELFROMGATEWAYENDPOINT._serialized_start=21665
    _DETACHMODELFROMGATEWAYENDPOINT._serialized_end=21759
    _DETACHMODELFROMGATEWAYENDPOINT_RESPONSE._serialized_start=1839
    _DETACHMODELFROMGATEWAYENDPOINT_RESPONSE._serialized_end=1849
    _CREATEGATEWAYENDPOINTBINDING._serialized_start=21762
    _CREATEGATEWAYENDPOINTBINDING._serialized_end=21938
    _CREATEGATEWAYENDPOINTBINDING_RESPONSE._serialized_start=21879
    _CREATEGATEWAYENDPOINTBINDING_RESPONSE._serialized_end=21938
    _DELETEGATEWAYENDPOINTBINDING._serialized_start=21940
    _DELETEGATEWAYENDPOINTBINDING._serialized_end=22047
    _DELETEGATEWAYENDPOINTBINDING_RESPONSE._serialized_start=1839
    _DELETEGATEWAYENDPOINTBINDING_RESPONSE._serialized_end=1849
    _LISTGATEWAYENDPOINTBINDINGS._serialized_start=22050
    _LISTGATEWAYENDPOINTBINDINGS._serialized_end=22206
    _LISTGATEWAYENDPOINTBINDINGS_RESPONSE._serialized_start=22146
    _LISTGATEWAYENDPOINTBINDINGS_RESPONSE._serialized_end=22206
    _SETGATEWAYENDPOINTTAG._serialized_start=22208
    _SETGATEWAYENDPOINTTAG._serialized_end=22292
    _SETGATEWAYENDPOINTTAG_RESPONSE._serialized_start=1839
    _SETGATEWAYENDPOINTTAG_RESPONSE._serialized_end=1849
    _DELETEGATEWAYENDPOINTTAG._serialized_start=22294
    _DELETEGATEWAYENDPOINTTAG._serialized_end=22366
    _DELETEGATEWAYENDPOINTTAG_RESPONSE._serialized_start=1839
    _DELETEGATEWAYENDPOINTTAG_RESPONSE._serialized_end=1849
    _GETSECRETSCONFIG._serialized_start=22368
    _GETSECRETSCONFIG._serialized_end=22425
    _GETSECRETSCONFIG_RESPONSE._serialized_start=22388
    _GETSECRETSCONFIG_RESPONSE._serialized_end=22425
    _MLFLOWSERVICE._serialized_start=22860
    _MLFLOWSERVICE._serialized_end=38738
  MlflowService = service_reflection.GeneratedServiceType('MlflowService', (_service.Service,), dict(
    DESCRIPTOR = _MLFLOWSERVICE,
    __module__ = 'service_pb2'
    ))

  MlflowService_Stub = service_reflection.GeneratedServiceStubType('MlflowService_Stub', (MlflowService,), dict(
    DESCRIPTOR = _MLFLOWSERVICE,
    __module__ = 'service_pb2'
    ))


  # @@protoc_insertion_point(module_scope)

